/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: c_projects.c
 *            DESCRIPTION: common (to any port) routines for XNotesPlus
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPWriteProjects, XPSetUpProjects
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_CPROJECTS_S

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include <ctype.h>

#ifdef DO_GTK
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#else
#include "motif.h"
#endif

/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */

/* === Private routine prototypes === */

/* === Global Variables === */
extern AppRes			app_res;
extern ProjectType	*projects_list;

/* === Static Variables === */


/*========================================================================
 *	Name:			XPSetUpProjects
 *	Prototype:	XPSetUpProjects()
 *					
 *	Description:
 * Read in the projects file into a link list of projects, if the file
 * exists.
 *
 *	Global Variables:
 *	ProjectType		*projects_list
 *
 *	Notes:
 * This routine gets called before app_res.note_dir is properly set up.
 *========================================================================*/
void
XPSetUpProjects()
{
#ifdef DEBUG
	char	*fname="XPSetUpProjects()";
#endif

	struct stat			*stat_buf;
	char					filename[MAXPATHLEN*2];
	FILE					*fp;
	char					buf[256], *string;
	ProjectType			*item, *lastitem=NULL;
	char					*name, *color;

#ifdef DO_GTK
	float					red, green, blue;
#endif

	DBGEnter();

	/*
	 * Check to see if the projects file exists.
	 */
	sprintf(filename, "%s/%s", app_res.note_dir, DefaultProjectFile);

	DBGPrintf(DBG_INFO,("projects file: %s\n", filename));

	stat_buf = (struct stat *) malloc( sizeof(struct stat) );
	if ( stat ( filename, stat_buf ) == -1 )
	{
		DBGPrintf(DBG_INFO, ("No projects file found.\n"));
		projects_list = NULL;
		return;
	}
	free(stat_buf);

	/*
	 * Open the file.  Read in each line and parse into its name/value
	 * pairs.  Add it to the link list.
	 */
	fp = fopen(filename, "r");

	/*
	 * Verify the file magic id.
	 */
	fgets(buf, 256, fp);
	string=strtok(buf,"\n");
	if ( strcmp(string, StickyProjMagic_v3) != 0 )
	{
		projects_list = NULL;
		return;
	}
	
	while ( fgets(buf, 256, fp) != NULL )
	{
		if ( ! feof(fp) )
		{
			string = XPSafeAlloc(strlen(buf)+1);
			strcpy(string, buf);

			/*
			 * Strip off the newline character.
			 */
			string=strtok(string,"\n");
			name = strtok(string,":");
			color = strtok(NULL,":");

#ifdef DO_GTK
			sscanf(color,"%f %f %f", &red, &green, &blue);
#endif

			item = (ProjectType *)XPSafeAlloc(sizeof(ProjectType));
			item->name = XPSafeAlloc(strlen(name)+1);
			strcpy(item->name, name);

#ifdef DO_GTK
			item->color[0] = (gdouble)red;
			item->color[1] = (gdouble)green;
			item->color[2] = (gdouble)blue;
#else
			item->color = XPSafeAlloc(strlen(color)+1);
			strcpy(item->color, color);
#endif

			item->next = NULL;

			free(string);

			/*
			 * Add it to link list.
			 */
			if ( projects_list == NULL )
				projects_list = item;
			else
				lastitem->next = item;
			lastitem = item;

		}
	}
	fclose(fp);

}


/*========================================================================
 *	Name:			XPWriteProjects
 *	Prototype:	XPWriteProjects()
 *					
 *	Description:
 * Write the projects file from the link list of projects.
 *
 *	Global Variables:
 *	ProjectType		*projects_list
 *
 *	Notes:
 * Since the projects list will always be relatively small (who has
 * 10,000 projects for a sticky notes program?) we can do this every
 * time a project entry is updated.  The overhead, from a user perspective,
 * is unnoticable.
 *========================================================================*/
void
XPWriteProjects()
{
#ifdef DEBUG
	char	*fname="XPWriteProjects()";
#endif

	char					filename[MAXPATHLEN*2];
	FILE					*fp;
	ProjectType			*item;

	DBGEnter();

	/*
	 * Check to see if the projects file exists.
	 */
	sprintf(filename, "%s/%s", app_res.note_dir, DefaultProjectFile);

	DBGPrintf(DBG_INFO,("projects file: %s\n", filename));

	/*
	 * Open the file.  
	 */
	fp = fopen(filename, "w");
	if ( fp == NULL )
		perror("fopen failed");

	/*
	 * Write the magic id to the file.
	 */
	fprintf(fp, "%s\n", StickyProjMagic_v3);

	/*
	 * Run the link list and write each link to the file.
	 */
	item = projects_list;
	while (item != NULL )
	{

#ifdef DO_GTK
		fprintf(fp, "%s:%f %f %f\n", 
			item->name, (float)item->color[0], 
			(float)item->color[1], (float)item->color[2]);
#else
		fprintf(fp, "%s:%s\n", item->name, item->color);
#endif

		item = item->next;
	}

	/*
	 * Close the projects file.
	 */
	fclose(fp);

}

