/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: c_utils.c
 *            DESCRIPTION: common (to any port) routines for XNotesPlus
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPSafeAlloc, XPSetNoteDir, XPMakeFname,
 *								 : XPPopUpList, XPNoop, XPRingBell, XPErrPopUp
 *								 : XPShowDisabledCmds, XPTestCmd, XPConvertCase
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_CUTILS_S

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include <ctype.h>

#ifdef DO_GTK
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#else
#include "motif.h"
#endif

/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */
void XPPopUpWindow ();


/* === Private routine prototypes === */

/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;

#ifdef DO_MOTIF
extern Display			*display;
#endif


/* === Static Variables === */



/*========================================================================
 *	Name:			XPSafeAlloc
 *	Prototype:	char *XPSafeAlloc ( gint nbytes )
 *					
 *	Description:
 * Allocate n bytes of memory, exit if we run out.
 *
 *	Input Arguments:
 *	gint	nbytes		number of bytes to allocate				
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
char *
XPSafeAlloc(
	register int nbytes
)
{
	register char *s;


	if ((s = (char *)malloc(nbytes)) == NULL) 
	{
		fprintf(stderr, "xnotes+: out of memory!\n");
		exit(1);
	}

	bzero(s, nbytes);
	return(s);
}


/*========================================================================
 *	Name:			XPSetNoteDir 
 *	Prototype:	void XPSetNoteDir()
 *					
 *	Description:
 * Determine the path to the note directory.  Validate it exists or create
 * it if necessary.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *		AppRes	app_res			global configuration parameters.
 *
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPSetNoteDir()
{
#ifdef DEBUG
	char	*fname="XPSetNoteDir()";
#endif

	char				*getenv();
	char				dirname[MAXPATHLEN];
	struct stat		*stat_buf;
	int				rc;

	DBGEnter();

	/*
	 * If it's an absolute path name,
	 * we're done.
	 */
	if (app_res.note_dir[0] != '/')
	{

		if (app_res.home_dir == NULL) 
		{
			printf(XP_E_NO_HOME_DIR_1, XP_S_PROG_NAME);
			printf(XP_E_NO_HOME_DIR_2, XP_S_PROG_NAME);
			printf(XP_E_NO_HOME_DIR_3, XP_S_PROG_NAME);
			exit(1);
		}

		/*
		 * Save the path.
		 */
		sprintf(dirname, "%s/%s", app_res.home_dir, app_res.note_dir);

/*
		Why doesn't this work anymore?  Probably due to the way X handles
		automatic command line parsing.
		if ( app_res.note_dir != NULL )
			free(app_res.note_dir);
*/

		app_res.note_dir = XPSafeAlloc(strlen(dirname) + 1);
		strcpy(app_res.note_dir, dirname);
	}
	
	/* now we have the name of the directory pointed at by app_res.note_dir
	 * so check if it exists, and if it doesn't, try to create it
	 */

	stat_buf = (struct stat *) malloc( sizeof(struct stat) );
	if ( ( rc = stat ( app_res.note_dir, stat_buf ) ) !=0 ) 
	{
		switch ( errno ) 
		{
			case EACCES:
				printf( XP_E_SEARCH_DENY_1, XP_S_PROG_NAME);
				printf( XP_E_SEARCH_DENY_2, XP_S_PROG_NAME, app_res.note_dir);
				printf( XP_E_SEARCH_DENY_3, XP_S_PROG_NAME);
				free ( stat_buf );
				exit ( -1 );
				break;

			case ENOTDIR:
				printf( XP_E_PATH_INVALID_1, XP_S_PROG_NAME, app_res.note_dir);
				printf( XP_E_PATH_INVALID_2 );
				free ( stat_buf );
				exit ( -1 );
				break;

			case ENOENT:
				/* if it doesn't exist, try to create it */
				printf( XP_E_PATH_EXIST_1, XP_S_PROG_NAME, app_res.note_dir);
				printf( XP_E_PATH_EXIST_2, XP_S_PROG_NAME);
				rc = mkdir ( app_res.note_dir, 0755 );
				if ( rc != 0 ) 
				{
					printf( XP_E_CREATE_FAIL, XP_S_PROG_NAME, app_res.note_dir);
					free ( stat_buf );
					exit ( -1 );
				}
				break;
			default:
				printf( XP_E_UNKNOWN_STAT, XP_S_PROG_NAME, errno, app_res.note_dir);
				exit ( -1 );
				break;
		}
	}
}


/*========================================================================
 *	Name:			XPMakeFname
 *	Prototype:	char *XPMakeFname(int index)
 *					
 *	Description:
 * Make a file name from a note index number.
 *
 *	Input Arguments:
 *		int	index			the notes index from its StickyNote structure.
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
char *
XPMakeFname(
	register int index
)
{
	char *str;
	char fname[MAXPATHLEN];

	sprintf(fname, "%s/%s%d", app_res.note_dir, StickyNoteFname, index);
	str = XPSafeAlloc(strlen(fname) + 1);
	strcpy(str, fname);
	return(str);
}


/*========================================================================
 *	Name:			XPPopUpList
 *	Prototype:	void XPPopUpList()
 *					
 *	Description:
 * Pop up the list of hidden notes.
 *
 *========================================================================*/
void
XPPopUpList(
)
{
	XPCreateFindNotePrompt(notes, TRUE);
}


/*========================================================================
 *	Name:			XPErrPopUp 
 *					
 *	Description:
 * Pop up a an error dialog box with the specified string.
 *
 *	Notes:
 * The original version of XPostiPlus had XPErrPopUp do all the work of
 * creating the dialog.  For various reasons, and to save me the effort
 * of changing all the calls to XPErrPopUp, it now just calls
 * XPMsgWindow().
 *========================================================================*/
void
XPErrPopUp(
	char			*string
)
{
	XPMsgWindow(
		string, XP_ERROR_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK, NULL, 
		NULL, NULL, NULL, NULL, NULL);
}


/*========================================================================
 *	Name:			XPFixUpDir
 *	Prototype:	XPFixUpDir(char **path)
 *
 *	Description:
 *	Fix up relative paths so they fall under the note_dir path.
 *
 *	Input Arguments:
 *	char	**path			pointer to path to fix up
 *
 *========================================================================*/
void
XPFixUpPath(
	char	**path
)
{
	char	*dir;

	/*
	 * If it's an absolute path name,
	 * we're done.
	 */
	if (*path[0] != '/')
	{
		/*
		 * If the current value is an absolute path, skip changing it.
		 */
		dir = *path;
		*path = 
			XPSafeAlloc(strlen(app_res.home_dir) + 
				strlen(app_res.note_dir) + strlen(dir) + 2 + 1);
		strcpy(*path, (char *)app_res.home_dir);
		strcat(*path, (char *)"/");
		strcat(*path, (char *)app_res.note_dir);
		strcat(*path, (char *)"/");
		strcat(*path, dir);
	}
	
}


/*========================================================================
 *	Name:			XPTestCmd
 *	Prototype:	XPTestCmd(char *cmd)
 *
 *	Description:
 *	Determine if the specified command can be found by this program.
 *
 *	Input Arguments:
 *	char	*cmd			name of command, possibly with fully qualified path
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
int
XPTestCmd(
	char	*command
)
{
#ifdef DEBUG
	char		*fname="XPTestCmd()";
#endif

	struct stat		*stat_buf;
	char				*envpath, *dir, *fqpath;
	char				*command_copy, *cmd;

	DBGEnter();

	if ( command == NULL )
		return (XP_E_FAILURE);

	command_copy = XPSafeAlloc(strlen(command) + 1);
	strcpy(command_copy, command);

	/*
	 * Parse out the first string from the command.  This will be the
	 * program name, either by itself or qualified with a path.
	 */
	cmd = strtok(command_copy, " ");
	if ( cmd[0] == '"' )
		cmd++;

	/*
	 * 1. if the cmd is fully qualified, just stat it.
	 */
	stat_buf = (struct stat *) malloc( sizeof(struct stat) );
	if ( cmd[0] == '/' )
		if ( stat ( cmd, stat_buf ) == -1 ) 
		{
			free(stat_buf);
			free(command_copy);
			return(XP_E_FAILURE);
		}
		else
		{
			free(stat_buf);
			free(command_copy);
			return(XP_E_SUCCESS);
		}

	/*
	 * 2. if the cmd is not fully qualified, prepend it with each
	 *    directory listed in the PATH environment variable and then stat it.
	 *
	 * Get a copy of the PATH environment variable.
	 */
	envpath=XPSafeAlloc(strlen(getenv("PATH")+1));
	strcpy(envpath, getenv("PATH"));
	
	/*
	 * Run through the PATH looking for the command in one of the directories.
	 */
	dir=strtok(envpath, ":");
	while(dir != NULL )
	{
		fqpath = XPSafeAlloc(strlen(dir)+strlen(cmd)+2);
		sprintf(fqpath,"%s/%s", dir, cmd);

		if ( stat ( fqpath, stat_buf ) == 0 ) 
		{
			/*
			 * Found it - tell the caller the command is available for use.
			 */
			DBGPrintf(DBG_INFO,("cmd %s was found\n", cmd));
			free(fqpath);
			free(stat_buf);
			free(command_copy);
			free(envpath);
			return(XP_E_SUCCESS);
		}

		dir=strtok(NULL, ":");
		free(fqpath);
	}

	/*
	 * If we get this far, then the command was not found in the path.
	 * The calling routine will know what to do in this case.
	 */
	DBGPrintf(DBG_INFO,("cmd %s was not found\n", cmd));
	free(stat_buf);
	free(command_copy);
	free(envpath);
	return(XP_E_FAILURE);
}



/*========================================================================
 *	Name:			XPShowDisabledCmds
 *	Prototype:	XPShowDisabledCmds(XtPointer user_data, XtIntervalId id)
 *
 *	Description:
 *	Tell user which commands have been disabled.
 *
 *	Notes:
 *========================================================================*/
#ifdef DO_GTK
int
XPShowDisabledCmds(
	gpointer 		user_data
)
#else
int
XPShowDisabledCmds(
	XtPointer 		user_data,
	XtIntervalId	*id
)
#endif
{
	char			string[2048];
	int			fd;

	/*
	 * If the command is NULL, add notice to string.
	 */
	if (	( app_res.print_cmd == NULL ) ||
			( app_res.calendar_cmd == NULL ) ||
			( app_res.email_cmd == NULL ) ||
			( app_res.audio_cmd == NULL ) ||
#ifdef DO_MOTIF
			( app_res.rgb_file == NULL ) ||
#endif
			( app_res.pilot_xfer == NULL ) ||
			( app_res.pilot_addr == NULL ) ||
			( app_res.pilot_memos == NULL ) ||
			( app_res.pilot_imemos == NULL )
		)
	{
		sprintf(string, XP_S_DISABLE_CMDS);

		if ( app_res.print_cmd == NULL )
			strcat(string, XP_S_DISABLE_PRINT);
		if ( app_res.calendar_cmd == NULL )
			strcat(string, XP_S_DISABLE_CAL);
		if ( app_res.email_cmd == NULL )
			strcat(string, XP_S_DISABLE_EMAIL);
		if ( app_res.audio_cmd == NULL )
			strcat(string, XP_S_DISABLE_AUDIO);

#ifdef DO_MOTIF
		if ( app_res.rgb_file == NULL )
			strcat(string, XP_S_DISABLE_PROJECTS);
#endif

		if ( app_res.pilot_xfer == NULL )
			strcat(string, XP_S_DISABLE_PILOT_SYNC);
		if ( app_res.pilot_addr == NULL )
			strcat(string, XP_S_DISABLE_PILOT_ADDR);
		if (	( app_res.pilot_memos == NULL ) ||
				( app_res.pilot_imemos == NULL ) )
			strcat(string, XP_S_DISABLE_PILOT_MEMOS);

		if ( getenv("PILOTPORT") == NULL )
			strcat(string, XP_S_DISABLE_PILOT_PORT);
		else
		{
			if ( (fd=open(getenv("PILOTPORT"), O_RDWR | O_NONBLOCK )) == -1 )
				strcat(string, XP_S_DISABLE_PILOT_PORT2);
			else
				close(fd);
		}
		if ( getenv("PILOTRATE") == NULL )
			strcat(string, XP_S_DISABLE_PILOT_RATE);

		/*
		 * Pop up a message dialog.
		 */
		XPMsgWindow(
			string, XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK, NULL, 
			NULL, NULL, NULL, NULL, NULL);
	}
	
	return(TRUE);
}


/*========================================================================
 *	Name:			XPRingBell
 *					
 *	Description:
 * Called in response to bad input in text fields.
 *========================================================================*/
void
XPRingBell()
{
#ifdef DO_MOTIF
	XBell(display, 80);
#else
	XBell(gdk_display, 80);
#endif
}


/*========================================================================
 *	Name:			XPNoop
 *					
 *	Description:
 * Dummy routine for overriding unwanted translations.
 *========================================================================*/
void
XPNoop() { }


/*========================================================================
 *	Name:			XPConvertCase
 *	Prototype:	XPConvertCase(char *buf, int type, int requested_length)
 *
 *	Description:
 *		Convert a string to upper or lower case.
 *
 *	Input Arguments:
 *		char	*buf						buffer to convert
 *		int	type						XP_UPPER or XP_LOWER
 *		int	requested_length		If not 0 then use this value for number
 *											of characters to convert.
 *
 *	Output Arguments:
 *		buf is converted in place
 *
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *		
 *========================================================================*/
void
XPConvertCase ( 
	char	*buf,
	int	type,
	int	requested_length
)
{
	int	i;
	char	ch1,ch2;
	int	length;


	if (requested_length == 0)
		length = strlen( buf );
	else
		length = requested_length;

	for ( i = 0; i < length; i++ )
	{
		strncpy ( (char *)&ch1, buf,1 );
		if ( type == XP_LOWER )
			ch2 = tolower ( ch1 ); 
		else
			ch2 = toupper ( ch1 ); 
		strncpy ( buf, (char *)&ch2, 1 );
		buf++;
	}
}

