/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: c_xnotes.c
 *            DESCRIPTION: more common modules 
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPValidateAppRes, XPShowDefaults
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_C_C

/* === System Headers === */
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <pwd.h>

#ifdef DO_GTK
#include "gtk/gtk.h"
#else
#include "motif.h"
#endif


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"

/* === external routines === */

/* === Public routine prototypes === */

/* === Private routine prototypes === */

/* === Global Variables === */
extern AppRes				app_res;

/* === Static Variables === */



/*========================================================================
 *	Name:			XPValidateAppRes()
 *
 *	Description:
 *	Validate we have a valid startup configuration.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *		AppRes	app_res			global configuration parameters.
 *
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPValidateAppRes()
{
#ifdef DEBUG
	char	*fname="ValidateAppRes()";
#endif

	struct stat		*stat_buf;
	int				rc;
	int				fd;

	DBGEnter();

	/*
	 * Test, create if necessary, the note directory.
	 */
	XPSetNoteDir();
	DBGPrintf(DBG_INFO,("returned from XPSetNoteDir()\n"));

	/*
	 * Test for the temporary directory.
	 */
	stat_buf = (struct stat *) malloc( sizeof(struct stat) );
	if ( ( rc = stat ( app_res.tmp_dir, stat_buf ) ) !=0 ) 
	{
		switch ( errno ) 
		{
			case EACCES:
				printf( XP_E_SEARCH_DENY_1, XP_S_PROG_NAME);
				printf( XP_E_SEARCH_DENY_2, XP_S_PROG_NAME, app_res.tmp_dir);
				printf( XP_E_SEARCH_DENY_3, XP_S_PROG_NAME);
				free ( stat_buf );
				exit ( -1 );
				break;

			case ENOTDIR:
				printf( XP_E_PATH_INVALID_1, XP_S_PROG_NAME, app_res.tmp_dir);
				printf( XP_E_PATH_INVALID_2);
				free ( stat_buf );
				exit ( -1 );
				break;

			case ENOENT:
				/* if it doesn't exist, try to create it */
				printf( XP_E_PATH_EXIST_1, XP_S_PROG_NAME, app_res.tmp_dir);
				printf( XP_E_PATH_EXIST_2, XP_S_PROG_NAME);
				if ( rc != 0 ) 
				{
					printf( XP_E_CREATE_FAIL, XP_S_PROG_NAME, app_res.tmp_dir);
					free ( stat_buf );
					exit ( -1 );
				}
				free ( stat_buf );
				break;

			default:
				printf( XP_E_UNKNOWN_STAT, XP_S_PROG_NAME, errno, app_res.tmp_dir);
				free ( stat_buf );
				exit ( -1 );
				break;
		}
	}
	free(stat_buf);

	/*
	 * Test for the bitmap directory.
	 */
	stat_buf = (struct stat *) malloc( sizeof(struct stat) );
	if ( ( rc = stat ( app_res.xpm_dir, stat_buf ) ) !=0 ) 
	{
		switch ( errno ) 
		{
			case EACCES:
				printf( XP_E_SEARCH_DENY_1, XP_S_PROG_NAME);
				printf( XP_E_SEARCH_DENY_2, XP_S_PROG_NAME, app_res.xpm_dir);
				printf( XP_E_SEARCH_DENY_3, XP_S_PROG_NAME);
				free ( stat_buf );
				exit ( -1 );
				break;

			case ENOTDIR:
				printf( XP_E_PATH_INVALID_1, XP_S_PROG_NAME, app_res.xpm_dir);
				printf( XP_E_PATH_INVALID_2);
				free ( stat_buf );
				exit ( -1 );
				break;

			case ENOENT:
				/* if it doesn't exist, we can't continue. */
				printf( XP_E_PATH_EXIST_1, XP_S_PROG_NAME, app_res.xpm_dir);
				free ( stat_buf );
				exit ( -1 );
				break;

			default:
				printf( XP_E_UNKNOWN_STAT, XP_S_PROG_NAME, errno, app_res.tmp_dir);
				free ( stat_buf );
				exit ( -1 );
				break;
		}
	}
	free(stat_buf);

	/*
	 * Now test the *_cmd options to see if we can find them.
	 * We do this by:
	 * 1. if the cmd is fully qualified, just stat it.
	 * 2. if the cmd is not fully qualified, prepend it with each
	 * directory listed in the PATH environment variable and then stat it.
	 * Commands to test:
	 * print_cmd
	 * calendar_cmd
	 * email_cmd
	 * audio_cmd
	 * If the command is not found, set its pointer to NULL.  We'll pop
	 * up a window after start up to notify the user of commands we've
	 * disabled.
	 */
	if ( XPTestCmd(app_res.print_cmd) == XP_E_FAILURE )
		app_res.print_cmd = NULL;
	if ( XPTestCmd(app_res.calendar_cmd) == XP_E_FAILURE )
		app_res.calendar_cmd = NULL;
	if ( XPTestCmd(app_res.email_cmd) == XP_E_FAILURE )
		app_res.email_cmd = NULL;
	if ( XPTestCmd(app_res.audio_cmd) == XP_E_FAILURE )
		app_res.audio_cmd = NULL;

	/*
	 * Test for Palm Pilot tools.
	 */
	app_res.pilot_xfer=DefaultPilotSync;
	app_res.pilot_addr=DefaultPilotAddr;
	app_res.pilot_memos=DefaultPilotMemos;
	app_res.pilot_imemos=DefaultPilotIMemos;
	if ( XPTestCmd(DefaultPilotSync) == XP_E_FAILURE )
		app_res.pilot_xfer = NULL;
	if ( XPTestCmd(DefaultPilotAddr) == XP_E_FAILURE )
		app_res.pilot_addr = NULL;
	if ( XPTestCmd(DefaultPilotMemos) == XP_E_FAILURE )
		app_res.pilot_memos = NULL;
	if ( XPTestCmd(DefaultPilotIMemos) == XP_E_FAILURE )
		app_res.pilot_imemos = NULL;

	/*
	 * Make sure the default pilot port works.
	 */
	if ( getenv("PILOTPORT") != NULL )
	{
		if ( (fd=open(getenv("PILOTPORT"), O_RDWR | O_NONBLOCK )) == -1 )
		{
			app_res.pilot_xfer=NULL;
			app_res.pilot_addr=NULL;
			app_res.pilot_memos=NULL;
			app_res.pilot_imemos=NULL;
		}
		else
			close(fd);
	}
	else
	{
		app_res.pilot_xfer=NULL;
		app_res.pilot_addr=NULL;
		app_res.pilot_memos=NULL;
		app_res.pilot_imemos=NULL;
	}

	/*
	 * Make sure we can find the rgb.txt file for setting project colors.
	 */
	stat_buf = (struct stat *) malloc( sizeof(struct stat) );
	if ( stat(app_res.rgb_file, stat_buf) != 0 )
		app_res.rgb_file = NULL;
	free(stat_buf);
}


/*========================================================================
 *	Name:			XPShowDefaults()
 *
 *	Description:
 *	Show the default values that have been configured.
 *
 *	Notes:
 * This is just so the user can get an idea of what their config is
 * before actually getting too far into the program.
 *========================================================================*/
void
XPShowDefaults()
{

#ifdef DEBUG
#ifdef DO_GTK
	printf("Debug value: %d\n", app_res.debuglevel);
#else
	printf("Debug value: %s\n", app_res.debuglevel);
#endif /* DO_GTK */
	printf("Debug file : %s\n", app_res.debugfile);
#endif /* DEBUG */

	printf("app_res.interval = %d\n", app_res.interval);
	printf("app_res.scroll_bar = %d\n", app_res.scroll_bar);
	printf("app_res.save_notes = %d\n", app_res.save_notes);
	printf("app_res.compatibility = %d\n", app_res.compatibility);
	printf("app_res.nosave = %d\n", app_res.nosave);
	printf("app_res.noalarm = %d\n", app_res.noalarm);
	printf("app_res.help = %d\n", app_res.help);
	printf("app_res.defaults = %d\n", app_res.defaults);
	printf("app_res.version = %d\n", app_res.version);
	printf("app_res.name_width = %d\n", app_res.name_width);
	printf("app_res.anchor_offset = %d\n", app_res.anchor_offset);
	printf("app_res.no_tips = %d\n", app_res.no_tips);

	if ( app_res.note_dir == NULL )
		printf("app_res.note_dir = NONE\n");
	else
		printf("app_res.note_dir = %s\n", app_res.note_dir);
	if ( app_res.addr_dir == NULL )
		printf("app_res.addr_dir = NONE\n");
	else
		printf("app_res.addr_dir = %s\n", app_res.addr_dir);
	if ( app_res.tmp_dir == NULL )
		printf("app_res.tmp_dir = NONE\n");
	else
		printf("app_res.tmp_dir = %s\n", app_res.tmp_dir);
	if ( app_res.print_cmd == NULL )
		printf("app_res.print_cmd = NONE\n");
	else
		printf("app_res.print_cmd = %s\n", app_res.print_cmd);
	if ( app_res.calendar_cmd == NULL )
		printf("app_res.calendar_cmd = NONE\n");
	else
		printf("app_res.calendar_cmd = %s\n", app_res.calendar_cmd);
	if ( app_res.audio_cmd == NULL )
		printf("app_res.audio_cmd = NONE\n");
	else
		printf("app_res.audio_cmd = %s\n", app_res.audio_cmd);
	if ( app_res.audio_dir == NULL )
		printf("app_res.audio_dir = NONE\n");
	else
		printf("app_res.audio_dir = %s\n", app_res.audio_dir);
	printf("app_res.noaudio = %d\n", app_res.noaudio);
	if ( app_res.email_cmd == NULL )
		printf("app_res.email_cmd = NONE\n");
	else
		printf("app_res.email_cmd = %s\n", app_res.email_cmd);
	if ( app_res.home_dir == NULL )
		printf("app_res.home_dir = NONE\n");
	else
		printf("app_res.home_dir = %s\n", app_res.home_dir);
	if ( app_res.xpm_dir == NULL )
		printf("app_res.xpm_dir = NONE\n");
	else
		printf("app_res.xpm_dir = %s\n", app_res.xpm_dir);
	if ( app_res.rgb_file == NULL )
		printf("app_res.rgb_file = NONE\n");
	else
		printf("app_res.rgb_file = %s\n", app_res.rgb_file);
	if ( app_res.pilot_dir == NULL )
		printf("app_res.pilot_dir = NONE\n");
	else
		printf("app_res.pilot_dir = %s\n", app_res.pilot_dir);
	exit(0);
}

