/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995, 1996, 1997, 1998
 *========================================================================
 *              FILE NAME: config.h
 *            DESCRIPTION: compile time configurable options for 
 *								 : XNotesPlus
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/

/*
 * Motif or GTK version?  Only one can be done at a time.  The header
 * files don't allow for more than one at a time.
 *
#define DO_GTK
 */
#define DO_MOTIF


/*
 * Where are the libraries and include files?  You only need to set one of
 * the following two sets of values (either the GTK ones or the Motif ones,
 * but not both).  
 * 
 * Uncomment and set these if your GTK libs and include files are installed 
 * in a nonstandard location.
#define NON_STD_GTK
GTK_LIB=/home/mhammel/lib/lib
GTK_INC=/home/mhammel/lib/include
 */

/*
 * Uncomment and set these if your Motif and/or XPM libs and include files
 * are installed in a nonstandard locations.
#define NON_STD_MOTIF
MOTIF_LIB=/usr/dt/lib
MOTIF_INC=/usr/dt/include
#define NON_STD_XPM
XPM_LIB=/home/mhammel/lib/lib
XPM_INC=/home/mhammel/lib/include
 */

/*
 * If you use the GTK libraries compiled with X Input enabled then
 * uncomment the following line.
#define USE_XINPUT
 */

/*
 * Need a statically linked Motif binary?  Uncomment the following.
 */
#define MAKE_STATIC

/*
 * Uncomment the following if you use a system that doesn't know about
 * bzero() - like Solaris, for example.
 */
#define NO_BZERO

/*
 * Uncomment this if you want to turn the debug macros off.  By default
 * the macros expand into real source code.  Note that even if
 * expanded, no debug output will be generated unless you use the appropriate
 * command line options: --debugfile <file> and --debuglevel <level>
#define DEBUGOFF
 */

/*
 * Uncomment this if you use gcc.  If your "cc" is a sym-link to gcc or
 * you've renamed gcc to cc, then uncomment this as well.  You don't
 * have to (I don't think), but it helps.
 */
#define HASGCC

/*
 * Define the tar command use for building a distributable package here.
 * Almost no one (except people like me who use two different versions of
 * tar) will need to change this.
TAR=gtar cvzf 
 */
TAR=tar cvzf 

/*=========================================================================
 * End of configurable options.
 * You don't need to change anything below this line.
 *=========================================================================*/

/*
 * Do we need to hack around the bzero() problem?
 */
#ifdef NO_BZERO
USE_MEMSET = -DSYSV_USE_MEMSET
#endif

/*
 * Put all those defines together.
 */
OTHER_DEFINES = $(USE_MEMSET)


/*
 * Compile and Link options; complex, but thats what we get when we don't
 * compile with the same compiler across platforms.
 * DEGBUG - turns on local debugging library
 * -Wall  - turns on exhaustive checking by gcc compiler
 * -Wno-unused  - turns off notices about unused parameters and variables.
 *		This is useful because the GUI has lots of unused parameters
 *		in callbacks and these are required by only some compilers.
 */

#ifdef DEBUGOFF
#    ifdef HASGCC
CC = gcc
/* DEFINES   = -g -Wall -Wno-unused $(OTHER_DEFINES)
 */
DEFINES   = -g -Wall $(OTHER_DEFINES)
#    else
DEFINES   = $(OTHER_DEFINES)
#    endif
#else
#    ifdef HASGCC
CC = gcc
/* DEFINES   = -g -Wall -Wno-unused -DDEBUG $(OTHER_DEFINES)
 */
DEFINES   = -g -Wall -DDEBUG $(OTHER_DEFINES)
#    else
DEFINES   = -DDEBUG $(OTHER_DEFINES)
#    endif
#endif

/*
 * Set up for which version to build.
 */
#ifdef DO_GTK
EXTRA_DEFINES = -DDO_GTK
#else
EXTRA_DEFINES = -DDO_MOTIF
#endif

/*
 * Add X Input library, if necessary.
 */
#ifdef USE_XINPUT
XINPUT = -lXi
#else
XINPUT = 
#endif

/*
 * For the masochistic types, who need a static binary.
 */
#ifdef MAKE_STATIC
EXTRA_LDOPTIONS = -static
#endif
