/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: audio.c
 *            DESCRIPTION: routines to handle audio for XNotesPlus
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPSetUpAudio
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_AUDIO_C

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include <time.h>

#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */


/* === Global Variables === */
extern GtkWidget		*toplevel;
extern AppRes			app_res;
extern StickyNote		*notes;

AudioFileType	*audio_files=NULL;


/* === Static Variables === */



/*========================================================================
 *	Name:			XPSetUpAudio
 *	Prototype:	void *XPSetAudio()
 *					
 *	Description:
 * Set up generic audio parameters, such as the file names.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPSetUpAudio()
{
#ifdef USE_DIR
	register struct direct *d;
#else
	register struct dirent *d;
#endif

	struct stat		stat_buf;
	char				buf[MAXPATHLEN+1];
	DIR				*dp;
	AudioFileType	*last_link=NULL, *audio_link;
	int				id;

	/*
	 * Verify audio directory exists.  If not, set link list to NULL.
	 */
	if ( stat(app_res.audio_dir, &stat_buf) != 0 )
	{
		audio_files = NULL;
		return;
	}

	/*
	 * Try to open the directory.
	 */
	if ((dp = opendir(app_res.audio_dir)) == NULL)
	{
		sprintf(buf, "%s(%s:%d): Can't open audio directory.\n",
			XP_S_PROG_NAME, __FILE__, __LINE__);
		fprintf(stderr, buf);
		audio_files=NULL;
		return;
	}

	/*
	 * Create a link list of audio file names.  Each link contains the 
	 * name of the file and an index id.
	 */
	id = 0;
	while ((d = readdir(dp)) != NULL) 
	{
		if (	( strcmp(d->d_name, ".") != 0 ) &&
				( strcmp(d->d_name, "..") != 0 ) )
		{
			/*
			 * Allocate a link.
			 */
			audio_link = (AudioFileType *)XPSafeAlloc(sizeof(AudioFileType));
	
			/*
			 * Add it to the link list.
			 */
			if ( audio_files == NULL )
				audio_files = audio_link;
			else
				last_link->next = audio_link;
			last_link = audio_link;
	
			/*
			 * Fill in the new link.
			 */
			audio_link->next = NULL;
			audio_link->id = id++;
			audio_link->file = 
				(char *)XPSafeAlloc(strlen(d->d_name) + 1);
			sprintf(audio_link->file, "%s", d->d_name);
		}
	}
}


