/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: email.c
 *            DESCRIPTION: email routines for XNotesPlus notes
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPCreateEmailNotePrompt
 *      PRIVATE FUNCTIONS: EmailPopDown, EmailNote
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES:
 *	
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_EMAIL_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <string.h>
#include <time.h>
#include <sys/param.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>


/* GTK/GDK specific includes */
#include <gtk/gtk.h>
#include <gdk/gdkx.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void EmailPopDown();
static void EmailNote();


/* === Global Variables === */
extern AppRes		app_res;
extern StickyNote	*notes;
extern GtkWidget	*toplevel;


/* === Static Variables === */
static GtkWidget		*email_widget=NULL;
static GtkWidget		*email_text;



/*========================================================================
 *	Name:			XPCreateEmailNotePrompt
 *	Prototype:	XPCreateEmailNotePrompt(StickyNote *pn)
 *					
 *	Description:
 * Create a window in which the email address can be entered.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPCreateEmailNotePrompt(
	GtkWidget	*widget,
	int			index
)
{
#ifdef DEBUG
	char	*fname="XPCreateEmailNotePrompt()";
#endif

	StickyNote		*pn;
	GtkWidget		*email_label, *email_cancel, *email_accept;
	GtkWidget		*vbox;
	GShell_T			gshell;

	DBGEnter();

	/*
	 * Only one copy at at time!
	 */
	if ( email_widget != NULL )
		return;

	/*
	 * Find the note we're opening.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Create a popup shell widget
	 */
	email_widget = XPCreateGenericShell(NotesEmailPrompt,
			ACCEPT_BUTTON | CANCEL_BUTTON, &gshell );

	gtk_signal_connect (GTK_OBJECT (email_widget), "destroy",
			GTK_SIGNAL_FUNC (EmailPopDown), NULL);

	vbox = gshell.vbox;
	email_cancel = gshell.cancel;
	email_accept = gshell.accept;


	/* the label window */
	email_label = gtk_label_new(XP_S_EMAIL);
	gtk_box_pack_start (GTK_BOX (vbox), email_label, TRUE, TRUE, 0);
	gtk_widget_show(email_label);

	/*
	 * The text entry area for the email address.
	 */
	email_text = gtk_entry_new();
	gtk_box_pack_start (GTK_BOX (vbox), email_text, TRUE, TRUE, 0);
	gtk_widget_show(email_text);
   gtk_entry_set_text(GTK_ENTRY(email_text), app_res.email_addr);

	/* 
	 * The Accept and Cancel buttons.
	 */
	gtk_signal_connect (GTK_OBJECT (email_accept), "clicked",
			GTK_SIGNAL_FUNC (EmailNote), (gpointer)pn);

	gtk_signal_connect (GTK_OBJECT (email_cancel), "clicked",
			GTK_SIGNAL_FUNC (EmailPopDown), (gpointer)NULL);


	/*
	 * Display the window.
	 */
	gtk_widget_realize(email_widget);
	XPCenterWindow(email_widget, NULL, email_widget);
	gtk_widget_show( email_widget );
}


/*========================================================================
 *	Name:			EmailPopDown
 *	Prototype:	EmailPopDown(GtkWidget *widget, gpointer data)
 *					
 *	Description:
 * Close the Email window
 *========================================================================*/
static void
EmailPopDown(
	GtkWidget	*widget,
	gpointer		data
)
{
	if ( email_widget != NULL )
	{
		gtk_widget_destroy(email_widget);
		email_widget = NULL;
	}
}



/*========================================================================
 *	Name:			EmailNote
 *	Prototype:	EmailNote(GtkWidget *widget, StickyNote *pn)
 *					
 *	Description:
 * Email a notes text to the specified recipient.
 *========================================================================*/
static void
EmailNote(
	GtkWidget	*widget,
	StickyNote	*pn
)
{
	FILE			*fp;
	int			len;
	char			tmpfile[512];
	char			syscmd[1024];
	char			emailcmd[1024];
	char			*addressee;

	/*
	 * Don't email blank notes.
	 */
	if (pn->pn_text != NULL )
		free(pn->pn_text);
	pn->pn_text = gtk_editable_get_chars(GTK_EDITABLE(pn->pn_textwidget), 0, -1);
	if ( strlen(pn->pn_text) == 0 )
	{
		XPErrPopUp("No text in note to email!");
		return;
	}

	/*
	 * Save the text to a temporary file
	 */
	sprintf(tmpfile,"%s/xp%d", app_res.tmp_dir, (int)getpid());
	if ((fp = fopen(tmpfile, "w")) == NULL) 
	{
		fprintf(stderr, "xnotes: ");
		perror(tmpfile);
		sprintf(syscmd, XP_S_TEMP_OPEN_FAIL, tmpfile);
		XPErrPopUp(syscmd);
		return;
	}

	len = strlen(pn->pn_text);
	fwrite(pn->pn_text, sizeof(char), len, fp);
	fclose(fp);

	/*
	 * Use the emailcmd resource to determine how to email the note
	 */
	addressee = gtk_entry_get_text(GTK_ENTRY(email_text));
	sprintf (emailcmd, app_res.email_cmd, pn->pn_name->str, addressee);
	sprintf (syscmd, "cat %s | %s", tmpfile, emailcmd );
	system ( syscmd );

	/*
	 * clean up the temporary file
	 */
	sprintf( syscmd, "rm %s", tmpfile );
	system ( syscmd );

	EmailPopDown(NULL, NULL);
}
