/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: envelope.c
 *            DESCRIPTION: routines to allow printing envelopes
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPEnvelope
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *	
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_ENVELOPE_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <string.h>
#include <time.h>
#include <sys/param.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

/* GTK/GDK specific includes */
#include <gtk/gtk.h>
#include <gdk/gdkx.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void	CloseEnvelope();
static void	PrintEnvelope();
static void	PTSizeCB();
static void	GetAddrList();
static void	UpdateToText();
static void	CloseList();
static void	SaveFromId();


/* === Global Variables === */
extern AppRes		app_res;
extern StickyNote	*notes;
extern GtkWidget	*toplevel;

extern PilotAddr_T	*Pilot_Address;

/* === Static Variables === */
GtkWidget		*envelope_dialog;
static int		pt_size_index;
static GtkWidget	*addr_to, *addr_from;
static GtkWidget	*printer_name, *env_type_6, *env_type_10;
static GtkWidget	*addr_dialog;
/*
static GtkWidget	*bold_button;
*/

static char *env_strings[]={
	"5",
	"10",
	"12",
	"14",
	"16",
	0
	};



/*========================================================================
 *	Name:			XPEnvelope
 *	Prototype:	XPEnvelope()
 *					
 *	Description:
 * Create dialog to allow inputting envelope details.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPEnvelope()
{
#ifdef DEBUG
	char	*fname="XPEnvelope()";
#endif

	GtkWidget	*frame_to, *frame_from, *frame_other;
	GtkWidget	/* *save_to, */ *save_from, *load_to;
	GtkWidget	*pt_size;
	GtkWidget	*vbox, *vbox1, *vbox2, *vbox3, *vbox4, *vbox5;
	GtkWidget	*hbox, *label;
	GtkWidget	*print_button, *close_button;
	GtkWidget	*menu, *menuitem;
	GtkWidget	*hsep;
	GtkWidget	*env_picture;
	char			buf[1024];
	GShell_T		gshell;
	GtkStyle		*style;

	GtkAdjustment	*vadj;
	int				index;
	struct stat		stat_buf;
	FILE				*fd;
	int				length;

	static gboolean		firsttime = TRUE;
	static GdkPixmap		*envelope_xbm = NULL;
	static GdkBitmap		*envelope_mask = NULL;

	/*
	 * Allow only one window at a time.
	 */
	if ( envelope_dialog != NULL )
		return;

	/*
	 * The dialog parent window.
	 */
	envelope_dialog = XPCreateGenericShell(NotesAddrPrompt,
			PRINT_BUTTON | CLOSE_BUTTON, &gshell, FALSE );

	gtk_signal_connect (GTK_OBJECT (envelope_dialog), "destroy",
			GTK_SIGNAL_FUNC (CloseEnvelope), NULL);

	vbox = gshell.vbox;
	close_button = gshell.cancel;
	print_button = gshell.accept;


	/*
	 * A horizontal box in which to place two vertical boxes side by side.
	 */
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(vbox), hbox);
	gtk_widget_show(hbox);

	
	/*
	 * A pair of vertical boxes.
	 */
	vbox1 = gtk_vbox_new(FALSE, 0);
	vbox2 = gtk_vbox_new(FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (vbox2), 5);
	gtk_box_pack_start (GTK_BOX (hbox), vbox1, FALSE, FALSE, 4);
	gtk_box_pack_start (GTK_BOX (hbox), vbox2, FALSE, FALSE, 4);
	gtk_widget_show(vbox1);
	gtk_widget_show(vbox2);

	/*
	 * Stuff two frames inside the left vbox, one inside the right box.
	 */
	frame_from = gtk_frame_new(XP_S_FROM_ADDR);
	frame_to = gtk_frame_new(XP_S_TO_ADDR);
	gtk_box_pack_start (GTK_BOX (vbox1), frame_from, FALSE, FALSE, 4);
	gtk_box_pack_start (GTK_BOX (vbox1), frame_to, FALSE, FALSE, 4);
	gtk_widget_show(frame_from);
	gtk_widget_show(frame_to);

	frame_other = gtk_frame_new(NULL);
	gtk_box_pack_start (GTK_BOX (vbox2), frame_other, FALSE, FALSE, 4);
	gtk_widget_show(frame_other);

	/*
	 * A text field and one button inside the From: frame.
	 */
	vbox3 = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(frame_from), vbox3);
	gtk_widget_show(vbox3);

   vadj = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 0.0, 0.0, 0.0, 0.0));
	addr_from = gtk_text_new(NULL, vadj);
	gtk_widget_set_name(addr_from, "addresses");
	gtk_text_set_editable (GTK_TEXT (addr_from), TRUE);
	gtk_widget_set_usize (addr_from, 200, 80);
	gtk_box_pack_start (GTK_BOX (vbox3), addr_from, FALSE, FALSE, 4);
	gtk_widget_show(addr_from);

	/*
	 * Insert the text from the ID file, if it exists.
	 */
	sprintf(buf,"%s/%s", app_res.note_dir, DefaultIdFile);
	if ( stat(buf, &stat_buf) == 0 )
	{
		DBGPrintf(DBG_INFO,("file size: %d", stat_buf.st_size));

		fd = fopen(buf, "r");
		fread(buf, stat_buf.st_size, 1, fd);
		fclose(fd);
		bcopy("\0", (char *)(buf+stat_buf.st_size), 1);

		DBGPrintf(DBG_INFO,("From address: \n%s\n", buf));

		gtk_widget_realize(addr_from);
		style = gtk_widget_get_style(addr_from);
		length = gtk_text_get_length(GTK_TEXT(addr_from));
		gtk_text_set_point(GTK_TEXT(addr_from), 0);
		gtk_text_forward_delete(GTK_TEXT(addr_from), length);
		gtk_text_freeze(GTK_TEXT(addr_from));
		gtk_text_insert (
			GTK_TEXT(addr_from),
			NULL, 									/* font */
			&style->fg[GTK_STATE_NORMAL], 	/* fg color */
			&style->bg[GTK_STATE_NORMAL],		/* bg color */
			buf, 										/* text to insert */
			-1);										/* -1: use strlen() */
		gtk_text_thaw(GTK_TEXT(addr_from));
	}

	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start (GTK_BOX (vbox3), hbox, FALSE, FALSE, 5);
	gtk_widget_show(hbox);

	label = gtk_label_new(" ");
	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);

	save_from = gtk_button_new();
	gtk_widget_set_name(save_from, "window_button");
	label = gtk_label_new(XP_S_SAVE);
	gtk_widget_set_name(label, "window_button");
	gtk_widget_show(label);
	gtk_container_add(GTK_CONTAINER(save_from), label);
	gtk_box_pack_start (GTK_BOX (hbox), save_from, FALSE, FALSE, 5);
	gtk_widget_show(save_from);

	gtk_signal_connect (GTK_OBJECT (save_from), "clicked",
		GTK_SIGNAL_FUNC (SaveFromId), NULL);

	/* Add a tool tip */
	gtk_signal_connect(GTK_OBJECT(save_from), "event",
		GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_ENV_SAVE_FROM);

	/*
	 * A text field and two buttons inside the To: frame.
	 */
	vbox4 = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(frame_to), vbox4);
	gtk_widget_show(vbox4);

	addr_to = gtk_text_new(NULL, vadj);
	gtk_widget_set_name(addr_to, "addresses");
	gtk_text_set_editable (GTK_TEXT (addr_to), TRUE);
	gtk_widget_set_usize (addr_to, 200, 80);
	gtk_box_pack_start (GTK_BOX (vbox4), addr_to, FALSE, FALSE, 4);
	gtk_widget_show(addr_to);

	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start (GTK_BOX (vbox4), hbox, FALSE, FALSE, 5);
	gtk_widget_show(hbox);

	label = gtk_label_new(" ");
	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);

/*
	save_to = gtk_button_new();
	gtk_widget_set_name(save_to, "window_button");
	label = gtk_label_new(XP_S_SAVE);
	gtk_widget_set_name(label, "window_button");
	gtk_widget_show(label);
	gtk_container_add(GTK_CONTAINER(save_to), label);
	gtk_box_pack_start (GTK_BOX (hbox), save_to, FALSE, FALSE, 5);
	gtk_widget_show(save_to);
*/

	load_to = gtk_button_new();
	gtk_widget_set_name(load_to, "window_button");
	label = gtk_label_new(XP_S_LOAD);
	gtk_widget_set_name(label, "window_button");
	gtk_widget_show(label);
	gtk_container_add(GTK_CONTAINER(load_to), label);
	gtk_box_pack_start (GTK_BOX (hbox), load_to, FALSE, FALSE, 5);
	gtk_widget_show(load_to);

	gtk_signal_connect (GTK_OBJECT (load_to), "clicked",
		GTK_SIGNAL_FUNC (GetAddrList), NULL);

	/* Add a tool tip */
	gtk_signal_connect(GTK_OBJECT(load_to), "event",
		GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_ENV_LOAD_TO);

	/*
	 * A vertical box inside the right side frame.
	 */
	vbox5 = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(frame_other), vbox5);
	gtk_widget_show(vbox5);

	/*
	 * ...stuffed with:
	 * 1. A pt. size option menu.
	 * 2. A set of envelope type toggle buttons
	 * 3. A printer name field.
	 */

	hbox = gtk_hbox_new(TRUE, 4);
	gtk_box_pack_start (GTK_BOX (vbox5), hbox, TRUE, TRUE, 4);
	gtk_widget_show(hbox);

	label = gtk_label_new(XP_S_PT_SIZE);
	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 2);
	gtk_widget_show(label);

	menu = gtk_menu_new();
	index=0;
	while ( env_strings[index] != NULL )
	{
		menuitem = gtk_menu_item_new_with_label(env_strings[index]);
		gtk_widget_set_name (menuitem, "option_menus");
		gtk_menu_append(GTK_MENU(menu), menuitem);
		gtk_widget_show(menuitem);
		gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
			GTK_SIGNAL_FUNC(PTSizeCB), (gpointer) index);
		index++;
	}
	pt_size = gtk_option_menu_new ();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (pt_size), menu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (pt_size), 1);
	gtk_box_pack_start (GTK_BOX (hbox), pt_size, FALSE, FALSE, 0);
	gtk_widget_show (pt_size);
 
	hsep = gtk_hseparator_new();
	gtk_box_pack_start (GTK_BOX (vbox5), hsep, FALSE, FALSE, 4);
	gtk_widget_show(hsep);

	label = gtk_label_new(XP_S_ENV_TYPE);
	gtk_box_pack_start (GTK_BOX (vbox5), label, FALSE, FALSE, 4);
	gtk_widget_show(label);

	hbox = gtk_hbox_new(TRUE, 4);
	gtk_box_pack_start (GTK_BOX (vbox5), hbox, FALSE, FALSE, 2);
	gtk_widget_show(hbox);

	env_type_6 = gtk_radio_button_new_with_label (NULL, "#6");
	gtk_box_pack_start (GTK_BOX (hbox), env_type_6, FALSE, FALSE, 0);
	gtk_widget_show (env_type_6);

	env_type_10 = gtk_radio_button_new_with_label (
			gtk_radio_button_group (GTK_RADIO_BUTTON (env_type_6)),
			"#10");
	gtk_box_pack_start (GTK_BOX (hbox), env_type_10, FALSE, FALSE, 0);
	gtk_widget_show (env_type_10);
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (env_type_6), TRUE);

	hsep = gtk_hseparator_new();
	gtk_box_pack_start (GTK_BOX (vbox5), hsep, FALSE, FALSE, 4);
	gtk_widget_show(hsep);

	label = gtk_label_new(XP_S_PRINTER_NAME);
	gtk_box_pack_start (GTK_BOX (vbox5), label, FALSE, FALSE, 4);
	gtk_widget_show(label);

	printer_name = gtk_entry_new();
	gtk_box_pack_start (GTK_BOX (vbox5), printer_name, FALSE, FALSE, 0);
	gtk_widget_show(printer_name);

	/*
	 * Lets get cute.
	 */
	if ( firsttime )
	{
		style = gtk_widget_get_style( vbox );
		sprintf(buf, "%s/%s", app_res.xpm_dir, XP_ENVELOPE_XPM);
		envelope_xbm = gdk_pixmap_create_from_xpm(
						toplevel->window,
						&envelope_mask,
						&style->bg[GTK_STATE_NORMAL],
						buf);
		firsttime = FALSE;
	}
	env_picture = gtk_pixmap_new(envelope_xbm, envelope_mask);
	gtk_box_pack_start (GTK_BOX (vbox5), env_picture, TRUE, TRUE, 5);
	gtk_widget_show( env_picture );

	/*
	 * Set the callbacks for the various buttons.
	 */
	gtk_signal_connect (GTK_OBJECT (close_button), "clicked",
		GTK_SIGNAL_FUNC (CloseEnvelope), NULL);
	gtk_signal_connect (GTK_OBJECT (print_button), "clicked",
		GTK_SIGNAL_FUNC (PrintEnvelope), NULL);


	/*
	 * Display the dialog.
	 */
	gtk_widget_realize(envelope_dialog);
	XPCenterWindow(envelope_dialog, NULL, envelope_dialog);
	gtk_widget_show(envelope_dialog);
}


/*========================================================================
 *	Name:			CloseEnvelope
 *	Prototype:	static void CloseEnvelope()
 *					
 *	Description:
 * Close the envelopes dialog.
 *
 *	Notes:
 *========================================================================*/
static void
CloseEnvelope()
{
	if ( envelope_dialog != NULL )
	{
		gtk_widget_destroy(envelope_dialog);
		envelope_dialog = NULL;
	}
	CloseList();
}


/*========================================================================
 *	Name:			PTSizeCB
 *	Prototype:	static void PTSizeCB()
 *					
 *	Description:
 * Keep track of which point size has been set.
 *
 *	Notes:
 *========================================================================*/
static void
PTSizeCB(
	GtkWidget	*widget,
	int			index
)
{
	pt_size_index = index;
}


/*========================================================================
 *	Name:			PrintEnvelope
 *	Prototype:	static void PrintEnvelope()
 *					
 *	Description:
 * Grab the configuration information from the dialog, then build
 * a nenscript command.  Write address info to a temporary file.
 * Issue the command and close the dialog.
 *
 *	Notes:
 *		nenscript command:
 *			For #6 envelopes:
 *				nenscript -fCourierxx -B -L21 -Pprinter tempfile
 *			For #10 envelopes:
 *				nenscript -fCourierxx -2 -B -L21 -Pprinter tempfile
 *			where
 *			xx			Pt size from pt_size_index global variable
 *			printer	from printer_name text field
 *			-2			two column output (CTRL-L forces start of 2nd column)
 *			-B			turns off page titles
 *			-L			number of lines to print, max.
 *========================================================================*/
static void
PrintEnvelope()
{
#ifdef DEBUG
	char	*fname="PrintEnvelope()";
#endif

	char				*printer, *string, *ptr;
	char				cmd[512];
	char				bold[512];
	char				tmpfile[MAXPATHLEN];
	struct stat		stat_buf;
	FILE				*fp;
	int				rc;

	DBGEnter();

	/*
	 * Grab printer name to use.
	 */
	printer = gtk_entry_get_text(GTK_ENTRY(printer_name));
	if ( strlen(printer) == 0 )
	{
		sprintf(cmd, XP_S_PRT_MISSING);
		XPMsgWindow(cmd, XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK,
			NULL, NULL, NULL, NULL, NULL, NULL);
		return;
	}

	/*
	 * Make sure address info has been supplied.
	 */
	if ( gtk_text_get_length(GTK_TEXT(addr_from)) == 0 )
	{
		sprintf(cmd, XP_S_PRT_FROM);
		XPMsgWindow(cmd, XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK,
			NULL, NULL, NULL, NULL, NULL, NULL);
		return;
	}
	if ( gtk_text_get_length(GTK_TEXT(addr_to)) == 0 )
	{
		sprintf(cmd, XP_S_PRT_TO);
		XPMsgWindow(cmd, XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK,
			NULL, NULL, NULL, NULL, NULL, NULL);
		return;
	}

	/*
	 * Get Bold button status.
	if (GTK_TOGGLE_BUTTON (bold_button)->active) 
		sprintf(bold,"-Bold");
	else
	 */
		bzero(bold,10);

	/*
	 * Build a temporary file name.
	 */
	sprintf(tmpfile,"%s/xpe.%d", app_res.note_dir, (int)getpid());

	/*
	 * Open the temp file.
	 */
	if ((fp = fopen(tmpfile, "w")) == NULL)
	{
		fprintf(stderr, "xnotes+: ");
		perror(tmpfile);
		sprintf(cmd,
			"Couldn't open tempfile (%s)\nfor printing envelope.\n", tmpfile);
		XPErrPopUp(cmd);
		return;
	} 

	/*
	 * Now we have to determine which envelope type we're using.
	 */
	if (GTK_TOGGLE_BUTTON (env_type_6)->active) 
	{
		/*
	 	 * Write the envelope headers to the temp file.
		 */
		string = gtk_editable_get_chars(GTK_EDITABLE(addr_from), 0, -1);
		fprintf(fp, "%s\n", string);
		fprintf(fp, "\n\n\n");
		free(string);

		string = gtk_editable_get_chars(GTK_EDITABLE(addr_to), 0, -1);
		ptr = strtok(string, "\n");
		while ( ptr != NULL )
		{
			fprintf(fp, "\t\t\t%s\n", ptr);
			ptr = strtok(NULL, "\n");
		}
		fclose(fp);
		free(string);


		/*
		 * Build the command string.
		 */
		sprintf(cmd, 
			"nenscript -fCourier%s%s -B -L21 -P%s %s",
			bold, env_strings[pt_size_index], printer, tmpfile);

	}
	else
	{
		/*
	 	 * Write the envelope headers to the temp file.
		 */
		string = gtk_editable_get_chars(GTK_EDITABLE(addr_from), 0, -1);
		fprintf(fp, "%s\n", string);
		fprintf(fp, "\n");
		fprintf(fp, "\n\n\n\n\n");

		string = gtk_editable_get_chars(GTK_EDITABLE(addr_to), 0, -1);
		fprintf(fp, "%s\n", string);
		fclose(fp);


		/*
		 * Build the command string.
		 */
		sprintf(cmd, 
			"nenscript -fCourier%s%s -2 -B -L21 -P%s %s",
			bold, env_strings[pt_size_index], printer, tmpfile);

	}

	/*
	 * Issue the command.
	 */
#ifdef DEBUG
	DBGPrintf(DBG_INFO,("Command used to print the envelope: %s\n", cmd));
	rc = system ( cmd );
	switch ( rc )
	{
		case 127:
			sprintf(cmd,"%s failed: errno=%d", cmd, errno);
			XPErrPopUp(cmd);
			break;
			
		case -1:
			sprintf(cmd,"%s failed: errno=%d", cmd, rc);
			XPErrPopUp(cmd);
			break;
			
		default:
			sprintf(cmd,"%s succeeded", cmd);
			XPMsgWindow(
				cmd, XP_INFO_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK, NULL, 
				NULL, NULL, NULL, NULL, NULL);
			break;
	}
#else
	system ( cmd );
#endif


	/*
 	 * Clean up the temporary file.
	 */
	if ( stat(tmpfile, &stat_buf) == 0 )
	{
		if ( unlink(tmpfile) != 0 )
		{
			sprintf(cmd,"Problems doing cleanup after printing envelope.\n"
				"%s could not be unlinked.\n", tmpfile);
			XPErrPopUp(cmd);
		}
	}
}


/*========================================================================
 *	Name:			GetAddrList
 *	Prototype:	static void GetAddrList()
 *					
 *	Description:
 * Retrieves the addresses currently available from the address database(s)
 * and presents them to the user to select one.
 *
 *	Notes:
 *========================================================================*/
static void
GetAddrList()
{
#ifdef DEBUG
	char	*fname="GetAddrList()";
#endif

	GtkWidget	*vbox, *scrolled_win, *close_button; 
	GtkWidget	*addr_list;
	PilotAddr_T	*current_link;
	char			buf[512];
	int			i;
	GShell_T		gshell;
	GtkWidget	*list_item;
	
	sprintf(buf, "%s/%s", app_res.pilot_dir, DefaultPilotDB);

	DBGPrintf(DBG_INFO,("pilot db: %s\n", buf));
	if ( XPPilotParseAddr(buf) != XP_E_SUCCESS )
	{
		XPErrPopUp(XP_S_LOCDB_FAIL);
		return;
	}

	/*
	 * Allow only one window at a time.
	 */
	if ( addr_dialog != NULL )
		return;

	/*
	 * The addr dialog parent window.
	 */
	addr_dialog = XPCreateGenericShell(AddrListPrompt, 
						CANCEL_BUTTON, &gshell, FALSE );

	gtk_signal_connect (GTK_OBJECT (addr_dialog), "destroy",
			GTK_SIGNAL_FUNC (CloseList), NULL);

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	vbox = gshell.vbox;
	close_button = gshell.cancel;

	/*
	 * A scrolled list of address.
	 */
	scrolled_win = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win),
		GTK_POLICY_AUTOMATIC,
		GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start (GTK_BOX (vbox), scrolled_win, TRUE, TRUE, 0);
	gtk_widget_show (scrolled_win);
	
	addr_list = gtk_list_new();
	gtk_list_set_selection_mode (GTK_LIST (addr_list), GTK_SELECTION_SINGLE);
	gtk_container_add (GTK_CONTAINER (scrolled_win), addr_list);
	gtk_widget_show (addr_list);

   gtk_container_set_focus_vadjustment (GTK_CONTAINER (addr_list),
		gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrolled_win)));

	gtk_widget_set_usize(scrolled_win, XP_ADDR_WIDTH, XP_ADDR_HEIGHT);

	/*
	 * Populate the list.
	 */
	current_link = Pilot_Address;
	i=1;
	while (current_link != NULL )
	{
		if ( ( current_link->last != NULL ) && ( current_link->first != NULL) )
		{
			if (  ( strlen(current_link->last) != 0 ) &&
					( strlen(current_link->first) != 0 ) )
				sprintf(buf, "%s %s", current_link->first, current_link->last);
			else
			if (  ( strlen(current_link->first) == 0 ) &&
					( strlen(current_link->last) != 0 ) )
				sprintf(buf, "%s", current_link->last);
			else
				sprintf(buf, "%s", current_link->first);
			}
		else
		if ( current_link->last != NULL )
			sprintf(buf, "%s", current_link->last);
		else
		if ( current_link->first != NULL )
			sprintf(buf, "%s", current_link->last);
		else
		if ( current_link->company != NULL )
			sprintf(buf, "%s", current_link->company);

		list_item = gtk_list_item_new_with_label (buf);
		gtk_container_add (GTK_CONTAINER (addr_list), list_item);
		gtk_object_set_user_data (GTK_OBJECT (list_item), (gpointer) i);
		gtk_widget_show (list_item);

		current_link = current_link->next;

		i++;
	}

	/*
	 * Callback to use when an address has been selected.
	 */
	gtk_signal_connect (GTK_OBJECT (addr_list), "selection_changed",
		GTK_SIGNAL_FUNC (UpdateToText), NULL);

	/*
	 * Callback to close the window.
	 */
	gtk_signal_connect (GTK_OBJECT (close_button), "clicked",
			GTK_SIGNAL_FUNC (CloseList), NULL);

	/*
	 * Display the dialog.
	 */
	gtk_widget_realize(addr_dialog);
	XPCenterWindow(addr_dialog, envelope_dialog, addr_dialog);
	gtk_widget_show(addr_dialog);

}


/*========================================================================
 *	Name:			CloseList
 *	Prototype:	static void CloseList()
 *					
 *	Description:
 * Closes the address selection dialog.
 *
 *	Notes:
 *========================================================================*/
static void
CloseList()
{
	if ( addr_dialog != NULL )
	{
		gtk_widget_destroy(addr_dialog);
		addr_dialog = NULL;
	}
}


/*========================================================================
 *	Name:			UpdateToText
 *	Prototype:	static void UpdateToText()
 *					
 *	Description:
 * Updates the To: address box with the selected entries data.
 *
 *	Notes:
 *========================================================================*/
static void
UpdateToText(
	GtkWidget	*list_widget,
	gpointer		client_data
)
{
#ifdef DEBUG
	char		*fname="UpdateToText()";
#endif

	GList				*list;
	GtkWidget		*list_item;
	GtkStyle			*style;

	int			i;
	PilotAddr_T	*current_link;
	char			name[1024];
	char			buf[2048];
	int			position, length;

	DBGEnter();
	
	/*
	 * Get the selected entry - there should only be one.
	 * The entries object data holds the index into the link list.
	 */
	list = (GTK_LIST(list_widget))->selection;
	if ( list == NULL )
		return;
	list_item = GTK_WIDGET(list->data);
	position = (int) gtk_object_get_user_data(GTK_OBJECT(list_item));

	/*
	 * Find the link list entry that holds the data we need.
	 */
	current_link = Pilot_Address;
	for ( i=1; i<position; i++ )
		current_link = current_link->next;

	/*
	 * Format the info for the Address To text field.
	 */
	if ( ( current_link->last != NULL ) && ( current_link->first != NULL ) )
	{
		if (	( strlen(current_link->last) != 0 ) && 
				( strlen(current_link->first) != 0 ) )
			sprintf(name, "%s %s", current_link->first, current_link->last);
		else
		if (	( strlen(current_link->first) == 0 ) && 
				( strlen(current_link->last) != 0 ) )
			sprintf(name, "%s", current_link->last);
		else
			sprintf(name, "%s", current_link->first);
	}
	else
	if ( current_link->last != NULL )
		sprintf(name, "%s", current_link->last);
	else
	if ( current_link->first != NULL )
		sprintf(name, "%s", current_link->last);
	else
	if ( current_link->company != NULL )
		sprintf(name, "%s", current_link->company);

	DBGPrintf(DBG_INFO, ("name: *%s*\n", name));

	sprintf(buf, "%s\n%s\n%s %s\n%s",
		name,
		current_link->address,
		current_link->city,
		current_link->state,
		current_link->zip);

	/*
	 * Delete any old text, then add the new text.
	 */
	style = gtk_widget_get_style(addr_to);
	length = gtk_text_get_length(GTK_TEXT(addr_to));
	if ( length > 0 )
	{
		gtk_text_set_point(GTK_TEXT(addr_to), 0);
		gtk_text_forward_delete(GTK_TEXT(addr_to), length);
	}
	gtk_text_freeze(GTK_TEXT(addr_to));
	gtk_text_insert (
		GTK_TEXT(addr_to),
		NULL, 									/* font */
		&style->fg[GTK_STATE_NORMAL], 	/* fg color */
		&style->bg[GTK_STATE_NORMAL],		/* bg color */
		buf, 										/* text to insert */
		-1);										/* -1: use strlen() */
	gtk_text_thaw(GTK_TEXT(addr_to));
}


/*========================================================================
 *	Name:			SaveFromId
 *	Prototype:	static void SaveFromId()
 *					
 *	Description:
 * Save the From address to the local id file.
 *
 *	Notes:
 *========================================================================*/
static void
SaveFromId()
{
#ifdef DEBUG
	char		*fname="SaveFromId()";
#endif

	FILE		*fd;
	char		*string;
	char		buf[MAXPATHLEN];

	DBGEnter();

	string = gtk_editable_get_chars(GTK_EDITABLE(addr_from), 0, -1);

	DBGPrintf(DBG_INFO,("From address: \n%s\n", string));

	if ( strlen(string) > 0 )
	{
		sprintf(buf,"%s/%s", app_res.note_dir, DefaultIdFile);
		fd = fopen(buf, "w");
		fprintf(fd, "%s", string);
		fclose(fd);
	}

	free(string);
}

