/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: findnote.c
 *            DESCRIPTION: routines for finding a note
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPCreateFindNotePrompt
 *      PRIVATE FUNCTIONS: FindPopDown, FindPopUpChild
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_FINDNOTE_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>

#include <gtk/gtk.h>
#include <gdk/gdkx.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void FindPopDown();
static void FindPopUpChild();


/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;
extern GtkWidget		*toplevel;


/* === Static Variables === */
static GtkWidget		*find_widget;
static GtkWidget		*find_viewport;



/*========================================================================
 *	Name:			XPCreateFindNotePrompt
 *	Prototype:	void XPCreateFindNotePrompt(StickyNote *notes, char hidden)
 *					
 *	Description:
 * Create a scrollable window in which all notes can be listed.
 *
 *	Input Arguments:
 *	StickyNote	*notes		link list of all notes
 *	char			hidden		TRUE:  only find hidden notes
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPCreateFindNotePrompt(
	StickyNote	*notes,
	char			hidden
)
{
#ifdef DEBUG
	char				*fname="XPCreateFindNotePrompt";
#endif

	char				found;
	GtkWidget		*cancel;
	GtkWidget		*vbox;
	GtkWidget		*list_item, *find_list;
	StickyNote		*pn;
	char				buf[256];
	GShell_T			gshell;

	DBGEnter();

	/*
	 * Allow only one copy at a time.
	 */
	if ( find_widget != NULL ) 
		return;

	/*
	 * Create a dialog, set its title, and position it at the cursor.
	 */
	if ( hidden )
		find_widget = XPCreateGenericShell(StickyNoteList,
			CANCEL_BUTTON, &gshell, FALSE );
	else
		 find_widget = XPCreateGenericShell(NotesFindPrompt,
			CANCEL_BUTTON, &gshell, FALSE );

	gtk_signal_connect (GTK_OBJECT (find_widget), "destroy",
			GTK_SIGNAL_FUNC (FindPopDown), find_widget);

	gtk_widget_realize(find_widget);

	vbox = gshell.vbox;
	cancel = gshell.cancel;

	/*
	 * A scrolled list to display the available notes.
	 */
	DBGPrintf(DBG_INFO,("Creating scrolled window.\n"));
	find_viewport = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_set_name (find_viewport, "find_list");
	gtk_scrolled_window_set_policy (
		GTK_SCROLLED_WINDOW (find_viewport),
		GTK_POLICY_AUTOMATIC,
		GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start (GTK_BOX (vbox), find_viewport, TRUE, TRUE, 0);
	gtk_widget_show (find_viewport);

	DBGPrintf(DBG_INFO,("Creating list.\n"));
	find_list = gtk_list_new();
	gtk_list_set_selection_mode (GTK_LIST (find_list), GTK_SELECTION_SINGLE);
	gtk_container_add (GTK_CONTAINER (find_viewport), find_list);
	gtk_widget_show (find_list);

	/*
	 * When the user makes a selection, call the routine for popping up
	 * the note.  The list item has a pointer to the notes data structure
	 * in its user data field.
	 */
	DBGPrintf(DBG_INFO,("Adding list callback: 0x%08x\n", (int)find_list));
	gtk_signal_connect (GTK_OBJECT (find_list), "selection_changed",
		GTK_SIGNAL_FUNC (FindPopUpChild), find_list);

	/*
	 * The Cancel button.
	 */
	gtk_signal_connect (GTK_OBJECT (cancel), "clicked",
			GTK_SIGNAL_FUNC (FindPopDown), find_widget);

	/*
	 * Populate the list.
	 */
	DBGPrintf(DBG_INFO,("Adding list items.\n"));
	found = FALSE;
	for (pn = notes; pn != NULL; pn = pn->pn_next)
	{
		if ( hidden ) 
		{
			if ( pn->pn_hidden )
			{

				list_item = gtk_list_item_new_with_label (pn->pn_name->str);
				gtk_widget_set_name (list_item, "find_list");
				gtk_container_add (GTK_CONTAINER (find_list), list_item);
				gtk_widget_show (list_item);
				gtk_object_set_user_data (GTK_OBJECT (list_item), (gpointer) pn);

				found=TRUE;
			}
		}
		else
		{
			list_item = gtk_list_item_new_with_label (pn->pn_name->str);
			gtk_widget_set_name (list_item, "find_list");
			gtk_container_add (GTK_CONTAINER (find_list), list_item);
			gtk_widget_show (list_item);
			gtk_object_set_user_data (GTK_OBJECT (list_item), (gpointer) pn);

			found=TRUE;
		}
	}

	if ( !found )
	{
		DBGPrintf(DBG_INFO,("No notes found - displaying error message.\n"));
		if ( hidden )
		{
			strcpy(buf, XP_S_NO_HIDDEN_NOTES);
			XPErrPopUp(buf);
		}
		else
		{
			strcpy(buf, XP_S_NO_NOTES);
			XPErrPopUp(buf);
		}
		FindPopDown();
		return;
	}

	DBGPrintf(DBG_INFO,("Showing the dialog.\n"));
	gtk_widget_set_usize (GTK_WIDGET (find_viewport), 
		XP_FIND_WIDTH, XP_FIND_HEIGHT);

	/*
	 * Stick the dialog in the middle of * the display.
	 */
	gtk_widget_realize(find_widget);
	XPCenterWindow(find_widget, NULL, find_widget);
	gtk_widget_show( find_widget );
}


/*========================================================================
 *	Name:			FindPopDown and FindChildPopUp
 *					
 *	Description:
 * FindPopDown - Destroy the Find window
 *	FindChildPopUp - Brind the requested note to the cursor
 *
 *========================================================================*/
static void
FindPopDown()
{
	if ( find_widget != NULL )
	{
		gtk_widget_destroy(find_widget);
		find_widget = NULL;
	}
}

static void
FindPopUpChild(
	GtkWidget	*list_widget,
	gpointer		*find_list
)
{
#ifdef DEBUG
	char	*fname="FindPopUpChild()";
#endif

	GList				*list;
	GtkWidget		*list_item;

	StickyNote		*pn;
	int 				root_x, root_y;

	DBGEnter();
	DBGPrintf(DBG_INFO,("find_list: 0x%08x\n", (int)find_list));

	/*
	 * Get the selection list.  There should only be one since we only
	 * allow a single selection.  Then grab the Widget pointer and use
	 * it to get the user data.  Thats the pointer to the note we want.
	 */
	list = (GTK_LIST(list_widget))->selection;
	if ( list == NULL )
		return;
	list_item = GTK_WIDGET(list->data);
	pn = (StickyNote *) gtk_object_get_user_data(GTK_OBJECT(list_item));

	/*
	 * If its a hidden note, then just unhide it.  Its location will
	 * be relative to the current screen, even on multi-workspace 
	 * window managers.
	 */
	if ( pn->pn_hidden )
		pn->pn_hidden = FALSE;
	else
		gtk_widget_hide( pn->pn_shellwidget );

	gtk_window_position (GTK_WINDOW (pn->pn_shellwidget), GTK_WIN_POS_MOUSE);
	gtk_widget_show( pn->pn_shellwidget );

	gdk_window_get_position(pn->pn_shellwidget->window, &root_x, &root_y);
	pn->pn_shellx = root_x;
	pn->pn_shelly = root_y;

	/* 
	 * Close the Find window
	 */
	FindPopDown();
}

