/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995, 1996, 1997, 1998.
 *========================================================================
 *              FILE NAME: findtext.c 
 *            DESCRIPTION: Find a text substring within a note
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPFindTextPrompt, XPFind
 *      PRIVATE FUNCTIONS: SearchIt, ClearFind, FindForward, FindBackward
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_FINDTEXT_C

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void		ClearFind();
static void		SearchIt();


/* === Global Variables === */

/* === Static Variables === */
static GtkWidget		*findwidget;
static GtkWidget		*case_toggle, *direction_toggle;
static GtkWidget		*search_term;
static gboolean		firsttime;

static int	start, end;
static char	*copy=NULL;



/*========================================================================
 *	Name:			XPFindTextPrompt
 *	Prototype:	XPFindTextPrompt()
 *					
 *	Description:
 * Put up a window that says who I am and what this thing is.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPFindTextPrompt(
	GtkWidget	*widget,
	int			index
)
{
#ifdef DEBUG
	char	fname[]="XPFindPrompt";
#endif

	GtkWidget		*vbox, *hbox;
	GtkWidget		*cancel_button, *accept_button;
	GtkWidget		*label;
	GShell_T			gshell;
	StickyNote		*pn;

	firsttime = TRUE;

	/*
	 * Allow only one copy at a time.
	 */
	if ( findwidget != NULL )
		return;

	/*
	 * Grab the note structure.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Create a dialog, set its title, and position it at the cursor.
	 */
	DBGPrintf(DBG_INFO,("Creating find dialog.\n"));
	findwidget = XPCreateGenericShell(FindTextDialog, 
						SEARCH_BUTTON | CLOSE_BUTTON, &gshell, FALSE );

	gtk_signal_connect (GTK_OBJECT (findwidget), "destroy",
			GTK_SIGNAL_FUNC (ClearFind), pn);

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	vbox = gshell.vbox;
	cancel_button = gshell.cancel;
	accept_button = gshell.accept;

	/*
	 * A label describing what this dialog is for.
	 */
	label = gtk_label_new(XP_S_SEARCH_PROMPT);
	gtk_widget_show(label);
	gtk_container_add(GTK_CONTAINER(vbox), label);

	/*
	 * An text entry widget in which the search term will go.
	 */
	search_term = gtk_entry_new();
	gtk_widget_show(search_term);
	gtk_container_add(GTK_CONTAINER(vbox), search_term);

	/*
	 * A horizontal box for the two toggle buttons.
	 */
	hbox = gtk_hbox_new(TRUE, 0);
	gtk_widget_show(hbox);
	gtk_container_add(GTK_CONTAINER(vbox), hbox);
	
	/*
	 * A toggle for case sensitve searches and
	 * a toggle for the direction to search.
	 */
	case_toggle = gtk_check_button_new_with_label(XP_S_CASE);
	gtk_widget_show(case_toggle);
	gtk_container_add(GTK_CONTAINER(hbox), case_toggle);

	direction_toggle = gtk_check_button_new_with_label(XP_S_DIRECTION);
	gtk_widget_show(direction_toggle);
	gtk_container_add(GTK_CONTAINER(hbox), direction_toggle);

	/*
	 * Search and Close buttons.
	 */
	gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
			GTK_SIGNAL_FUNC (ClearFind), pn);
	gtk_signal_connect (GTK_OBJECT (accept_button), "clicked",
			GTK_SIGNAL_FUNC (SearchIt), pn);

	/*
	 * Stick the dialog in the middle of the display.
	 */
	gtk_widget_realize( findwidget );
	XPCenterWindow(findwidget, pn->pn_shellwidget, findwidget);
	gtk_widget_show(findwidget);

}



/*========================================================================
 *	Name:			ClearFind
 *	Prototype:	ClearFind()
 *					
 *	Description:
 * Close the window and remove the highlight, if one exists.
 *
 *========================================================================*/
static void
ClearFind(
	GtkWidget	*widget,
	StickyNote	*pn
)
{
	GtkStyle		*style;

	if ( findwidget != NULL )
	{
		gtk_widget_destroy( findwidget );
		findwidget = NULL;
	}

	if ( start != -1 )
	{
		style = gtk_widget_get_style(pn->pn_textwidget);

		if ( copy != NULL )
		{
			gtk_text_set_point(GTK_TEXT(pn->pn_textwidget), start);
			gtk_text_forward_delete(GTK_TEXT(pn->pn_textwidget), strlen(copy));
			gtk_text_freeze(GTK_TEXT(pn->pn_textwidget));
			gtk_text_insert (
				GTK_TEXT(pn->pn_textwidget),
				NULL, 									/* font */
				&style->fg[GTK_STATE_NORMAL], 	/* fg color */
				&style->bg[GTK_STATE_NORMAL],		/* bg color */
				copy, 									/* text to insert */
				-1);										/* -1: use strlen() */
			gtk_text_thaw(GTK_TEXT(pn->pn_textwidget));

			free(copy);
			copy=NULL;
			start=end=-1;
		}
	}
}



/*========================================================================
 *	Name:			SearchIt
 *	Prototype:	SearchIt()
 *					
 *	Description:
 * Do the actual search and update the display to highlight the text,
 * if found.
 *========================================================================*/
static void
SearchIt(
	GtkWidget	*widget,
	StickyNote	*pn
)
{
#ifdef DEBUG
	char	*fname="SearchIt()"; 
#endif

	gboolean		case_sensitive;
	int			direction;
	char			*text;
	int			rc;
	int			offset;
	GtkStyle		*style;

	DBGEnter();

	/*
	 * Reinitialize last location pointers.
	 */
	if ( firsttime )
	{
		start = end = -1;
		firsttime = FALSE;
		if ( copy != NULL )
		{
			free(copy);
			copy = NULL;
		}
	}

	/*
	 * Get toggle states.
	 */
	if (GTK_TOGGLE_BUTTON (case_toggle)->active) 
	{
		DBGPrintf(DBG_INFO,("case sensitive search\n"));
		case_sensitive = TRUE;
	}
	else
	{
		DBGPrintf(DBG_INFO,("case insensitive search\n"));
		case_sensitive = FALSE;
	}
	if (GTK_TOGGLE_BUTTON (direction_toggle)->active) 
	{
		DBGPrintf(DBG_INFO,("search backward\n"));
		direction = XP_BACKWARD;
	}
	else
	{
		DBGPrintf(DBG_INFO,("search forward\n"));
		direction = XP_FORWARD;
	}

	/*
	 * Get search term.
	 */
	text = gtk_entry_get_text(GTK_ENTRY(search_term));

	/*
	 * Get the current cursor location.
	 */
	offset = gtk_text_get_point(GTK_TEXT(pn->pn_textwidget));
	DBGPrintf(DBG_INFO,("initial offset: %d\n", offset));

	/*
	 * If searching backwards, back up the length of the search string.
	 */
	if ( direction == XP_BACKWARD )
	{
		offset -= strlen(text);
		if ( offset < 0 )
			offset = 0;
	}

	/*
	 * Search for text.
	 */
	DBGPrintf(DBG_INFO,("text to search for: %s\n", text));
	rc = XPFind(direction, case_sensitive, offset, text, pn);

	DBGPrintf(DBG_INFO,("returned offset: %d\n", rc));

	/*
	 * If found, update the current cursor location and highlight the text.
	 */
	if ( rc != -1 )
	{
		/*
		 * First, reset the last location, if any.
		 */
		if ( start != -1 )
		{
			style = gtk_widget_get_style(pn->pn_textwidget);

			gtk_text_set_point(GTK_TEXT(pn->pn_textwidget), start);
			gtk_text_freeze(GTK_TEXT(pn->pn_textwidget));
			gtk_text_forward_delete(GTK_TEXT(pn->pn_textwidget), strlen(copy));
			gtk_text_insert (
				GTK_TEXT(pn->pn_textwidget),
				NULL, 									/* font */
				&style->fg[GTK_STATE_NORMAL], 	/* fg color */
				&style->bg[GTK_STATE_NORMAL],		/* bg color */
				copy, 									/* text to insert */
				-1);										/* -1: use strlen() */
			gtk_text_thaw(GTK_TEXT(pn->pn_textwidget));

			free(copy);
		}

		/*
		 * Save the old text.
		 */
		if ( pn->pn_text != NULL )
			free (pn->pn_text);
		pn->pn_text = 
			gtk_editable_get_chars(GTK_EDITABLE(pn->pn_textwidget), 0, -1);
		copy = XPSafeAlloc(strlen(text) + 1);
		strncpy(copy, (char *)(pn->pn_text+rc), strlen(text));
		strncpy((char *)(copy+strlen(text)),"\0", 1);

		/*
		 * Now highlight the new location.
		 */
		gtk_text_set_point(GTK_TEXT(pn->pn_textwidget), rc);
		gtk_text_forward_delete(GTK_TEXT(pn->pn_textwidget), strlen(text));
		gtk_text_freeze(GTK_TEXT(pn->pn_textwidget));
		gtk_text_insert (
			GTK_TEXT(pn->pn_textwidget),
			NULL, 										/* font */
			&pn->pn_textwidget->style->white, 	/* fg color */
			&pn->pn_textwidget->style->black,	/* bg color */
			copy, 										/* text to insert */
			-1);											/* -1: use strlen() */
		gtk_text_thaw(GTK_TEXT(pn->pn_textwidget));
		gtk_text_set_point(GTK_TEXT(pn->pn_textwidget), rc+strlen(text));

		/*
		 * Set the start and end pointers to this highlighted location.
		 */
		start = rc;
		end = rc + strlen(text);
	}
	else
		XPErrPopUp(XP_S_NOT_FOUND);
}

