/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: inserts.c
 *            DESCRIPTION: routines that insert simple text data into notes
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPInsertCalendar, XPInsertDates
 *      PRIVATE FUNCTIONS: CreateInsertPrompt, InsertAccept, InsertPopDown,
 *								 : InsertLeftArrowCB, 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_INSERTS_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include <time.h>

#include <gtk/gtk.h>
#include <gdk/gdkx.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void InsertPopDown();
static void InsertAccept();
static void InsertLeftArrowCB();
static void CreateInsertPrompt();


/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;
extern GtkWidget		*toplevel;


/* === Static Variables === */
static GtkWidget 		*insert_widget=NULL;
static GtkWidget		*insert_boxes[12];
static GtkWidget		*insert_buttons[2];
static GtkWidget		*insert_leftarrows[11];
static int				active_item;



/*========================================================================
 *	Name:			XPInsertCalendar
 *	Prototype:	void XPInsertCalendar(GtkWidget *widget, int index)
 *					
 *	Description:
 * Insert this months calendar into the note
 *
 *	Notes:
 *========================================================================*/
void
XPInsertCalendar(
	GtkWidget	*widget,
	int			index
)
{
#ifdef DEBUG
	char	*fname="XPInsertCalendar()";
#endif

	StickyNote	*pn;
	char			tmpfile[512];
	char			cmd[1024];
	int			rc;

	DBGEnter();

	/*
	 * Find the note we're adding the calender to.
	 */
	DBGPrintf(DBG_INFO,("savewidget index: %d\n", index));
	if ((pn = XPFindNote(index)) == NULL)
		return;
	DBGPrintf(DBG_INFO,("pn: 0x%08x\n", (int)pn));
	DBGPrintf(DBG_INFO,("savewidget(0): 0x%08x\n", pn->pn_savewidget));

	/*
	 * Run the calendar generating program to get the text.
	 * We use system() instead of popen() because the latter
	 * doesn't reliably tell us if the process actually ran.
	 * system() will provide the return value of the invoked
	 * program.
	 */
	sprintf(tmpfile,"%s/xp%d", app_res.tmp_dir, (int)getpid());
	sprintf (cmd, "%s > %s", app_res.calendar_cmd, tmpfile );
	if ( (rc=system ( cmd )) != 0 ) 
	{

		sprintf(cmd, XP_S_CANT_RUN_CMD, app_res.calendar_cmd, rc);
		XPErrPopUp(cmd);

		/* cleanup, if necessary */
		sprintf (cmd, "rm %s", tmpfile );
		system ( cmd );
		return;
	}

	/* stuff the output into the note */
	DBGPrintf(DBG_INFO,("savewidget(1): 0x%08x\n", pn->pn_savewidget));
	XPInsertText(pn, tmpfile, TRUE, FILE_TYPE);

	/*
	 * Changes occured to the note - set its save widget sensitive.
	 */
	DBGPrintf(DBG_INFO,("savewidget(2): 0x%08x\n", pn->pn_savewidget));
	XPSetSaveSensitive(NULL, NULL, pn);
}



/*========================================================================
 *	Name:			XPInsertDates
 *	Prototype:	void XPInsertDates(GtkWidget *widget, int index)
 *					
 *	Description:
 * Callback for inserting a formatted date into the note.
 *
 *	Notes:
 *========================================================================*/
void
XPInsertDates(
	GtkWidget	*widget,
	int			index
)
{
	StickyNote 	*pn;

	/*
	 * Find the note we're hiding.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	CreateInsertPrompt(pn);

}


/*========================================================================
 *	Name:			CreateInsertPrompt
 *	Prototype:	static void CreateInsertPrompt(StickyNote *pn)
 *					
 *	Description:
 * Create a prompt for insert information
 *
 *	Notes:
 *========================================================================*/
static void
CreateInsertPrompt(
	StickyNote	*pn
)
{
	GtkWidget		*label[11];
	GtkWidget		*vbox;
	GtkWidget		*insert_button;
	int				i;
	int				max_width=0;
	int				max_height=0;
	char				buf[256];
	GtkRequisition	attributes;
	GShell_T			gshell;

	/*
	 * Prevent multiple windows from being opened at once.
	 */
	if ( insert_widget != NULL )
		InsertPopDown();

	/* initialize marker */
	active_item=0;

	/*
	 * Create a popup shell widget
	 */
	insert_widget = XPCreateGenericShell(NotesInsertPrompt,
			ACCEPT_BUTTON | CANCEL_BUTTON, &gshell, FALSE );

	gtk_signal_connect (GTK_OBJECT (insert_widget), "destroy",
			GTK_SIGNAL_FUNC (InsertPopDown), NULL);

	vbox = gshell.vbox;
	insert_buttons[0] = gshell.accept;
	insert_buttons[1] = gshell.cancel;

	/*
	 * The set of button/labels that provide the various date formats
	 * to use.
	 */
	for ( i=0; i<11; i++)
	{
		switch(i)
		{
			case 0: sprintf(buf, "mm/dd/yy"); break;
			case 1: sprintf(buf, "<mm/dd/yy>"); break;
			case 2: sprintf(buf, "(mm/dd/yy)"); break;
			case 3: sprintf(buf, "mm/dd/yy - hh:mm"); break;
			case 4: sprintf(buf, "mm/dd/yy(hh:mm)"); break;
			case 5: sprintf(buf, "Jan 1, 1996"); break;
			case 6: sprintf(buf, "Jan 1, 1996 - hh:mm"); break;
			case 7: sprintf(buf, "Jan 1, 1996 (hh:mm)"); break;
			case 8: sprintf(buf, "Tuesday Jan 1, 1996"); break;
			case 9: sprintf(buf, "Tuesday Jan 1, 1996 - hh:mm"); break;
			case 10: sprintf(buf, "Tuesday Jan 1, 1996 (hh:mm)"); break;
		}

		/*
		 * Each row starts with a horizontal box.
		 */
		insert_boxes[i] = gtk_hbox_new(FALSE, 12);
		gtk_box_pack_start(GTK_BOX(vbox), insert_boxes[i], FALSE, FALSE, 5);
		gtk_widget_show(insert_boxes[i]);

		/*
		 * First comes a button and a label for each button.
		 */
		insert_button = gtk_button_new();
		gtk_widget_set_name (insert_button, "xp_button");
		insert_leftarrows[i] = gtk_label_new ("  -  ");
		gtk_widget_set_name (insert_leftarrows[i], "xp_button");
		gtk_box_pack_start(GTK_BOX(insert_boxes[i]), insert_button, 
				FALSE, FALSE, 2);
		gtk_container_add(GTK_CONTAINER(insert_button), insert_leftarrows[i]);
		gtk_widget_show(insert_button);
		gtk_widget_show(insert_leftarrows[i]);
		gtk_signal_connect (GTK_OBJECT (insert_button), "clicked",
			GTK_SIGNAL_FUNC (InsertLeftArrowCB), (gpointer)i);

		/*
		 * Then a label with the format type.
		 */
		label[i] = gtk_label_new(buf);
		gtk_box_pack_start(GTK_BOX(insert_boxes[i]), label[i], FALSE, FALSE, 2);
		gtk_widget_show(label[i]);

		/*
		 * Determine the width of the label - we need to make them all match
		 * later.
		 */
		gtk_widget_size_request(label[i], &attributes);
		if ( attributes.width > max_width )
		{
			max_width = attributes.width;
			max_height = attributes.height;
		}
	}

	/*
	 * the Cancel/Accept command buttons
	 */
	gtk_signal_connect (GTK_OBJECT (insert_buttons[0]), "clicked",
			GTK_SIGNAL_FUNC (InsertAccept), (gpointer)pn);

	gtk_signal_connect (GTK_OBJECT (insert_buttons[1]), "clicked",
			GTK_SIGNAL_FUNC (InsertPopDown), (gpointer)NULL);

	/*
	 * Pop up the window.
	 */
	gtk_widget_realize(insert_widget);
	XPCenterWindow(insert_widget, pn->pn_shellwidget, insert_widget);
	gtk_widget_show( insert_widget );
}


/*========================================================================
 *	Name:			InsertAccept
 *	Prototype:	void InsertAccept(GtkWidget *widget, int index)
 *					
 *	Description:
 * Accept selected date value. This routine has the potential to be 
 * very OS specific.
 *
 *	Notes:
 *========================================================================*/
void
InsertAccept(
	GtkWidget	*widget,
	StickyNote 	*pn
)
{
#ifdef DEBUG
	char	*fname="InsertAccept()";
#endif

	char			cmd[1024];
	time_t		time_tp;
	struct tm	*tm_tp;
	char			dayname[12];
	char			year[5];
	char			month[10];

	DBGEnter();

	/* get todays date and current time */
	time(&time_tp);
	tm_tp = localtime(&time_tp);

	/*
	 * convert numeric values to strings now, even if we don't use them,
	 * since we need these in more than one place.
	 */
	switch ( tm_tp->tm_wday )
	{
		case 0:	sprintf(dayname, "Sunday"); break;
		case 1:	sprintf(dayname, "Monday"); break;
		case 2:	sprintf(dayname, "Tuesday"); break;
		case 3:	sprintf(dayname, "Wednesday"); break;
		case 4:	sprintf(dayname, "Thursday"); break;
		case 5:	sprintf(dayname, "Friday"); break;
		case 6:	sprintf(dayname, "Saturday"); break;
	}
	switch ( tm_tp->tm_mon )
	{
		case 0: sprintf(month, "Jan"); break;
		case 1: sprintf(month, "Feb"); break;
		case 2: sprintf(month, "Mar"); break;
		case 3: sprintf(month, "Apr"); break;
		case 4: sprintf(month, "May"); break;
		case 5: sprintf(month, "Jun"); break;
		case 6: sprintf(month, "Jul"); break;
		case 7: sprintf(month, "Aug"); break;
		case 8: sprintf(month, "Sept"); break;
		case 9: sprintf(month, "Oct"); break;
		case 10: sprintf(month, "Nov"); break;
		case 11: sprintf(month, "Dec"); break;
	}
	if ( tm_tp->tm_year > 100 )
		sprintf(year,"20");
	else
		sprintf(year,"19");

	/*
	 * increment the numeric month, since localtime()
	 * returns the month from 0 to 11.
	 */
	tm_tp->tm_mon++;

	switch (active_item)
	{
		case 0:
			sprintf (cmd, 
				"%02d/%02d/%02d", tm_tp->tm_mon, tm_tp->tm_mday, tm_tp->tm_year );
			break;

		case 1:
			sprintf (cmd, 
				"<%02d/%02d/%02d>", tm_tp->tm_mon, tm_tp->tm_mday, tm_tp->tm_year );
			break;

		case 2:
			sprintf (cmd, 
				"(%02d/%02d/%02d)", tm_tp->tm_mon, tm_tp->tm_mday, tm_tp->tm_year );
			break;

		case 3:
			sprintf (cmd, 
				"%02d/%02d/%02d - %02d:%02d", 
				tm_tp->tm_mon, tm_tp->tm_mday, tm_tp->tm_year, 
				tm_tp->tm_hour, tm_tp->tm_min );
			break;

		case 4:
			sprintf (cmd, 
				"%02d/%02d/%02d(%02d:%02d)", 
				tm_tp->tm_mon, tm_tp->tm_mday, tm_tp->tm_year, 
				tm_tp->tm_hour, tm_tp->tm_min );
			break;

		case 5:
			sprintf (cmd, 
				"%s %d, %s%d", month, tm_tp->tm_mday, year, tm_tp->tm_year);
			break;

		case 6:
			sprintf (cmd, 
				"%s %d, %s%d - %02d:%02d", 
				month, tm_tp->tm_mday, year, tm_tp->tm_year,
				tm_tp->tm_hour, tm_tp->tm_min );
			break;

		case 7:
			sprintf (cmd, 
				"%s %d, %s%d (%02d:%02d)", 
				month, tm_tp->tm_mday, year, tm_tp->tm_year,
				tm_tp->tm_hour, tm_tp->tm_min );
			break;

		case 8:
			sprintf (cmd, 
				"%s %s %d, %s%d", 
				dayname, month, tm_tp->tm_mday, year, tm_tp->tm_year);
			break;

		case 9:
			sprintf (cmd, 
				"%s %s %d, %s%d - %02d:%02d", 
				dayname, month, tm_tp->tm_mday, year, tm_tp->tm_year,
				tm_tp->tm_hour, tm_tp->tm_min );
			break;

		case 10:
			sprintf (cmd, 
				"%s %s %d, %s%d (%02d:%02d)", 
				dayname, month, tm_tp->tm_mday, year, tm_tp->tm_year,
				tm_tp->tm_hour, tm_tp->tm_min );
			break;

	}

	/* stuff the output into the note */
	DBGPrintf(DBG_INFO, ("pn: 0x%08x\n", (int)pn));
	XPInsertText(pn, cmd, FALSE, TEXT_TYPE);

	gtk_widget_set_sensitive ( pn->pn_savewidget, TRUE );
	InsertPopDown();
}

/*
 * InsertPopDown - removes the Insert Dialog box
 */
static void
InsertPopDown()
{
	if ( insert_widget != NULL )
	{
		gtk_widget_destroy( insert_widget );
		insert_widget = NULL;
	}
}


/*
 * InsertLeftArrowcB - handle a left arrow click
 */
static void
InsertLeftArrowCB(
	GtkWidget	*widget,
	int			index
)
{
	int	i;

	for (i=0; i<11; i++)
	{
		if ( i == index )
			gtk_label_set(GTK_LABEL(insert_leftarrows[i]),"  >  ");
		else
			gtk_label_set(GTK_LABEL(insert_leftarrows[i]),"  -  ");
	}

	active_item=index;
}
