/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: loadsave.c
 *            DESCRIPTION: Handle note reading and writing
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPLoadSavedNotes
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_LOADSAVE_C

/* === System Headers === */
#include <sys/param.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include <gtk/gtk.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static int	GetNoteVersion();


/* === Global Variables === */
StickyEnvelope		*envelopes;



/*========================================================================
 *	Name:			XPLoadSavedNotes
 *	Prototype:	void XPLoadSavedNotes()
 *					
 *	Description:
 * Load any notes the user has saved from previous invokations.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *		
 *========================================================================*/
void
XPLoadSavedNotes()
{
#ifdef DEBUG
	char			*fname="XPLoadSavedNotes";
#endif

#ifdef USE_DIR
	register struct direct *d;
#else
	register struct dirent *d;
#endif

	register StickyNote *pn;

	DIR			*dp;
	FILE			*fp;
	char			buf[BUFSIZ], filename[MAXPATHLEN];
	int			n, len, nlen, shellx, shelly;
	int			texth, textw, hidden, anchor, namelen;
	int			pnalarm, month, day, hour, minute;
	int			save_screenx, save_screeny;
	int			version;
	int			note_type, alarm_repeat, volume;
	char			audio_file[BUFSIZ], project[BUFSIZ];
	char			*buf2;
	time_t		time_tp;
	struct tm	*tm_tp;
	int			curr_screenx, curr_screeny;

	DBGEnter();

	/*
	 * Grab the current screen height and width.
	 */
	curr_screenx = gdk_screen_width();
	curr_screeny = gdk_screen_height();

	/*
	 * Try to open the directory.
	 */
	DBGPrintf(DBG_INFO,("loading notes from %s\n", app_res.note_dir));
	if ((dp = opendir(app_res.note_dir)) == NULL)
	{
		fprintf(stderr, "xnotes: ");
		perror(app_res.note_dir);
		return;
	}

	nlen = strlen(StickyNoteFname);

	/*
	 * For each entry...
	 */
	while ((d = readdir(dp)) != NULL) 
	{
		/*
		 * Skip over anything which doesn't match our
		 * file naming scheme.
		 */
		if (strncmp(d->d_name, StickyNoteFname, nlen) != 0)
			continue;

		/*
		 * Make the full path name.
		 */
		sprintf(filename, "%s/%s", app_res.note_dir, d->d_name);

		/*
		 * Open the file.
		 */
		if ((fp = fopen(filename, "r")) == NULL)
			continue;

		/*
		 * Look for the magic cookie identifying this as
		 * a Sticky note.
		 */
		if (fscanf(fp, "%s", buf) == EOF) 
		{
			fclose(fp);
			continue;
		}

		/*
		 * I changed the magic cookie so it didn't conflict with
		 * the Postscript magic cookie, but we need to maintain
		 * compatibility, if requested.
		 */
		if ( (version=GetNoteVersion(buf)) == -1 )
		{
			DBGPrintf(DBG_INFO,("%s version unknown\n", buf));
			fclose(fp);
			continue;
		}

		switch (version)
		{

			/*
			 * Initial version, way back in the dark ages of X11R4.
			 */
			case 1: 
				fgets(buf, sizeof(buf), fp);
				n = sscanf(buf, "%d %d %d %d %d %d %d %d", 
						&shellx, &shelly, &texth, &textw, &len, 
						&hidden, &save_screenx, &save_screeny);
				/*
				 * Bad format; skip it.
				 */
				if (n < 5) 
				{
					fclose(fp);
					continue;
				}
				break;

			/*
			 * My first major release.
			 */
			case 2: 
				fgets(buf, sizeof(buf), fp);
				n = sscanf(buf, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d", 
					&shellx, &shelly, &texth, &textw, &len, 
					&hidden, &anchor, &save_screeny, &save_screenx,
					&pnalarm, &month, &day, &hour, &minute);

				/*
				 * Bad format; skip it.
				 */
				if (n < 14) 
				{
					fclose(fp);
					continue;
				}
				break;

			/*
			 * The first GTK/Motif-based release.  We have to do this 
			 * a little different because the project can be a multi-word
			 * string.
			 */
			case 3: 
				n = fscanf(fp, 
					"%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %s ", 
					&shellx, &shelly, &texth, &textw, &len, 
					&hidden, &anchor, &save_screeny, &save_screenx,
					&pnalarm, &month, &day, &hour, &minute,
					&note_type, &alarm_repeat, &volume, audio_file
					);
				fgets(project, sizeof(project), fp);
				buf2=strtok(project,"\n");
				strcpy(project, buf2);


				/*
				 * Bad format; skip it.
				 */
				if (n < 18) 
				{
					fclose(fp);
					continue;
				}
		}

		/*
		 * Adjust notes position if it was saved on different sized
		 * screens.
		 */
		if (curr_screenx!=save_screenx || curr_screeny!=save_screeny) 
		{
			shellx = (shellx * curr_screenx) / save_screenx;
			shelly = (shelly * curr_screeny) / save_screeny;
		}

		/*
		 * Get the index number of this note.
		 */
		n = atoi(&(d->d_name[nlen]));

		/*
		 * Get a note structure.
		 */
		pn = XPAllocNote(n);

		/*
		 * Get the name of the note
		 */
		fgets(buf, sizeof(buf), fp);
		namelen = strlen ( buf ) - 1;

		/* remove newline character */
		bcopy ( (void *)"\0", (void *)(buf+namelen), 1 );

		/* Get the notes title (aka Name) */
		if ( pn->pn_name )
			g_string_free(pn->pn_name, TRUE);
		pn->pn_name = g_string_new ( buf );

		/*
		 * Set the rest of the note information.
		 */
		pn->pn_shellx = shellx;
		pn->pn_shelly = shelly;
		pn->pn_textwidth = textw;
		pn->pn_textheight = texth;
		pn->pn_positionit = TRUE;
		pn->pn_hidden = (char)hidden;
		switch(version)
		{
			/*
			 * Version 1 didn't have alarms or anchors.
			 * It also didn't have audio alarms, projects, note types
			 * or alarm repeat counts.
			 */
			case 1: 
				pn->pn_anchor = FALSE;
				pn->pn_alarm = FALSE;
				time(&time_tp);
				tm_tp = localtime(&time_tp);
				pn->pn_alarm_mon = tm_tp->tm_mon;
				pn->pn_alarm_day = tm_tp->tm_mday;
				pn->pn_alarm_hour = tm_tp->tm_hour;
				pn->pn_alarm_min = tm_tp->tm_min;
				pn->pn_alarm_audio = NULL;
				pn->pn_alarm_repeat = 0;
				pn->pn_alarm_volume = 50;
				pn->pn_project = NULL;
				pn->pn_type = 0;
				break;

			/*
			 * Version 2 didn't have audio alarms, projects, note types
			 * or alarm repeat counts.
			 */
			case 2:
				pn->pn_anchor = (char)anchor;
				pn->pn_alarm = (char)pnalarm;
				pn->pn_alarm_mon = month;
				pn->pn_alarm_day = day;
				pn->pn_alarm_hour = hour;
				pn->pn_alarm_min = minute;
				pn->pn_alarm_audio = NULL;
				pn->pn_alarm_repeat = 0;
				pn->pn_alarm_volume = 50;
				pn->pn_project = NULL;
				pn->pn_type = 0;
				break;

			/*
			 * Version 3 is the answer to everyones prayers, except
			 * maybe those waiting for Elvis to return.
			 */
			case 3:
				pn->pn_anchor = (char)anchor;
				pn->pn_alarm = (char)pnalarm;
				pn->pn_alarm_mon = month;
				pn->pn_alarm_day = day;
				pn->pn_alarm_hour = hour;
				pn->pn_alarm_min = minute;
				pn->pn_type = note_type;
				pn->pn_alarm_volume = volume;

				if ( strcmp(audio_file, XP_S_NO_AUDIO) == 0)
				{
					pn->pn_alarm_audio = NULL;
				}
				else
				{
					pn->pn_alarm_audio = (char *)XPSafeAlloc(strlen(audio_file)+1);
					strcpy(pn->pn_alarm_audio, audio_file);
				}
				pn->pn_alarm_repeat = alarm_repeat;

				DBGPrintf(DBG_INFO,("Project name: %s\n", project));
				if ( strcmp(project, XP_S_NO_PROJECT) == 0)
					pn->pn_project = NULL;
				else
				{
					pn->pn_project = (char *)XPSafeAlloc(strlen(project)+1);
					strcpy(pn->pn_project, project);
				}
				break;

		}

		/*
		 * Save the file name.
		 */
		if ( pn->pn_file )
			g_string_free(pn->pn_file, TRUE);
		pn->pn_file = g_string_new(filename);

		/*
		 * Allocate a buffer in which we can save the file text until
		 * we actually create the Text widget.
		 */
		if ((pn->pn_text = (char *)XPSafeAlloc(len+1)) == NULL) 
		{
			fprintf(stderr, "xnotes: out of memory.\n");
			XPShutdown();
		}

		/*
		 * Read in the text.
		 */
		fread(pn->pn_text, sizeof(char), len, fp);
		fclose(fp);

		/*
		 * Make a widget for this note.
		 */
		XPMakeNoteWidget(pn);
	}

	closedir(dp);
}

static int
GetNoteVersion(
	char	*buf
)
{
	/*
	 * If its the current version, life is simple.
	 */
	if (strcmp(buf, StickyNoteMagic_v3) == 0) 
		return(3);

	/*
	 * If compatibility mode is turned on...
	 */
	if (app_res.compatibility) 
	{
		/*
		 * Is it a version 2 note?
		 */
		if (strcmp(buf, StickyNoteMagic_v2) == 0) 
			return(2);

		/*
		 * Must be that ancient history version.
		 */
		return(1);
	}
	return(-1);
}



/*========================================================================
 *	Name:			XPLoadEnvelopes
 *	Prototype:	void XPLoadEnvelopes()
 *					
 *	Description:
 * Load the envelope entries into a link list.
 *
 *	Notes:
 *		
 *========================================================================*/
void
XPLoadEnvelopes()
{
	struct stat		stat_buf;
	char				buf[MAXPATHLEN];
	char				filename[MAXPATHLEN];
	FILE				*fp;

	/*
	 * Make the full path name.
	 */
	sprintf(filename, "%s/%s", app_res.note_dir, StickyEnvelopeFname);

	/*
	 * Verify the file exists.  If not, the link list of envelope entries
	 * will simply be NULL.
	 */
	if ( stat(filename, &stat_buf) == -1 )
	{
		envelopes = NULL;
		return;
	}

	/*
	 * Open the file.
	 */
	if ((fp = fopen(filename, "r")) == NULL)
	{
		envelopes = NULL;
		return;
	}

	/*
	 * Look for the magic cookie identifying this as
	 * an XNotesPlus file.
	 */
	if (fgets(buf, MAXPATHLEN, fp) == NULL) 
	{
		fclose(fp);
		envelopes = NULL;
		return;
	}
	if ( GetNoteVersion(buf) == -1 )
	{
		fclose(fp);
		envelopes = NULL;
		return;
	}

	/*
	 * Read each record and insert it into the link list in alphabetical
	 * order.
	 */
	while (fgets(buf, MAXPATHLEN, fp) != NULL) 
	{
		/*
		 * Allocate a link.
		 */

		/*
		 * Parse the fields of the line.
		 */

		/*
		 * Insert the link in the list of envelopes.
		 */
	}

	/*
	 * Close the envelopes flat file.
	 */
	fclose(fp);
	
}


/*========================================================================
 *	Name:			XPInsertLink
 *	Prototype:	void XPInsertLink()
 *					
 *	Description:
 * Insert an envelope link into the list of envelope records in 
 * alphabetical order.
 *
 *	Notes:
 *		
 *========================================================================*/
void
XPInsertLink(
	StickyEnvelope		*env
)
{
}
