/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: menus.c
 *            DESCRIPTION: menus module for XNotesPlus
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPButtonPress, XPCreateMainMenu
 *      PRIVATE FUNCTIONS: HandleMenuSelection
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_MENUS_C

/* === System Headers === */
#include <signal.h>
#include <string.h>
#include <gtk/gtk.h>

/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"

#define MenuCreate_1p5x2	100
#define MenuCreate_2x3		101
#define MenuCreate_3x3		102
#define MenuCreate_3x4		103
#define MenuCreate_3x5		104
#define MenuCreate_4x6		105

#define NoteTypes				0
#define EndNoteTypes			1
#define MenuRaiseAll			2
#define MenuLowerAll			3
#define MenuSaveAll			4
#define MenuShowHidden		5
#define MenuUnHideAll		6
#define MenuHideAll			7
#define MenuCascade			8
#define MenuFindANote		9
#define EndNoteFunctions	10
#define MenuEnvelopes		11
#define MenuProjects			12
#define MenuPilotTool		13
#define MenuToolTips			14
#define EndOtherFunctions	15
#define MenuAbout				16
#define MenuExit				17


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void				HandleMenuSelection();
static void				ToggleTips();


/* === Global Variables === */
extern StickyNote		*notes;

int	button1_down;
int	button2_down;
int	button3_down;


/* === Static Variables === */
static GtkWidget		*tooltips_label;

static char *top_menu_strings[] = {
	"Create Notes",
	" ",
	"Raise All Notes",
	"Lower All Notes",
	"Save All Notes",
	"Hidden Notes",
	"Unhide All Notes",
	"Hide All Notes",
	"Cascade Notes",
	"Find A Note",
	" ",
	"Envelopes",
	"Projects",
	"Pilot Tool",
	"Disable ToolTips",
	" ",
	"About",
	"Exit",
	NULL,
};
static char *note_type_strings[] = {
	"Create 1.5x2 Note",
	"Create 2x3 Note",
	"Create 3x3 Note",
	"Create 3x4 Note",
	"Create 3x5 Note",
	"Create 4x6 Note",
	NULL,
};


/*========================================================================
 *	Name:			XPButtonPress
 *	Prototype:	XPButtonPress ( GtkWidget *widget, GdkEvent *event )
 *					
 *
 *	Description:
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
int 
XPButtonPress (
	GtkWidget *widget, 
	GdkEvent *event
)
{
	GdkEventButton *bevent = (GdkEventButton *) event; 

	switch (event->type)
	{
		case GDK_BUTTON_PRESS:
			switch( (int) (((GdkEventButton *)event)->button) )
			{
				case 1:
					button1_down = TRUE;
					break;
				case 2:
					XPLowerAllNotes();
					break;
				case 3:
					gtk_menu_popup (GTK_MENU(widget), NULL, NULL, NULL, NULL,
						bevent->button, bevent->time);
					break;
			}
			/*
			 * Tell calling code that we have handled this event; the buck
			 * stops here.
			 */
			return TRUE;
			break;

		case GDK_BUTTON_RELEASE:
			switch( (int) (((GdkEventButton *)event)->button) )
			{
				case 1:
					if ( button1_down == TRUE )
					{
						XPRaiseAllNotes();
						button1_down = FALSE;
					}
					break;
				case 2:
					break;
				case 3:
					break;
			}
			return TRUE;
			break;

		case GDK_LEAVE_NOTIFY:
			switch( (int) (((GdkEventButton *)event)->button) )
			{
				case 1:
					if ( button1_down == TRUE )
					{
						XPCreateNewNote(StickyNote_3x4, NULL);
						button1_down = FALSE;
					}
					break;
				case 2:
					break;
				case 3:
					break;
			}
			return TRUE;
			break;

		default:
			break;
	}

	/*
	 * Tell calling code that we have not handled this event; pass it
	 * on.
	 */
	return FALSE;
}


/*========================================================================
 *	Name:			XPCreateMainMenu
 *	Prototype:	XPCreateMainMenu ( GtkWidget *button )
 *
 *	Description:
 *	Create the menu in the plaid window.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPCreateMainMenu(
	GtkWidget	*button
)
{
	int	i,j;

	GtkWidget *menu, *submenu;
	GtkWidget *menu_items, *submenu_items;

	menu = gtk_menu_new();
	gtk_widget_set_name (menu, "main_menus");
	
	
	i = 0;
	while ( top_menu_strings[i] != NULL )
	{
		switch ( i )
		{
			case NoteTypes: 
				/* Create a new menu-item with a label in it */
				menu_items = gtk_menu_item_new_with_label(top_menu_strings[i]);
				gtk_widget_set_name (menu_items, "main_menus");

				/* ...and add it to the menu. */
				gtk_menu_append(GTK_MENU (menu), menu_items);

				submenu = gtk_menu_new();
				j=0;
				while ( note_type_strings[j] != NULL )
				{
					/* Create a new menu-item with a label in it */
					submenu_items = 
						gtk_menu_item_new_with_label(note_type_strings[j]);
					gtk_widget_set_name (submenu_items, "main_menus");
					gtk_menu_append(GTK_MENU (submenu), submenu_items);
					gtk_widget_show(submenu_items);

					/* Add a callback for this item. */
					gtk_signal_connect_object(GTK_OBJECT(submenu_items), "activate",
						GTK_SIGNAL_FUNC(HandleMenuSelection), (gpointer) j+100);

					j++;
				}
				gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_items), submenu);

				/* Display the menu item */
				gtk_widget_show(menu_items);

				break;

				/*
			case MenuProjects: 
				 * Projects are disabled until I figure out how to handle
				 * colors in Gtk/Gdk.
				break;
				 */

			case MenuPilotTool: 
				/*
				 * Disable Pilot Tool if the tools are not availalbe or
				 * the port has not been specified.
				 */
				if (	(	( app_res.pilot_xfer != NULL ) ||
							( app_res.pilot_addr != NULL ) ||
							(	( app_res.pilot_memos != NULL ) &&
								( app_res.pilot_imemos != NULL ) )
						) &&
						( getenv("PILOTPORT") != NULL )
					)
				{
					/* Create a new menu-item with a label in it */
					menu_items = gtk_menu_item_new_with_label(top_menu_strings[i]);
					gtk_widget_set_name (menu_items, "main_menus");

					/* ...and add it to the menu. */
					gtk_menu_append(GTK_MENU (menu), menu_items);

					/* Add a callback for this item. */
					gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
						GTK_SIGNAL_FUNC(HandleMenuSelection), (gpointer) i);

					/* Display the menu item */
					gtk_widget_show(menu_items);
				}
				break;

			case MenuToolTips: 
				/*
				 * Create a label seperate from the menu item so that we
				 * can change it on the fly.
				 */
				menu_items = gtk_menu_item_new();
				gtk_widget_set_name (menu_items, "main_menus");
				tooltips_label = gtk_label_new(top_menu_strings[i]);
				gtk_widget_set_name (menu_items, "main_menus");
				gtk_container_add(GTK_CONTAINER(menu_items), tooltips_label);

				/* ...and add it to the menu. */
				gtk_menu_append(GTK_MENU (menu), menu_items);

				/* Add a callback for this item. */
				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(HandleMenuSelection), (gpointer) i);

				/* Display the menu item */
				gtk_widget_show(tooltips_label);
				gtk_widget_show(menu_items);

				break;

			default:
				/* Create a new menu-item with a label in it */
				menu_items = gtk_menu_item_new_with_label(top_menu_strings[i]);
				gtk_widget_set_name (menu_items, "main_menus");

				/* ...and add it to the menu. */
				gtk_menu_append(GTK_MENU (menu), menu_items);

				/* Add a callback for this item. */
				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(HandleMenuSelection), (gpointer) i);

				if ( strlen(top_menu_strings[i]) == 0 )
					 gtk_widget_set_sensitive (menu_items, FALSE);

				/* Display the menu item */
				gtk_widget_show(menu_items);

				break;
		}

		i++;
	}

	gtk_signal_connect_object(GTK_OBJECT(button), "event",
		GTK_SIGNAL_FUNC (XPButtonPress), GTK_OBJECT(menu));
}



/*========================================================================
 *	Name:			HandleMenuSelection 			
 *	Prototype:	HandleMenuSelection(int)
 *
 *	Description:
 * Callback from menu widget to handle a selection.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
static void
HandleMenuSelection(
	int	menu_item
)
{
	/*
	 * Dispatch the command.
	 */
	switch (menu_item) {
	case MenuCreate_1p5x2:
		XPCreateNewNote(StickyNote_1p5x2, NULL);
		break;
	case MenuCreate_2x3:
		XPCreateNewNote(StickyNote_2x3, NULL);
		break;
	case MenuCreate_3x3:
		XPCreateNewNote(StickyNote_3x3, NULL);
		break;
	case MenuCreate_3x4:
		XPCreateNewNote(StickyNote_3x4, NULL);
		break;
	case MenuCreate_3x5:
		XPCreateNewNote(StickyNote_3x5, NULL);
		break;
	case MenuCreate_4x6:
		XPCreateNewNote(StickyNote_4x6, NULL);
		break;
	case MenuRaiseAll:
		XPRaiseAllNotes();
		break;
	case MenuLowerAll:
		XPLowerAllNotes();
		break;
	case MenuSaveAll:
		XPSaveAllNotes(TRUE);
		break;
	case MenuShowHidden:
		XPPopUpList();
		break;
	case MenuUnHideAll:
		XPUnHideAllNotes();
		break;
	case MenuHideAll:
		XPHideAllNotes();
		break;
	case MenuCascade:
		XPCascadeNotes();
		break;
	case MenuFindANote:
		XPCreateFindNotePrompt(notes, FALSE);
		break;
	case MenuEnvelopes:
		XPEnvelope();
		break;
	case MenuProjects:
		XPProject(XP_PROJ_CONFIG, NULL);
		break;
	case MenuPilotTool:
		XPPilot();
		break;
	case MenuToolTips:
		ToggleTips();
		break;
	case MenuAbout:
		XPAbout();
		break;
	case MenuExit:
		XPShutdown();
		break;
	}
}


/*========================================================================
 *	Name:			ToggleTips
 *	Prototype:	ToggleTips()
 *
 *	Description:
 *	Toggle between Tool Tips On and Tool Tips Off.
 *
 *========================================================================*/
static void
ToggleTips()
{
	if ( app_res.no_tips == TRUE )
	{
		app_res.no_tips = FALSE;
		gtk_label_set(GTK_LABEL(tooltips_label), XP_S_DISABLE_TIPS);
	}
	else
	{
		app_res.no_tips = TRUE;
		gtk_label_set(GTK_LABEL(tooltips_label), XP_S_ENABLE_TIPS);
	}
}
