/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995.
 *========================================================================
 *              FILE NAME: name.c 
 *            DESCRIPTION: handle note naming requests made by the user.
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPNameNote
 *      PRIVATE FUNCTIONS: ClearName, UpdateName
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_NAME_C

/* === System Headers === */
#include <stdio.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>


/* === Project Headers === */
#include "XNotesPlus.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void			ClearName();
static void			UpdateName();


/* === Global Variables === */
extern GtkWidget		*toplevel;

static GtkWidget		*dialogwidget;
static GtkWidget		*namewidget;



/*========================================================================
 *	Name:			XPNameNote
 *	Prototype:	XPNameNote(StickyNote *pn)
 *					
 *	Description:
 * Put up a window prompt for the notes name
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPNameNote(
	GtkWidget	*widget,
	int			index
)
{
	GtkWidget		*vbox;
	GtkWidget		*confirm_button, *cancel_button;
	StickyNote		*pn;
	GShell_T			gshell;


	/*
	 * Grab the note structure.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * If we need to construct the confirmation box do that,
	 * otherwise just reset the position and callbacks and
	 * put it up again.
	 */
	namewidget = XPCreateGenericShell(StickyNoteDialog,
			ACCEPT_BUTTON | CANCEL_BUTTON, &gshell, FALSE );

	gtk_signal_connect (GTK_OBJECT (namewidget), "destroy",
			GTK_SIGNAL_FUNC (ClearName), NULL);

	vbox = gshell.vbox;
	cancel_button = gshell.cancel;
	confirm_button = gshell.accept;

	/*
	 * An text entry widget in which the name will go.
	 * If the text already has a name, use it as the default text.
	 */
	dialogwidget = gtk_entry_new();
	gtk_widget_show(dialogwidget);
	gtk_container_add(GTK_CONTAINER(vbox), dialogwidget);
	if ( pn->pn_name->str != NULL )
		gtk_entry_set_text(GTK_ENTRY(dialogwidget), pn->pn_name->str);

	/*
	 * Confirmation button.
	 */
	gtk_signal_connect (GTK_OBJECT (confirm_button), "clicked",
			GTK_SIGNAL_FUNC (UpdateName), (gpointer)index);

	/*
	 * Cancellation button.
	 */
	gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
			GTK_SIGNAL_FUNC (ClearName), NULL);

	/*
	 * Pop up the confirmation box.
	 */
	gtk_widget_realize(namewidget);
	XPCenterWindow(namewidget, pn->pn_shellwidget, namewidget);
	gtk_widget_show( namewidget );

}

/*
 * ClearName - get rid of the confirmation box.
 */
static void
ClearName()
{
	if ( namewidget != NULL )
	{
		gtk_widget_destroy( namewidget );
		namewidget = NULL;
	}
}


static void
UpdateName(
	GtkWidget	*widget,
	int			index
)
{
	char			*text;
	StickyNote	*pn;

	if ((pn = XPFindNote(index)) == NULL)
	{
		XPErrPopUp(XP_S_NOTE_PTR_MISSING);
	}
	else
	{
		text = gtk_entry_get_text(GTK_ENTRY(dialogwidget));
		gtk_window_set_title (GTK_WINDOW (pn->pn_shellwidget), text );
		if ( pn->pn_name != NULL )
		{
			g_string_free(pn->pn_name, TRUE);
			pn->pn_name = g_string_new(text);
		}
	}

	ClearName();
}
