/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: note.c
 *            DESCRIPTION: note handler for XNotesPlus
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPCreateNewNote, XPAllocNote, XPFindNote,
 *								 : XPMakeNoteWidget, XPDestroyNote, XPSaveAllNotes,
 *								 : XPRaiseAllNotes, XPHideAllNotes, XPUnHideAllNotes
 *								 : XPLowerAllNotes, XPToggleRaise, XPToggleShow,
 *								 : XPMakeNewNote, XPSetSaveSensitive, XPCascadeNotes
 *								 : XPSaveNote, XPGetNotePosition
 *      PRIVATE FUNCTIONS: NoteIndex
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_NOTES_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <string.h>
#include <time.h>
#include <sys/param.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>

/* GTK/GDK specific includes */
#include <gtk/gtk.h>
#include <gdk/gdkx.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */
StickyNote	*XPFindNote();
void			XPDestroyNote();
void			XPMakeNoteWidget();
StickyNote *XPAllocNote();
void			XPSetSaveSensitive();
void			XPSaveNote();
void			XPEraseNote();
void			XPGetNotePosition();


/* === Private routine prototypes === */
static gint 		NoteIndex();
static void 		DestroyNoteHandler();
static void 		AnchorNote();
static void 		UnAnchorNote();
static void 		HideNote();


/* === Global Variables === */
extern AppRes		app_res;
extern GtkWidget	*toplevel;

extern ProjectType	*projects_list;

StickyNote	*notes = NULL;
GdkPixmap	*alarmon_xbm = NULL;
GdkPixmap	*alarmoff_xbm = NULL;
GdkPixmap	*anchoron_xbm = NULL;
GdkPixmap	*anchoroff_xbm = NULL;
GdkPixmap	*file_xbm = NULL;
GdkPixmap	*note_xbm = NULL;
GdkBitmap	*anchor_mask = NULL;
GdkBitmap	*anchoroff_mask = NULL;
GdkBitmap	*alarmon_mask = NULL;
GdkBitmap	*alarmoff_mask = NULL;
GdkBitmap	*file_mask = NULL;
GdkBitmap	*note_mask = NULL;

static int	raisestatus = 1;
static int	showstatus = 1;


/* === Static Variables === */
static char		*File_Menu_Items[] = {
	XP_FILE_SAVE,
	XP_FILE_OPEN,
	XP_FILE_EXPORT,
	XP_FILE_FIND,
	XP_FILE_PRINT,
	XP_FILE_EMAIL,
	NULL
	};
static char		*Options_Menu_Items[] = {
	XP_OPTIONS_HIDE,
	XP_OPTIONS_ERASE,
	XP_OPTIONS_DESTROY,
	XP_OPTIONS_NAME,
	XP_OPTIONS_PROJECT,
	XP_OPTIONS_CALENDAR,
	XP_OPTIONS_DATE,
	NULL
	};
static char		*Alarm_Menu_Items[] = {
	XP_ALARM_ON,
	XP_ALARM_OFF,
	NULL
	};
static char		*Anchor_Menu_Items[] = {
	XP_ANCHOR_ON,
	XP_ANCHOR_OFF,
	NULL
	};



/*========================================================================
 *	Name:			XPCreateNewNote 
 *	Prototype:	XPCreateNewNote(int type, StickyNote **caller_pn)
 *					
 *	Description:
 * Create a new note of the specified type.
 *
 *	Input Arguments:
 *	int	type			used as an index to determine the initial size of 
 *							the note.
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPCreateNewNote(
	int 			type,
	StickyNote	**caller_pn
)
{
#ifdef DEBUG
	char	*fname="XPCreateNewNote()";
#endif

	static gint hpi = 0;
	static gint wpi = 0;
	register StickyNote *pn;
	register gint hmm, wmm, hpixel, wpixel;

	DBGEnter();

	/*
	 * Find out the number of pixels per inch on the screen.  We
	 * can get the number of pixels, and the size in millimeters.
	 * Then we convert to pixels/inch using the formula
	 *
	 *  2.54 cm     10 mm     pixels     pixels
	 *	--------- x ------- x -------- = --------
	 *	  inch        cm         mm        inch
	 *
	 * The only problem with this is that some servers (like Xsun)
	 * lie about what these numbers really are.
	 */
	if ((hpi == 0) || (wpi == 0)) 
	{
		hpixel = gdk_screen_height();
		wpixel = gdk_screen_width();

		/*
		 * Gtk/Gdk do not (yet) provide MM values for the screen, but
		 * its a simple fix to add it.
		 */
		hmm = DisplayHeightMM (gdk_display, gdk_screen);
		wmm = DisplayWidthMM(gdk_display, gdk_screen);

		hpi = (int) ((25.4 * hpixel) / (float) hmm + 0.5);
		wpi = (int) ((25.4 * wpixel) / (float) wmm + 0.5);
	}

	/*
	 * Calculate sizes for the note.
	 */
	switch (type) {
	case StickyNote_1p5x2:
		hpixel = 1.5 * hpi;
		wpixel = 2 * wpi;
		break;
	case StickyNote_2x3:
		hpixel = 2 * hpi;
		wpixel = 3 * wpi;
		break;
	case StickyNote_3x3:
		hpixel = 3 * hpi;
		wpixel = 3 * wpi;
		break;
	case StickyNote_3x4:
		hpixel = 3 * hpi;
		wpixel = 4 * wpi;
		break;
	case StickyNote_3x5:
		hpixel = 3 * hpi;
		wpixel = 5 * wpi;
		break;
	case StickyNote_4x6:
		hpixel = 4 * hpi;
		wpixel = 6 * wpi;
		break;
	default:
		hpixel = 1.5 * hpi;
		wpixel = 2 * wpi;
		break;
	}

	/*
	 * Allocate a new note structure.
	 */
	pn = XPAllocNote(NewIndex);

	/*
	 * Set the text window size.
	 */
	pn->pn_textwidth = wpixel;
	pn->pn_textheight = hpixel;

	/*
	 * Make the widget for the note.
	 */
	XPMakeNoteWidget(pn);
	DBGPrintf(DBG_INFO,("pn: 0x%08x\n", pn));
	DBGPrintf(DBG_INFO,("savewidget id: 0x%08x\n", pn->pn_savewidget));

	/*
	 * Return the pointer to the new note if the caller requested it.
	 */
	if ( caller_pn != NULL ) 
		*caller_pn = pn;
}


/*========================================================================
 *	Name:			XPAllocNote 
 *	Prototype:	StickyNote *XPAllocNote ( int index )
 *					
 *	Description:
 * Allocate a new note structure and insert in into the list of notes.
 *
 *	Input Arguments:
 *	int	index				
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
StickyNote *
XPAllocNote(
	int index
)
{
#ifdef DEBUG
	char	*fname="XPAllocNote()";
#endif

	register StickyNote	*pn;
	char						label[15];		
	time_t					time_tp;
	struct tm				*tm_tp;

	/*
	 * Allocate a structure.
	 */
	if (notes == NULL) 
	{
		notes = (StickyNote *) XPSafeAlloc(sizeof(StickyNote));
		pn = notes;
	}
	else 
	{
		for (pn = notes; pn->pn_next != NULL; pn = pn->pn_next)
			;

		pn->pn_next = (StickyNote *) XPSafeAlloc(sizeof(StickyNote));
		pn = pn->pn_next;
	}
	DBGPrintf(DBG_INFO,("pn: 0x%08x\n", pn));

	/*
	 * Initialize the note.
	 */
	pn->pn_positionit = FALSE;
	pn->pn_hidden = FALSE;
	pn->pn_anchor = FALSE;
	pn->pn_type = 0;
	pn->pn_alarm = FALSE;
	pn->pn_alarm_audio = NULL;
	pn->pn_alarm_repeat = 0;
	pn->pn_alarm_volume = 50;
	pn->pn_project = NULL;
	pn->pn_file = NULL;

	/*
	 * Initial position is undefined.
	 */
	pn->pn_shellx = 0xffff;
	pn->pn_shelly = 0xffff;

	/* set todays date in the alarm entries */
	time(&time_tp);
	tm_tp = localtime(&time_tp);
	pn->pn_alarm_mon = tm_tp->tm_mon;
	pn->pn_alarm_day = tm_tp->tm_mday;
	pn->pn_alarm_hour = tm_tp->tm_hour;
	pn->pn_alarm_min = tm_tp->tm_min;

	DBGPrintf(DBG_INFO,("month: %2d\n", pn->pn_alarm_mon));
	DBGPrintf(DBG_INFO,("day  : %2d\n", pn->pn_alarm_day));
	DBGPrintf(DBG_INFO,("hour : %2d\n", pn->pn_alarm_hour));
	DBGPrintf(DBG_INFO,("min  : %2d\n", pn->pn_alarm_min));

	/*
	 * Don't hide by default.
	 */
	pn->pn_hide_index = -1;
	pn->pn_textsize = app_res.buf_size;
	pn->pn_text = NULL;

	/*
	 * If the index number was given, use it.  Otherwise,
	 * get a new index number.
	 */
	pn->pn_index = (index == NewIndex ? NoteIndex() : index);

	/* save the name of the note */
	sprintf (label, "Note %d", pn->pn_index);
	pn->pn_name = g_string_new ( label );

	return(pn);
}


/*========================================================================
 *	Name:			NoteIndex 
 *	Prototype:	gint NoteIndex ()
 *					
 *	Description:
 * Find the lowest free index number.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
static gint
NoteIndex()
{
	register gint index;
	register StickyNote *pn;

	/*
	 * This is O(n**2), but the list should be small.
	 */
	for (index = 1; ; index++) 
	{
		if ((pn = XPFindNote(index)) == NULL)
			return(index);
	}
}


/*========================================================================
 *	Name:			XPFindNote
 *	Prototype:	StickyNote *XPFindNote (gint index)
 *					
 *	Description:
 * Find the note structure with the given index number.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
StickyNote *
XPFindNote(
	register gint index
)
{
	register StickyNote *pn;

	for (pn = notes; pn != NULL; pn = pn->pn_next) 
	{
		if (pn->pn_index == index)
			return(pn);
	}

	return(NULL);
}


/*========================================================================
 *	Name:			XPMakeNoteWidget 
 *	Prototype:	void XPMakeNoteWidget ( StickyNote *pn )
 *					
 *	Description:
 * Create a new note window.
 *
 *	Input Arguments:
 *		StickyNote		*pn		pointer to note's data structure
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPMakeNoteWidget(
	StickyNote *pn
)
{
#ifdef DEBUG
	char				*fname="XPMakeNoteWidget";
#endif

	GtkWidget		*menubar, *menu_items=NULL;
	GtkWidget		*file_menu, *options_menu;
	GtkWidget		*vbox, *hbox, *text;
	GtkWidget		*text_hbox, *text_scrollbar;
	GtkWidget		*alarm_button, *alarm_menu;
	GtkWidget		*anchor_button, *anchor_menu;
	GtkWidget		*file_button;
	GtkWidget		*notes_button;
	GtkWidget		*note_pixmap, *file_pixmap;
	GtkStyle			*style, *style1, *style2;
	GdkColor			gdk_color;
	GdkColormap		*colormap;
	ProjectType		*item;
	char				buf[100256];
	int				i;

	static char		firsttime = TRUE;

	DBGEnter();

	/*
	 * Make the shell widget for this note.  We just use
	 * a pop-up shell for this.
	 */
	pn->pn_shellwidget = gtk_window_new (GTK_WINDOW_DIALOG);
	gtk_widget_set_uposition(pn->pn_shellwidget, pn->pn_shellx, pn->pn_shelly);
	gtk_window_set_wmclass(
		GTK_WINDOW(pn->pn_shellwidget),
		"StickyNote",
		StickyNoteClass);

	/*
	 * Force use of Destroy option instead of window manager close options.
	 */
	gtk_signal_connect (GTK_OBJECT (pn->pn_shellwidget), "delete_event",
			GTK_SIGNAL_FUNC (gtk_false), (gpointer) pn->pn_shellwidget);
	gtk_widget_realize (pn->pn_shellwidget);
/*
	gdk_window_set_decorations (pn->pn_shellwidget->window, 
		GDK_DECOR_ALL | GDK_DECOR_MENU );
*/

	gtk_window_set_title (GTK_WINDOW (pn->pn_shellwidget), 
			(gchar *)(pn->pn_name)->str);


	/* A vertical box inside which the menubar and text area will go */
	vbox = gtk_vbox_new(FALSE, 1);
	gtk_container_border_width (GTK_CONTAINER (vbox), 0);
	gtk_container_add(GTK_CONTAINER(pn->pn_shellwidget), vbox);
	gtk_widget_show(vbox);

	/* A horizontal box for the menubar, and the alarm and anchor icons */
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (hbox), 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
	gtk_widget_show(hbox);

	/*
	 * The Alarm button.
	 */
	pn->pn_alarmwidget = 
		alarm_button = gtk_menu_item_new();

	/*
	 * The anchor button.
	 */
	anchor_button = gtk_menu_item_new();

	/*
	 * Only create the pixmaps once - we can reuse them.
	 */
	if ( firsttime )
	{
		style = gtk_widget_get_style( alarm_button );
		sprintf(buf, "%s/%s", app_res.xpm_dir, XP_ALARM_ON_XPM);
		alarmon_xbm = gdk_pixmap_create_from_xpm(
						toplevel->window,
						&alarmon_mask,
						&style->bg[GTK_STATE_NORMAL],
						buf);

		sprintf(buf, "%s/%s", app_res.xpm_dir, XP_ALARM_OFF_XPM);
		anchoroff_xbm = alarmoff_xbm = gdk_pixmap_create_from_xpm(
						toplevel->window,
						&alarmoff_mask,
						&style->bg[GTK_STATE_NORMAL],
						buf);

		style = gtk_widget_get_style( anchor_button );
		sprintf(buf, "%s/%s", app_res.xpm_dir, XP_ANCHOR_XPM);
		anchoron_xbm = gdk_pixmap_create_from_xpm(
						toplevel->window,
						&anchor_mask,
						&style->bg[GTK_STATE_NORMAL],
						buf);

		anchoroff_mask = alarmoff_mask;

		sprintf(buf, "%s/%s", app_res.xpm_dir, XP_FILE_XPM);
		file_xbm = gdk_pixmap_create_from_xpm(
						toplevel->window,
						&file_mask,
						&style->bg[GTK_STATE_NORMAL],
						buf);

		sprintf(buf, "%s/%s", app_res.xpm_dir, XP_NOTES_XPM);
		note_xbm = gdk_pixmap_create_from_xpm(
						toplevel->window,
						&note_mask,
						&style->bg[GTK_STATE_NORMAL],
						buf);

		firsttime = FALSE;
	}

	if ( pn->pn_alarm )
		pn->pn_alarm_pixmapid = gtk_pixmap_new(alarmon_xbm, alarmon_mask);
	else
		pn->pn_alarm_pixmapid = gtk_pixmap_new(alarmoff_xbm, alarmoff_mask);

	gtk_container_add(GTK_CONTAINER(alarm_button), pn->pn_alarm_pixmapid);
	gtk_widget_show( pn->pn_alarm_pixmapid );

	if ( pn->pn_anchor )
		pn->pn_anchor_pixmapid = gtk_pixmap_new(anchoron_xbm, anchor_mask);
	else
		pn->pn_anchor_pixmapid = gtk_pixmap_new(anchoroff_xbm, anchoroff_mask);
	gtk_widget_show( pn->pn_anchor_pixmapid );
	gtk_container_add(GTK_CONTAINER(anchor_button), pn->pn_anchor_pixmapid);

	/*
	 * Show the menu buttons.
	 */
	gtk_widget_show(alarm_button);
	gtk_widget_show(anchor_button);

	/*
	 * Add the alarm and anchor menus.
	 */
	alarm_menu = gtk_menu_new();
	anchor_menu = gtk_menu_new();

	gtk_menu_item_set_submenu(GTK_MENU_ITEM (anchor_button), anchor_menu);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM (alarm_button), alarm_menu);

	for(i = 0; Alarm_Menu_Items[i] != NULL; i++)
	{
		menu_items = gtk_menu_item_new_with_label(Alarm_Menu_Items[i]);
		gtk_widget_set_name (menu_items, "option_menus");
		gtk_menu_append(GTK_MENU (alarm_menu), menu_items);
		gtk_widget_show(menu_items);
		switch (i) 
		{
			case XP_ALARM_ON_ID:
				pn->pn_settimewidget = menu_items;
				gtk_widget_set_name (menu_items, "alarm_button");
				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(XPCreateTimerPrompt), (gpointer) pn->pn_index);
				break;
			case XP_ALARM_OFF_ID:
				pn->pn_unsettimewidget = menu_items;
				gtk_widget_set_name (menu_items, "alarm_button");
				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(XPAlarmOff), (gpointer) pn->pn_index);
				break;
		}
	}

	for(i = 0; Anchor_Menu_Items[i] != NULL; i++)
	{
		menu_items = gtk_menu_item_new_with_label(Anchor_Menu_Items[i]);
		gtk_widget_set_name (menu_items, "option_menus");
		gtk_menu_append(GTK_MENU (anchor_menu), menu_items);
		gtk_widget_show(menu_items);
		switch (i) 
		{
			case XP_ANCHOR_ON_ID:
				pn->pn_anchorwidget = menu_items;
				gtk_widget_set_name (menu_items, "anchor_button");
				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(AnchorNote), (gpointer) pn->pn_index);
				break;
			case XP_ANCHOR_OFF_ID:
				pn->pn_unanchorwidget = menu_items;
				gtk_widget_set_name (menu_items, "anchor_button");
				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(UnAnchorNote), (gpointer) pn->pn_index);
				break;
		}
	}

	/*
	 * Set anchor sensitivity appropriately.
	 */
	if (pn->pn_anchor)
	{
		gtk_widget_set_sensitive ( pn->pn_anchorwidget, FALSE );
		gtk_widget_set_sensitive ( pn->pn_unanchorwidget, TRUE );
	}
	else
	{
		gtk_widget_set_sensitive ( pn->pn_anchorwidget, TRUE );
		gtk_widget_set_sensitive ( pn->pn_unanchorwidget, FALSE );
	}

	if (pn->pn_alarm)
	{
		gtk_widget_set_sensitive ( pn->pn_settimewidget, FALSE );
		gtk_widget_set_sensitive ( pn->pn_unsettimewidget, TRUE );
	}
	else
	{
		gtk_widget_set_sensitive ( pn->pn_settimewidget, TRUE );
		gtk_widget_set_sensitive ( pn->pn_unsettimewidget, FALSE );
	}

	/*
	 * Create the menu bar, with all its menus and buttons.  Add
	 * it to the note window.
	 */
	file_menu = gtk_menu_new();
	options_menu = gtk_menu_new();
/*
	file_button = gtk_menu_item_new_with_label(XP_FILE_LABEL);
*/
	file_button = gtk_menu_item_new();
	file_pixmap = gtk_pixmap_new(file_xbm, file_mask);
	gtk_container_add(GTK_CONTAINER(file_button), file_pixmap );
	gtk_widget_show( file_pixmap );

/*
	notes_button = gtk_menu_item_new_with_label(XP_NOTES_LABEL);
*/
	notes_button = gtk_menu_item_new();
	note_pixmap = gtk_pixmap_new(note_xbm, note_mask);
	gtk_container_add(GTK_CONTAINER(notes_button), note_pixmap );
	gtk_widget_show( note_pixmap );

	gtk_widget_show(file_button);
	gtk_widget_show(notes_button);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM (file_button), file_menu);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM (notes_button), options_menu);

	/* The menu bar - stuff it in the vertical box */
	menubar = gtk_menu_bar_new();
	gtk_box_pack_start(GTK_BOX(hbox), menubar, TRUE, TRUE, 0);
	gtk_widget_show(menubar);

	/* Add the menus to the menu bar */
	gtk_menu_bar_append(GTK_MENU_BAR (menubar), alarm_button);
	gtk_menu_bar_append(GTK_MENU_BAR (menubar), anchor_button);
	gtk_menu_bar_append(GTK_MENU_BAR (menubar), file_button);
	gtk_menu_bar_append(GTK_MENU_BAR (menubar), notes_button);

	/*
	 * Adjust the size of the menu bar components - make it as small as
	 * possible.
	 */
	gtk_widget_set_usize(menubar, -1 , 24);
	gtk_widget_set_usize(alarm_button, -1 , 24);
	gtk_widget_set_usize(anchor_button, -1 , 24);


	/* The actual menus */
	for(i = 0; File_Menu_Items[i] != NULL; i++)
	{
		switch(i)
		{
			case XP_FILE_SAVE_ID:
				menu_items = gtk_menu_item_new_with_label(File_Menu_Items[i]);
				gtk_widget_set_name (menu_items, "option_menus");

				pn->pn_savewidget = menu_items;
				DBGPrintf(DBG_TRACE2,
					("pn/savewidget id/index: 0x%08x/0x%08x/%d\n", 
						(int)pn, pn->pn_savewidget, pn->pn_index));
				gtk_widget_set_sensitive(pn->pn_savewidget, FALSE);
				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(XPSaveNote), (gpointer) pn->pn_index);

				gtk_menu_append(GTK_MENU (file_menu), menu_items);
				gtk_widget_show(menu_items);
				break;

			case XP_FILE_OPEN_ID:
				menu_items = gtk_menu_item_new_with_label(File_Menu_Items[i]);
				gtk_widget_set_name (menu_items, "option_menus");

				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(XPOpenNote), (gpointer) pn->pn_index);

				gtk_menu_append(GTK_MENU (file_menu), menu_items);
				gtk_widget_show(menu_items);
				break;

			case XP_FILE_EXPORT_ID:	
				menu_items = gtk_menu_item_new_with_label(File_Menu_Items[i]);
				gtk_widget_set_name (menu_items, "option_menus");

				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(XPExportNote), (gpointer) pn->pn_index);

				gtk_menu_append(GTK_MENU (file_menu), menu_items);
				gtk_widget_show(menu_items);
				break;

			case XP_FILE_FIND_ID:
				menu_items = gtk_menu_item_new_with_label(File_Menu_Items[i]);
				gtk_widget_set_name (menu_items, "option_menus");

				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(XPFindTextPrompt), (gpointer) pn->pn_index);

				gtk_menu_append(GTK_MENU (file_menu), menu_items);
				gtk_widget_show(menu_items);
				break;

			case XP_FILE_PRINT_ID:
				/*
				 * If the print command was not found, we dont' give access
				 * to this feature.  After all, it wouldn't work anyway, right?
				 */
				if ( app_res.print_cmd != NULL )
				{
					menu_items = gtk_menu_item_new_with_label(File_Menu_Items[i]);
					gtk_widget_set_name (menu_items, "option_menus");

					gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
						GTK_SIGNAL_FUNC(XPPrintNote), (gpointer) pn->pn_index);

					gtk_menu_append(GTK_MENU (file_menu), menu_items);
					gtk_widget_show(menu_items);
				}
				break;

			case XP_FILE_EMAIL_ID:
				menu_items = gtk_menu_item_new_with_label(File_Menu_Items[i]);
				gtk_widget_set_name (menu_items, "option_menus");

				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(XPCreateEmailNotePrompt), 
					(gpointer) pn->pn_index);

				gtk_menu_append(GTK_MENU (file_menu), menu_items);
				gtk_widget_show(menu_items);
				break;
		}
	}
	for(i = 0; Options_Menu_Items[i] != NULL; i++)
	{
		menu_items = gtk_menu_item_new_with_label(Options_Menu_Items[i]);
		gtk_widget_set_name (menu_items, "option_menus");

		switch(i)
		{
			case XP_OPTIONS_HIDE_ID:
				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(HideNote), (gpointer) pn->pn_index);
				break;
			case XP_OPTIONS_ERASE_ID:
				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(XPEraseNote), (gpointer) pn);
				break;
			case XP_OPTIONS_DESTROY_ID:
				gtk_widget_set_name (menu_items, "destroy_button");
				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(XPDestroyNote), (gpointer) pn->pn_index);
				break;
			case XP_OPTIONS_NAME_ID:
				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(XPNameNote), (gpointer) pn->pn_index);
				break;
			case XP_OPTIONS_PROJ_ID:
				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(XPNoteProject), (gpointer) pn->pn_index);
				break;
			case XP_OPTIONS_CALENDAR_ID:
				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(XPInsertCalendar), (gpointer) pn->pn_index);
				break;
			case XP_OPTIONS_DATE_ID:
				gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
					GTK_SIGNAL_FUNC(XPInsertDates), (gpointer) pn->pn_index);
				break;
		}

		gtk_menu_append(GTK_MENU (options_menu), menu_items);
		gtk_widget_show(menu_items);
	}

	/*
	 * Add the scrolled text field to the note.
	 */
	text_hbox = gtk_hbox_new(FALSE, 1);
	gtk_container_add (GTK_CONTAINER (vbox), text_hbox);
	gtk_widget_show(text_hbox);
	pn->pn_texthbox = text_hbox;

	text = gtk_text_new (NULL, NULL);
	gtk_widget_set_usize(text, pn->pn_textwidth, pn->pn_textheight);
	gtk_box_pack_start(GTK_BOX(text_hbox), text, TRUE, TRUE, 0);
	gtk_text_set_editable (GTK_TEXT (text), TRUE);
	gtk_text_set_word_wrap(GTK_TEXT(text), TRUE);
	gtk_widget_realize(text);
	pn->pn_textwidget = text;

	/*
	 * Get a copy of the original style so we can change background colors
	 * on individual notes later.
	 */
	style = gtk_style_copy( gtk_widget_get_style( text ) );
	gtk_widget_set_style(text, style);

	/*
	 * At this point the pn->pn_text structure member should hold the
	 * any data we need in the text widget.  After we add this to the
	 * text field we clean up that storage.  We will be using the
	 * GtkText widgets storage directly from here on.
	 *
	*/
	if ( pn->pn_text != NULL )
	{
		gtk_text_freeze(GTK_TEXT(text));
		gtk_text_insert (
			GTK_TEXT (text), 			/* widget */
			NULL, 						/* font */
			&text->style->black, 	/* fg color */
			NULL,							/* bg color */
			pn->pn_text, 				/* text to insert */
			-1);							/* -1: use strlen(); >0: # chars to insert */
		free(pn->pn_text);
		pn->pn_text = NULL;
		gtk_text_thaw (GTK_TEXT (text));
	}
	gtk_widget_show( text );

	/*
	 * A scrollbar to control scrolling of the text window.
	 */
	pn->pn_scrollwidget = 
		text_scrollbar = gtk_vscrollbar_new(GTK_TEXT(text)->vadj);
	gtk_widget_show(text_scrollbar);
	gtk_box_pack_start(GTK_BOX(text_hbox), text_scrollbar, FALSE, FALSE, 0);
	gtk_widget_set_usize(text_scrollbar, 11, -1);

	/*
	 * Update the notes color.  This is a hack - the GTK v1.0.x Text
	 * widget doesn't accept changes to the background colors, so we
	 * only get the border changed.  In order to show the project color
	 * we have to set the background of the scroll widget too.  Bleck.
	 */
	if ( pn->pn_project != NULL )
	{
		item = projects_list;
		while ( item != NULL )
		{
			DBGPrintf(DBG_TRACE2,("item/project: *%s*/*%s*\n", 
				item->name, pn->pn_project));
			if ( strcmp(item->name, pn->pn_project) == 0 )
			{
				DBGPrintf(DBG_TRACE3,("found match.\n"));
				style1 = gtk_widget_get_style( pn->pn_textwidget );
				style2 = gtk_style_copy( style1 );
				colormap = gdk_window_get_colormap (toplevel->window);
				gdk_color.red = (guint16)(item->color[0]*65535.0);
				gdk_color.green = (guint16)(item->color[1]*65535.0);
				gdk_color.blue = (guint16)(item->color[2]*65535.0);
				gdk_color_alloc (colormap, &gdk_color);
				style2->base[GTK_STATE_NORMAL].red = gdk_color.red;
				style2->base[GTK_STATE_NORMAL].green = gdk_color.green;
				style2->base[GTK_STATE_NORMAL].blue = gdk_color.blue;
				style2->bg[GTK_STATE_NORMAL].red = gdk_color.red;
				style2->bg[GTK_STATE_NORMAL].green = gdk_color.green;
				style2->bg[GTK_STATE_NORMAL].blue = gdk_color.blue;
				gtk_widget_set_style(pn->pn_textwidget, style2);

				gdk_window_set_background(pn->pn_scrollwidget->window, &gdk_color);

			}
			item = item->next;
		}
	}
#ifdef DEBUG
	else
		DBGPrintf(DBG_INFO,("project name is NULL\n"));
#endif


	/*
	 * Event handlers for the text area.
	 */
	DBGPrintf(DBG_INFO,("text: 0x%08x\n", (int)text));
	DBGPrintf(DBG_INFO,("pn: 0x%08x\n", (int)pn));
	DBGPrintf(DBG_INFO,("savewidget id: 0x%08x\n", pn->pn_savewidget));
	gtk_signal_connect (
		GTK_OBJECT(text), "key_press_event",
		GTK_SIGNAL_FUNC(XPSetSaveSensitive), pn);

	
	/*
	 * Position the new window, if it hasn't been positioned previously.
	 */
	if ( ( pn->pn_shellx == 0xffff ) && ( pn->pn_shellx == 0xffff ) )
		XPCenterWindow(pn->pn_shellwidget, NULL, pn->pn_shellwidget);

	/*
	 * Show the note.
	 */
	if ( pn->pn_hidden == FALSE )
		gtk_widget_show( pn->pn_shellwidget );
}


/*========================================================================
 *	Name:			XPDestroyNote and DestroyNoteHandler
 *					
 *	Description:
 * Destroy a note window.  The former just opens a confirmation box and
 * the latter is the routine that gets called if the user confirms the
 * destroy request.
 *
 *========================================================================*/
void
XPDestroyNote(
	GtkWidget	*widget,
	int			index
)
{
	StickyNote	*pn;

	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Create a confirmation box.
	 */
	XPMsgWindow(
		XP_S_DESTROY_CONFIRM, XP_CONFIRM_TYPE | XP_MSG_NOHELP, NULL,
		pn->pn_shellwidget, DestroyNoteHandler, pn->pn_index, NULL, NULL);

}

void
DestroyNoteHandler(
	GtkWidget	*widget,
	int			index
)
{
#ifdef DEBUG
	char		*fname="DestroyNoteHandler";
#endif

	register StickyNote	*pn, *prev;

	DBGEnter();
	DBGPrintf(DBG_INFO,("index=%d",index));

	/*
	 * Find the note we're destroying.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;
	
	DBGPrintf(DBG_INFO,("Found note."));

	if ( pn != NULL )
	{
		gtk_widget_destroy(pn->pn_shellwidget);

		/*
		 * Get rid of the note structure.
		 */
		if (pn != notes) 
		{

			/*
			 * first, free up the name buffer
			 */
			if ( pn->pn_name != NULL )
				g_string_free ( pn->pn_name, TRUE );
	
			for (prev = notes; prev->pn_next; prev = prev->pn_next) 
			{
				if (prev->pn_next == pn)
					break;
			}

			prev->pn_next = pn->pn_next;
		}
		else 
			notes = pn->pn_next;

		/*
		 * Get rid of the file.
		 */
		if (pn->pn_file) 
		{
			unlink(pn->pn_file->str);
			g_string_free(pn->pn_file, TRUE);
		}

		/*
		 * Free the note text memory that was allocated.
		 */
		free(pn);
	}

}


/*========================================================================
 *	Name:			XPSaveAllNotes
 *	Prototype:	void XPSaveAllNotes (int showmsg)
 *					
 *	Description:
 * Save all notes.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPSaveAllNotes(
	int	showmsg
)
{
	register StickyNote *pn;
	char found;

	found = FALSE;
	for (pn = notes; pn != NULL; pn = pn->pn_next)
	{
		XPSaveNote(pn->pn_shellwidget, pn->pn_index);
		found = TRUE;
	}

	if ( showmsg )
		if ( !found )
			XPErrPopUp("There are no notes to save.");
}


/*========================================================================
 *	Name:			XPSaveNote
 *	Prototype:	void XPSaveNote(GtkWidget *window, int index) 
 *					
 *	Description:
 * Use temporary files for saves in case the system crashes or xnotes
 * exits unexpectedly during a save.  This will preserve, at a minimum,
 * the last copy of the note.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPSaveNote(
	GtkWidget	*window,
	int			index
)
{
	register StickyNote	*pn;
	GtkRequisition			win_attributes;
	struct stat				stat_buf;

	FILE		*fp;
	int		len, shellx, shelly, texth, textw;
	int		xpos, ypos;
	char		saveit;
	char		tmpfile1[512], tmpfile2[512];
	char		*audio_file, *project;
	char		*filename;
	char		cmd[512];

	static char		*noaudio=XP_S_NO_AUDIO;
	static char		*noproject=XP_S_NO_PROJECT;

	/*
	 * Find the note we're saving.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/* by default, we're not going to save the note */
	saveit = FALSE;

	/*
	 * If the note has changed, the sensitivity of the note
	 * save option will be True.
	 */
	if ( GTK_WIDGET_IS_SENSITIVE(pn->pn_savewidget) == TRUE )
		saveit = TRUE;

	/*
	 * Get the position of the shell window.
	 */
	XPGetNotePosition(pn->pn_shellwidget, &shellx, &shelly);
	gdk_window_get_position(pn->pn_shellwidget->window, &xpos, &ypos);
	shellx += xpos;
	shelly += ypos;

	if ( (pn->pn_shellx != shellx ) || (pn->pn_shelly != shelly ) )
	{
		pn->pn_shellx = shellx;
		pn->pn_shelly = shelly;
		saveit = TRUE;
	}

	/*
	 * if nothing about the note has changed, don't waste effor saving it
	 */
	if ( !saveit )
		return;

	/*
	 * If it doesn't have a file name, make one.
	 */
	if (pn->pn_file == NULL)
	{
		filename = XPMakeFname(pn->pn_index);
		pn->pn_file = g_string_new ( filename );
		free(filename);
	}


	/*
	 * Get the size of the text window.
	 */
	gtk_widget_size_request(pn->pn_textwidget, &win_attributes);
	textw = win_attributes.width;
	texth = win_attributes.height;

	/*
	 * Get the length of the text in the window.
	 */
	if (pn->pn_text != NULL )
		free(pn->pn_text);
	pn->pn_text = gtk_editable_get_chars(GTK_EDITABLE(pn->pn_textwidget), 0, -1);
	len = strlen(pn->pn_text);

	/*
	 * Create the temporary filenames.  These temporary files go in
	 * the same directory as the real notes to avoid problems with
	 * renaming files across file system boundaries, etc.
	 */
	sprintf(tmpfile1,"%s/xp1.%d", app_res.note_dir, (int)getpid());
	sprintf(tmpfile2,"%s/xp2.%d", app_res.note_dir, (int)getpid());

	/*
	 * Open the first temporary file for writing
	 */
	if ((fp = fopen(tmpfile1, "w")) == NULL) {
		fprintf(stderr, "xnotes: ");
		perror(tmpfile1);
		return;
	}

	/*
	 * Determine what to output for the audio file and the project
	 * name.
	 */
	if ( pn->pn_alarm_audio == NULL )
		audio_file = noaudio;
	else
		audio_file = pn->pn_alarm_audio;
	if ( pn->pn_project == NULL )
		project = noproject;
	else
		project = pn->pn_project;

	/*
	 * Print out the information needed to recreate the note.
	 */
	fprintf(fp, 
		"%s %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %s %s\n%s\n", 
		StickyNoteMagic_v3, shellx, shelly, texth, textw, 
		len, pn->pn_hidden, pn->pn_anchor,
		gdk_screen_height(), gdk_screen_width(),
		pn->pn_alarm,
		pn->pn_alarm_mon, pn->pn_alarm_day, 
		pn->pn_alarm_hour, pn->pn_alarm_min, 
		pn->pn_type, pn->pn_alarm_repeat, pn->pn_alarm_volume,
		audio_file, project,
		pn->pn_name->str);

	/*
	 * Write out the text of the note.
	 */
	if (len)
		fwrite(pn->pn_text, sizeof(char), len, fp);

	fclose(fp);

	/* rename the old note (pn->pn_file, if it exists) to tmp2 */
	if ( stat(pn->pn_file->str, &stat_buf) == 0 )
	{
		if ( rename(pn->pn_file->str, tmpfile2) != 0 )
		{
			/* error - couldn't do save */
			sprintf(cmd, "xnotes: %s-%s", pn->pn_file->str, tmpfile2);
			perror(cmd);
			sprintf(cmd,"Couldn't rename %s\nto %s.\nSave of note failed.\n"
				"%s should contain valid copy of note.", 
				pn->pn_file->str, tmpfile2, tmpfile2);
			XPErrPopUp(cmd);
			return;
		}

		/* check that rename worked (due to NFS problems) */
		if ( stat(tmpfile2, &stat_buf) != 0 )
		{
			/* rename didn't work! Goddam NFS.... */
			sprintf(cmd,"Rename %s to\n %s failed.\n"
				"Might be NFS problems.\nSave of note failed.", 
				pn->pn_file->str, tmpfile2);
			XPErrPopUp(cmd);
			return;
		}
	}

	/* rename tmp1 note to old note */
	if ( rename(tmpfile1, pn->pn_file->str) != 0 )
	{
		/* error - couldn't do save - clean up */
		sprintf(cmd, "xnotes: %s-%s", tmpfile1, pn->pn_file->str);
		perror(cmd);
		sprintf(cmd,"Couldn't rename %s\nto %s.\n"
			"Save of note failed."
			"%s should contain valid copy of note.", 
			tmpfile1, pn->pn_file->str, tmpfile1);
		XPErrPopUp(cmd);
		return;
	}

	/* check that rename worked (due to NFS problems) */
	if ( stat(pn->pn_file->str, &stat_buf) != 0 )
	{
		/* rename didn't work! Goddam NFS.... */
		sprintf(cmd,"Rename %s to\n %s failed.\n"
			"Might be NFS problems.\nSave of note failed.", 
			tmpfile1, pn->pn_file->str );
		XPErrPopUp(cmd);
		return;
	}

	/* unlink tmp2 */
	if ( stat(tmpfile2, &stat_buf) == 0 )
	{
		if ( unlink(tmpfile2) != 0 )
		{
			sprintf(cmd,"Problems doing cleanup after save.\n"
				"%s could not be unlinked.\n", tmpfile2);
			XPErrPopUp(cmd);
		}
	}

	/* reset save button to be insensitive */
	gtk_widget_set_sensitive ( pn->pn_savewidget, FALSE );
}


/*========================================================================
 *	Name:			XPGetNotePosition
 *	Prototype:	XPGetNotePosition(GtkWidget *window, int *x, int *y)
 *					
 *	Description:
 * Find the position of the widget window, taking into
 *	account any borders stuck on by reparenting window
 *	managers.
 *
 *	Input Arguments:
 *		GtkWidget	*window		window for which coordinates are needed
 *		int			*x				where to save the x coordinate of window
 *		int			*y				where to save the y coordinate of window
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 * The old version of this used to try to calculate the offset based on
 * any borders added by the window manager.  This version uses
 * XTranslateCoordinates() to translate the absolute position of the 
 * notes shell widget from the shell of the window managers frame.  This
 * appears to do a better job of getting note positions correct.
 *
 * At least I think it does.
 *========================================================================*/
void
XPGetNotePosition(
	GtkWidget	*widget,
	int			*x, 
	int			*y
)
{
#ifdef DEBUG
	char			*fname="XPGetNotePosition";
#endif

	Window		*children;
	int			status, nchildren;
	Window		here, parent, root;
	XWindowAttributes win_attributes;
	int			xoffset, yoffset;

	DBGEnter();

	/*
	 * Start with the X Window of the widget passed to us.
	 */
	parent = GDK_WINDOW_XWINDOW(widget->window);

	/*
	 * Walk up the tree looking for a parent of this window whose
	 * parent is the root.  That'll either be this window (if there
	 * is no window manager window) or the window added by the
	 * window manager.
	 */
	do {
		here = parent;
		status = XQueryTree(gdk_display, here, &root, &parent, &children,
						&nchildren);

		if (!status)
			break;

		if (children)
			XFree(children);
	} while (parent != root);

	/*
	 * Now get the window attributes for this window.
	 */
	XGetWindowAttributes(gdk_display, here, &win_attributes);

	/*
	 * Translate the coordinates for the widget passed in relative
	 * to the windows whose attributes we just fetched.  The offset will
	 * be 0,0 if there is no window manager frame, otherwise the offsets
	 * will be the width of the left/right and height of the top/bottom
	 * window frame borders.
	 */
	XTranslateCoordinates(gdk_display, 
		here,
		GDK_WINDOW_XWINDOW(widget->window),
		0,0, &xoffset, &yoffset, &parent);

	/*
	 * Pass back the true absolute coordinates for the widget passed in.
	*x = win_attributes.x - xoffset;
	*y = win_attributes.y - yoffset;
	 */
	*x = xoffset;
	*y = yoffset;
}


/*========================================================================
 *	Name:			XPRaiseAllNotes
 *	Prototype:	void XPRaiseAllNotes()
 *					
 *	Description:
 *	Raise all the notes by raising their shell windows.
 *					
 *	Global Variables:
 *	int	raisestatus		1: notes currently raised (used to toggle status)
 *========================================================================*/
void
XPRaiseAllNotes()
{
	register StickyNote *pn;
	char		found;

	found = False;
	raisestatus = 1;
	for (pn = notes; pn != NULL; pn = pn->pn_next)
	{
		gdk_window_raise(pn->pn_shellwidget->window);
		found = TRUE;
	}

	if ( !found )
		XPErrPopUp(XP_S_NO_NOTES_TO_RAISE);
}


/*========================================================================
 *	Name:			XPHideAllNotes
 *	Prototype:	void XPHideAllNotes()
 *					
 *	Description:
 * Hide all the notes.
 *					
 *	Global Variables:
 *	int	raisestatus		1: notes currently raised (used to toggle status)
 *========================================================================*/
void
XPHideAllNotes()
{
	register StickyNote *pn;
	char		found;

	found = False;
	showstatus = 0;
	for (pn = notes; pn != NULL; pn = pn->pn_next)
	{
		if (pn->pn_hidden == FALSE)
		{
			gtk_widget_hide ( pn->pn_shellwidget );
			pn->pn_hidden = TRUE;
			found = TRUE;
		}
	}

	if ( !found )
		XPErrPopUp(XP_S_NO_NOTES_TO_HIDE);
}


/*========================================================================
 *	Name:			XPUnHideAllNotes
 *	Prototype:	void XPUnHideAllNotes()
 *					
 *	Description:
 * Unhide all hidden notes, if any.
 *					
 *	Global Variables:
 *	int	raisestatus		1: notes currently raised (used to toggle status)
 *========================================================================*/
void
XPUnHideAllNotes()
{
	register StickyNote *pn;
	char		found;

	found = FALSE;
	showstatus = 1;
	for (pn = notes; pn != NULL; pn = pn->pn_next)
		if ( pn->pn_hidden )
		{
			gtk_widget_show(pn->pn_shellwidget);
			if ( ( pn->pn_shellx == 0xffff ) && ( pn->pn_shellx == 0xffff ) )
				XPCenterWindow(pn->pn_shellwidget, NULL, pn->pn_shellwidget);
			pn->pn_hidden = FALSE;
			found = TRUE;
		}

	if ( !found )
		XPErrPopUp(XP_S_NO_NOTES_TO_UNHIDE);
}


/*========================================================================
 *	Name:			XPLowerAllNotes
 *	Prototype:	void XPLowerAllNotes()
 *					
 *	Description:
 * Lower all the notes by lowering their shell windows.
 *					
 *	Global Variables:
 *	int	raisestatus		1: notes currently raised (used to toggle status)
 *========================================================================*/
void
XPLowerAllNotes()
{
	register StickyNote *pn;
	char		found;

	found = FALSE;
	raisestatus = 0;
	for (pn = notes; pn != NULL; pn = pn->pn_next)
	{
		XLowerWindow(gdk_display, 
			GDK_WINDOW_XWINDOW(pn->pn_shellwidget->window));
		found = TRUE;
	}

	if ( !found )
		XPErrPopUp(XP_S_NO_NOTES_TO_LOWER);
}


/*========================================================================
 *	Name:			XPToggleRaise
 *	Prototype:	void XPToggleRaise()
 *					
 *	Description:
 * Toggle raise status
 *					
 *	Global Variables:
 *	int	raisestatus		1: notes currently raised (used to toggle status)
 *========================================================================*/
void
XPToggleRaise() 
{
	if(!showstatus) 
	{
		XPUnHideAllNotes(); 
		XPRaiseAllNotes();
	} 
	else
		raisestatus ? XPLowerAllNotes() : XPRaiseAllNotes();
}


/*========================================================================
 *	Name:			XPToggleShow
 *	Prototype:	void XPToggleShow()
 *					
 *	Description:
 * Toggle show status
 *					
 *	Global Variables:
 *	int	showstatus		1: Hide notes 0: show notes
 *========================================================================*/
void
XPToggleShow() 
{
	showstatus ? XPHideAllNotes() : XPUnHideAllNotes();
}


/*========================================================================
 *	Name:			XPMakeNewNote
 *	Prototype:	void XPMakeNewNote()
 *					
 *	Description:
 * This gets called when the user "tears off" a note from the plaid window.
 *					
 *	Global Variables:
 *========================================================================*/
void
XPMakeNewNote(
)
{
	XPCreateNewNote(StickyNote_1p5x2, NULL);
}


/*========================================================================
 *	Name:			XPSetSaveSensitive
 *	Prototype:	void XPSetSaveSensitive(GtkWidget *widget, 
 *							GdkEventKey *event, gpointer data)
 *					
 *	Description:
 * This gets called when the state of the note changes.  "State" can be
 * changes to text, anchoring, alarms, and so forth.
 *					
 *	Global Variables:
 *========================================================================*/
void
XPSetSaveSensitive(
	GtkWidget	*widget,
	GdkEventKey	*event,
	StickyNote	*pn
)
{
#ifdef DEBUG
	char	*fname="XPSetSaveSensitive()";
#endif

	DBGPrintf(DBG_INFO,("savewidget: 0x%08x\n", pn->pn_savewidget));
	gtk_widget_set_sensitive(pn->pn_savewidget, TRUE);
}


/*========================================================================
 *	Name:			XPCascadeNotes
 *	Prototype:	void XPCascadeNotes()
 *					
 *	Description:
 * Cascade the notes on any anchor notes.
 * This isn't the best, but it distributes the notes among the notes
 * marked as anchors fairly evenly.
 *					
 *	Global Variables:
 *========================================================================*/
void
XPCascadeNotes()
{
#ifdef DEBUG
	char *fname="XPCascadeNotes";
#endif

	register StickyNote	*pn;
	PNLinks 					*linklist, *pnlink, *prevlink;
	int						shellx, shelly;
	int						xpos, ypos;

	DBGEnter();

	/*
	 * get a list of anchor notes
	 */
	linklist = pnlink = prevlink = NULL;
	for (pn = notes; pn != NULL; pn = pn->pn_next)
	{
		if ( pn->pn_anchor )
		{

			pnlink = (PNLinks *)XPSafeAlloc(sizeof (PNLinks));
			pnlink->pn = pn;
			pnlink->next = NULL;

			/*
			 * Get the windows location on the root.
			 */
			XPGetNotePosition(pn->pn_shellwidget, &shellx, &shelly);
			DBGPrintf(DBG_INFO,("shell offsets: %d %d\n", shellx, shelly));
			gdk_window_get_position(pn->pn_shellwidget->window, &xpos, &ypos);
			DBGPrintf(DBG_INFO,("shell positions: %d %d\n", xpos, ypos));
			shellx = xpos - shellx;
			shelly = ypos - shelly;
			DBGPrintf(DBG_INFO,("anchor note position: %d %d\n", shellx, shelly));

			pnlink->x = (int)shellx;
			pnlink->y = (int)shelly;
			if ( linklist == NULL )
				linklist = pnlink;
			if ( prevlink != NULL )
				prevlink->next = pnlink;
			prevlink = pnlink;
		}
	}

	/*
	 * Now run the list of notes and put any non-anchored notes over the
	 * anchored notes.
	 */
	if ( linklist != NULL )
	{
		pnlink = linklist;
		for (pn = notes; pn != NULL; pn = pn->pn_next)
		{
			if ( ( ! pn->pn_anchor ) && ( ! pn->pn_hidden ) )
			{
				/*
				 * reposition the non-anchor down and to
				 * the right of the last note.
				 */
				gtk_widget_set_uposition(
					pn->pn_shellwidget,
					pnlink->x+app_res.anchor_offset,
					pnlink->y+app_res.anchor_offset);
				gdk_window_raise(pn->pn_shellwidget->window);
				DBGPrintf(DBG_INFO,("cascaded note position: %d %d\n", 
					pnlink->x+app_res.anchor_offset, 
					pnlink->y+app_res.anchor_offset));

				/*
				 * reset the anchor position
				 */
				pnlink->x = pnlink->x + app_res.anchor_offset;
				pnlink->y = pnlink->y + app_res.anchor_offset;

				/*
				 * Move to the next anchor.  Rotate through the list.
				 */
				if ( pnlink->next != NULL )
					pnlink = pnlink->next;
				else
					pnlink = linklist;
			}
		}
	}

	/*
	 * Clean up that storage
	 */
	if ( linklist != NULL )
	{
		for ( pnlink = linklist, prevlink = linklist; pnlink != NULL; 
			prevlink = pnlink->next, free(pnlink), pnlink = prevlink );
		linklist = NULL;
	}
	else 
		XPErrPopUp(XP_S_NO_ANCHORED_NOTES);
}



/*========================================================================
 *	Name:			AnchorNote and UnAnchorNote
 *					
 *	Description:
 * Set/Unset the anchor option for a note.
 *					
 *	Global Variables:
 *========================================================================*/
static void
AnchorNote(
	GtkWidget	*widget,
	int			index
)
{
	StickyNote *pn;

	/*
	 * Find the note we're naming.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	pn->pn_anchor = TRUE;
	gtk_widget_set_sensitive ( pn->pn_unanchorwidget, TRUE );
	gtk_widget_set_sensitive ( pn->pn_anchorwidget, FALSE );
	gtk_widget_set_sensitive ( pn->pn_savewidget, TRUE );

	gtk_pixmap_set(GTK_PIXMAP(pn->pn_anchor_pixmapid), 
		anchoron_xbm, NULL);
}

static void
UnAnchorNote(
	GtkWidget		*widget,
	int				index
)
{
#ifdef DEBUG
	char	*fname="UnAnchorNote";
#endif

	StickyNote *pn;

	DBGEnter();

	/*
	 * Find the note we're naming.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	pn->pn_anchor = FALSE;
	gtk_widget_set_sensitive ( pn->pn_unanchorwidget, FALSE );
	gtk_widget_set_sensitive ( pn->pn_anchorwidget, TRUE );
	gtk_widget_set_sensitive ( pn->pn_savewidget, TRUE );

	gtk_pixmap_set(GTK_PIXMAP(pn->pn_anchor_pixmapid), 
		anchoroff_xbm, NULL);
}


/*========================================================================
 *	Name:			HideNote
 *					
 *	Description:
 * Hide a note
 *					
 *	Global Variables:
 *========================================================================*/
static void
HideNote(
	GtkWidget	*widget,
	int			index
)
{
	StickyNote 	*pn;

	/*
	 * Find the note we're hiding.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/* mark it hidden */
	pn->pn_hidden = True;

	/*
	 * pop the window down
	 */
	gtk_widget_hide( pn->pn_shellwidget);
}


/*========================================================================
 *	Name:			XPEraseNote
 *					
 *	Description:
 * Erase a note
 *					
 *	Global Variables:
 *========================================================================*/
void
XPEraseNote(
	GtkWidget	*widget,
	StickyNote 	*pn
)
{
	int	len;

	len = gtk_text_get_length(GTK_TEXT(pn->pn_textwidget));
	gtk_text_set_point(GTK_TEXT(pn->pn_textwidget), 0);
	gtk_text_forward_delete(GTK_TEXT(pn->pn_textwidget), len);
	if ( pn->pn_text != NULL )
		free(pn->pn_text);
	pn->pn_text = NULL;
}

