/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: pilot.c
 *            DESCRIPTION: PalmPilot(r) interface routines
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPPilot
 *      PRIVATE FUNCTIONS: see "Private routine prototypes" below
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_PILOT_C

/* === System Headers === */
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdk.h>

/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"

/* === external routines === */

/* === Public routine prototypes === */

/* === Private routine prototypes === */
static void ClosePilot();
static void ModeCB();
static void	FetchPilot();
static void	FetchAddresses();
static void FetchCB();
static void ProcessMemos();
static void ToggleChange();
static void HotSyncPrompt();
static void PilotParseSubjects();
static void ParseMemosIntoNotes();
static void ParseNotesIntoMemos();
static void CreateIOPrompt();
static void IOCancelCB();
static void IOAcceptCB();
static void CheckDirEntry();
static void RemoveDir();
static void RemoveDirCB();
static void RenameDir();
static void UpdateName();
static void CloseName();
static void DoPilotSync();

/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;
extern GtkWidget		*toplevel;

GtkWidget	*pilot_widget;
GtkWidget	*memos_list, *memos_fetch, *memos_process;
GtkWidget	*addr_fetch;
GtkWidget	*go_button, *remove_button, *rename_button;
GtkWidget	*file_button, *file_text;
GtkWidget	*import_button, *export_button;
GtkWidget	*name_text;
int			mode_index;

static GtkWidget	*io_widget;
static GtkWidget	*namewidget=NULL;

static char		*pilot_strings[]={
	XP_S_PILOT_MENU_BU,
	XP_S_PILOT_MENU_RESTORE,
	XP_S_PILOT_MENU_SYNC,
	XP_S_PILOT_MENU_UPDATE,
	NULL
};


/*========================================================================
 *	Name:			XPPilot
 *	Prototype:	Widget XPPilot()
 *					
 *	Description:
 *	Provide a front end to the set of Pilot tools accessible from XNotesPlus.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPPilot()
{
#ifdef DEBUG
	char	fname[]="XPPilot";
#endif

	GtkWidget		*vbox, *hbox, *eventbox, *scrolled_win;
	GtkWidget		*cancel_button;
	GtkWidget		*left_form, *right_form;
	GtkWidget		*radio_box;
	GtkWidget		*frame, *frame2, *label;
	GtkWidget		*mode_form, *menu;
	GtkWidget		*menuitem, *submenu;
	GtkWidget		*file_pixmap;
	GtkStyle			*style;
	char				buf[256];
	int				index;
	int				width;
	struct stat		stat_buf;
	GShell_T			gshell;

	static GdkPixmap	*file_xbm;
	static GdkBitmap	*file_mask = NULL;
	static GdkPixmap	*pilot_xbm;
	static GdkBitmap	*pilot_mask = NULL;
	static gboolean	firsttime=True;

	DBGEnter();

	/*
	 * Allow only one copy at a time.
	 */
	if ( pilot_widget != NULL )
		return;

	/*
	 * Create a dialog, set its title, and position it at the cursor.
	 */
	pilot_widget = XPCreateGenericShell(PilotDialog, 
							CANCEL_BUTTON, &gshell, FALSE );

	gtk_signal_connect (GTK_OBJECT (pilot_widget), "destroy",
			GTK_SIGNAL_FUNC (ClosePilot), NULL);

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	vbox = gshell.vbox;
	cancel_button = gshell.cancel;


	/*
	 * Left and right side forms - the boxes inside the rest of the 
	 * components of the window will be placed.
	 */
	hbox = gtk_hbox_new(FALSE,0);
	gtk_widget_show(hbox);
	gtk_container_add(GTK_CONTAINER(vbox), hbox);

	left_form = gtk_vbox_new(FALSE,0);
	gtk_widget_show(left_form);
	gtk_container_add(GTK_CONTAINER(hbox), left_form);

	right_form = gtk_vbox_new(FALSE,0);
	gtk_widget_show(right_form);
	gtk_container_add(GTK_CONTAINER(hbox), right_form);


	/*
	 * Label above scrolled window.
	 */
	label = gtk_label_new(XP_S_PILOT_MEMOS);
	gtk_widget_show(label);
	gtk_box_pack_start (GTK_BOX (left_form), label, FALSE, FALSE, 0);
	
	/*
	 * A scrolled window in the left form for the list of xnotes or
	 * the list of Pilot memos available - depending on the situation.
	 */
	eventbox = gtk_event_box_new();
	gtk_box_pack_start (GTK_BOX (left_form), eventbox, TRUE, TRUE, 0);
	gtk_widget_show (eventbox);

	scrolled_win = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win),
		GTK_POLICY_AUTOMATIC,
		GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER (eventbox), scrolled_win);
	gtk_widget_show (scrolled_win);
	
	memos_list = gtk_list_new();
	gtk_list_set_selection_mode (GTK_LIST (memos_list), GTK_SELECTION_MULTIPLE);
	gtk_container_add (GTK_CONTAINER (scrolled_win), memos_list);
	gtk_widget_show (memos_list);

   gtk_container_set_focus_vadjustment (GTK_CONTAINER (memos_list),
		gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrolled_win)));

	/* Add a tool tip */
	gtk_signal_connect(GTK_OBJECT(eventbox), "event",
		GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_PILOT_MEMO);

	/*
	 * Radio buttons to determine if we're selecting xnotes to export
	 * to the Pilot, or importing memos to xnotes.
	 */
	radio_box = gtk_hbox_new(TRUE, 4);
	gtk_box_pack_start (GTK_BOX (left_form), radio_box, FALSE, FALSE, 2);
	gtk_widget_show(radio_box);

	import_button = gtk_radio_button_new_with_label (NULL, XP_S_IMPORT);
	gtk_box_pack_start (GTK_BOX (radio_box), import_button, FALSE, FALSE, 0);
	gtk_widget_show (import_button);

	export_button = gtk_radio_button_new_with_label (
			gtk_radio_button_group (GTK_RADIO_BUTTON (import_button)),
			XP_S_EXPORT);
	gtk_box_pack_start (GTK_BOX (radio_box), export_button, FALSE, FALSE, 0);
	gtk_widget_show (export_button);
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (import_button), TRUE);

	gtk_signal_connect (GTK_OBJECT (import_button), "toggled",
		GTK_SIGNAL_FUNC (ToggleChange), NULL);
	gtk_signal_connect (GTK_OBJECT (export_button), "toggled",
		GTK_SIGNAL_FUNC (ToggleChange), NULL);

	/*
	 * A button to fetch the memos or xnotes into the scrolled list.
	 */
	hbox = gtk_hbox_new(FALSE, 4);
	gtk_box_pack_start (GTK_BOX (left_form), hbox, FALSE, FALSE, 5);
	gtk_widget_show(hbox);

	memos_fetch = gtk_button_new();
	gtk_widget_set_name(memos_fetch, "window_button");
	label = gtk_label_new(XP_S_FETCH);
	gtk_widget_set_name(label, "window_button");
	gtk_widget_show(label);
	gtk_container_add(GTK_CONTAINER(memos_fetch), label);
	gtk_widget_show(memos_fetch);
	gtk_box_pack_start (GTK_BOX (hbox), memos_fetch, 
		FALSE, FALSE, 5);

	/* Add a tool tip */
	gtk_signal_connect(GTK_OBJECT(memos_fetch), "event",
		GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_PILOT_MFET);


	/*
	 * A button to fetch the memos or xnotes into the scrolled list.
	 */
	memos_process = gtk_button_new();
	gtk_widget_set_name(memos_process, "window_button");
	label = gtk_label_new(XP_S_PROCESS);
	gtk_widget_set_name(label, "window_button");
	gtk_widget_show(label);
	gtk_container_add(GTK_CONTAINER(memos_process), label);
	gtk_widget_show(memos_process);
	gtk_box_pack_start (GTK_BOX (hbox), memos_process, 
		FALSE, FALSE, 5);

	/* Add a tool tip */
	gtk_signal_connect(GTK_OBJECT(memos_process), "event",
		GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_PILOT_MPRO);


	/*
	 * A framed button to fetch the Pilot addresses database.
	 */
	frame= gtk_frame_new(XP_S_PILOT_ADDR);
	gtk_container_border_width (GTK_CONTAINER (frame), 5);
	gtk_widget_show(frame);
	gtk_box_pack_start (GTK_BOX (right_form), frame, 
		FALSE, FALSE, 5);
	hbox = gtk_hbox_new(FALSE, 4);
	gtk_container_add(GTK_CONTAINER(frame), hbox);
	gtk_widget_show(hbox);

	addr_fetch = gtk_button_new();
	gtk_widget_set_name(addr_fetch, "window_button");
	label = gtk_label_new(XP_S_FETCH);
	gtk_widget_set_name(label, "window_button");
	gtk_widget_show(label);
	gtk_container_add(GTK_CONTAINER(addr_fetch), label);
	gtk_widget_show(addr_fetch);
	gtk_box_pack_start (GTK_BOX (hbox), addr_fetch, 
		FALSE, FALSE, 5);

	gtk_signal_connect (GTK_OBJECT (addr_fetch), "clicked",
		GTK_SIGNAL_FUNC (FetchAddresses), NULL);

	/* Add a tool tip */
	gtk_signal_connect(GTK_OBJECT(addr_fetch), "event",
		GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_PILOT_ADDR);


	/*
	 * A frame around the Pilot Sync features.
	 */
	frame2= gtk_frame_new(XP_S_PILOT_SYNC);
	gtk_container_border_width (GTK_CONTAINER (frame2), 5);
	gtk_widget_show(frame2);
	gtk_box_pack_start (GTK_BOX (right_form), frame2, 
		FALSE, FALSE, 5);

	/*
	 * A form inside the frame to place the Sync window components.
	 */
	mode_form = gtk_vbox_new(FALSE,0);
	gtk_widget_show(mode_form);
	gtk_container_add(GTK_CONTAINER(frame2), mode_form);

	/*
	 * The options menu (with label and cascade button) is created
	 * here.
	 */
	hbox = gtk_hbox_new(FALSE, 4);
	gtk_container_add(GTK_CONTAINER(mode_form), hbox);
	gtk_container_border_width (GTK_CONTAINER (hbox), 5);
	gtk_widget_show(hbox);

	label = gtk_label_new(XP_S_PILOT_MODE);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
	gtk_widget_show(label);

	submenu = gtk_menu_new();
	index=0;
	while ( pilot_strings[index] != NULL )
	{
		menuitem = gtk_menu_item_new_with_label(pilot_strings[index]);
		gtk_widget_set_name (menuitem, "option_menus");
		gtk_menu_append(GTK_MENU(submenu), menuitem);
		gtk_widget_show(menuitem);
		gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
			GTK_SIGNAL_FUNC(ModeCB), (gpointer) index);
		index++;
	}
	menu = gtk_option_menu_new ();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (menu), submenu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (menu), 0);
	gtk_box_pack_start (GTK_BOX (hbox), menu, FALSE, FALSE, 0);
	gtk_widget_show (menu);
 
	/*
	 * Pushbutton and text field for setting directory.
	 */
	hbox = gtk_hbox_new(FALSE, 4);
	gtk_container_add(GTK_CONTAINER(mode_form), hbox);
	gtk_container_border_width (GTK_CONTAINER (hbox), 5);
	gtk_widget_show(hbox);

	file_button = gtk_button_new();
	gtk_widget_set_name(file_button, "window_button");
	gtk_widget_show(file_button);
	gtk_box_pack_start (GTK_BOX (hbox), file_button, 
		FALSE, FALSE, 5);

	gtk_signal_connect (GTK_OBJECT (file_button), "clicked",
		GTK_SIGNAL_FUNC (CreateIOPrompt), NULL);

	/* Add a tool tip */
	gtk_signal_connect(GTK_OBJECT(file_button), "event",
		GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_DIR);


	/*
	 * First time through, create the file pixmap.
	 */
	if ( firsttime )
	{
		style = gtk_widget_get_style( file_button );
		sprintf(buf, "%s/%s", app_res.xpm_dir, XP_FILE_XPM);
		file_xbm = gdk_pixmap_create_from_xpm(
						toplevel->window,
						&file_mask,
						&style->bg[GTK_STATE_NORMAL],
						buf);

		sprintf(buf, "%s/%s", app_res.xpm_dir, XP_PILOT_XPM);
		pilot_xbm = gdk_pixmap_create_from_xpm(
						toplevel->window,
						&pilot_mask,
						&style->bg[GTK_STATE_NORMAL],
						buf);

		firsttime = False;
	}
	file_pixmap = gtk_pixmap_new(file_xbm, file_mask);
	gtk_container_add(GTK_CONTAINER(file_button), file_pixmap);
	gtk_widget_show( file_pixmap );

	file_text = gtk_entry_new();
	gtk_widget_show(file_text);
	gtk_container_add(GTK_CONTAINER(hbox), file_text);

	gtk_entry_set_text(GTK_ENTRY(file_text), app_res.pilot_dir);

	gtk_signal_connect (GTK_OBJECT (file_text), "key_press_event",
		GTK_SIGNAL_FUNC (CheckDirEntry), NULL);

	/*
	 * If the backup directory does not exist, create it.
	 */
	if ( stat(app_res.pilot_dir, &stat_buf) != 0 )
		mkdir(app_res.pilot_dir, 0755);

	/*
	 * Backup management buttons (starts sync process, rename dir, remove dir)
	 */
	hbox = gtk_hbox_new(FALSE, 4);
	gtk_container_add(GTK_CONTAINER(mode_form), hbox);
	gtk_container_border_width (GTK_CONTAINER (hbox), 5);
	gtk_widget_show(hbox);

	remove_button = gtk_button_new();
	gtk_widget_set_name(remove_button, "window_button");
	label = gtk_label_new(XP_S_PILOT_REMOVE);
	gtk_widget_set_name(label, "window_button");
	gtk_widget_show(label);
	gtk_container_add(GTK_CONTAINER(remove_button), label);
	gtk_widget_show(remove_button);
	gtk_box_pack_start (GTK_BOX (hbox), remove_button, FALSE, FALSE, 5);

	gtk_signal_connect (GTK_OBJECT (remove_button), "clicked",
		GTK_SIGNAL_FUNC (RemoveDir), NULL);

	rename_button = gtk_button_new();
	gtk_widget_set_name(rename_button, "window_button");
	label = gtk_label_new(XP_S_PILOT_RENAME);
	gtk_widget_set_name(label, "window_button");
	gtk_widget_show(label);
	gtk_container_add(GTK_CONTAINER(rename_button), label);
	gtk_widget_show(rename_button);
	gtk_box_pack_start (GTK_BOX (hbox), rename_button, FALSE, FALSE, 5);

	gtk_signal_connect (GTK_OBJECT (rename_button), "clicked",
		GTK_SIGNAL_FUNC (RenameDir), NULL);

	go_button = gtk_button_new();
	gtk_widget_set_name(go_button, "window_button");
	label = gtk_label_new(XP_S_PILOT_START);
	gtk_widget_set_name(label, "window_button");
	gtk_widget_show(label);
	gtk_container_add(GTK_CONTAINER(go_button), label);
	gtk_widget_show(go_button);
	gtk_box_pack_start (GTK_BOX (hbox), go_button, FALSE, FALSE, 5);

	gtk_signal_connect (GTK_OBJECT (go_button), "clicked",
		GTK_SIGNAL_FUNC (DoPilotSync), NULL);

	/*
	 * The Palm Pilot Label
	 */
	label = gtk_pixmap_new(pilot_xbm, pilot_mask);
	gtk_box_pack_start (GTK_BOX (mode_form), label, TRUE, FALSE, 5);
	gtk_widget_show( label );


	/*
	 * Set the callbacks for the various buttons.
	 */
	gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
		GTK_SIGNAL_FUNC (ClosePilot), NULL);

	gtk_signal_connect (GTK_OBJECT (memos_fetch), "clicked",
		GTK_SIGNAL_FUNC (FetchCB), NULL);

	gtk_signal_connect (GTK_OBJECT (memos_process), "clicked",
		GTK_SIGNAL_FUNC (ProcessMemos), NULL);

	/*
	 * Disable items for which the pilot-link tool could not be found.
	 */
	if ( app_res.pilot_xfer == NULL )
		gtk_widget_set_sensitive( frame2, FALSE);
	if ( app_res.pilot_addr == NULL )
		gtk_widget_set_sensitive( frame, FALSE);
	if ( ( app_res.pilot_memos == NULL ) || ( app_res.pilot_imemos == NULL ) )
		gtk_widget_set_sensitive( left_form, FALSE);

	/*
	 * Resize window to make room for the text windows.
	 */
	gtk_widget_realize(pilot_widget);
	GTK_WIDGET_GET_USIZE_WIDTH ( pilot_widget, width);
	gtk_widget_set_usize(pilot_widget, width+80, -1);


	/*
	 * Display the dialog.
	 */
	XPCenterWindow(pilot_widget, NULL, pilot_widget);
	gtk_widget_show(pilot_widget);

}


/*========================================================================
 *	Name:			ClosePilot
 *	Prototype:	static void ClosePilot()
 *					
 *	Description:
 * Close the envelopes dialog.  Clean up any extra storage we might have
 * created here, including temp files.
 *
 *	Notes:
 *========================================================================*/
static void
ClosePilot()
{
	char				buf[256];
	struct stat		stat_buf;

	if ( pilot_widget != NULL )
	{
		gtk_widget_destroy(pilot_widget);
		pilot_widget = NULL;
	}

	sprintf(buf, "/tmp/xnmemos.%d", (int)getpid());
	if ( stat (buf, &stat_buf ) == 0 )
		unlink(buf);
}


/*========================================================================
 *	Name:			ModeCB
 *	Prototype:	static void ModeCB()
 *					
 *	Description:
 * Close the envelopes dialog.
 *
 *	Notes:
 *========================================================================*/
static void
ModeCB(
	GtkWidget	*widget,
	int			index
)
{
	mode_index = index;
}


/*========================================================================
 *	Name:			ToggleChange
 *	Prototype:	static void ToggleChange()
 *					
 *	Description:
 * The import/export toggles have been changed.  Clear the scrolled list.
 *
 *	Notes:
 *========================================================================*/
static void
ToggleChange(
	GtkWidget	*widget,
	int			index
)
{
	gtk_list_clear_items(GTK_LIST(memos_list), 0, -1);
}


/*========================================================================
 *	Name:			FetchCB
 *	Prototype:	static void FetchCB()
 *					
 *	Description:
 * Retrieve either the list of available notes or the set of memos from
 *	the Palm Pilot, depending on which toggle (export or immport,
 *	respectively) is set.
 *
 *	Notes:
 *========================================================================*/
static void
FetchCB(
	GtkWidget	*widget,
	gpointer	client_data
)
{
	StickyNote	*pn;
	GtkWidget	*list_item;

	/*
	 * Clear the scrolled list.
	 */
	gtk_list_clear_items(GTK_LIST(memos_list), 0, -1);


	/*
	 * Determine from where to retrieve list items.
	 */
	if (GTK_TOGGLE_BUTTON (import_button)->active) 
	{
		HotSyncPrompt(XP_PILOT_MEMO);
	}
	else
	{
		/*
		 * Grab the names of the existing notes and add them to
		 * the scrolled list window.
		 */
		for (pn=notes; pn!=NULL; pn=pn->pn_next)
		{
			/*
			 * All notes should have default names - this better not crash!
			 */
			list_item = gtk_list_item_new_with_label (pn->pn_name->str);
			gtk_container_add (GTK_CONTAINER (memos_list), list_item);
			gtk_object_set_user_data (GTK_OBJECT (list_item), (gpointer) pn);
			gtk_widget_show (list_item);
		}
	}
}


/*========================================================================
 *	Name:			FetchPilot
 *	Prototype:	static int FetchPilot(GtkWidget *widget, int type)
 *					
 *	Description:
 * Retrieve data from the Palm Pilot.  This will handle memos, addresses,
 * and backups/restores/syncs/updates.
 *
 * Input Arguments:
 *	int	type			one of XP_PILOT_MEMO, XP_PILOT_INST, XP_PILOT_SYNC,
 *							or XP_PILOT_ADDR
 *
 *	Notes:
 * Called when the user clicks on the ok button of the HotSyncPrompt()
 * dialog.
 *========================================================================*/
static void
FetchPilot(
	GtkWidget	*widget,
	int			type
)
{
	FILE				*fd;
	char				buf[MAXPATHLEN];
	char				cmd[MAXPATHLEN];
	char				pilot_cmd[MAXPATHLEN];
	char				title[MAXPATHLEN];
	char				ok_text[MAXPATHLEN];
	struct stat		stat_buf;
	int				rc;
	gboolean			failed;
	DIR				*dp;
	char				mode[5];
	char				*string=NULL;

#ifdef USE_DIR
	register struct direct *d;
#else
	register struct dirent *d;
#endif


	failed = False;

	/*
	 * Type dependent setup for this routine.
	 */
	switch (type)
	{
		case XP_PILOT_MEMO:
			/*
			 * Clean up any old files first.
			 */
			sprintf(buf, "/tmp/xnmemos.%d", (int)getpid());
			if ( stat (buf, &stat_buf ) == 0 )
				unlink(buf);

			sprintf(title, XP_S_PTITLE_MEMO);
			sprintf(pilot_cmd, app_res.pilot_memos);
			sprintf(ok_text, XP_S_PILOT_MEMO_OK);

			break;

		case XP_PILOT_INST:
			sprintf(title, XP_S_PTITLE_IMEMO);
			sprintf(pilot_cmd, app_res.pilot_imemos);
			sprintf(ok_text, XP_S_PILOT_IMEMO_OK);
			break;

		case XP_PILOT_ADDR:
			sprintf(title, XP_S_PTITLE_ADDR);
			sprintf(pilot_cmd, app_res.pilot_addr);
			sprintf(ok_text, XP_S_PILOT_ADDR_OK);
			break;

		case XP_PILOT_SYNC:
			/*
			 * Check for the directory specified.
			 * Create it if its not there.
			 */
			string = gtk_entry_get_text(GTK_ENTRY(file_text));
		
			if ( strncmp(string, "/", 1) != 0 )
			{
				sprintf(buf, XP_S_PATH_INVALID);
				XPErrPopUp(buf);
				free(string);
				return;
			}
		
			if ( stat(string, &stat_buf) != 0 )
				mkdir(string, 0755);
			else
				if ( ! (stat_buf.st_mode & S_IFDIR) )
				{
					sprintf(buf, "%s exists and is not a directory!", string);
					XPErrPopUp(buf);
					free(string);
					return;
				}

			switch (mode_index)
			{
				case 0: sprintf(mode, "-b"); break;
				case 1: sprintf(mode, "-r"); break;
				case 2: sprintf(mode, "-s"); break;
				case 3: sprintf(mode, "-u"); break;
			}
			sprintf(title, XP_S_PTITLE_SYNC);
			sprintf(pilot_cmd, app_res.pilot_xfer);
			sprintf(ok_text, XP_S_PILOT_SYNC_OK);
			break;
	}

	/*
	 * Build the shell script to run.  We have to run a script because
	 * 1. we're trying to avoid incorporating the PilotLink software 
	 *		directly in this code (makes it easier to upgrade)
	 * 2. allow shell expansion of "*" to identify all files instead
	 *		of having to list them all with fully-qualified paths.
	 */
	sprintf(cmd, "%s/xnmemos.sh", app_res.pilot_dir);
	fd = fopen(cmd, "w");

	fprintf(fd, "#!/bin/sh\n");
	fprintf(fd, title);
	fprintf(fd, "# This script is automatically generated by xnotes+.\n");
	fprintf(fd, "# \n\n");

	switch (type)
	{
		case XP_PILOT_MEMO:
			fprintf(fd, "%s > /tmp/xnmemos.%d 2>/dev/null \n", 
				app_res.pilot_memos, (int)getpid());
			break;

		case XP_PILOT_INST:
			fprintf(fd, "cd %s/xnmemos\n", app_res.pilot_dir);
			fprintf(fd, "%s -q -c xnotes %s * 2> /dev/null \n", 
				app_res.pilot_imemos, (char *)getenv("PILOTPORT"));
			break;

		case XP_PILOT_ADDR:
			fprintf(fd, "%s %s -c all -w %s/%s 2>/dev/null \n", 
				app_res.pilot_addr, (char *)getenv("PILOTPORT"),
				app_res.pilot_dir, DefaultPilotDB);
			break;

		case XP_PILOT_SYNC:
			fprintf(fd, "pilot-xfer %s %s 2> /dev/null 1>/dev/null \n", 
				mode, string);
			free(string);
			break;
	}

	fprintf(fd, "echo $? > %s/xnmemos.rc\n", app_res.pilot_dir);

	fclose(fd);
	chmod(cmd, S_IRWXU);

	/*
	 * Start the upload process.  This will block other xnotes updates,
	 * including window refreshes, but for now its simpler than trying
	 * to coordinate an exec'd script to handle messaging (ie I'm too
	 * lazy to add signal handling, etc right now).
	 */
	rc=system(cmd);
	switch ( rc )
	{
		case 127:
			sprintf(buf,"%s failed: errno=%d", cmd, errno);
			XPErrPopUp(buf);
			failed=True;
			break;
		case -1:
			sprintf(buf,"%s failed: errno=%d", cmd, rc);
			XPErrPopUp(buf);
			failed=True;
			break;
		default:
			sprintf(buf, "%s/xnmemos.rc", app_res.pilot_dir);
			if ( stat (buf, &stat_buf) == 0 )
			{
				fd = fopen(buf, "r");
				fscanf(fd, "%d", &rc);
				fclose(fd);
				if ( rc != 0 )
				{
					sprintf(buf,XP_S_CANT_RUN_CMD, pilot_cmd, rc);
					XPErrPopUp(buf);
					failed=True;
				}
				else
				{
					XPMsgWindow(ok_text, 
						XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK, NULL, pilot_widget,
						RemoveDirCB, NULL, NULL, NULL);
				}
			}
			else
			{
				sprintf(buf,XP_S_CANT_RUN_CMD2, pilot_cmd);
				XPErrPopUp(buf);
				failed=True;
			}
			break;
	}

	switch ( type )
	{
		case XP_PILOT_MEMO:
			break;

		case XP_PILOT_INST:
			/*
			 * Clean up the temporary files.
			 */
			sprintf(buf, "%s/xnmemos", app_res.pilot_dir);
			if ((dp = opendir(buf)) == NULL)
			{
				fprintf(stderr, "xnotes: ");
				perror(buf);
			}
			else
			{
				while ((d = readdir(dp)) != NULL) 
				{
					if (strcmp(d->d_name, ".") == 0)
						continue;
					if (strcmp(d->d_name, "..") == 0)
						continue;
		
					sprintf(buf, "%s/xnmemos/%s", app_res.pilot_dir, d->d_name);
					unlink (buf);
				}
				closedir(dp);
			}
			break;
	}

	/*
	 * Generic cleanup.
	 */
	sprintf(buf, "%s/xnmemos.sh", app_res.pilot_dir);
	unlink (buf);
	sprintf(buf, "%s/xnmemos.rc", app_res.pilot_dir);
	unlink (buf);


	if ( type == XP_PILOT_MEMO )
	{
		/*
		 * Parse out the subject headers and
		 * add them to the scrolled list window.
		 */
		if ( !failed )
			PilotParseSubjects();
	}
}



/*========================================================================
 *	Name:			ProcessMemos
 *	Prototype:	static void ProcessMemos()
 *					
 *	Description:
 *	Run through the list of items (either xnotes or pilot memos) in the
 * scrolled window and process them for import/export, depending on the
 * state of the toggle buttons.
 *
 *	Notes:
 *========================================================================*/
static void
ProcessMemos(
	GtkWidget	*widget,
	gpointer		client_data
)
{
	int				count;
	struct stat		stat_buf;
	char				buf[256];
	GList				*list;


	/*
	 * Get the list of items to process.
	 * If there are no items to process, simply return.
	 */
	list = (GTK_LIST(memos_list))->selection;
	if ( list == NULL )
	{
		XPMsgWindow(
			XP_S_LIST_EMPTY, XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOCANCEL, 
			NULL, NULL, NULL, NULL, NULL, NULL);
		return;
	}
	/*
	 * Count the number of items in the list.
	 */
	count=0;
	while ( list != NULL )
	{
		count++;
		list = list->next;
	}
	list = (GTK_LIST(memos_list))->selection;
	

	/*
	 * Determine from where to retrieve list items.
	 */
	if (GTK_TOGGLE_BUTTON (import_button)->active) 
	{
		/*
		 * These are the subject headers from the memos downloaded
		 * from the Palm Pilot.  Parse their respective memos from the
		 * temporary file and turn them into new notes.
		 */
		ParseMemosIntoNotes(list, count);

		/*
		 * Remove temp file.
		 */
		sprintf(buf, "/tmp/xnmemos.%d", (int)getpid());
		if ( stat (buf, &stat_buf ) == 0 )
			unlink(buf);
	}
	else
	{
		/*
		 * These are notes to be exported to the Palm Pilot. Write them
		 * to temp files and upload them to the Palm Pilot using "install-memo".
		 */
		ParseNotesIntoMemos(list, count);
	}
}



/*========================================================================
 *	Name:			HotSyncPrompt
 *	Prototype:	static void HotSyncPrompt(int type)
 *					
 *	Description:
 * Prompt the user to press the HotSync button on the Pilot's cradle.
 *
 *	Notes:
 *========================================================================*/
static void
HotSyncPrompt(
	int	type
)
{
	XPMsgWindow(
		XP_S_PILOT_HOTSYNC, XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOCANCEL, 
		NULL, NULL, FetchPilot, type, NULL, NULL);
}


/*========================================================================
 *	Name:			PilotParseSubjects
 *	Prototype:	static void PilotParseSubjects()
 *					
 *	Description:
 * Parse the subject headers from the mail-formatted Pilot memos.
 *
 *	Notes:
 *========================================================================*/
static void
PilotParseSubjects(
)
{
#ifdef DEBUG
	char	*fname="PilotParseSubjects()";
#endif

	FILE		*fd;
	char		buf[256];
	char		*token;
	int		i;

	GtkWidget		*list_item;
	struct stat		stat_buf;

	DBGEnter();

	/*
	 * Make sure the temp file is still there.
	 */
	sprintf(buf, "/tmp/xnmemos.%d", (int)getpid());
	if ( stat(buf, &stat_buf) != 0)
	{
		DBGPrintf(DBG_INFO,("Can't find memos temporary file!"));
		XPErrPopUp("Can't find memos temporary file!");
		return;
	}

	/*
	 * Open the temporary file.
	 */
	fd = fopen(buf, "r");

	/*
	 * Read each line, one by one, and test for the Subject: header tag.
	 */
	i=1;
	while (fgets(buf, 256, fd) != NULL)
	{
		if ( !feof(fd) )
		{
			token=strtok(buf, ":");
			if (strcmp(token, "Subject") == 0 )
			{
				/*
				 * Add subjects to scrolled list window.
				 */
				token=strtok(NULL, "\n");
				list_item = gtk_list_item_new_with_label (token);
				gtk_container_add (GTK_CONTAINER (memos_list), list_item);
				gtk_object_set_user_data (GTK_OBJECT (list_item), (gpointer) i);
				gtk_widget_show (list_item);
				i++;
			}
		}
	}

	/*
	 * Close the temporary file.
	 */
	fclose(fd);
}


/*========================================================================
 *	Name:			ParseMemosIntoNotes
 *	Prototype:	static void ParseMemosIntoNotes()
 *					
 *	Description:
 * Parse the memos from the temp file into new notes.
 *
 *	Notes:
 *========================================================================*/
static void
ParseMemosIntoNotes(
	GList		*positions,
	int		list_count
)
{
#ifdef DEBUG
	char	*fname="ParseMemosIntoNotes()";
#endif

	FILE		*fd;
	char		*ptr;
	char		buf[256];
	char		*token, *bufcpy;
	char		*textbuf=NULL;
	char		subject[1024];
	char		*newbuf=NULL;
	int		subject_count, id;
	gboolean	keep;

	GList				*list;
	GtkWidget		*list_item;
	StickyNote		*pn;
	struct stat		stat_buf;

	DBGEnter();

	/*
	 * Make sure the temp file is still there.
	 */
	sprintf(buf, "/tmp/xnmemos.%d", (int)getpid());
	DBGPrintf(DBG_INFO,("memos file: %s\n", buf));
	if ( stat(buf, &stat_buf) != 0)
	{
		DBGPrintf(DBG_INFO,("Can't find memos temporary file!"));
		XPErrPopUp("Can't find memos temporary file!");
		return;
	}

	/*
	 * Open the temporary file.
	 */
	fd = fopen(buf, "r");

	/*
	 * Read each line, one by one.  Look for the "Received:" tag, which will
	 * start a new mail message (skip the "From" since it has 
	 * no colon delimiter and all other colon-delimited tags).
	 */
	keep = False;
	subject_count=0;
	while (fgets(buf, 256, fd) != NULL)
	{
		if ( !feof(fd) )
		{
			/*
			 * Create a copy of the line.
			 */
			bufcpy=(char *)XPSafeAlloc(strlen(buf)+1);
			strcpy(bufcpy, buf);

			/*
			 * Get the token from the line.
			 */
			token=strtok(bufcpy, ":");

			if (strcmp(token, "Received") == 0 )
			{
				/*
				 * We're staring a new memo - reset state info.
				 */
				keep = FALSE;

				/*
				 * Process previous memo, if any.
				 */
				if ( textbuf != NULL )
				{
					/*
					 * Allocate a new note structure.
					 */
					XPCreateNewNote(StickyNote_3x3, &pn);

					/*
					 * Add the text to the window.
					 */
					if ( pn->pn_text != NULL )
						free(pn->pn_text);
					XPInsertText(pn, textbuf, FALSE, TEXT_TYPE);

					/*
					 * Set the notes title.
					 */
					if ( pn->pn_name != NULL )
						g_string_free ( pn->pn_name, TRUE );
					pn->pn_name = g_string_new ( subject );
					gtk_window_set_title (GTK_WINDOW (pn->pn_shellwidget), 
							(gchar *)(pn->pn_name)->str);

					/*
					 * Clean up.
					 */
					free(textbuf);
					textbuf = NULL;
				}
				continue;
			}

			/*
			 * If its one of the other headers, except "Subject:", just
			 * skip the line.
			 */
			if	(	( strcmp(token, "To") == 0 ) ||
					( strcmp(token, "Date") == 0 ) ||
					( strncmp(buf, "From ", 5) == 0 ) )
					
				continue;

			/*
			 * If its the Subject: tag, test it against the list of
			 * subjects we want to turn into notes.  If it matches any,
			 * turn on the "keep" flag.  These are the memos we're going
			 * to turn into notes.
			 */
			if ( strcmp(token, "Subject") == 0 )
			{
				subject_count++;

				ptr=strtok(NULL,"\n");
				bzero(subject, 1024);
				strcpy(subject, ptr);

				list = positions;
				while ( list != NULL )
				{
					list_item = GTK_WIDGET(list->data);
					id = (int) gtk_object_get_user_data(GTK_OBJECT(list_item));
					if ( subject_count == id )
					{
						keep = TRUE;
						list = NULL;
					}
					else
						list = list->next;
				}
			}
			else
			{
				/*
				 * This must be a text line.  If "keep" is true, add it to 
				 * the temp buffer.
				 */
				if (keep)
				{
					if ( textbuf == NULL )
					{
						textbuf = XPSafeAlloc(strlen(buf) + 1);
						strcpy ( textbuf, buf );
					}
					else
					{
						newbuf = XPSafeAlloc(strlen(buf) + strlen(textbuf) + 1);
						strcpy(newbuf, textbuf);
						strcat(newbuf, buf);
						free(textbuf);
						textbuf = newbuf;
					}
				}
			}

			/*
			 * Free up the copy of the buffer.
			 */
			free(bufcpy);
		}
		else
		{
			DBGPrintf(DBG_INFO,("found EOF\n"));

			/*
			 * One last check, in case the last item in the list was selected.
			 */
			if ( textbuf != NULL )
			{
				/*
				 * Allocate a new note structure.
				 */
				XPCreateNewNote(StickyNote_3x3, &pn);

				/*
				 * Add the text to the window.
				 */
				if ( pn->pn_text != NULL )
					free(pn->pn_text);
				XPInsertText(pn, textbuf, FALSE, TEXT_TYPE);

				/*
				 * Set the notes title.
				 */
				if ( pn->pn_name != NULL )
					g_string_free ( pn->pn_name, TRUE );
				pn->pn_name = g_string_new ( subject );
				gtk_window_set_title (GTK_WINDOW (pn->pn_shellwidget), 
						(gchar *)(pn->pn_name)->str);

				/*
				 * Clean up.
				 */
				free(textbuf);
				textbuf = NULL;
			}
		}
	}

	/*
	 * One last check, in case the last item in the list was selected.
	 */
	if ( textbuf != NULL )
	{
		/*
		 * Allocate a new note structure.
		 */
		XPCreateNewNote(StickyNote_3x3, &pn);

		/*
		 * Add the text to the window.
		 */
		if ( pn->pn_text != NULL )
			free(pn->pn_text);
		XPInsertText(pn, textbuf, FALSE, TEXT_TYPE);

		/*
		 * Set the notes title.
		 */
		if ( pn->pn_name != NULL )
			g_string_free ( pn->pn_name, TRUE );
		pn->pn_name = g_string_new ( subject );
		gtk_window_set_title (GTK_WINDOW (pn->pn_shellwidget), 
				(gchar *)(pn->pn_name)->str);

		/*
		 * Clean up.
		 */
		free(textbuf);
		textbuf = NULL;
	}

	/*
	 * Close the memos temporary file.
	 */
	fclose(fd);

	/*
	 * Clear out the list.
	 */
	gtk_list_clear_items(GTK_LIST(memos_list), 0, -1);
}


/*========================================================================
 *	Name:			ParseNotesIntoMemos
 *	Prototype:	static void ParseNotesIntoMemos()
 *					
 *	Description:
 * Parse notes into a format for uploading as memos.  Upload those new
 * memos.
 *
 *	Notes:
 * This routine builds a script to run install-memo and runs it with the
 * system command.  Its a round about way of doing things, but allows
 * me to use install-memo and not embed code for doin the same thing here
 * in xnotes.  This way I don't have to add the overhead of the Pilot
 * headers, libs, etc just to get xnotes to work.
 *
 * Also, once started the install-memo script will block all other 
 * xnotes work until it completes.  Its a tradeoff between this and having 
 * to get fancy with exec'd scripts and signalling.
 *
 * Finally, because of the use of a script with system(), it doesn't matter 
 * if the install-memo doesn't work or not.  Thats a bonus.
 *========================================================================*/
static void
ParseNotesIntoMemos(
	GList		*positions,
	int		list_count
)
{
#ifdef DEBUG
	char	*fname="ParseNotesIntoMemos()";
#endif

	FILE				*fd;
	char				filename[MAXPATHLEN];
	char				buf[MAXPATHLEN];
	char				alarm[10];
	char				hidden[10];
	char				anchor[10];
	char				project[512];
	StickyNote		*pn;
	struct stat		stat_buf;
	GList				*list;
	GtkWidget		*list_item;

	DBGEnter();

	/*
	 * Check for the memos directory in the users xnotes directory.
	 * Create it if its not there.
	 */
	sprintf(buf, "%s/xnmemos", app_res.pilot_dir);
	if ( stat(buf, &stat_buf) != 0 )
		mkdir(buf, 0755);
	else
		if ( ! (stat_buf.st_mode & S_IFDIR) )
		{
			sprintf(buf, "%s exists and is not a directory!", buf);
			XPErrPopUp(buf);
			return;
		}

	/*
	 * For each note selected in the list, write the text and related
	 * information to a temp file, formatted like this:
	 * Note Title
	 * XNProject: <setting>
	 * XNAlarm: <setting>
	 * XNAnchor: <setting>
	 * XNHidden: <yes or no>
	 * <note text>
	 */
	list = positions;
	while ( list != NULL )
	{
		list_item = GTK_WIDGET(list->data);
		pn = (StickyNote *) gtk_object_get_user_data(GTK_OBJECT(list_item));

		/*
		 * Found a matching note - write it to a temp file.
		 */
		sprintf(filename, 
			"%s/xnmemos/%d.%d", app_res.pilot_dir, pn->pn_index, (int)getpid());

		fd = fopen(filename, "w" );

		if ( pn->pn_alarm )
			sprintf(alarm, "True");
		else
			sprintf(alarm, "False");
		if ( pn->pn_anchor )
			sprintf(anchor, "True");
		else
			sprintf(anchor, "False");
		if ( pn->pn_hidden )
			sprintf(hidden, "True");
		else
			sprintf(hidden, "False");
		if ( pn->pn_project == NULL )
			sprintf(project, "None");
		else
			sprintf(project, "%s", pn->pn_project);

		if (pn->pn_text != NULL )
			free(pn->pn_text);
		pn->pn_text = 
			gtk_editable_get_chars(GTK_EDITABLE(pn->pn_textwidget), 0, -1);

		fprintf(fd, 
			"%s\n"
			"XNProject: %s\n"
			"XNAlarm: %s\n"
			"XNAnchor: %s\n"
			"XNHidden: %s\n"
			"%s",
			pn->pn_name->str, project, alarm, anchor, hidden, 
			pn->pn_text);

		fclose(fd);

		list = list->next;
	}

	/*
	 * Install the notes onto the Pilot.
	 */
	HotSyncPrompt(XP_PILOT_INST);

	/*
	 * Clear out the list.
	 */
	gtk_list_clear_items(GTK_LIST(memos_list), 0, -1);
}


/*========================================================================
 *	Name:			CreateIOPrompt
 *	Prototype:	CreateIOPrompt()
 *					
 *	Description:
 * Create a file selection widget
 *
 *	Notes:
 *========================================================================*/
static void
CreateIOPrompt(
	GtkWidget 	*widget,
	gpointer		client_data
)
{
	/*
	 * Only one copy at a time allowed.
	 */
	if ( io_widget != NULL )
		return;

	/*
	 * Create File Selection Box.
	 */
	io_widget = gtk_file_selection_new (NotesPilotBUPrompt);
	gtk_file_selection_set_filename 
		(GTK_FILE_SELECTION(io_widget), app_res.pilot_dir);

	/* The Cancel/Accept buttons */
	gtk_signal_connect_object (
		GTK_OBJECT (GTK_FILE_SELECTION (io_widget)->cancel_button), "clicked", 
		(GtkSignalFunc) IOCancelCB, NULL);
	gtk_signal_connect_object (
		GTK_OBJECT (GTK_FILE_SELECTION (io_widget)->ok_button), "clicked", 
		(GtkSignalFunc) IOAcceptCB, NULL);

	/*
	 * pop up the window.
	 */
	gtk_widget_realize(io_widget);
	XPCenterWindow(io_widget, NULL, io_widget);
	gtk_widget_show(io_widget);
}


/*========================================================================
 *	Name:			IOCancelCB
 *	Prototype:	IOCancelCB()
 *					
 *	Description:
 * Close the file selection dialog.
 *
 *	Notes:
 *========================================================================*/
static void
IOCancelCB()
{
	if (io_widget != NULL )
	{
		gtk_widget_destroy(io_widget);
		io_widget=NULL;
	}
}


/*========================================================================
 *	Name:			IOAcceptCB
 *	Prototype:	IOAcceptCB()
 *					
 *	Description:
 * Handle File Selection Accept case - save directory selected in text
 * field, update buttons sensitivity, close file selection dialog.
 *
 *	Notes:
 *========================================================================*/
static void
IOAcceptCB()
{
	char				*string;
	struct stat		stat_buf;

	/*
	 * Retrieve the directory name from the File Selection dialog
	 * and update the directory text field.
	 */
	string = gtk_file_selection_get_filename (GTK_FILE_SELECTION(io_widget));
	gtk_entry_set_text(GTK_ENTRY(file_text), string);

	/*
	 * If the directory exists, set the remove/rename buttons to 
	 * accept button presses.
	 */
	if ( stat(string, &stat_buf) == 0 )
	{
		gtk_widget_set_sensitive(remove_button, TRUE);
		gtk_widget_set_sensitive(rename_button, TRUE);
	}
	else
	{
		gtk_widget_set_sensitive(remove_button, FALSE);
		gtk_widget_set_sensitive(rename_button, FALSE);
	}

	/*
	 * Close the File Selection window.
	 */
	IOCancelCB();
}


/*========================================================================
 *	Name:			CheckDirEntry
 *	Prototype:	CheckDirEntry()
 *					
 *	Description:
 * Test to see if the directory specified exists.  Do appropriate things
 * if it does.
 *
 *	Notes:
 *========================================================================*/
static void
CheckDirEntry()
{
	struct stat		stat_buf;
	char				*string;

	string = gtk_entry_get_text(GTK_ENTRY(file_text));

	if ( stat(string, &stat_buf) == 0 )
	{
		gtk_widget_set_sensitive(remove_button, TRUE);
		gtk_widget_set_sensitive(rename_button, TRUE);
	}
	else
	{
		gtk_widget_set_sensitive(remove_button, FALSE);
		gtk_widget_set_sensitive(rename_button, FALSE);
	}
}


/*========================================================================
 *	Name:			RenameDir
 *	Prototype:	RenameDir()
 *					
 *	Description:
 * Prompt user for the new name of the directory.
 *
 *	Notes:
 * We can't get here unless the directory already exists.
 *========================================================================*/
static void
RenameDir()
{
#ifdef DEBUG
	char	fname[]="RenameDir";
#endif

	GtkWidget		*vbox;
	GtkWidget		*confirm_button, *cancel_button;
	char				*string;
	GShell_T			gshell;


	DBGEnter();

	/*
	 * Allow only one copy at a time.
	 */
	if ( namewidget != NULL )
		return;

	/*
	 * Create a dialog, set its title, and position it at the cursor.
	 */
	namewidget = XPCreateGenericShell(RenameDirDialog, 
					ACCEPT_BUTTON | CANCEL_BUTTON, &gshell, FALSE );

	gtk_signal_connect (GTK_OBJECT (namewidget), "destroy",
			GTK_SIGNAL_FUNC (CloseName), NULL);

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	vbox = gshell.vbox;
	cancel_button = gshell.cancel;
	confirm_button = gshell.accept;
 
	/*
	 * An text entry widget in which the name will go.
	 */
	name_text = gtk_entry_new();
	gtk_widget_show(name_text);
	gtk_container_add(GTK_CONTAINER(vbox), name_text);

	string = gtk_entry_get_text(GTK_ENTRY(file_text));
	gtk_entry_set_text(GTK_ENTRY(name_text), string);

	/*
	 * Confirmation button.
	 */
	gtk_signal_connect (GTK_OBJECT (confirm_button), "clicked",
		GTK_SIGNAL_FUNC (UpdateName), NULL);

	/*
	 * Cancellation button.
	 */
	gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
		GTK_SIGNAL_FUNC (CloseName), NULL);

	/*
	 * Stick the dialog in the middle of the notes shell widget.
	 */
	gtk_widget_realize( namewidget );
	XPCenterWindow(namewidget, pilot_widget, namewidget);
	gtk_widget_show(namewidget);

}


/*========================================================================
 *	Name:			UpdateName
 *	Prototype:	UpdateName()
 *					
 *	Description:
 * Change the name of the existing directory.
 *
 *	Notes:
 *========================================================================*/
static void
UpdateName()
{
	char		*old_dir, *new_dir;
	char		buf[256];

	new_dir = gtk_entry_get_text(GTK_ENTRY(name_text));
	old_dir = gtk_entry_get_text(GTK_ENTRY(file_text));

	if ( rename(old_dir, new_dir) != 0 )
	{
		sprintf(buf, XP_S_RENAME_FAIL, old_dir, new_dir, errno);
		XPErrPopUp(buf);
	}
	else
	{
		gtk_entry_set_text(GTK_ENTRY(file_text), new_dir);
	}

	CloseName();
}


/*========================================================================
 *	Name:			CloseName
 *	Prototype:	CloseName()
 *					
 *	Description:
 * Close the directory rename dialog.
 *
 *	Notes:
 *========================================================================*/
static void
CloseName()
{
	if (namewidget != NULL )
	{
		gtk_widget_destroy(namewidget);
		namewidget=NULL;
	}
}


/*========================================================================
 *	Name:			RemoveDir
 *	Prototype:	RemoveDir()
 *					
 *	Description:
 * Prompt user to verify the removal of an existing directory and its files.
 *
 *	Notes:
 * We can't get here unless the directory already exists.
 *========================================================================*/
static void
RemoveDir()
{
	char				*string;
	char				buf[512];

	string = gtk_entry_get_text(GTK_ENTRY(file_text));

	sprintf(buf, XP_S_DELETE_PROMPT, string);
	XPMsgWindow(buf, XP_MSG_TYPE | XP_MSG_NOHELP, NULL, pilot_widget,
		RemoveDirCB, NULL, NULL, NULL);
}


/*========================================================================
 *	Name:			RemoveDirCB
 *	Prototype:	RemoveDirCB()
 *					
 *	Description:
 * Remove an existing directory and its files.
 *
 *	Notes:
 * We can't get here unless the directory already exists.
 *========================================================================*/
static void
RemoveDirCB()
{
	char				*string;
	char				buf[MAXPATHLEN];
	DIR				*dp;

#ifdef USE_DIR
	register struct direct *d;
#else
	register struct dirent *d;
#endif

	string = gtk_entry_get_text(GTK_ENTRY(file_text));

	if ((dp = opendir(string)) == NULL)
	{
		fprintf(stderr, "xnotes: ");
		perror(string);
		return;
	}

	while ((d = readdir(dp)) != NULL) 
	{
		if (strcmp(d->d_name, ".") == 0)
			continue;
		if (strcmp(d->d_name, "..") == 0)
			continue;

		sprintf(buf, "%s/%s", string, d->d_name);
		unlink (buf);
	}
	closedir(dp);
	rmdir(string);

	/*
	 * Update the text field.
	 */
	gtk_entry_set_text(GTK_ENTRY(file_text), app_res.pilot_dir);
}


/*========================================================================
 *	Name:			DoPilotSync
 *	Prototype:	DoPilotSync()
 *					
 *	Description:
 * Start the file sync process with the Pilot.
 *
 *	Notes:
 * We *can* get here with a directory specified that doesn't exist!
 *
 * Directories for backups must be fully qualified.  Makes life easier
 * for me.
 *
 * This is just a front end to FetchPilot() for doing backups, etc.
 *========================================================================*/
static void
DoPilotSync()
{
	HotSyncPrompt(XP_PILOT_SYNC);
}


/*========================================================================
 *	Name:			FetchAddresses
 *	Prototype:	static void FetchAddresses()
 *					
 *	Description:
 *
 *	Notes:
 *	Front end to FetchPilot() for fetching addresses from the Pilot.
 *========================================================================*/
static void
FetchAddresses()
{
	HotSyncPrompt(XP_PILOT_ADDR);
}
