/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: plaid.c
 *            DESCRIPTION: Create top level windows, plaid button
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPCreatePlaidWidget
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_PLAID_C

/* === System Headers === */
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <gtk/gtk.h>

/* === Project Headers === */
#include "XNotesPlus.h"

/* === external routines === */

/* === Public routine prototypes === */

/* === Private routine prototypes === */

/* === Global Variables === */
GtkWidget	*toplevel;


/*========================================================================
 *	Name:			XPCreatePlaidWidget
 *	Prototype:	GtkWidget *XPCreatePlaidWidget()
 *					
 *	Description:
 *	Create the plaid widget, the top level button from which the user
 *	initiates all note actions.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *		GtkWidget		pointer to the visible widget (a button)
 *
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *		
 *========================================================================*/
GtkWidget *
XPCreatePlaidWidget()
{
	GtkWidget	*window;
	GtkWidget	*box;
	GtkWidget	*button;
	GtkStyle		*style;
	GdkPixmap	*plaid_xpm;
	GdkBitmap	*mask;
	GtkWidget	*pixmapid;

	char			buf[256];

	/*
	 * Create the top level window and set it up to properly exit on
	 * a delete request from the OS or Window Manager.  Then set the
	 * title for the window (which for most people will probably never
	 * get used) and set the internal border width (for any widgets we
	 * pack inside) to 0 (so they butt up against the edges of our main
	 * window).
	 */
	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_show(window);
	toplevel = window;

	/* This handles window manager Delete/Close events */
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
			GTK_SIGNAL_FUNC (XPShutdown), NULL);
	gtk_signal_connect (GTK_OBJECT (window), "destroy",
			GTK_SIGNAL_FUNC (XPShutdown), NULL);

	gtk_window_set_title (GTK_WINDOW (window), XP_S_PROG_NAME);
	gtk_container_border_width (GTK_CONTAINER (window), 0);
	gtk_window_set_policy (GTK_WINDOW (window), TRUE, FALSE, FALSE);
	

	/*
	 * Create a box to pack the plaid window into.
	 * Put the box into the main window.
	 */
	box = gtk_hbox_new(FALSE, 0);
	gtk_container_add (GTK_CONTAINER (window), box);
	
	/*
	 * Creates a button and stuff it inside of the box.
	 */
	button = gtk_button_new();
	gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
	
	/*
	 * Add plaid image to button.
	 */
	style = gtk_widget_get_style( window );
	sprintf(buf, "%s/%s", app_res.xpm_dir, XP_PLAID_XPM);
	plaid_xpm = gdk_pixmap_create_from_xpm(
						window->window,
						&mask,
						&style->bg[GTK_STATE_NORMAL],
						buf);
	pixmapid = gtk_pixmap_new(plaid_xpm, mask);
	gtk_widget_show( pixmapid);
	gtk_container_add( GTK_CONTAINER(button), pixmapid );

	/*
	 * Show the buttons, boxes and windows.
	 */
	gtk_widget_show(button);
	gtk_widget_show(box);

	return(button);

}
