/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995.
 *========================================================================
 *              FILE NAME: projects.c 
 *            DESCRIPTION: handle note project list.
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPProject XPUpdateColorBox XPNoteProject
 *      PRIVATE FUNCTIONS: UpdateProject ClearProject UpdateRGB ResetWindow
 *								 : ColorSelect ClearColorSelect PNoteUpdate
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_PROJECT_C

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void		UpdateProject();
static void		ClearProject();
static void		UpdateRGB();
static void		ResetWindow();
static void		ColorSelect();
static void		ClearColorSelect();
static void		PNoteUpdate();

/* === Global Variables === */
ProjectType			*projects_list=NULL;


/* === Static Variables === */
/*
static GtkWidget		*pnote_widget;
static GtkWidget		*pnote_color;
static GtkWidget		*pnote_list;
*/

static GtkWidget		*project_widget;
static GtkWidget		*project_eventbox;
static GtkWidget		*project_color;
static GtkWidget		*project_list;
static GtkWidget		*project_text;
static GtkWidget		*scrolled_win;

static gdouble			prj_color[4];
static int				project_type;

static GtkWidget 		*color_dialog;


/*========================================================================
 *	Name:			XPProject
 *	Prototype:	XPProject(int type, StickyNote *pn)
 *					
 *	Description:
 * Dialog window for managing projects and their colors.
 *
 *	Input Arguments:
 *	int			type	XP_PROJ_CONFIG or XP_PROJ_USER
 *	StickyNote	*pn	pointer to note structure (for XP_PROJ_USER only)
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPProject(
	int			type,
	StickyNote	*pn
)
{
#ifdef DEBUG
	char	fname[]="XPProject";
#endif

	GtkWidget		*vbox, *hbox, *vbox2, *eventbox;
	GtkWidget		*confirm_button, *cancel_button;
	GtkWidget		*label;
	GtkWidget		*list_item;
	ProjectType		*item;
	GShell_T			gshell;
	GdkColor			color;

	DBGEnter();

	/*
	 * Allow only one copy at a time.
	 */
	if ( project_widget != NULL )
		return;

	/*
	 * Save the type - other routines in this module will need it.
	 */
	project_type = type;

	/*
	 * Create a dialog, set its title, and position it at the cursor.
	 */
	project_widget = XPCreateGenericShell(ProjectDialog, 
					ACCEPT_BUTTON | CLOSE_BUTTON, &gshell, FALSE );

	gtk_signal_connect (GTK_OBJECT (project_widget), "destroy",
			GTK_SIGNAL_FUNC (ClearProject), NULL);

	vbox = gshell.vbox;
	cancel_button = gshell.cancel;
	confirm_button = gshell.accept;

	/*
	 * A horizontal box - we'll have two sides to this dialog.
	 */
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(vbox), hbox);
	gtk_widget_show(hbox);

	/*
	 * A vertical alignment within the hbox.
	 */
	vbox2 = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(hbox), vbox2);
	gtk_container_border_width (GTK_CONTAINER (vbox2), 3);
	gtk_widget_show(vbox2);

	/*
 	 * A label over the list of project names.
	 */
	label = gtk_label_new(XP_S_PROJ_CHOICES);
	gtk_widget_show(label);
	gtk_container_add(GTK_CONTAINER(vbox2), label);

	/*
	 * An scrolled list in which the current set of project names will go.
	 */
	eventbox = gtk_event_box_new();
	gtk_container_add(GTK_CONTAINER(vbox2), eventbox);
	gtk_widget_show (eventbox);

	scrolled_win = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win),
		GTK_POLICY_AUTOMATIC,
		GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER (eventbox), scrolled_win);
	gtk_widget_show (scrolled_win);
	
	project_list = gtk_list_new();
	gtk_list_set_selection_mode (GTK_LIST (project_list), GTK_SELECTION_SINGLE);
	gtk_container_add (GTK_CONTAINER (scrolled_win), project_list);
	gtk_widget_show (project_list);
	gtk_widget_set_usize (scrolled_win, XP_PROJ_WIDTH, XP_PROJ_HEIGHT);

   gtk_container_set_focus_vadjustment (GTK_CONTAINER (project_list),
		gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrolled_win)));

	/*
	 * Add the projects to the scrolled window.
	 */
	item = projects_list;
	while ( item != NULL )
	{
		DBGPrintf(DBG_INFO,("item name: %s\n", item->name));
		list_item = gtk_list_item_new_with_label (item->name);
		gtk_container_add (GTK_CONTAINER (project_list), list_item);
		gtk_widget_show (list_item);
		gtk_object_set_user_data (GTK_OBJECT (list_item), (gpointer) item);

		item = item->next;
	}

	/*
	 * Set the first entry in the project list to selected.
	 */
	gtk_list_select_item(GTK_LIST(project_list), 0);

	/*
	 * A text field for the project name.
	 */
	if ( type == XP_PROJ_CONFIG )
	{
		eventbox = gtk_event_box_new();
		gtk_container_add(GTK_CONTAINER(vbox2), eventbox);
		gtk_widget_show (eventbox);

		project_text = gtk_entry_new();
		gtk_container_add (GTK_CONTAINER (eventbox), project_text);
		gtk_widget_show (project_text);

		/* Add a tool tip */
		gtk_signal_connect(GTK_OBJECT(eventbox), "event",
			GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_PRJ_TEXT);
	}

	/*
	 * A colored bar showing the current selected color.
	 */
	vbox2 = gtk_vbox_new(TRUE, 0);
	gtk_container_add(GTK_CONTAINER(hbox), vbox2);
	gtk_container_border_width (GTK_CONTAINER (vbox2), 3);
	gtk_widget_show(vbox2);

	project_eventbox = gtk_event_box_new();
	gtk_container_add(GTK_CONTAINER(vbox2), project_eventbox);
	gtk_widget_show (project_eventbox);

	if ( type != XP_PROJ_USER )
		gtk_signal_connect(GTK_OBJECT(project_eventbox), "event",
			GTK_SIGNAL_FUNC (ColorSelect), (gpointer)project_eventbox);

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(project_eventbox), hbox);
	gtk_container_border_width (GTK_CONTAINER (hbox), 5);
	gtk_widget_show (hbox);

	/*
	 * If a projects list exists, grab the first entries name and colors.
	 */
	if ( projects_list != NULL )
	{
		if ( type == XP_PROJ_CONFIG )
			gtk_entry_set_text(GTK_ENTRY(project_text), projects_list->name);

		prj_color[0] = projects_list->color[0];
		prj_color[1] = projects_list->color[1];
		prj_color[2] = projects_list->color[2];

		color.red = (guint16)(prj_color[0]*65535.0);
		color.green = (guint16)(prj_color[1]*65535.0);
		color.blue = (guint16)(prj_color[2]*65535.0);

		gtk_widget_realize(project_eventbox);
		XPUpdateColorBox(project_widget, project_eventbox, &color);
	}

	/*
	 * The colored label to show what the text will look like in the note.
	 * The extra (blank) labels are just spacers.
	 */

	project_color = gtk_label_new(XP_S_SAMPLE_NOTE);
	gtk_container_add(GTK_CONTAINER(hbox), project_color);
	gtk_widget_show (project_color);

	/*
	 * Confirmation button.
	 */
	if ( type == XP_PROJ_CONFIG )
		gtk_signal_connect(GTK_OBJECT(confirm_button), "clicked",
			GTK_SIGNAL_FUNC (UpdateProject), (gpointer)NULL);
	else
		gtk_signal_connect(GTK_OBJECT(confirm_button), "clicked",
			GTK_SIGNAL_FUNC (PNoteUpdate), (gpointer)pn);

	/*
	 * Cancellation button.
	 */
	gtk_signal_connect(GTK_OBJECT(cancel_button), "clicked",
		GTK_SIGNAL_FUNC (ClearProject), (gpointer)NULL);

	/*
	 * List selection changes get handled here.  We have to add it here
	 * because if we add the callback when the list widget is created
	 * then when we update it we crash because other widgets had not been
	 * created yet.  Details, details.
	 */
	gtk_signal_connect (GTK_OBJECT (project_list), "selection_changed",
		GTK_SIGNAL_FUNC (ResetWindow), NULL);


	if ( type == XP_PROJ_CONFIG )
	{
		/* Add a tool tip */
		gtk_signal_connect(GTK_OBJECT(confirm_button), "event",
			GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_PRJ_SAVE);

		/* Add a tool tip */
		gtk_signal_connect(GTK_OBJECT(cancel_button), "event",
			GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_PRJ_CANCEL);
	}

	/*
	 * Stick the dialog in the middle of the display.
	 */
	gtk_widget_realize( project_widget );
	if ( type == XP_PROJ_CONFIG )
		XPCenterWindow(project_widget, NULL, project_widget);
	else
		XPCenterWindow(project_widget, pn->pn_shellwidget, project_widget);
	gtk_widget_show(project_widget);

}


/*========================================================================
 *	Name:			UpdateProject
 *	Prototype:	UpdateProject(GtkWidget *widget, gpointer client_data)
 *					
 *	Description:
 * Updates the display and/or lists when the Accept key is hit.
 *
 *	Notes:
 * Grab the name from the text field.  If its an existing project,
 * then update its color to match the current color for the color label.
 * If its a new project, add it to the link list of projects and the 
 * project list.  Don't exit.
 *========================================================================*/
static void
UpdateProject(
	GtkWidget	*widget,
	gpointer		client_data
)
{
	ProjectType		*item;
	gboolean			found;
	char				*name;
	GtkWidget		*list_item;
	GtkAdjustment	*vadj;

	/*
	 * Grab the name from the text field.
	 */
	name = gtk_entry_get_text(GTK_ENTRY(project_text));

	/*
	 * Find this project in the link list.
	 */
	item = projects_list;
	found = False;
	if ( item != NULL )
	{
		while ( (!found) && (item != NULL) )
		{
			if ( strcmp(item->name, name) == 0 )
				found = True;
			else
				item = item->next;
		}
	}


	if ( found )
	{
		/*
		 * Project already exists - update its color.
		 */
		item->color[0] = prj_color[0];
		item->color[1] = prj_color[1];
		item->color[2] = prj_color[2];
		XPMsgWindow(XP_S_PROJ_UPD, XP_INFO_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK,
			NULL, NULL, NULL, NULL, NULL, NULL);

		/*
		 * Update projects file.
		 */
		XPWriteProjects();

	}
	else
	{
		/*
		 * New project - add to end of list and select it.
		 */
		item = projects_list;
		if ( item != NULL )
		{
			while (item->next != NULL)
				item = item->next;
			item->next = (ProjectType *)XPSafeAlloc(sizeof(ProjectType));
			item = item->next;
		}
		else
		{
			item = (ProjectType *)XPSafeAlloc(sizeof(ProjectType));
			projects_list = item;
		}
		item->next = NULL;
		item->color[0] = prj_color[0];
		item->color[1] = prj_color[1];
		item->color[2] = prj_color[2];
		item->name = XPSafeAlloc(strlen(name)+1);
		strcpy(item->name, name);

		list_item = gtk_list_item_new_with_label (item->name);
		gtk_container_add (GTK_CONTAINER (project_list), list_item);
		gtk_widget_show (list_item);
		gtk_object_set_user_data (GTK_OBJECT (list_item), (gpointer) item);

		/*
		 * Select the new item and scroll down to it.
		 */
		gtk_list_select_item(GTK_LIST(project_list), 
			gtk_list_child_position(GTK_LIST(project_list), list_item));

		vadj = gtk_scrolled_window_get_vadjustment(
					GTK_SCROLLED_WINDOW (scrolled_win));

		gtk_adjustment_set_value(vadj, vadj->upper);

		/*
		 * Update projects file.
		 */
		XPWriteProjects();

	}

}


/*========================================================================
 *	Name:			ClearProject
 *	Prototype:	ClearProject(GtkWidget *widget, gpointer client_data)
 *					
 *	Description:
 * Closes the Projects window.
 *
 *	Notes:
 *========================================================================*/
static void
ClearProject(
	GtkWidget	*widget,
	gpointer		client_data
)
{
	if ( project_widget != NULL )
	{
		gtk_widget_destroy(project_widget);
		project_widget = NULL;
	}
}


/*========================================================================
 *	Name:			UpdateRGB
 *	Prototype:	UpdateRGB(GtkWidget *widget, gpointer client_data)
 *					
 *	Description:
 * Retrieve the color selected from the color selection dialog and
 * update the color box.
 *
 *	Notes:
 *========================================================================*/
static void
UpdateRGB(
	GtkWidget					*widget,
	GtkColorSelectionDialog	*cs
)
{
#ifdef DEBUG
	char		*fname="UpdateRGB()";
#endif

	GtkColorSelection	*colorsel;
	gdouble	color[4];
	GdkColor	gdk_color;

	DBGEnter();

	/*
	 * Retrieve colors of selected color from Color Selection dialog.
	 */
	colorsel = GTK_COLOR_SELECTION(cs->colorsel);
	gtk_color_selection_get_color (colorsel,color);

	/*
	 * Save colors from selection box so we can save to the projects file.
	 */
	prj_color[0]= color[0];
	prj_color[1]= color[1];
	prj_color[2]= color[2];

	/*
	 * Stuff colors into GdkColor struct and pass to routine for updating
	 * the color label.
	 */
	gdk_color.red = (guint16)(color[0]*65535.0);
	gdk_color.green = (guint16)(color[1]*65535.0);
	gdk_color.blue = (guint16)(color[2]*65535.0);
	XPUpdateColorBox(project_widget, project_eventbox, &gdk_color);
}



/*========================================================================
 *	Name:			XPUpdateColorBox
 *	Prototype:	void XPUpdateColorBox(Widget parent, Widget child, 
 *										GdkColor *color)
 *					
 *	Input Arguments:
 *	Widget	parent		parent of child, just make it the shell widget
 *	Widget	child			widget whose background color is to be updated.
 *	char		*string		color name.
 *
 *	Description:
 * Updates the colored box based on color structure passed in.
 *
 *	Notes:
 *========================================================================*/
void
XPUpdateColorBox(
	GtkWidget	*parent,
	GtkWidget	*child,
	GdkColor		*color
)
{
#ifdef DEBUG
	char	*fname="XPUpdateColorBox()";
#endif

	GdkColormap		*colormap;

	DBGEnter();

	if (GTK_WIDGET_REALIZED (child) )
	{
		DBGPrintf(DBG_TRACE3,("updating color box\n"));
		colormap = gdk_window_get_colormap (child->window);
		gdk_color_alloc (colormap, color);
		gdk_window_set_background (child->window, color);
		gdk_window_clear(child->window);
		gtk_widget_queue_draw(child);
	}
 
}


/*========================================================================
 *	Name:			ResetWindow
 *	Prototype:	ResetWindow(GtkWidget *widget, gpointer client_data)
 *					
 *	Description:
 * Update the color box and the text field when an
 * existing project is selected from the projects scrolled list.
 *
 *	Notes:
 *========================================================================*/
static void
ResetWindow(
	GtkWidget	*widget,
	gpointer		client_data
)
{
#ifdef DEBUG
	char		*fname="ResetWindow()";
#endif

	ProjectType		*item;
	GList				*list;
	GtkWidget		*list_item;
	GdkColor			gdk_color;

	DBGEnter();

	/*
	 * Retrieve the selected project from the link list of projects.
	 */
	list = (GTK_LIST(project_list))->selection;
	if ( list == NULL )
		return;

	list_item = GTK_WIDGET(list->data);
	item = (ProjectType *) gtk_object_get_user_data(GTK_OBJECT(list_item));

	/*
	 * Save the current projects colors.
	 */
	prj_color[0]= item->color[0];
	prj_color[1]= item->color[1];
	prj_color[2]= item->color[2];

	/*
	 * Update the color box accordingly.
	 */
	gdk_color.red = (guint16)(item->color[0]*65535.0);
	gdk_color.green = (guint16)(item->color[1]*65535.0);
	gdk_color.blue = (guint16)(item->color[2]*65535.0);
	XPUpdateColorBox(project_widget, project_eventbox, &gdk_color);

	/*
	 * Update the text entry field with the new name.
	 */
	if ( project_type == XP_PROJ_CONFIG )
		gtk_entry_set_text(GTK_ENTRY(project_text), item->name);
}


/*========================================================================
 *	Name:			ColorSelect
 *	Prototype:	ColorSelect(GtkWidget *widget, gpointer client_data)
 *					
 *	Description:
 * Open the color selection dialog.
 *
 *	Notes:
 *========================================================================*/
static void
ColorSelect(
	GtkWidget	*widget,
	GdkEvent		*event,
	GtkWidget	*child
)
{
#ifdef DEBUG
	char		*fname="ColorSelect()";
#endif

	GtkWidget *colorsel;

	DBGEnter();

	if ( color_dialog != NULL )
		return;

	if (event->type == GDK_BUTTON_PRESS )
	{
		color_dialog = gtk_color_selection_dialog_new("Select background color");
		colorsel = GTK_COLOR_SELECTION_DIALOG(color_dialog)->colorsel;

		gtk_signal_connect(GTK_OBJECT(colorsel), "color_changed",
			(GtkSignalFunc)UpdateRGB, (gpointer)color_dialog);

		gtk_signal_connect (
				GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(color_dialog)->ok_button),
				"clicked",
				GTK_SIGNAL_FUNC(UpdateRGB),
				color_dialog);

		gtk_signal_connect_object (
				GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(color_dialog)->cancel_button),
				"clicked",
				GTK_SIGNAL_FUNC(ClearColorSelect),
				GTK_OBJECT (color_dialog));

		gtk_widget_show(color_dialog);
	}
	else
	{
		/*
		 * Other events may require window refreshes, so lets for it just in
		 * case.
		 */
		gdk_window_clear(child->window);
		gtk_widget_queue_draw(child);
	}
}


/*========================================================================
 *	Name:			ClearColorSelect
 *	Prototype:	ClearColorSelect(GtkWidget *widget, gpointer client_data)
 *					
 *	Description:
 * Close the color selection dialog.
 *
 *	Notes:
 *========================================================================*/
static void
ClearColorSelect(
	GtkWidget	*widget,
	gpointer		client_data
)
{
	if ( color_dialog != NULL )
	{
		gtk_widget_destroy(color_dialog);
		color_dialog = NULL;
	}
}


/*========================================================================
 *	Name:			XPNoteProject
 *	Prototype:	XPNoteProject(GtkWidget *widget, gpointer client_data)
 *					
 *	Description:
 * Close the color selection dialog.
 *
 *	Notes:
 *========================================================================*/
void
XPNoteProject(
	GtkWidget	*widget,
	int			index
)
{
	StickyNote	*pn;

	/*
	 * Grab the note structure.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return; 

	/*
	 * Present the projects dialog to the user.
	 */
	XPProject(XP_PROJ_USER, pn);
}


/*========================================================================
 *	Name:			PNoteUpdate
 *	Prototype:	PNoteUpdate(GtkWidget *widget, gpointer client_data)
 *					
 *	Description:
 * Close the color selection dialog.
 *
 *	Notes:
 *========================================================================*/
static void
PNoteUpdate(
	GtkWidget	*widget,
	StickyNote	*pn
)
{
	GdkColormap		*colormap;
	ProjectType		*item;
	GList				*list;
	GtkWidget		*list_item;
	GdkColor			gdk_color;
	GtkStyle			*style1, *style2;

	/*
	 * Retrieve the selected project from the link list of projects.
	 */
	list = (GTK_LIST(project_list))->selection;
	if ( list == NULL )
		return;

	list_item = GTK_WIDGET(list->data);
	item = (ProjectType *) gtk_object_get_user_data(GTK_OBJECT(list_item));

	/*
	 * Save the current projects colors.
	 */
	prj_color[0]= item->color[0];
	prj_color[1]= item->color[1];
	prj_color[2]= item->color[2];

	gdk_color.red = (guint16)(item->color[0]*65535.0);
	gdk_color.green = (guint16)(item->color[1]*65535.0);
	gdk_color.blue = (guint16)(item->color[2]*65535.0);

	/*
	 * Update the notes color.  This is a hack - the GTK v1.0.x Text
	 * widget doesn't accept changes to the background colors, so we
	 * only get the border changed.  In order to show the project color
	 * we have to set the background of the scroll widget too.  Bleck.
	 */
	colormap = gdk_window_get_colormap (pn->pn_textwidget->window);
	gdk_color_alloc (colormap, &gdk_color);

	style1 = gtk_widget_get_style( pn->pn_textwidget );
	style2 = gtk_style_copy( style1 );
	style2->base[GTK_STATE_NORMAL].red = gdk_color.red;
	style2->base[GTK_STATE_NORMAL].green = gdk_color.green;
	style2->base[GTK_STATE_NORMAL].blue = gdk_color.blue;
	style2->bg[GTK_STATE_NORMAL].red = gdk_color.red;
	style2->bg[GTK_STATE_NORMAL].green = gdk_color.green;
	style2->bg[GTK_STATE_NORMAL].blue = gdk_color.blue;
	gtk_widget_set_style(pn->pn_textwidget, style2);

	gdk_window_set_background(pn->pn_scrollwidget->window, &gdk_color);

	gdk_window_clear(pn->pn_textwidget->window);
	gtk_widget_queue_draw(pn->pn_textwidget);
	gdk_window_clear(pn->pn_scrollwidget->window);
	gtk_widget_queue_draw(pn->pn_scrollwidget);

	/*
	 * Set its project name.
	 */
	if (pn->pn_project != NULL )
		free(pn->pn_project);
	pn->pn_project = XPSafeAlloc(strlen(item->name) + 1);
	strcpy(pn->pn_project, item->name);

	/*
	 * Mark the note as changed.
	 */
	XPSetSaveSensitive(NULL, NULL, pn);
}

