/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: timer_cb.c
 *            DESCRIPTION: callbacks for timers
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPAutoSave, XPAlarmCheck, XPMakeAlarmPopup,
 *								 : XPCreateTimerPrompt, XPAlarmOff
 *      PRIVATE FUNCTIONS: TimerLeftArrowCB, TimerCancel,
 *								 : TimerRightArrowCB, TimerAccept, UpdateAudioID,
 *								 : Reset
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_TIMERCB_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <signal.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */
void XPMakeAlarmPopup();


/* === Private routine prototypes === */
static void TimerRightArrowCB();
static void TimerLeftArrowCB();
static void TimerCancel();
static void TimerAccept();
static void UpdateAudioID();
static void Reset();


/* === Global Variables === */
extern GtkWidget		*toplevel;
extern AppRes			app_res;
extern StickyNote		*notes;
extern GdkPixmap		*alarmon_xbm;
extern GdkPixmap		*alarmoff_xbm;
extern GdkBitmap		*alarmon_mask;
extern GdkBitmap		*alarmoff_mask;
extern AudioFileType	*audio_files;


/* === Static Variables === */
static int				audio_id=-1;
static int				reset_val = 0;
static GtkWidget		*timer_text[5];
static GtkWidget		*timer_widget=NULL;



/*========================================================================
 *	Name:			XPAutoSave
 *	Prototype:	void *XPAutoSave ( gpointer *user_data )
 *					
 *	Description:
 * Save all notes automatically when timer expires
 *
 *	Input Arguments:
 *	
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
int
XPAutoSave(
	gpointer *user_data
)
{
	XPSaveAllNotes(FALSE);
	return(TRUE);
}


/*========================================================================
 *	Name:			XPAlarmCheck
 *	Prototype:	void *XPAlarmCheck ( gpointer *user_data )
 *					
 *	Description:
 * Check the notes for any alarms that need to go off.
 *
 *	Input Arguments:
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
int
XPAlarmCheck(
	gpointer *user_data
)
{
	char	buf[1024];
	char	buf2[MAXPATHLEN];
	char	*earg[257];
	int	i;

	register StickyNote	*pn;
	time_t					time_tp;
	struct tm				*tm_tp;

	for (pn = notes; pn != NULL; pn = pn->pn_next)
	{
		if (pn->pn_alarm)
		{
			time(&time_tp);
			tm_tp = localtime(&time_tp);
			if ( (pn->pn_alarm_mon == tm_tp->tm_mon) &&
			     (pn->pn_alarm_day == tm_tp->tm_mday) &&
			     (pn->pn_alarm_hour == tm_tp->tm_hour) &&
			     (pn->pn_alarm_min == tm_tp->tm_min) )
			{

				/*
				 * Display the alarm message.
				 */
				XPMakeAlarmPopup(pn);

				/*
				 * Test to see if we need to try to play the audio alarm.
				 */
				if(( app_res.noalarm == FALSE ) && 
					( app_res.audio_cmd != NULL) &&
					( app_res.noaudio != FALSE) &&
					( pn->pn_alarm_audio != NULL) )
				{
					/*
					 * Fork a second process to play the alarm audio,
					 * if requested.
					 */
					if ( fork() == 0 )
					{
						/*
						 * The child process plays the audio.  We fork() because
						 * playing the audio can take a moment and we don't want
						 * to block the rest of the program.
						 *
						 * Errors get written to stderr only.
						 */
						sprintf(buf2, "%s/%s", app_res.audio_dir, pn->pn_alarm_audio);
						sprintf(buf, app_res.audio_cmd, buf2, 
							(pn->pn_alarm_volume/100.0));
						earg[0] = strtok(buf," ");
						for (i=1; i<256; i++)
						{
							earg[i] = strtok(NULL," ");
							if ( earg[i] == NULL )
								break;
						}

						/*
						 * Make sure we NULL terminate the list of args.
						 */
						if (i==256)
						{
							earg[254] = "\"";
							earg[255] = NULL;
						}

						/*
						 * Now exec() the audio command.  We use exec() because
						 * we don't want the child to actually exit - that can
						 * cause problems with widgets shared with the parent
						 * that would go away.
						 */
						if ( execv(earg[0], &earg[0] ) == -1 )
						{
							fprintf(stderr,"%s: command failed (errno=%d)- %s\n",
								XP_S_PROG_NAME, errno, buf);
						}
						/*
						 * Just so its understood - the child stops here!
						 * (Except it should never actually get here.)
						 */
						exit(0);
					}
				}
			}
		}
	}

	return(TRUE);
}


/*========================================================================
 *	Name:			XPMakeAlarmPopup
 *	Prototype:	void *XPMakeAlarmPopup ( StickyNote *pn )
 *					
 *	Description:
 * Create dialog for when alarm expires.
 *
 *	Input Arguments:
 *	StickyNote		*pn		pointer to note for which alarm has expired
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPMakeAlarmPopup(
	StickyNote *pn
)
{
	char	string[512];
	int	i;

	/*
	 * Build the text message.
	 */
	sprintf(string, "%s %s %s", 
		XP_S_EXPIRED_1, XP_S_EXPIRED_2, pn->pn_name->str);

	XPMsgWindow(string, 
		XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOCANCEL, NULL, pn->pn_shellwidget,
		NULL, NULL, NULL, NULL);

	/* double-ring bell 3 times */
	for (i=0; i<3; i++)
		XBell(gdk_display, 80);
}


/*========================================================================
 *	Name:			XPCreateTimerPrompt
 *	Prototype:	void XPCreateTimerPrompt(GtkWidget *widget, int index)
 *					
 *	Description:
 * Create a prompt for setting an alarm.
 *
 *	Input Arguments:
 *	GtkWidget	*widget		widget for which this callback was set
 * int			index			index of note associated with widget
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPCreateTimerPrompt(
	GtkWidget	*widget,
	int			index
)
{
#ifdef DEBUG
	char			*fname="XPCreateTimerPrompt";
#endif 

	StickyNote		*pn;
	GtkWidget		*timer_label[4];
	GtkWidget		*eventbox;
	GtkWidget		*timer_leftarrow, *timer_rightarrow;
	GtkWidget		*timer_buttons[2];
	GtkWidget		*button;
	GtkWidget		*timer_boxes[4];
	GtkWidget		*frame1, *frame2, *frame3, *vbox, *vbox2, *vbox3, *vbox4;
	GtkWidget		*hsep;
	GtkWidget		*label, *hbox;
	GtkWidget		*scrolled_window, *audio_list, *list_item;
	GtkWidget		*current_setting;
	char				buf[128];
	int				i;
	AudioFileType	*audio_link;

	GtkRequisition	attributes;
	int				width, height;
	time_t			time_tp;
	struct tm		*tm_tp;
	GShell_T			gshell;

	DBGEnter();

	/*
	 * Don't open more than one alarm window.
	 */
	if ( timer_widget != NULL )
		return;

	/*
	 * Find the note of interest.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Create a popup shell widget
	 */
	timer_widget = XPCreateGenericShell(NotesTimerPrompt,
			ACCEPT_BUTTON | CANCEL_BUTTON, &gshell, FALSE );

	gtk_signal_connect (GTK_OBJECT (timer_widget), "destroy",
			GTK_SIGNAL_FUNC (TimerCancel), NULL);

	vbox = gshell.vbox;
	timer_buttons[0] = gshell.accept;
	timer_buttons[1] = gshell.cancel;


	/*
	 * 3 Frames around:
	 * 1. The current alarm setting.
	 * 2. Buttons to set the alarm.
	 * 3. Alarm audio setting.
	 */
	frame1 = gtk_frame_new(XP_S_CURRENT_TIME);
	gtk_frame_set_shadow_type (GTK_FRAME(frame1), GTK_SHADOW_ETCHED_OUT); 
	gtk_box_pack_start(GTK_BOX(vbox), frame1, FALSE, FALSE, 0);
	gtk_widget_show(frame1);

	frame2 = gtk_frame_new(XP_S_SET_TIME);
	gtk_frame_set_shadow_type (GTK_FRAME(frame2), GTK_SHADOW_ETCHED_OUT); 
	gtk_box_pack_start(GTK_BOX(vbox), frame2, FALSE, FALSE, 0);
	gtk_widget_show(frame2);

	frame3 = gtk_frame_new(XP_S_SET_AUDIO);
	gtk_frame_set_shadow_type (GTK_FRAME(frame3), GTK_SHADOW_ETCHED_OUT); 
	gtk_box_pack_start(GTK_BOX(vbox), frame3, FALSE, FALSE, 0);
	gtk_widget_show(frame3);

	/*
	 * Verticle boxes to place the components inside the frames.
	 */
	vbox2 = gtk_vbox_new(TRUE, 2);
	gtk_container_border_width (GTK_CONTAINER (vbox2), 5);
	gtk_container_add(GTK_CONTAINER(frame1), vbox2);
	gtk_widget_show(vbox2);

	vbox3 = gtk_vbox_new(TRUE, 2);
	gtk_container_border_width (GTK_CONTAINER (vbox3), 5);
	gtk_container_add(GTK_CONTAINER(frame2), vbox3);
	gtk_widget_show(vbox3);

	vbox4 = gtk_vbox_new(FALSE, 2);
	gtk_container_border_width (GTK_CONTAINER (vbox4), 1);
	gtk_container_add(GTK_CONTAINER(frame3), vbox4);
	gtk_widget_show(vbox4);

	/*
	 * The current alarm setting in a label.
	 */
	sprintf(buf, "%02d/%02d %02d:%02d", 
		pn->pn_alarm_mon+1, pn->pn_alarm_day,
		pn->pn_alarm_hour, pn->pn_alarm_min);
	current_setting = gtk_label_new(buf);
	gtk_box_pack_start(GTK_BOX(vbox2), current_setting, TRUE, TRUE, 0);
	gtk_widget_show(current_setting);

	/*
	 * Horizontal boxes, one for each line of that sets part of the alarm
	 * setting, plus one for the Accept/Cancel buttons.
	 */
	for ( i=0; i<4; i++)
	{
		switch (i)
		{
			case 0:
			case 1:
			case 2:
			case 3:
				timer_boxes[i] = gtk_hbox_new(FALSE, 2);
				gtk_box_pack_start(GTK_BOX(vbox3), timer_boxes[i], TRUE, TRUE, 0);
				gtk_widget_show(timer_boxes[i]);
				break;
		}

		/*
		 * A horizontal separator. which goes before the buttons.
		 */
		if ( i == 3 )
		{
			hsep = gtk_hseparator_new();
			gtk_widget_show(hsep);
			gtk_box_pack_start (GTK_BOX (vbox), hsep, FALSE, FALSE, 0);
		}
	}

	/*
	 * A reset button - to make the spin values match the last setting.
	 */
	reset_val = 0;
	button = gtk_button_new();
	gtk_widget_set_name(button, "window_button");
	label = gtk_label_new(XP_S_TOGGLE);
	gtk_widget_set_name(label, "window_button");
	gtk_widget_show(label);
	gtk_container_add(GTK_CONTAINER(button), label);
	gtk_box_pack_start(GTK_BOX(vbox3), button, TRUE, TRUE, 2);
	gtk_widget_show(button);
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (Reset), (gpointer)pn);

	/* Add a tool tip */
	gtk_signal_connect(GTK_OBJECT(button), "event",
		GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_AUDIO_TOG);

	/* The labels of each box */
	for ( i=0; i<4; i++)
	{
		switch(i)
		{
			case 0: sprintf(buf, XP_S_MONTH ); break;
			case 1: sprintf(buf, XP_S_DAY); break;
			case 2: sprintf(buf, XP_S_HOUR); break;
			case 3: sprintf(buf, XP_S_MIN); break;
		}
		timer_label[i] = gtk_label_new(buf);
		gtk_widget_show(timer_label[i]);
		gtk_box_pack_start (GTK_BOX (timer_boxes[i]), 
			timer_label[i], TRUE, FALSE, 0);
	}
	gtk_widget_size_request(timer_label[0], &attributes);
	width = attributes.width;
	height = attributes.height;
	for ( i=1; i<4; i++ )
		gtk_widget_set_usize(timer_label[i], width, height);

	/*
	 * Get the current time.
	 */
	time(&time_tp);
	tm_tp = localtime(&time_tp);

	/*
	 * The arrow buttons and text fields in each box
	 */
	for ( i=0; i<4; i++)
	{
		sprintf( buf, " < ");
		timer_leftarrow = gtk_button_new();
		gtk_widget_set_name(timer_leftarrow, "xp_button");
		label = gtk_label_new(buf);
		gtk_widget_set_name(label, "xp_button");
		gtk_widget_show(label);
		gtk_widget_show(timer_leftarrow);
		gtk_container_add(GTK_CONTAINER(timer_leftarrow), label);
		gtk_signal_connect (GTK_OBJECT (timer_leftarrow), "clicked",
			GTK_SIGNAL_FUNC (TimerLeftArrowCB), (gpointer)i);

		/*
		 * Set the text fields to the notes stored values
		 */
		switch (i)
		{
			case 0: sprintf(buf, "%02d", tm_tp->tm_mon+1); break;
			case 1: sprintf(buf, "%02d", tm_tp->tm_mday); break;
			case 2: sprintf(buf, "%02d",  tm_tp->tm_hour); break;
			case 3: sprintf(buf, "%02d", tm_tp->tm_min); break;
			default : sprintf(buf, "   "); break;
		}
	
		timer_text[i] = gtk_label_new(buf);
		gtk_widget_show(timer_text[i]);
		gtk_widget_set_usize(timer_text[i], width, height);

		sprintf( buf, " > ");
		timer_rightarrow = gtk_button_new();
		gtk_widget_set_name(timer_rightarrow, "xp_button");
		label = gtk_label_new(buf);
		gtk_widget_set_name(label, "xp_button");
		gtk_widget_show(label);
		gtk_widget_show(timer_rightarrow);
		gtk_container_add(GTK_CONTAINER(timer_rightarrow), label);
		gtk_signal_connect (GTK_OBJECT (timer_rightarrow), "clicked",
			GTK_SIGNAL_FUNC (TimerRightArrowCB), (gpointer)i);

		gtk_box_pack_start(GTK_BOX(timer_boxes[i]), 
			timer_leftarrow, FALSE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(timer_boxes[i]), 
			timer_text[i], FALSE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(timer_boxes[i]), 
			timer_rightarrow, FALSE, TRUE, 0);
	}

	/*
	 * A scrolled window for the audio list.
	 */
	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
	gtk_container_border_width (GTK_CONTAINER (scrolled_window), 3);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(vbox4), scrolled_window, FALSE, FALSE, 0);
	gtk_widget_show (scrolled_window);


	/*
	 * The audio list.
	 */
	audio_list = gtk_list_new();
	gtk_list_set_selection_mode (GTK_LIST (audio_list), GTK_SELECTION_SINGLE);
	gtk_container_add (GTK_CONTAINER (scrolled_window), audio_list);
	gtk_widget_show (audio_list);
	gtk_signal_connect (GTK_OBJECT (audio_list), "selection_changed",
		GTK_SIGNAL_FUNC (UpdateAudioID), audio_list);

	/*
	 * Populate the list.
	 */
	audio_id = -1;
	audio_link = audio_files;
	while (audio_link != NULL)
	{
		list_item = gtk_list_item_new_with_label (audio_link->file);
		gtk_container_add (GTK_CONTAINER (audio_list), list_item);
		gtk_widget_show (list_item);
		gtk_object_set_user_data (GTK_OBJECT (list_item), 
			(gpointer) audio_link->id);

		/*
		 * If the alarm already has an audio file selected, preselect it.
	 	 */
		if ( pn->pn_alarm_audio != NULL )
		{
			if ( strcmp(pn->pn_alarm_audio, audio_link->file) == 0 )
			{
			/*	gtk_item_select (GTK_ITEM (list_item));
			 */
				gtk_list_select_item (GTK_LIST(audio_list), audio_link->id);

				audio_id = audio_link->id;
			}
		}

		audio_link = audio_link->next;
	}

	/*
	 * A spinbox for setting the repeat amount.
	 */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox4), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	label = gtk_label_new(XP_S_REPEAT_TEXT);
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

	sprintf( buf, " < ");
	timer_leftarrow = gtk_button_new();
	gtk_widget_set_name(timer_leftarrow, "xp_button");
	label = gtk_label_new(buf);
	gtk_widget_set_name(label, "xp_button");
	gtk_widget_show(label);
	gtk_widget_show(timer_leftarrow);
	gtk_container_add(GTK_CONTAINER(timer_leftarrow), label);
	gtk_signal_connect (GTK_OBJECT (timer_leftarrow), "clicked",
		GTK_SIGNAL_FUNC (TimerLeftArrowCB), (gpointer)i);
	gtk_box_pack_start(GTK_BOX(hbox), timer_leftarrow, FALSE, FALSE, 0);

	eventbox = gtk_event_box_new();
	gtk_box_pack_start(GTK_BOX(hbox), eventbox, FALSE, FALSE, 0);
	gtk_widget_show(eventbox);

	sprintf(buf, "%d", pn->pn_alarm_repeat);
	timer_text[i] = gtk_label_new(buf);
	gtk_widget_show(timer_text[i]);
	gtk_widget_set_usize(timer_text[i], width, height);
	gtk_container_add(GTK_CONTAINER(eventbox), timer_text[i]);

	/* Add a tool tip */
	gtk_signal_connect(GTK_OBJECT(eventbox), "event",
		GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_AUDIO_REP);

	sprintf( buf, " > ");
	timer_rightarrow = gtk_button_new();
	gtk_widget_set_name(timer_rightarrow, "xp_button");
	label = gtk_label_new(buf);
	gtk_widget_set_name(label, "xp_button");
	gtk_widget_show(label);
	gtk_widget_show(timer_rightarrow);
	gtk_container_add(GTK_CONTAINER(timer_rightarrow), label);
	gtk_signal_connect (GTK_OBJECT (timer_rightarrow), "clicked",
		GTK_SIGNAL_FUNC (TimerRightArrowCB), (gpointer)i);
	gtk_box_pack_start(GTK_BOX(hbox), timer_rightarrow, FALSE, FALSE, 0);


	/*
	 * create the Cancel/Accept command buttons 
	 */
	gtk_signal_connect (GTK_OBJECT (timer_buttons[0]), "clicked",
			GTK_SIGNAL_FUNC (TimerAccept), (gpointer)pn);

	gtk_signal_connect (GTK_OBJECT (timer_buttons[1]), "clicked",
			GTK_SIGNAL_FUNC (TimerCancel), (gpointer)NULL);

	/*
	 * Pop up the window.
	 */
	gtk_widget_realize(timer_widget);
	XPCenterWindow(timer_widget, pn->pn_shellwidget, timer_widget);
	gtk_widget_show( timer_widget );
}


/*========================================================================
 *	Name:			TimerLeftArrowcB
 *	Prototype:	void TimerLeftArrowcB(GtkWidget *widget, int index)
 *					
 *	Description:
 * Handle a left arrow click
 *========================================================================*/
void
TimerLeftArrowCB(
	GtkWidget	*widget,
	int			index
)
{
	int		max=0, min=0;
	int		value;
	char		*string;
	char		buf[10];

	/* set limits based on which field is being updated */
	switch ( index )
	{
		case 0: max=12; min=1; break; /* month */
		case 2: max=23; min=0; break; /* hour */
		case 3: max=59; min=0; break; /* minute */
		case 4: max=20; min=0; break; /* repeat */

		/*
		 * setting the day max is dependent on which month
		 * is currently set
		 */
		case 1:
			min=1;  
			gtk_label_get(GTK_LABEL(timer_text[0]), &string);
			value = (int)atoi(string);
			switch ( value )
			{
				case 1:
				case 3:
				case 5:
				case 7:
				case 8:
				case 10:
				case 12:	max=31; break;
				case 2:	max=29; break;
				default:	max=30; break;
			}
			break;
	}

	/* Decrement text field */
	gtk_label_get(GTK_LABEL(timer_text[index]), &string);
	value = (int)atoi(string) - 1;
	if ( value < min )
		value = max;
	sprintf ( buf, "%02d", value);
	gtk_label_set(GTK_LABEL(timer_text[index]), buf);

}


/*========================================================================
 *	Name:			TimerRightArrowcB
 *	Prototype:	void TimerRightArrowcB(GtkWidget *widget, int index)
 *					
 *	Description:
 * Handle a right arrow click
 *========================================================================*/
void
TimerRightArrowCB(
	GtkWidget	*widget,
	int			index
)
{
	int	max=0, min=0;
	char	*string;
	char	buf[25];
	int	value;

	/* set limits based on which field is being updated */
	switch ( index )
	{
		case 0: max=12; min=1; break; /* month */
		case 2: max=23; min=0; break; /* hour */
		case 3: max=59; min=0; break; /* minute */
		case 4: max=20; min=0; break; /* repeat */

		/*
		 * setting the day max is dependent on which month
		 * is currently set
		 */
		case 1: 
			min = 1;
			gtk_label_get(GTK_LABEL(timer_text[0]), &string);
			value = (int)atoi(string);
			switch ( value )
			{
				case 1:
				case 3:
				case 5:
				case 7:
				case 8:
				case 10:
				case 12:	max=31; break;
				case 2:	max=29; break;
				default:	max=30; break;
			}
			break;
	}

	/* increment text field */
	gtk_label_get(GTK_LABEL(timer_text[index]), &string);
	value = (int)atoi(string) + 1;
	if ( value > max )
		value = min;
	sprintf ( buf, "%02d", value);
	gtk_label_set(GTK_LABEL(timer_text[index]), buf);
}


/*========================================================================
 *	Name:			TimerCancel
 *	Prototype:	void TimerCancel()
 *					
 *	Description:
 * Removes the "Set Alarm" Dialog box
 *========================================================================*/
static void
TimerCancel()
{
	if ( timer_widget != NULL )
	{
		gtk_widget_destroy( timer_widget );
		timer_widget = NULL;
	}
}


/*========================================================================
 *	Name:			TimerAccept
 *	Prototype:	void TimerAccept()
 *					
 *	Description:
 * Accept alarm timer settings.
 *========================================================================*/
static void
TimerAccept(
	GtkWidget	*widget,
	StickyNote	*pn
)
{
#ifdef DEBUG
	char			*fname="TimerAccept";
#endif 

	int				i, value;
	char				*string;
	AudioFileType	*audio_link;

	DBGEnter();
	
	/*
	 * Fetch the configured alarm values.
	 */
	for (i=0; i<4; i++)
	{
		gtk_label_get(GTK_LABEL(timer_text[i]), &string);
		value = (int)atoi(string);
		switch (i)
		{
			/* Month has to be adjusted to what localtime() uses. */
			case 0: pn->pn_alarm_mon = value-1; break;
			case 1: pn->pn_alarm_day = value; break;
			case 2: pn->pn_alarm_hour = value; break;
			case 3: pn->pn_alarm_min = value; break;
		}
	}

	/*
	 * Get the repeat count.
	 */
	gtk_label_get(GTK_LABEL(timer_text[i]), &string);
	pn->pn_alarm_repeat = atoi(string);

	/*
	 * Get the alarm audio setting, if any.
	 */
	audio_link = audio_files;
	while ( audio_link != NULL )
	{
		if (audio_link->id == audio_id)
		{
			if ( pn->pn_alarm_audio != NULL )
				free(pn->pn_alarm_audio);
			pn->pn_alarm_audio = (char *)XPSafeAlloc(strlen(audio_link->file)+1);
			strcpy(pn->pn_alarm_audio, audio_link->file);
			break;
		}
		audio_link = audio_link->next;
	}

	/*
	 * Turn on the alarm flag.
	 */
	pn->pn_alarm = TRUE;

	/*
	 * Set the sensitivity of various widgets appropriately.
	 */
	gtk_widget_set_sensitive ( pn->pn_unsettimewidget, TRUE );
	gtk_widget_set_sensitive ( pn->pn_settimewidget, FALSE );
	gtk_widget_set_sensitive ( pn->pn_savewidget, TRUE );

	/* set the alarm icon on for this note */
	gtk_pixmap_set(GTK_PIXMAP(pn->pn_alarm_pixmapid), alarmon_xbm,
		NULL);

	/*
	 * Close the alarm timer window.
	 */
	TimerCancel();
}


/*========================================================================
 *	Name:			XPAlarmOff
 *	Prototype:	void XPAlarmOff()
 *					
 *	Description:
 * Turn off alarm for a note.
 *
 * Notes:
 * Just like TimerExcept except there is no dialog to fetch data from or 
 * to close.  Also, this routine is called directly as a callback, whereas
 * TimerExcept is called from the timer dialog.
 *========================================================================*/
void
XPAlarmOff(
	GtkWidget	*widget,
	int			index
)
{
	StickyNote 	*pn;
	
	/*
	 * Get the note structure.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Turn off the alarm flag.
	 */
	pn->pn_alarm = FALSE;

	/*
	 * Set the sensitivity of various widgets appropriately.
	 */
	gtk_widget_set_sensitive ( pn->pn_unsettimewidget, FALSE );
	gtk_widget_set_sensitive ( pn->pn_settimewidget, TRUE );
	gtk_widget_set_sensitive ( pn->pn_savewidget, TRUE );

	/* set the alarm icon on for this note */
	gtk_pixmap_set(GTK_PIXMAP(pn->pn_alarm_pixmapid), 
		alarmoff_xbm, NULL);

}



/*========================================================================
 *	Name:			UpdateAudioId
 *	Prototype:	void UpdateAudioID()
 *					
 *	Description:
 * When an audio file is selected, update the global audio id.
 *========================================================================*/
static void
UpdateAudioID(
	GtkWidget	*list_widget,
	gpointer		function_data
)
{
	GList				*list;
	GtkWidget		*list_item;

	/*
	 * Get the selection list.  There should only be one since we only
	 * allow a single selection.  Then grab the Widget pointer and use
	 * it to get the user data.  Thats the pointer to the note we want.
	 */
	list = (GTK_LIST(list_widget))->selection;
	if ( list == NULL )
		return;
	list_item = GTK_WIDGET(list->data);
	audio_id = (int) gtk_object_get_user_data(GTK_OBJECT(list_item));
}


/*========================================================================
 *	Name:			Reset
 *	Prototype:	void Reset()
 *					
 *	Description:
 * Toggle the alarm spin values between the current time and the last
 * setting.
 *========================================================================*/
static void
Reset(
	GtkWidget	*widget,
	StickyNote	*pn
)
{
	int			i;
	char			buf[25];
	time_t		time_tp;
	struct tm	*tm_tp;

	/*
	 * The arrow buttons and text fields in each box
	 */
	for ( i=0; i<4; i++)
	{
		if ( reset_val == 0 )
		{
			/*
			 * Set the text fields to the notes stored values
			 */
			switch (i)
			{
				case 0: sprintf(buf, "%02d", pn->pn_alarm_mon+1); break;
				case 1: sprintf(buf, "%02d", pn->pn_alarm_day); break;
				case 2: sprintf(buf, "%02d", pn->pn_alarm_hour); break;
				case 3: sprintf(buf, "%02d", pn->pn_alarm_min); break;
			}
			gtk_label_set(GTK_LABEL(timer_text[i]), buf);
		}
		else
		{
			/*
			 * Set the text fields to the notes stored values
			 */
			time(&time_tp);
			tm_tp = localtime(&time_tp);
			switch (i)
			{
				case 0: sprintf(buf, "%02d", tm_tp->tm_mon+1); break;
				case 1: sprintf(buf, "%02d", tm_tp->tm_mday); break;
				case 2: sprintf(buf, "%02d", tm_tp->tm_hour); break;
				case 3: sprintf(buf, "%02d", tm_tp->tm_min); break;
			}
			gtk_label_set(GTK_LABEL(timer_text[i]), buf);
		}
	}

	if ( reset_val == 0 )
		reset_val = 1;
	else
		reset_val = 0;
}
