/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: tooltips.c
 *            DESCRIPTION: pop up windows that give help for certain features
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPToolTips
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_TT_C

/* === System Headers === */
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "gtk/gtk.h"

/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"

/* === external routines === */

/* === Public routine prototypes === */

/* === Private routine prototypes === */

/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;

static char	*ToolTipText[] = {
	XP_S_TT_DIR,
	XP_S_TT_PRJ_TEXT,
	XP_S_TT_AUDIO_REP,
	XP_S_TT_AUDIO_TOG,
	XP_S_TT_PILOT_ADDR,
	XP_S_TT_PILOT_MFET,
	XP_S_TT_PILOT_MPRO,
	XP_S_TT_PILOT_MEMO,
	XP_S_TT_ENV_SAVE_FROM,
	XP_S_TT_ENV_LOAD_TO,
	XP_S_TT_PRJ_SAVE,
	XP_S_TT_PRJ_CANCEL,
	NULL
};


/*========================================================================
 *	Name:			XPToolTips
 *	Prototype:	Widget XPToolTips()
 *					
 *	Description:
 *	Pop up windows with tips on using certain features.
 *
 *	Notes:
 *		
 *========================================================================*/
int
XPToolTips(
	GtkWidget	*widget, 
	GdkEvent		*event,
	int			details
)
{
#ifdef DEBUG
	char			fname[]="XPToolTip()";
#endif

	GdkColor				color;
	GdkColormap			*colormap;
	static int			first_time = TRUE;
	static GtkWidget	*detail_popup, *detail_label;
	static GtkWidget	*detail_bg, *detail_border, *detail_hbox;

	int		xpos, ypos, width, height;


	DBGEnter();

	/*
	 * Don't show detail windows if user has turned this feature off.
	 */
	if ( app_res.no_tips )
			return (FALSE);

	if (first_time)
	{
		DBGPrintf(DBG_TRACE3,("Creating tool tip window.\n"));
		/*
		 * Create the popup shell and label for our messages
		 * in the pop down window
		 */
		detail_popup = gtk_window_new(GTK_WINDOW_POPUP);
		gtk_widget_set_name(detail_popup, "tool_tip"); 
		gtk_window_set_policy (GTK_WINDOW (detail_popup), FALSE, FALSE, TRUE);
		gtk_widget_realize (detail_popup);

		/*
		 * Force a border by sticking a GtkFixed widget behind the label that
		 * is a bit larger.  Its a trick - I don't know how (or if its
		 * possible) to add a border color and specify its width for windows.
		 */
		detail_border = gtk_fixed_new();
		gtk_container_border_width (GTK_CONTAINER (detail_border), 2);
		gtk_widget_show(detail_border);
		gtk_container_add(GTK_CONTAINER(detail_popup), detail_border);

		color.red = 0;
		color.green = 0;
		color.blue = 0;
		gtk_widget_realize(detail_border);
		colormap = gdk_window_get_colormap (detail_border->window);
		gdk_color_alloc (colormap, &color);
		gdk_window_set_background (detail_border->window, &color);

		/*
		 * An eventbox to provide the background color of the tooltips.
		 */
		detail_bg = gtk_event_box_new();
		gtk_widget_set_name(detail_bg, "tool_tip"); 
		gtk_widget_show(detail_bg);
		gtk_container_add(GTK_CONTAINER(detail_border), detail_bg);

		/*
		 * An hbox to force some space between the label and the border.
		 */
		detail_hbox = gtk_hbox_new(TRUE, 3);
		gtk_container_border_width (GTK_CONTAINER (detail_hbox), 3);
		gtk_widget_set_name(detail_hbox, "tool_tip"); 
		gtk_widget_show(detail_hbox);
		gtk_container_add(GTK_CONTAINER(detail_bg), detail_hbox);

		/*
		 * The label, where we will stick the detail text.
		 */
		detail_label = gtk_label_new("testing");
		gtk_widget_set_name(detail_label, "tool_tip"); 
		gtk_widget_show(detail_label);
		gtk_container_add(GTK_CONTAINER(detail_hbox), detail_label);
		gtk_label_set_justify (GTK_LABEL(detail_label), GTK_JUSTIFY_LEFT); 

		first_time = FALSE;
	}

	switch (event->type)
	{
		case GDK_ENTER_NOTIFY:

			DBGPrintf(DBG_TRACE3,("EnterNotify event caught.\n"));

			/*
			 * Its possible to get in here with the popup already displayed.
			 * In that case, don't do anything - just return.
		 	 */
			if ( GTK_WIDGET_VISIBLE(detail_popup) )
				return (FALSE);

			/*
			 * Set the help message in the label of the tool tip.
			 */
			DBGPrintf(DBG_TRACE3,("setting label: %s\n", ToolTipText[details]));
			gtk_label_set(GTK_LABEL(detail_label), ToolTipText[details]);

		
			/*
			 * Determine real coordinates for popup window
			 * We want to put it on the lower left edge of the icon, but 
			 * not the lower left edge of the label (since the icon is 
			 * rectangular, but appears non-rectangular, we need the pop-down
			 * window to be near the visible part of the icon).
			 */
			gdk_window_get_origin(widget->window, &xpos, &ypos);
			gdk_window_get_geometry(widget->window, NULL, NULL, 
				&width, &height, NULL);
			DBGPrintf(DBG_TRACE3,("window position: %d/%d (%dx%d\n", 
					xpos,ypos, width, height));
		
			/*
			 * Set the positon for the popup
			 */
			DBGPrintf(DBG_TRACE3,("showing window\n"));
			gtk_widget_set_uposition (detail_popup, 
					xpos+(.75*width), ypos+height-2);
			gtk_widget_show(detail_popup);
			return(FALSE);
			break;

		/* 
		 * Mouse leaves window or button pressed, then hide the tool tip.
		 */
		case GDK_LEAVE_NOTIFY:
		case GDK_BUTTON_PRESS:
			DBGPrintf(DBG_TRACE3,("Non-EnterNotify event caught.\n"));
			if ( ! first_time )
				if ( GTK_WIDGET_VISIBLE(detail_popup) )
					gtk_widget_hide(detail_popup);
			return(FALSE);
			break;

		/*
		 * Ignore all others.
		 */
		default:
			return(FALSE);
			break;
	}

}


