/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: utils.c
 *            DESCRIPTION: generic routines for XNotesPlus
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPMultiLineText, XPInsertText, XPShutdown,
 *                       : XPGtkNameToWidget, XPCenterWindow
 *      PRIVATE FUNCTIONS: SOECancel, SOEAccept
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_UTILS_C

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>

#include <gtk/gtk.h>
#include <gdk/gdkx.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */
void XPCenterWindow();


/* === Private routine prototypes === */
static void SOEAccept();
static void SOECancel();


/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;
extern GtkWidget		*toplevel;
extern int				disabled_cmds_timer;


/* === Static Variables === */



/*========================================================================
 *	Name:			XPMultiLineText
 *	Prototype:	void XPMultiLineText(char *string, int maxlength)
 *					
 *	Description:
 * Place a string into a series of labels.  The maxlength value determines
 * how many characters can go into each label.
 *
 *	Input Arguments:
 *		char	*string		string to place into the labels
 *		int	maxlength	maximum number of characters that should go into
 *								each label.
 *
 *	Return Values:
 *		GtkWidget *			a box widget with the labels stuffed vertically inside
 *
 * Notes:
 *	This doesn't work quite right just yet.  It only copies in based
 * on number of characters - not by full words.  I'll fix it later.
 *
 *========================================================================*/
GtkWidget *
XPMultiLineText(
	char	*string,
	int	maxlength
)
{
#ifdef DEBUG
	char			*fname="XPMultiLineText";
#endif

	char			*newstring, *freeptr, *tmpstring;
	GtkWidget	*vbox, *label;

	DBGEnter();


	/*
	 * Create a copy of the original string and a temporary buffer for
	 * holding maxlength portions of the original.  Hang on to a 
	 * pointer to the copies original location, so we can free it up 
	 * later.
	 */
	DBGPrintf(DBG_TRACE3,("Allocating new buffers.\n\t*%s*\n", string));
	newstring = (char *)XPSafeAlloc(strlen(string)+1);
	tmpstring = (char *)XPSafeAlloc(maxlength+1);
	freeptr = newstring;
	strcpy(newstring, string);

	/*
	 * Create the vertical box that the labels will go in.
	 */
	DBGPrintf(DBG_TRACE3,("Creating vbox.\n"));
	vbox = gtk_vbox_new(FALSE, 0);

/*
	spacer = gtk_label_new(" ");
	gtk_widget_show(spacer);
	gtk_widget_set_usize(spacer, 10, 3);
	gtk_box_pack_start (GTK_BOX (vbox), spacer, TRUE, FALSE, 0);
*/
	while (strlen(newstring)>0)
	{
		if (strlen(newstring) > maxlength)
		{
			/*
			 * Plenty of string left.  Copy maxlength characters to a temp
			 * holder, then move the newstring pointer up by maxlength
			 * characters.
			 */
			DBGPrintf(DBG_TRACE3,
				("Grabbing %d characters for label.\n", maxlength));
			strncpy(tmpstring, newstring, maxlength);
			strncpy((char *)(tmpstring+maxlength), "\0", 1);
			newstring = newstring+maxlength;
		}
		else
		{
			/*
			 * Whats left is less than the maxlength, so just copy it
			 * to the temp location and zero out the newstring.
			 */
			DBGPrintf(DBG_TRACE3, ("Grabbing whats left for label.\n"));
			strcpy(tmpstring, newstring);
			strcpy(newstring, "\0");
		}

		/*
		 * Create the label and add it to the horizontal box.
		 */
		DBGPrintf(DBG_TRACE3, ("Creating new label.\n"));
		label = gtk_label_new(tmpstring);
		gtk_widget_set_name(label, "multiline_text");
		gtk_widget_show(label);
		gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
		
	}
/*
	spacer = gtk_label_new(" ");
	gtk_widget_show(spacer);
	gtk_widget_set_usize(spacer, 10, 3);
	gtk_box_pack_start (GTK_BOX (vbox), spacer, FALSE, FALSE, 0);
*/

	/*
	 * Free up temporary storage.
	 */
	DBGPrintf(DBG_TRACE3, ("Freeing up temp buffers.\n"));
	free(tmpstring);
	free(freeptr);
	
	/*
	 * Pass a pointer to the horizontal box back to the caller.
	 */
	DBGExit();
	return(vbox);
}



/*========================================================================
 *	Name:			XPInsertText
 *	Prototype:	void InsertText(StickyNote *pn, char *tmpfile, int remove
 *							int type)
 *					
 *	Description:
 * Generic routine for inserting text into a note from a file
 *
 *	Input Arguments:
 *		StickyNote	*pn			pointer to note to update
 *		char			*tmpfile		name of file to read from.
 *		int			remove		remove tmpfile? 1: yes, 0: no
 *		int			type			FILE_TYPE: read from a real file
 *										otherwise the file was a temp file we
 *										created on our own.
 *
 * Notes:
 *========================================================================*/
void
XPInsertText(
	StickyNote	*pn,
	char			*tmpfile,
	int			remove,
	int			type
)
{
#ifdef DEBUG
	char	*fname="XPInsertText()";
#endif

	int				fp;
	char				*buf;
	struct stat		stat_buf;
	char				cmd[1024];
	int				position;
	GtkStyle			*style;

	DBGEnter();

	if ( type == FILE_TYPE )
	{

		/*
		 * Check for existance of file.
		 */
		if ( stat(tmpfile, &stat_buf) != 0 )
		{
			sprintf(cmd,"Can't stat file: %s", tmpfile);
			XPErrPopUp(cmd);
			if ( remove )
			{
				sprintf (cmd, "rm %s", tmpfile );
				system ( cmd );
			}
			return;
		}
		if ((fp = open(tmpfile, O_RDONLY)) == -1)
		{
			fprintf(stderr, "xnotes: ");
			perror(tmpfile);
			sprintf(cmd,"Can't open file for\nreading: %s", tmpfile);
			XPErrPopUp(cmd);
			if ( remove )
			{
				sprintf (cmd, "rm %s", tmpfile );
				system ( cmd );
			}
			return;
		}
		buf = (char *) XPSafeAlloc((int)stat_buf.st_size+2);
		while ( read(fp, buf, (int)stat_buf.st_size ) != 0 )
		{
			/*
			 * Insert the text at the current cursor location
			 */
			style = gtk_widget_get_style( pn->pn_textwidget );
			gtk_text_freeze (GTK_TEXT (pn->pn_textwidget));
			DBGPrintf(DBG_INFO,("inserting calendar into note\n%s\n", buf));
			gtk_text_insert (
				GTK_TEXT (pn->pn_textwidget),
				NULL, 					/* font */
				&style->black, 		/* fg color */
				NULL,						/* bg color */
				buf, 						/* text to insert */
				-1);						/* -1: use strlen(); >0: #chars to insert */
			gtk_text_thaw (GTK_TEXT (pn->pn_textwidget));

			/*
			 * Update the position of the cursor in the text buffer.
			 *
			 */
			DBGPrintf(DBG_INFO,("setting new cursor position\n"));
			position = gtk_text_get_length (GTK_TEXT (pn->pn_textwidget));
			gtk_text_set_point (GTK_TEXT (pn->pn_textwidget), position);
		}
		close (fp);
		free(buf);
	
		/*
		 * cleanup
		 */
		if ( remove )
		{
			sprintf (cmd, "rm %s", tmpfile );
			system ( cmd );
		}
	}
	else
	{
		/*
		 * Insert the text at the current cursor location
		 * "tmpfile" is actually a text buffer that we want to insert.
		 * This was added as an afterthought (does it show?)
		 */
		position = gtk_text_get_point (GTK_TEXT (pn->pn_textwidget));
		DBGPrintf(DBG_INFO,("cursor pos: %d\n", position));
		gtk_text_freeze (GTK_TEXT (pn->pn_textwidget));
		gtk_text_insert (
			GTK_TEXT (pn->pn_textwidget),
			NULL, 					/* font */
			&(pn->pn_textwidget)->style->black, /* fg color */
			NULL,						/* bg color */
			tmpfile, 				/* text to insert */
			-1);						/* -1: use strlen(); >0: #chars to insert */
		gtk_text_thaw (GTK_TEXT (pn->pn_textwidget));

		/*
		 * Update the position of the cursor in the text buffer.
		 *
		 */
		position = gtk_text_get_length (GTK_TEXT (pn->pn_textwidget));
		gtk_text_set_point (GTK_TEXT (pn->pn_textwidget), position);
	}
}
	

/*========================================================================
 *	Name:			XPShutdown
 *	Prototype:	void XPShutdown(GtkWidget *widget, gpointer *data)
 *					
 *	Description:
 *	Graceful exit of XNotesPlus.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void 
XPShutdown(
	GtkWidget *widget, 
	gpointer *data
)
{
	GtkWidget		*soe_widget, *soe_label, *soe_buttons[2];
	GtkWidget		*vbox;
	static			int running = FALSE;
	GShell_T			gshell;

	if (app_res.save_notes)
	{
		XPSaveAllNotes(FALSE);

		gtk_main_quit();
		exit(0);
	}

	/*
	 * Ask the user if they want to save the notes
	 * This is a compile time option.
	 */
	else 
	{
		/*
		 * Only one copy of this window allowed at a time!
		 */
		if ( running )
			return;

		running = TRUE;

		/*
		 * Create a popup shell widget
		 */
		soe_widget = XPCreateGenericShell(NotesSaveOnExitPrompt,
			YES_BUTTON | NO_BUTTON, &gshell, TRUE );

		gtk_signal_connect (GTK_OBJECT (soe_widget), "destroy",
			GTK_SIGNAL_FUNC (SOECancel), NULL);

		vbox = gshell.vbox;
		soe_buttons[0] = gshell.accept;
		soe_buttons[1] = gshell.cancel;
	
		/*
		 * The windows message.
		 */
		soe_label = gtk_label_new(XP_S_SAVE_NOTES);
		gtk_box_pack_start (GTK_BOX (vbox), soe_label, TRUE, FALSE, 0);
		gtk_widget_show(soe_label);

		/*
		 * create the Yes/No command buttons 
		 */
		gtk_signal_connect (GTK_OBJECT (soe_buttons[0]), "clicked",
			GTK_SIGNAL_FUNC (SOEAccept), (gpointer)NULL);
	
		gtk_signal_connect (GTK_OBJECT (soe_buttons[1]), "clicked",
			GTK_SIGNAL_FUNC (SOECancel), (gpointer)NULL);

		/*
		 * Display the window.
		 */
		gtk_widget_realize(soe_widget);
		XPCenterWindow(soe_widget, NULL, soe_widget);
		gtk_widget_show( soe_widget );

	}

}


/*========================================================================
 *	Name:			SOECancel and SOEAccept
 *					
 *	Description:
 *	Handle users response to save notes before exiting.
 * SOECancel - exit without saving notes
 * SOEAccept - saving notes and then exit
 *========================================================================*/
static void
SOECancel(
	GtkWidget	*widget,
	gpointer		data
)
{
	gtk_main_quit();
	exit(0);
}

static void
SOEAccept(
	GtkWidget	*widget,
	StickyNote	*pn
)
{
	XPSaveAllNotes(FALSE);
	gtk_main_quit();
	exit(0);
}


/*========================================================================
 *	Name:			XPCenterWindow
 *	Prototype:	XPCenterWindow(GtkWidget *child, GtkWidget *parent, 
 *							GtkWidget *shell)
 *
 *	Description:
 *	Position a dialog in the middle of a parent window or in the middle of the
 * display.
 *
 *	Input Arguments:
 *	GtkWidget	*child	window that needs positioning
 *	GtkWidget	*parent	window in which child is to be centered
 *	GtkWidget	*shell	window shell that receives positioning for child
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *	If parent is NULL, then the child is positioned in the middle of the 
 * display.  The child and shell arguments must be specified, but parent.
 * is optional.
 *		
 *========================================================================*/
void
XPCenterWindow(
	GtkWidget	*child,
	GtkWidget	*parent,
	GtkWidget	*shell
)
{
#ifdef DEBUG
	char	*fname="XPCenterWindow()";
#endif

	gint		xpos, ypos;
	gint		child_width, child_height;
	gint		parent_width, parent_height;
	gint		shellx, shelly;

	DBGEnter();

	/*
	 * Position the child within the parent.  Be certain
	 * the box will be visible if centering it will place it off the edge
	 * of the screen.
	 */
	if ( parent != NULL )
	{
		DBGPrintf(DBG_INFO,("parent is not NULL\n"));

		/*
		 * Get parents width/height and position on the screen.
		 */
      gdk_window_get_position(parent->window, &xpos, &ypos);
		GTK_WIDGET_GET_USIZE_WIDTH(parent, parent_width);
		GTK_WIDGET_GET_USIZE_HEIGHT(parent, parent_height);

		/*
		 * Get offsets for window manager border.
		 */
		XPGetNotePosition(parent, &shellx, &shelly);
		shellx = abs(shellx);
		shelly = abs(shelly);

		/*
		 * Get childs width/height.
		 */
		GTK_WIDGET_GET_USIZE_WIDTH(child, child_width);
		GTK_WIDGET_GET_USIZE_HEIGHT(child, child_height);

		/*
		 * Compute childs position from parents/screen.
		 */
		xpos = xpos + (parent_width/2) - (child_width/2);
		ypos = ypos + (parent_height/2) - (child_height/2);

		/*
		 * Rubber band it - make sure window doesn't go off edge
		 * of screen.
		 */
		if (xpos < 1)
			xpos = 1;
		if ( (xpos+child_width+2*shellx) >  gdk_screen_width() )
			xpos =  gdk_screen_width() - child_width - 10;

		if (ypos < 1)
			ypos = 1;
		if ( (ypos+child_height+shelly+shellx) > gdk_screen_height() )
			ypos =  gdk_screen_height() - child_height - shelly - shellx - 4;

	 	gtk_widget_set_uposition( shell, xpos, ypos );
	}
	else
	{
		DBGPrintf(DBG_INFO,("parent is NULL\n"));

		/*
		 * Get childs width/height.
		 */
		GTK_WIDGET_GET_USIZE_WIDTH(child, child_width);
		GTK_WIDGET_GET_USIZE_HEIGHT(child, child_height);

		xpos = (gdk_screen_width()/2) - (child_width/2);
		ypos = (gdk_screen_height()/2) - (child_height/2);

	 	gtk_widget_set_uposition( shell, xpos, ypos );
	}

}


/*========================================================================
 *	Name:		int XPNoop2()
 *					
 *	Description:
 * Noop routine for Gtk.
 *========================================================================*/
int
XPNoop2()
{
	return FALSE;
}


/*========================================================================
 *	Name:		int XPGTKShowDisabledCmds()
 *					
 *	Description:
 * Since timers stick around until manually removed in Gtk, we have to 
 * add a front end to the XPShowDisabledCmds() routine for Gtk so we can
 * disable the timer after it expires the first time.
 *========================================================================*/
int
XPGTKShowDisabledCmds()
{
	XPShowDisabledCmds();
	gtk_timeout_remove(disabled_cmds_timer);
	return TRUE;
}


/*========================================================================
 *	Name:		int XPFind(GtkWidget *widget, StickyNote *pn)
 *					
 *	Description:
 * Search for text in string.
 *
 *	Input Arguments:
 * int	direction			XP_FORWARD or XP_BACKWARD
 * int	case_sensitve		TRUE or FALSE
 * int	initial_offset		offset from start of text to start search
 * char	*search_string		string to search for
 * StickyNote	*pn			pointer to note with text.
 *
 *	Return Values:
 * Returns the offset into string where a match was found.
 *
 *	Notes:
 *========================================================================*/
int
XPFind(
	int			direction,
	int			case_sensitive,
	int			initial_offset,
	char			*search_string,
	StickyNote	*pn
)
{
	char		*string, *string_copy, *search_copy;
	char		*start, *end, *ptr;
	gboolean	found;


	string = gtk_editable_get_chars(GTK_EDITABLE(pn->pn_textwidget), 0, -1);

	/*
	 * Validate parameters.
	 */
	if ( strlen(string) < initial_offset )
		return (-1);

	/*
	 * Get a copy of the text in the note and a copy of the search string.
	 */
	string_copy = XPSafeAlloc(strlen(string)+1);
	strcpy(string_copy, string);

	search_copy = XPSafeAlloc(strlen(search_string)+1);
	strcpy(search_copy, search_string);

	/*
	 * Determine start and end points of search.
	 */
	if ( direction == XP_FORWARD )
	{
		start = (char *)(string_copy + initial_offset);
		end = (char *)(string_copy + strlen(string_copy) );
	}
	else
	{
		start = (char *)(string_copy + initial_offset );
		end = string_copy;
		strncpy((char *)(start+1), "\0", 1);
	}

	/*
	 * If the search will be case insensitive, convert it to lowercase
	 * first.
	 */
	if ( ! case_sensitive )
	{
		XPConvertCase(string_copy, XP_LOWER, 0);
		XPConvertCase(search_copy, XP_LOWER, 0);
	}

	/*
	 * Find the substring.
	 */
	found = FALSE;
	if ( direction == XP_FORWARD )
	{
		for (ptr=start; ptr < end; ptr++ )
		{
			if ( ptr[0] == search_copy[0] )
				if ( strncmp(search_copy, ptr, strlen(search_copy)) == 0 )
				{
					found = TRUE;
					break;
				}
		}
	}
	else
	{
		for (ptr=start; ptr > end; ptr-- )
		{
			if ( ptr[0] == search_copy[0] )
				if ( strncmp(search_copy, ptr, strlen(search_copy)) == 0 )
				{
					found = TRUE;
					break;
				}
		}
	}
	
	if ( found )
	{
		return((int)(ptr-string_copy));
	}
	else
		return (-1);
}

