/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: xnotes.c
 *            DESCRIPTION: Main module for XNotesPlus
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: main
 *      PRIVATE FUNCTIONS: ParseCmdLineOptions, InitConfigParameters,
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_C
#define MAIN_C

/* === System Headers === */
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <pwd.h>
#include <gtk/gtk.h>

/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"

/* === external routines === */

/* === Public routine prototypes === */

/* === Private routine prototypes === */

/* === Global Variables === */
extern AudioFileType		*audio_files;
AppRes						app_res;		/* configurable parameters */
int							disabled_cmds_timer;
GtkTooltips					*Tool_Tips;


/* === Static Variables === */
option app_defaults[] = {
	{"bs",			1,	0,	0},	/*	0  = blocksize */
	{"notedir",		1,	0,	0},	/*	1  = notes directory */
	{"interval",	1,	0,	0},	/*	2  = autosave timeout interval */
	{"sb",			0,	0,	0},	/*	3  = turn on scrollbars for all notes */
	{"sv",			0,	0,	0},	/*	4  = enables save on exit */
	{"nw",			1,	0,	0},	/*	5  = name width */
	{"ns",			0,	0,	0},	/*	6  = disables auto save feature */
	{"na",			0,	0,	0},	/*	7  = disable alarm feature */
	{"help",			0,	0,	0},	/*	8  = print help message */
	{"version",		0,	0,	0},	/*	9  = print version number */
	{"ao",			1,	0,	0},	/*	10 = cascade offset */
	{"tmpdir",		1,	0,	0},	/*	11 = directory to use for temporary files */
	{"printcmd",	1,	0,	0},	/*	12 = printed command in printf format */
	{"calendarcmd",1,	0,	0},	/*	13 = calendar command */
	{"emailcmd",	1,	0,	0},	/*	14 = comand to use to send email */
	{"homedir",		1,	0,	0},	/*	15 = default home directory */
	{"debugfile",	1,	0,	0},	/*	16 = debug output file */
	{"debuglevel",	1,	0,	0},	/*	17 = debug level */
	{"xpmdir",		1,	0,	0},	/*	18 = where to find bitmaps */
	{"defaults",	0,	0,	0},	/*	19 = show all defaults */
	{"audiocmd",	1,	0,	0},	/*	20 = how to play audio files */
	{"audiodir",	1,	0,	0},	/*	21 = audio files directory */
	{"noaudio",		0,	0,	0},	/*	22 = turn off alarm audio */
	{"addrdir",		0,	0,	0},	/*	23 = directory for address files */
	{"gtkrc",		1,	0,	0},	/*	24 = gtk rc file */
	{"pilotdir",	1,	0,	0},	/*	25 = directory for pilot spec. stuff */
	{"notips",		0,	0,	0},	/*	26 = turn off Tool Tips */
	{"nn",			0,	0,	0},	/*	27 = turn off dialog of disabled functions */
	{"emailaddr",	1,	0,	0},	/*	28 = default email address */
	{0,				0,	0,	0}		/* End of options */
	/*
	 * Other options that take single dash format:
	 *
	 * -c : compatibility mode (reads old notes)
	 * -? : print help message
	 * -h : print help message
	 */
};


/*========================================================================
 *	Name:			ParseCmdLineOptions
 *	Prototype:	ParseCmdLineOptions(int argc, char **argv)
 *					
 *	Description:
 *	Parse the command line for user specified options.  These take
 *	precedence over any values initialized by XNotesPlus.
 *
 *	Input Arguments:
 *	int	arc			number of arguments in argv
 *	char	**argv		command line arguments
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *	This routine requires getopt_long().
 *========================================================================*/
static void
ParseCmdLineOptions(
	int	argc,
	char	**argv
)
{
	char c;
	int option_index;

	/*
	 * Suppress error messages from getopt_long() for unrecognized 
	 * options.
	 */
	opterr = 0;

	/*
	 * Parse the command line.
	 */
	while ((c = getopt_long(argc, argv, "ch?",
		app_defaults, &option_index )) != EOF)
	{
		switch (c)
		{
			case 0:
				{
					switch(option_index)
					{
						case 0: /* Text buffer size */
							app_res.buf_size = atoi(optarg);
							break;
	
						case 1: /* Notes directory */
							if ( app_res.note_dir )
								free(app_res.note_dir);
							app_res.note_dir = XPSafeAlloc(strlen(optarg)+1);
							strcpy(app_res.note_dir, (char *)optarg);
							break;
	
						case 2: /* auto-save interval */
							app_res.interval = atoi(optarg);
							break;
	
						case 3: /* turn on scrollbars for all notes */
							app_res.scroll_bar = TRUE;
							break;
	
						case 4: /* save notes on exit */
							app_res.save_notes = TRUE;
							break;
	
						case 5: /* name width in pixels */
							app_res.name_width = atoi(optarg);
							break;
	
						case 6: /* Disable autosave feature */
							app_res.nosave = TRUE;
							break;
	
						case 7: /* Disable alarm */
							app_res.noalarm = TRUE;
							break;
	
						case 8: /* Display help message */
							printf(USAGE);
							exit(0);
							break;
	
						case 9: /* Display version information */
							printf( "xnotes+ v%s\n", VERSION );
							printf( "Copyright 1994-1998 Michael J. Hammel\n\n");
							exit(0);
							break;
	
						case 10: /* Cascade offset in pixels */
							app_res.anchor_offset = atoi(optarg);
							break;
	
						case 11: /* dir to use for temp files */
							if ( app_res.tmp_dir )
								free( app_res.tmp_dir );
							app_res.tmp_dir = XPSafeAlloc(strlen(optarg)+1);
							strcpy(app_res.tmp_dir, (char *)optarg);
							break;
	
						case 12: /* Print command */
							if ( app_res.print_cmd )
								free( app_res.print_cmd );
							app_res.print_cmd = XPSafeAlloc(strlen(optarg)+1);
							strcpy(app_res.print_cmd, (char *)optarg);
							break;
	
						case 13: /* Calendar command */
							if ( app_res.calendar_cmd )
								free( app_res.calendar_cmd );
							app_res.calendar_cmd = XPSafeAlloc(strlen(optarg)+1);
							strcpy(app_res.calendar_cmd, (char *)optarg);
							break;
	
						case 14: /* Email cmd */
							if ( app_res.email_cmd )
								free( app_res.email_cmd );
							app_res.email_cmd = XPSafeAlloc(strlen(optarg)+1);
							strcpy(app_res.email_cmd, (char *)optarg);
							break;
	
						case 15: /* Default Home directory of user */
							if ( app_res.home_dir )
								free( app_res.home_dir );
							app_res.home_dir = XPSafeAlloc(strlen(optarg)+1);
							strcpy(app_res.home_dir, (char *)optarg);
							break;
	
						case 16: /* Debug output file */
							if ( app_res.debugfile )
								free( app_res.debugfile );
							app_res.debugfile = XPSafeAlloc(strlen(optarg)+1);
							strcpy(app_res.debugfile, (char *)optarg);
							break;
	
						case 17: /* Debug level */
							app_res.debuglevel = DBGStringToInt(optarg);
							if ( app_res.debuglevel == -1 )
							{
								printf("XNotesPlus: Invalid debug level: %s\n", optarg);
								exit(1);
							}
							break;
	
						case 18: /* where to find bitmap files */
							if ( app_res.xpm_dir )
								free( app_res.xpm_dir );
							app_res.xpm_dir = XPSafeAlloc(strlen(optarg)+1);
							strcpy(app_res.xpm_dir, (char *)optarg);
							break;
	
						case 19: /* Show defaults */
							app_res.defaults = TRUE;
							break;
	
						case 20: /* Audio cmd */
							if ( app_res.audio_cmd )
								free( app_res.audio_cmd );
							app_res.audio_cmd = XPSafeAlloc(strlen(optarg)+1);
							strcpy(app_res.audio_cmd, (char *)optarg);
							break;
	
						case 21: /* Audio dir */
							if ( app_res.audio_dir )
								free( app_res.audio_dir );
							app_res.audio_dir = XPSafeAlloc(strlen(optarg)+1);
							strcpy(app_res.audio_dir, (char *)optarg);
							break;
	
						case 22: /* Turn off alarm audio */
							app_res.noaudio = TRUE;
							break;
	
						case 23: /* Addresses dir */
							if ( app_res.addr_dir )
								free( app_res.addr_dir );
							app_res.addr_dir = XPSafeAlloc(strlen(optarg)+1);
							strcpy(app_res.addr_dir, (char *)optarg);
							break;
	
						case 24: /* Gtk rc file */
							if ( app_res.gtk_rc )
								free( app_res.gtk_rc );
							app_res.gtk_rc = XPSafeAlloc(strlen(optarg)+1);
							strcpy(app_res.gtk_rc, (char *)optarg);
							break;
	
						case 25: /* Pilot directory */
							if ( app_res.pilot_dir )
								free( app_res.pilot_dir );
							app_res.pilot_dir = XPSafeAlloc(strlen(optarg)+1);
							strcpy(app_res.pilot_dir, (char *)optarg);
							break;
	
						case 26: /* Turn off alarm audio */
							app_res.no_tips = TRUE;
							break;
	
						case 27: /* Turn off dialog of disabled functions */
							app_res.nonotice = TRUE;
							break;
	
						case 28: /* Email address */
							if ( app_res.email_addr )
								free( app_res.email_addr );
							app_res.email_addr = XPSafeAlloc(strlen(optarg)+1);
							strcpy(app_res.email_addr, (char *)optarg);
							break;
	
					}
				}
				break;
			case 'c':
				app_res.compatibility = TRUE;
				break;
			case 'h':
			case '?':
			default:
				/* Print usage message */
				printf(USAGE);
				exit(0);
				break;
		}
	}
}



/*========================================================================
 *	Name:			InitConfigParameters
 *	Prototype:	void InitConfigParameters()
 *					
 *	Description:
 *	Initialize the configurable parameters structure.
 *
 *	Notes:
 *========================================================================*/

void
InitConfigParameters()
{
	char				*home;
	struct passwd	*pwd;

	/*
	 * Initialize configurable parameters.
	 */
	app_res.buf_size = DefaultBufSize;
	app_res.interval = DefaultInterval;
	app_res.scroll_ovf = 0;
	app_res.scroll_bar = FALSE;
	app_res.save_notes = FALSE;
	app_res.compatibility = FALSE;
	app_res.nosave = FALSE;
	app_res.noalarm = FALSE;
	app_res.help = FALSE;
	app_res.version = FALSE;
	app_res.name_width = DefaultNameWidth;
	app_res.anchor_offset = DefaultAnchorOffset;
	app_res.defaults = FALSE;
	app_res.noaudio = FALSE;
	app_res.no_tips = FALSE;
	app_res.nonotice = FALSE;


	app_res.note_dir = XPSafeAlloc(strlen(DefaultNoteDir)+1);
	strcpy(app_res.note_dir, (char *)DefaultNoteDir);

	app_res.tmp_dir = XPSafeAlloc(strlen(DefaultTmpDir)+1);
	strcpy(app_res.tmp_dir, (char *)DefaultTmpDir);

	app_res.print_cmd = XPSafeAlloc(strlen(DefaultPrintCmd)+1);
	strcpy(app_res.print_cmd, (char *)DefaultPrintCmd);

	app_res.calendar_cmd = XPSafeAlloc(strlen(DefaultCalendarCmd)+1);
	strcpy(app_res.calendar_cmd, (char *)DefaultCalendarCmd);

	app_res.email_cmd = XPSafeAlloc(strlen(DefaultEmailCmd)+1);
	strcpy(app_res.email_cmd, (char *)DefaultEmailCmd);

	app_res.email_addr = XPSafeAlloc(strlen(DefaultEmailAddr)+1);
	strcpy(app_res.email_addr, (char *)DefaultEmailAddr);

	app_res.audio_cmd = XPSafeAlloc(strlen(DefaultAudioCmd)+1);
	strcpy(app_res.audio_cmd, (char *)DefaultAudioCmd);

	home = getenv("HOME");
	if ( home != NULL )
	{
		app_res.home_dir = XPSafeAlloc(strlen(home)+1);
		strcpy(app_res.home_dir, (char *)home);
	}
	else
	{
		/*
		 * Find the user's home directory.
		 */
		if ((pwd = getpwuid(getuid())) == NULL) 
			app_res.home_dir = NULL;
		home = pwd->pw_dir;
		app_res.home_dir = XPSafeAlloc(strlen(home)+1);
		strcpy(app_res.home_dir, (char *)home);
	}


	app_res.xpm_dir = XPSafeAlloc(strlen(DefaultXPMDir)+1);
	strcpy(app_res.xpm_dir, DefaultXPMDir);

	app_res.audio_dir = XPSafeAlloc(strlen(DefaultAudioDir)+1);
	strcpy(app_res.audio_dir, DefaultAudioDir);

	app_res.addr_dir = XPSafeAlloc(strlen(DefaultAddrDir)+1);
	strcpy(app_res.addr_dir, DefaultAddrDir);

	app_res.gtk_rc = XPSafeAlloc(strlen(DefaultGtkRC)+1);
	strcpy(app_res.gtk_rc, DefaultGtkRC);

	app_res.pilot_dir = XPSafeAlloc(strlen(DefaultPilotDir)+1);
	strcpy(app_res.pilot_dir, DefaultPilotDir);

	app_res.debugfile = (char *)XPSafeAlloc(strlen("/dev/null")+1);
	strcpy(app_res.debugfile, "/dev/null");
	app_res.debuglevel = 0;

}


/*========================================================================
 *	Name:			main()
 *
 *	Description:
 *	Just get things rolling.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/

int 
main(
	int argc, 
	char *argv[]
)
{
#ifdef DEBUG
	char	*fname="main()";
#endif

	gboolean setsigs = FALSE;
	GtkWidget	*button;
	int			timer, timer_interval;
	int			alarm_timer, alarm_interval;


	/*
	 * Ignore signals for now, but record whether they were
	 * already ignored or not so we can catch them later if
	 * need be.
	 */
	if (	(signal(SIGQUIT, SIG_IGN) != SIG_IGN) &&
			(signal(SIGINT, SIG_IGN) != SIG_IGN) )
	{
		setsigs = TRUE;
	}

	/*
	 * Ignore child processes exits.
	 */
	signal(SIGCHLD, SIG_IGN);

	/*
	 * Initialize configurable parameters.
	 */
	InitConfigParameters();

	/*
	 * Parse the remainder of the command line arguments.
	 */
	ParseCmdLineOptions(argc, argv);

	/*
	 * More initialization:
	 * Read the project configuration file, if any.
	 */
	XPReadConfigFile();

	/*
	 * Fix any relative paths.
	 */
	XPFixUpPath(&app_res.xpm_dir);
	XPFixUpPath(&app_res.audio_dir);
	XPFixUpPath(&app_res.addr_dir);
	XPFixUpPath(&app_res.gtk_rc);
	XPFixUpPath(&app_res.pilot_dir);

	/*
	 * Set up the audio files for use with the notes.
	 */
	XPSetUpAudio();

	/*
	 * This is called in all GTK applications.  Arguments are parsed
	 * from the command line and are returned to the application.
	 */
	gtk_init (&argc, &argv);
	gtk_rc_parse (app_res.gtk_rc);

	/*
	 * Show the configured defaults, if requested.
	 */
	if ( app_res.defaults )
		XPShowDefaults();


#ifdef DEBUG

	/*
	 * set the debug file, if any
	 */
	if ( app_res.debugfile != NULL )
		DBGFile = app_res.debugfile;

	/*
	 * Set the debug level, if any.  The debug level must be set the
	 * same way it is in debug.h - using bit flags.
	 */
	GMDebugLevel = app_res.debuglevel;

#endif

	/*
	 * Debug output is to /dev/console by default if -DDEBUG was set at compile
	 * time, otherwise the DBGxxx macros have no effect.
	 */
	DBGOpen( DEBUG_FD );
	DBGEnter();


	/*
	 * Validate application resources that have been set by user.
	 */
	DBGPrintf(DBG_TRACE3, ("Validating Application Resources.\n"));
	XPValidateAppRes();

	/*
	 * Read in the projects list, if any.  We do this now so we can save
	 * debug info if necessary, plus the app_res.note_dir gets set by
	 * XPValidateApRes();
	 */
	XPSetUpProjects();

	/*
	 * Create the Plaid window.
	 */
	DBGPrintf(DBG_TRACE3, ("Creating plaid widget.\n"));
	button = XPCreatePlaidWidget();

	/*
	 * Add pop up menu to Plaid button.
	 */
	DBGPrintf(DBG_TRACE3, ("Adding the popup menu.\n"));
	XPCreateMainMenu(button);

	/*
	 * Establish Tool Tips.
	 */
	Tool_Tips = gtk_tooltips_new ();
	gtk_tooltips_enable(Tool_Tips);

	/*
	 * Load saved notes.
	 */
	DBGPrintf(DBG_TRACE3, ("Loading saved notes.\n"));
	XPLoadSavedNotes();

	/*
	 * Set the autosave timer.
	 */
	DBGPrintf(DBG_TRACE3, ("Starting timers.\n"));
	if ( !app_res.nosave )
	{
		timer_interval = app_res.interval * 1000;
		timer = gtk_timeout_add (timer_interval, (GtkFunction)XPAutoSave, NULL);
	}
	
	/*
	 * Set the alarm timer (expires once a minute). 
	 */
	if ( !app_res.noalarm )
	{
		alarm_interval = XP_ALARM_PERIOD * 1000;
		alarm_timer = gtk_timeout_add (alarm_interval, 
			(GtkFunction)XPAlarmCheck, NULL);
	}

	/*
	 * Set timer to notify user of disabled functions.
	 */
	if ( !app_res.nonotice )
	{
		disabled_cmds_timer = gtk_timeout_add (1500, 
			(GtkFunction)XPGTKShowDisabledCmds, NULL);
	}

	/*
	 * Sit and spin.
	 */
	DBGPrintf(DBG_TRACE3, ("sit and spin...\n"));
	gtk_main ();
	
	return 0;
}
