/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: bindings.c
 *            DESCRIPTION: handle key bindings for XNotesPlus
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPActionAlarm XPActionAnchor XPActionDestroy
 *                       : XPActionHide XPActionErase XPActionCal
 *                       : XPActionSave XPActionPrint XPActionImport
 *                       : XPActionEnvelope XPActionProject
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_BINDINGS_C

/* === System Headers === */
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "motif.h"

/* === Project Headers === */
#include "XNotesPlus.h"

/* === external routines === */

/* === Public routine prototypes === */

/* === Private routine prototypes === */
static StickyNote	*NoteFromWidget();

/* === Global Variables === */
Widget				Top_Level;
extern StickyNote	*notes;


/*========================================================================
 *	Name:			NoteFromWidget
 *	Prototype:	static StickyNote *NoteFromWidget()
 *					
 *	Description:
 *	Call Alarm window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
static StickyNote *
NoteFromWidget(
	Widget		widget
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	for ( pn = notes; pn!= NULL; pn=pn->pn_next )
	{
		if (pn->pn_shellwidget == widget)
			return(pn);
		if (pn->pn_textwidget == widget)
			return(pn);
	}
	return(NULL);
}


/*========================================================================
 *	Name:			XPActionAlarm
 *	Prototype:	void XPActionAlarm()
 *					
 *	Description:
 *	Call Alarm window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionAlarm(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Toggle alarm status for note.
	 */
	if ( pn->pn_alarm )
		XPAlarmOff(widget, (XtPointer)pn->pn_index, NULL);
	else
		XPCreateTimerPrompt(widget, (XtPointer)pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionAnchor
 *	Prototype:	void XPActionAnchor()
 *					
 *	Description:
 *	Call Anchor window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionAnchor(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Toggle Anchor status for note.
	 */
	if ( pn->pn_anchor )
		XPUnAnchorNote(widget, (XtPointer)pn->pn_index, NULL);
	else
		XPAnchorNote(widget, (XtPointer)pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionDestroy
 *	Prototype:	void XPActionDestroy()
 *					
 *	Description:
 *	Call Destroy window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionDestroy(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 *  Destroy current note.
	 */
	XPDestroyNote(widget, pn, NULL);
}


/*========================================================================
 *	Name:			XPActionCal
 *	Prototype:	void XPActionCal()
 *					
 *	Description:
 *	Call Calendar window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionCal(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Insert a calendar into the current note.
	 */
	XPInsertCalendar(widget, pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionEnvelope
 *	Prototype:	void XPActionEnvelope()
 *					
 *	Description:
 *	Call Envelope window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionEnvelope(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	/*
	 * Open Envelope dialog.
	 */
	XPEnvelope();
}


/*========================================================================
 *	Name:			XPActionProject
 *	Prototype:	void XPActionProject()
 *					
 *	Description:
 *	Call Project window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionProject(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	/*
	 * Open Envelope dialog.
	 */
	XPProject();
}


/*========================================================================
 *	Name:			XPActionHide
 *	Prototype:	void XPActionHide()
 *					
 *	Description:
 *	Call Hide window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionHide(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Hide note.
	 */
	XPHideNote(widget, pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionErase
 *	Prototype:	void XPActionErase()
 *					
 *	Description:
 *	Call Erase window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionErase(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Erase note contents.
	 */
	XPEraseNote(widget, pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionPrint
 *	Prototype:	void XPActionPrint()
 *					
 *	Description:
 *	Call Print window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionPrint(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Print note contents.
	 */
	XPPrintNote(widget, pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionOpen
 *	Prototype:	void XPActionOpen()
 *					
 *	Description:
 *	Call Open window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionOpen(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Open file into note contents.
	 */
	XPOpenNote(widget, pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionSave
 *	Prototype:	void XPActionSave()
 *					
 *	Description:
 *	Call Save window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionSave(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Save note to file.
	 */
	XPSaveNote(widget, pn->pn_index, NULL);
}

