/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: dialog.c
 *            DESCRIPTION: Generic Dialog handler routines
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPMsgWindow
 *      PRIVATE FUNCTIONS: CloseMsgWindow
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_DIALOG_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>

#include "motif.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void CloseMsgWindow();


/* === Global Variables === */
extern Widget	Top_Level;


/* === Static Variables === */
static Widget	messagebox=NULL;
static Pixmap	error_xbm;
static Pixmap	info_xbm;
static Pixmap	msg_xbm;



/*========================================================================
 *	Name:			XPMsgWindow
 *	Prototype:	XPMsgWindow(char *msg, int type, 
 *							Widget *return_widget, Widget parent,
 *							XtPointer ok_cb, XtPointer cancel_cb, XtPointer help_cb)
 *
 *	Description:
 *		This routine creates and pops up a generic message window.
 *		The content of the message is passed from the caller and the
 *		type of icon to use with the message is also defined by the
 *		caller.
 *
 *	Input Arguments:
 *		char 			*msg			Message to display
 *		int 			type			Defines which icon to use in window
 *										Valid values are:
 *										XP_ERROR_TYPE    - Error message
 *										XP_MSG_TYPE	     - Generic message
 *										XP_INFO_TYPE     - Informational message
 *										XP_MSG_NOCANCEL  - Don't use cancel button
 *										XP_MSG_NOHELP    - Don't use Help button
 *										XP_MSG_NOOK      - Don't use OK button
 *		Widget		parent		Parent widget, in the center of which the message
 *										window will be opened.
 *		XtPointer	ok_cb			callback to use for OK button
 *		XtPointer	cancel_cb	callback to use for Close button
 *		XtPointer	help_cb		currently unused
 *
 *	Output Arguments:
 *		Widget	*return_widget	if non-NULL, return dialog widget to caller here
 *
 *	Return Values:
 *		None
 *
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 * This routine does allow multiple dialogs to be opened.  If you 
 * call this routine you should save the dialog widget id and close
 * the dialog yourself.
 *		
 *========================================================================*/
void
XPMsgWindow(
	char			*msg,
	int			type,
	Widget		*return_widget,
	Widget		parent,
	XtPointer	ok_cb,
	XtPointer	ok_data,
	XtPointer	cancel_cb,
	XtPointer	help_cb
)
{
#ifdef DEBUG
	char			fname[]="XPMsgWindow()";
#endif

	Arg				args[5];
	int				nargs;
	XmString			xm_string;
	char				buf[256];
	char				title[256];
	Widget			vbox, accept_button, close_button;
	Widget			label, label_pixmap;
	int				shell_type;
	XpmAttributes	attributes;

	static			firsttime = True;


	DBGEnter();

	/*
	 * Create the dialog. We'll destroy it when it is closed.
	 */
	if (type & XP_ERROR_TYPE)
		(void) sprintf ( title, "%s", XP_ERROR_TITLE );
	else
	if (type & XP_MSG_TYPE)
		(void) sprintf ( title, "%s", XP_MSG_TITLE );
	else
	if (type & XP_INFO_TYPE)
		(void) sprintf ( title, "%s", XP_INFO_TITLE );
	else
		(void) sprintf ( title, "%s", XP_INFO_TITLE );

	if ( type & XP_MSG_NOCANCEL & XP_MSG_NOHELP & XP_MSG_NOOK )
	{
		shell_type = 0;
	}
	else
	{
		shell_type =  ACCEPT_BUTTON | CLOSE_BUTTON | HELP_BUTTON;

		if ( type & XP_MSG_NOCANCEL )
			shell_type = shell_type ^ CLOSE_BUTTON;
	
		if ( type & XP_MSG_NOHELP )
			shell_type = shell_type ^ HELP_BUTTON;
	
		if ( type & XP_MSG_NOOK )
			shell_type = shell_type ^ ACCEPT_BUTTON;
	}

	messagebox = XPCreateGenericShell(title, shell_type );

	sprintf(buf, "*GenericForm%d", (int)messagebox );
	vbox = XtNameToWidget(messagebox, buf);
	sprintf(buf, "*GenericCancelB%d", (int)messagebox);
	close_button = XtNameToWidget(messagebox, buf);
	sprintf(buf, "*GenericAcceptB%d", (int)messagebox);
	accept_button = XtNameToWidget(messagebox, buf);

	/*
	 * Set the message boxes pixmap.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelType, XmPIXMAP ); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM); nargs++;
	label_pixmap = XmCreateLabel(
				vbox,
				"dialogLabel",
				args, nargs);
	XtManageChild(label_pixmap);

	/*
	 * Turn the message into a multiline string, if necessary.
	 */
	XPMultiLineText(msg, &xm_string);

	/*
	 * Set the message boxes text string.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string ); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNleftWidget, label_pixmap); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM); nargs++;
	label = XmCreateLabel(
				vbox,
				"dialogLabel",
				args, nargs);
	XtManageChild(label);
	XmStringFree(xm_string);

	/*
	 * First time through create the pixmaps - we need to do this after the
	 * label has been created so we can use it as the color to map the
	 * background colors to.
	 */
	if ( firsttime )
	{
   	XPCreatePixmap(&msg_xbm, NULL, XP_MSG_XPM, 
			&attributes, label);
   	XPCreatePixmap(&info_xbm, NULL, XP_INFO_XPM, 
			&attributes, label);
   	XPCreatePixmap(&error_xbm, NULL, XP_ERROR_XPM, 
			&attributes, label);
		firsttime = False;
	}

	/*
	 * Setup the buttons in the window based on the type of message
	 * window requested.
	 */
	if ( type & XP_MSG_TYPE )
	{
		nargs=0;
		XtSetArg ( args[nargs], XmNlabelPixmap, msg_xbm ); nargs++;
		XtSetValues( label_pixmap, args, nargs );
	}

	if ( type & XP_INFO_TYPE )
	{
		nargs=0;
		XtSetArg ( args[nargs], XmNlabelPixmap, info_xbm ); nargs++;
		XtSetValues( label_pixmap, args, nargs );
	}

	if ( type & XP_ERROR_TYPE )
	{
		nargs=0;
		XtSetArg ( args[nargs], XmNlabelPixmap, error_xbm ); nargs++;
		XtSetValues( label_pixmap, args, nargs );
	}

	/*
	 * Set the callbacks for the buttons.
	 */
	if ( ! (type & XP_MSG_NOCANCEL) )
	{
		if ( cancel_cb != NULL )
		{
			XtAddCallback ( close_button, XmNactivateCallback,
				(XtCallbackProc) cancel_cb, messagebox);
		}
		XtAddCallback ( close_button, XmNactivateCallback,
			(XtCallbackProc) CloseMsgWindow, messagebox);
	}
	if ( ! (type & XP_MSG_NOOK) )
	{
		if ( ok_cb != NULL )
		{
			DBGPrintf(DBG_INFO,("ok_data: %d\n", ok_data));
			XtAddCallback ( accept_button, XmNactivateCallback,
				(XtCallbackProc) ok_cb, ok_data);
		}
		XtAddCallback ( accept_button, XmNactivateCallback,
			(XtCallbackProc) CloseMsgWindow, messagebox);
	}
 

	/*
	 * Manage the message box.
	 */
	XtRealizeWidget(messagebox);
	XPCenterWindow(messagebox, parent, messagebox);

	/*
	 * Pop up the dialog shell.
	 */
	XtPopup(messagebox, XtGrabNone);

	/*
	 * Pass back the messagebox widgets id, if requested, so the caller
	 * can pop down the dialog.
	 */
	if ( return_widget != NULL )
		*return_widget = messagebox;
	
	DBGExit();
}


/*========================================================================
 *	Name:			CloseMsgWindow
 *	Prototype:	CloseMsgWindow()
 *
 *	Description:
 *		Default callback for the OK and Close buttons that closes the
 *		dialog window.  This gets called even if the user specifies
 *		there own callback for the buttons, so they don't have to worry
 *		about closing the window.
 *
 *	Notes:
 *		
 *========================================================================*/
static void
CloseMsgWindow(
	Widget		widget,
	Widget		shell,
	XtPointer	call_data
)
{
	if ( shell != NULL )
		XtDestroyWidget(shell);
}


