/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: email.c
 *            DESCRIPTION: email routines for XNotesPlus notes
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPCreateEmailNotePrompt
 *      PRIVATE FUNCTIONS: EmailPopDown, EmailNote
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES:
 *	
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_EMAIL_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <string.h>
#include <time.h>
#include <sys/param.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>


#include "motif.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void EmailPopDown();
static void EmailNote();


/* === Global Variables === */
extern AppRes		app_res;
extern StickyNote	*notes;
extern Widget		Top_Level;


/* === Static Variables === */
static Widget		email_widget=NULL;
static Widget		email_text;



/*========================================================================
 *	Name:			XPCreateEmailNotePrompt
 *	Prototype:	XPCreateEmailNotePrompt(Widget widget, int index, 
 *						XtPointer call_data)
 *					
 *	Description:
 * Create a window in which the email address can be entered.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPCreateEmailNotePrompt(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	StickyNote		*pn;
	Widget			email_label, email_cancel, email_accept;
	Widget			vbox;
	Arg				args[5];
	int				nargs;
	XmString			xm_string;
	char				buf[128];
	Dimension		width, height;

	/*
	 * Only one copy at at time!
	 */
	if ( email_widget != NULL )
		EmailPopDown();

	/*
	 * Find the note we're opening.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Create a popup shell widget
	 */
	email_widget = XPCreateGenericShell(NotesEmailPrompt, 
					ACCEPT_BUTTON | CANCEL_BUTTON );

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	sprintf(buf, "*GenericForm%d", (int)email_widget );
	vbox = XtNameToWidget(email_widget, buf);
	sprintf(buf, "*GenericCancelB%d", (int)email_widget);
	email_cancel = XtNameToWidget(email_widget, buf);
	sprintf(buf, "*GenericAcceptB%d", (int)email_widget);
	email_accept = XtNameToWidget(email_widget, buf);


	/* the label window */
	xm_string = XmStringCreateLtoR(XP_S_EMAIL, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
	email_label=XmCreateLabel(
			vbox,
			"label",
			args, nargs);
	XmStringFree(xm_string);
	XtManageChild(email_label);

	/*
	 * The text entry area for the email address.
	 */
	nargs=0;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, email_label); nargs++;
	email_text = XmCreateTextField(
							vbox,
							"textField",
							args, nargs
							);
	XtManageChild(email_text);
	if ( app_res.email_addr != NULL )
		XmTextFieldSetString(email_text, app_res.email_addr);

	/* 
	 * The Accept and Cancel button callbacks.
	 */
	XtAddCallback( email_accept, XmNactivateCallback, 
			EmailNote, (XtPointer)pn);
	XtAddCallback( email_cancel, XmNactivateCallback, 
			EmailPopDown, (XtPointer)NULL);

	/*
	 * Display the window.
	 */
	XtRealizeWidget(email_widget);
	XPCenterWindow(email_widget, pn->pn_shellwidget, email_widget);
	XtPopup( email_widget, XtGrabNone );

	/*
	 * This bit of code forces the pop up window to stay its
	 * initial size, despite the users attempts to resize it.
	 * By doing this, I avoid having to muck extensively with
	 * aligning objects after a resize.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, &width); nargs++;
	XtSetArg ( args[nargs], XmNheight, &height); nargs++;
	XtGetValues ( email_widget, args, nargs);
	nargs = 0;
	XtSetArg ( args[nargs], XmNmaxWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNmaxHeight, height); nargs++;
	XtSetArg ( args[nargs], XmNminWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNminHeight, height); nargs++;
	XtSetValues ( email_widget, args, nargs); 

}


/*========================================================================
 *	Name:			EmailPopDown
 *	Prototype:	EmailPopDown(Widget widget, XtPointer client_data, 
 *							XtPointer call_data)
 *					
 *	Description:
 * Close the Email window
 *========================================================================*/
static void
EmailPopDown(
	Widget		widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
	if ( email_widget != NULL )
	{
		XtDestroyWidget(email_widget);
		email_widget = NULL;
	}
}



/*========================================================================
 *	Name:			EmailNote
 *	Prototype:	EmailNote(Widget widget, StickyNote *pn, XtPointer call_data)
 *					
 *	Description:
 * Email a notes text to the specified recipient.
 *========================================================================*/
static void
EmailNote(
	Widget		widget,
	StickyNote	*pn,
	XtPointer	call_data
)
{
	FILE			*fp;
	int			len;
	char			tmpfile[512];
	char			syscmd[1024];
	char			emailcmd[1024];
	char			*addressee;

	/*
	 * Don't email blank notes.
	 */
	if ( pn->pn_text != NULL )
		free(pn->pn_text);
	pn->pn_text = XmTextGetString(pn->pn_textwidget);
	if ( strlen(pn->pn_text) == 0 )
	{
		XPErrPopUp("No text in note to email!");
		return;
	}

	/*
	 * Save the text to a temporary file
	 */
	sprintf(tmpfile,"%s/xp%d", app_res.tmp_dir, (int)getpid());
	if ((fp = fopen(tmpfile, "w")) == NULL) 
	{
		fprintf(stderr, "xnotes: ");
		perror(tmpfile);
		sprintf(syscmd, XP_S_TEMP_OPEN_FAIL, tmpfile);
		XPErrPopUp(syscmd);
		return;
	}

	if ( pn->pn_text != NULL )
		free(pn->pn_text);
	pn->pn_text = XmTextGetString(pn->pn_textwidget);
	len = strlen(pn->pn_text);
	fwrite(pn->pn_text, sizeof(char), len, fp);
	fclose(fp);

	/*
	 * Use the emailcmd resource to determine how to email the note
	 */
	addressee = XmTextFieldGetString(email_text);
	sprintf (emailcmd, app_res.email_cmd, pn->pn_name, addressee);
	sprintf (syscmd, "cat %s | %s", tmpfile, emailcmd );
	system ( syscmd );

	/*
	 * clean up the temporary file
	 */
	sprintf( syscmd, "rm %s", tmpfile );
	system ( syscmd );

	EmailPopDown(NULL, NULL, NULL);
}
