/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: envelope.c
 *            DESCRIPTION: routines to allow printing envelopes
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPEnvelope
 *      PRIVATE FUNCTIONS: CloseEnvelope PTSizeCB
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *	
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_ENVELOPE_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <string.h>
#include <time.h>
#include <sys/param.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include "motif.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void	CloseEnvelope();
static void	PrintEnvelope();
static void	PTSizeCB();
static void	GetAddrList();
static void	UpdateToText();
static void	CloseList();
static void	SaveFromId();


/* === Global Variables === */
extern AppRes		app_res;
extern StickyNote	*notes;
extern Widget		Top_Level;

extern PilotAddr_T	*Pilot_Address;


/* === Static Variables === */
static Widget	envelope_dialog;
static int		pt_size_index;
static Widget	addr_to, addr_from;
static Widget	printer_name, env_type_6, env_type_10;
static Widget	addr_dialog;
/*
static Widget	bold_button;
*/

static char *env_strings[]={
	"5",
	"10",
	"12",
	"14",
	"16",
	0
	};



/*========================================================================
 *	Name:			XPEnvelope
 *	Prototype:	XPEnvelope()
 *					
 *	Description:
 * Create dialog to allow inputting envelope details.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPEnvelope()
{
#ifdef DEBUG
	char		*fname="XPEnvelope()";
#endif

	Widget	frame_to, frame_from;
	Widget	/* save_to, */ save_from, load_to;
	Widget	vbox1, vbox2, vbox3, vbox4;
	Widget	hbox, label;
	Widget	radio_box;
	Widget	print_button, close_button;
	Widget	menu;
	Widget	envelope_xpm;
	char		buf[1024];
	Arg		args[10];
	int		nargs;
	XmString	xm_string;
	XmString	env_xm_string[15];

	FILE			*fd;
	struct stat	stat_buf;

	Dimension	width, height;
	int			index;

	XpmAttributes	attributes;
	static Pixmap	envelope_xbm;
	static int		firsttime = True;

	/*
	 * Allow only one window at a time.
	 */
	if ( envelope_dialog != NULL )
		return;

	/*
	 * The dialog parent window.
	 */
	envelope_dialog = XPCreateGenericShell(NotesAddrPrompt, 
					ACCEPT_BUTTON | CANCEL_BUTTON );

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	sprintf(buf, "*GenericForm%d", (int)envelope_dialog );
	hbox = XtNameToWidget(envelope_dialog, buf);
	sprintf(buf, "*GenericCancelB%d", (int)envelope_dialog);
	close_button = XtNameToWidget(envelope_dialog, buf);
	sprintf(buf, "*GenericAcceptB%d", (int)envelope_dialog);
	print_button = XtNameToWidget(envelope_dialog, buf);

	/*
	 * A pair of vertical boxes.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNborderWidth, 0 ); nargs++;
	XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM ); nargs++;
	vbox1 = XmCreateForm(
				hbox,
				"form",
				args, nargs);
	XtManageChild(vbox1);
	nargs=0;
	XtSetArg ( args[nargs], XmNborderWidth, 0 ); nargs++;
	XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM ); nargs++;
	vbox2 = XmCreateForm(
				hbox,
				"form",
				args, nargs);
	XtManageChild(vbox2);
	nargs=0;
	XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_WIDGET ); nargs++;
	XtSetArg ( args[nargs], XmNrightWidget, vbox2 ); nargs++;
	XtSetValues( vbox1, args, nargs );

	/*
	 * Stuff two frames inside the left vbox, one inside the right box.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN ); nargs++;
	XtSetArg ( args[nargs], XmNshadowThickness, 4 ); nargs++;
	frame_from = XmCreateFrame(
						vbox1,
						"frame",
						args, nargs);
	XtManageChild(frame_from);
	xm_string = XmStringCreateLtoR(XP_S_FROM_ADDR, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetArg ( args[nargs], XmNchildType, XmFRAME_TITLE_CHILD); nargs++;
	label= XmCreateLabel(
					frame_from,
					"frame",
					args, nargs);
	XmStringFree(xm_string);
	XtManageChild(label);
	nargs=0;
	XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET ); nargs++;
	XtSetArg ( args[nargs], XmNtopWidget, frame_from ); nargs++;
	XtSetArg ( args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN ); nargs++;
	XtSetArg ( args[nargs], XmNshadowThickness, 4 ); nargs++;
	frame_to = XmCreateFrame(
						vbox1,
						"frame",
						args, nargs);
	XtManageChild(frame_to);
	xm_string = XmStringCreateLtoR(XP_S_TO_ADDR, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetArg ( args[nargs], XmNchildType, XmFRAME_TITLE_CHILD); nargs++;
	label= XmCreateLabel(
					frame_to,
					"frame",
					args, nargs);
	XmStringFree(xm_string);
	XtManageChild(label);

	/*
	 * A text field and one button inside the From: frame.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNborderWidth, 0 ); nargs++;
	vbox3 = XmCreateForm(
				frame_from,
				"form",
				args, nargs);
	XtManageChild(vbox3);

	nargs=0;
	XtSetArg ( args[nargs], XmNeditMode, XmMULTI_LINE_EDIT); nargs++;
	XtSetArg ( args[nargs], XmNeditable, True); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrows, 4); nargs++;
	addr_from = XmCreateScrolledText(
				vbox3,
				"addresses",
				args, nargs);
	XtManageChild(addr_from);
	sprintf(buf,"%s/%s", app_res.note_dir, DefaultIdFile);
	if ( stat(buf, &stat_buf) == 0 )
	{
		DBGPrintf(DBG_INFO,("file size: %d", stat_buf.st_size));

		fd = fopen(buf, "r");
		fread(buf, stat_buf.st_size, 1, fd);
		fclose(fd);
		bcopy("\0", (char *)(buf+stat_buf.st_size), 1);

		DBGPrintf(DBG_INFO,("From address: \n%s\n", buf));

		XmTextSetString(addr_from, buf);
	}

	xm_string = XmStringCreateSimple(XP_S_SAVE);
	nargs=0;
	XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, addr_from); nargs++;
	save_from = XmCreatePushButton(
						vbox3,
						"button",
						args, nargs);
	XtManageChild(save_from);
	XmStringFree(xm_string);
	XtAddCallback( save_from, XmNactivateCallback, 
			SaveFromId, (XtPointer)NULL);

	/* Add a tool tip */
	XtAddEventHandler(save_from,
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_ENV_SAVE_FROM);

	/*
	 * A text field and two buttons inside the To: frame.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNborderWidth, 0 ); nargs++;
	vbox4 = XmCreateForm(
				frame_to,
				"form",
				args, nargs);
	XtManageChild(vbox4);

	nargs=0;
	XtSetArg ( args[nargs], XmNeditMode, XmMULTI_LINE_EDIT); nargs++;
	XtSetArg ( args[nargs], XmNeditable, True); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrows, 4); nargs++;
	addr_to = XmCreateScrolledText(
				vbox4,
				"addresses",
				args, nargs);
	XtManageChild(addr_to);

	xm_string = XmStringCreateSimple(XP_S_LOAD);
	nargs=0;
	XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, addr_to); nargs++;
	load_to = XmCreatePushButton(
						vbox4,
						"button",
						args, nargs);
	XtManageChild(load_to);
	XtAddCallback( load_to, XmNactivateCallback, 
			GetAddrList, (XtPointer)NULL);

	/* Add a tool tip */
	XtAddEventHandler(load_to,
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_ENV_LOAD_TO);

/*
	xm_string = XmStringCreateSimple(XP_S_SAVE);
	nargs=0;
	XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNrightWidget, load_to); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, addr_to); nargs++;
	save_to = XmCreatePushButton(
						vbox4,
						"button",
						args, nargs);
	XtManageChild(save_to);
*/

	/*
	 * A vertical box inside the right side frame.
	nargs=0;
	XtSetArg ( args[nargs], XmNborderWidth, 0 ); nargs++;
	vbox5 = XmCreateForm(
				frame_other,
				"form",
				args, nargs);
	XtManageChild(vbox5);
	 */

	/*
	 * ...stuffed with:
	 * 1. A pt. size option menu.
	 * 2. A set of envelope type toggle buttons
	 * 3. A printer name field.
	 */

	/*
	 * The options menu (with label and cascade button) is created
	 * here.
	 */
	index=0;
	while ( env_strings[index] != NULL )
	{
		env_xm_string[index] = XmStringCreateSimple(env_strings[index]);
		index++;
	}

	xm_string = XmStringCreateSimple(XP_S_PT_SIZE);
	nargs = 0;
	XtSetArg ( args[nargs], XmNbuttons, env_xm_string); nargs++;
	XtSetArg ( args[nargs], XmNoptionLabel, xm_string); nargs++;
	XtSetArg ( args[nargs], XmNbuttonSet, 1); nargs++;
	XtSetArg ( args[nargs], XmNbuttonCount, index); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNsimpleCallback, PTSizeCB); nargs++;
	menu = XmCreateSimpleOptionMenu(
				vbox2,
				"option_menus",
				args, nargs);
	XmStringFree(xm_string);
	XtManageChild(menu);
	pt_size_index=1;

	index=0;
	while ( env_strings[index] != NULL )
	{
		XmStringFree(env_xm_string[index]);
		index++;
	}

	/*
	 * Label for "Envelope Type".
	 */
	xm_string = XmStringCreateLtoR(XP_S_ENV_TYPE, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, menu); nargs++;
	label= XmCreateLabel(
					vbox2,
					"label",
					args, nargs);
	XmStringFree(xm_string);
	XtManageChild(label);

	/*
	 * Radio buttons for the envelopes types.
	 */
	nargs=0;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, label); nargs++;
	XtSetArg ( args[nargs], XmNorientation, XmHORIZONTAL); nargs++;
	radio_box = XmCreateRadioBox(
					vbox2,
					"label",
					args, nargs);
	xm_string = XmStringCreateLtoR("#6", XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	env_type_6 = XmCreateToggleButton(
						radio_box,
						"toggleButton",
						args, nargs);
	XtManageChild(env_type_6);
	XmStringFree(xm_string);

	xm_string = XmStringCreateLtoR("#10", XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	env_type_10 = XmCreateToggleButton(
						radio_box,
						"toggleButton",
						args, nargs);
	XtManageChild(env_type_10);
	XmStringFree(xm_string);
	XtManageChild(radio_box);

	/*
	 * Make the #6 envelope the default type.
	 */
	XmToggleButtonSetState(env_type_6, True, False);


	xm_string = XmStringCreateLtoR(XP_S_PRINTER_NAME, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, radio_box); nargs++;
	label= XmCreateLabel(
					vbox2,
					"label",
					args, nargs);
	XmStringFree(xm_string);
	XtManageChild(label);

	nargs=0;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, label); nargs++;
	printer_name = XmCreateTextField(
							vbox2,
							"textField",
							args, nargs
							);
	XtManageChild(printer_name);

	/*
	 * Bold option. - this isn't supported by nenscript like the man
	 * pages suggests it is.
	xm_string = XmStringCreateLtoR("Bold", XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, printer_name); nargs++;
	bold_button = XmCreateToggleButton(
						vbox2,
						"toggleButton",
						args, nargs);
	XtManageChild(bold_button);
	XmStringFree(xm_string);
	 */

	/*
	 * A pixmap, just to fill some space and make it look clever.
	 */
	if ( firsttime )
	{
   	XPCreatePixmap(&envelope_xbm, NULL, XP_ENVELOPE_XPM, 
			&attributes, vbox2);
		firsttime = False;
	}
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelType, XmPIXMAP); nargs++;
	XtSetArg ( args[nargs], XmNlabelPixmap, envelope_xbm); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, printer_name); nargs++;
   XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM); nargs++;
	envelope_xpm = XmCreateLabel(
				vbox2,
				"pixmap",
				args, nargs
				);
	XtManageChild(envelope_xpm);

	/*
	 * Set the callbacks for the various buttons.
	 */
	XtAddCallback( close_button, XmNactivateCallback, 
			CloseEnvelope, (XtPointer)NULL);
	XtAddCallback( print_button, XmNactivateCallback, 
			PrintEnvelope, (XtPointer)NULL);

	/*
	 * Resize window to make room for the text windows.
	 */
	XtRealizeWidget(envelope_dialog);
	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, &width); nargs++;
	XtSetArg ( args[nargs], XmNheight, &height); nargs++;
	XtGetValues ( envelope_dialog, args, nargs);

	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, width+80); nargs++;
	XtSetValues ( envelope_dialog, args, nargs);

	/*
	 * Display the dialog.
	 */
	XtPopup(envelope_dialog, XtGrabNone);
	XPCenterWindow(envelope_dialog, NULL, envelope_dialog);

	/*
	 * This bit of code forces the pop up window to stay its
	 * initial size, despite the users attempts to resize it.
	 * By doing this, I avoid having to muck extensively with
	 * aligning objects after a resize.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, &width); nargs++;
	XtSetArg ( args[nargs], XmNheight, &height); nargs++;
	XtGetValues ( envelope_dialog, args, nargs);
	nargs = 0;
	XtSetArg ( args[nargs], XmNmaxWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNmaxHeight, height); nargs++;
	XtSetArg ( args[nargs], XmNminWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNminHeight, height); nargs++;
	XtSetValues ( envelope_dialog, args, nargs); 

}


/*========================================================================
 *	Name:			CloseEnvelope
 *	Prototype:	static void CloseEnvelope()
 *					
 *	Description:
 * Close the envelopes dialog.
 *
 *	Notes:
 *========================================================================*/
static void
CloseEnvelope()
{
	if ( envelope_dialog != NULL )
	{
		XtDestroyWidget(envelope_dialog);
		envelope_dialog = NULL;
	}
	CloseList();
}


/*========================================================================
 *	Name:			PTSizeCB
 *	Prototype:	static void PTSizeCB()
 *					
 *	Description:
 * Keep track of which point size has been set.
 *
 *	Notes:
 *========================================================================*/
static void
PTSizeCB(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	pt_size_index = index;
}


/*========================================================================
 *	Name:			PrintEnvelope
 *	Prototype:	static void PrintEnvelope()
 *					
 *	Description:
 * Grab the configuration information from the dialog, then build
 * a nenscript command.  Write address info to a temporary file.
 * Issue the command and close the dialog.
 *
 *	Notes:
 *		nenscript command:
 *			For #6 envelopes:
 *				nenscript -fCourierxx -B -L21 -Pprinter tempfile
 *			For #10 envelopes:
 *				nenscript -fCourierxx -2 -B -L21 -Pprinter tempfile
 *			where
 *			xx			Pt size from pt_size_index global variable
 *			printer	from printer_name text field
 *			-2			two column output (CTRL-L forces start of 2nd column)
 *			-B			turns off page titles
 *			-L			number of lines to print, max.
 *========================================================================*/
static void
PrintEnvelope()
{
#ifdef DEBUG
	char	*fname="PrintEnvelope()";
#endif

	char				*printer, *string, *ptr;
	char				cmd[512];
	char				bold[512];
	char				tmpfile[MAXPATHLEN];
	struct stat		stat_buf;
	FILE				*fp;
	int				rc;

	DBGEnter();

	/*
	 * Grab printer name to use.
	 */
	printer = XmTextFieldGetString(printer_name);
	if ( strlen(printer) == 0 )
	{
		sprintf(cmd, XP_S_PRT_MISSING);
		XPMsgWindow(cmd, XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK,
			NULL, NULL, NULL, NULL, NULL, NULL);
		return;
	}

	/*
	 * Make sure address info has been supplied.
	 */
	if ( strlen(XmTextGetString(addr_from)) == 0 )
	{
		sprintf(cmd, XP_S_PRT_FROM);
		XPMsgWindow(cmd, XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK,
			NULL, NULL, NULL, NULL, NULL, NULL);
		return;
	}
	if ( strlen(XmTextGetString(addr_to)) == 0 )
	{
		sprintf(cmd, XP_S_PRT_TO);
		XPMsgWindow(cmd, XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK,
			NULL, NULL, NULL, NULL, NULL, NULL);
		return;
	}

	/*
	 * Get Bold button status.
	if ( XmToggleButtonGetState(bold_button) == True )
		sprintf(bold,"-Bold");
	else
	 */
		bzero(bold,10);

	/*
	 * Build a temporary file name.
	 */
	sprintf(tmpfile,"%s/xpe.%d", app_res.note_dir, (int)getpid());

	/*
	 * Open the temp file.
	 */
	if ((fp = fopen(tmpfile, "w")) == NULL)
	{
		fprintf(stderr, "xnotes+: ");
		perror(tmpfile);
		sprintf(cmd,
			"Couldn't open tempfile (%s)\nfor printing envelope.\n", tmpfile);
		XPErrPopUp(cmd);
		return;
	} 

	/*
	 * Now we have to determine which envelope type we're using.
	 */
	if ( XmToggleButtonGetState(env_type_6) == True )
	{
		/*
	 	 * Write the envelope headers to the temp file.
		 */
		fprintf(fp, "%s\n", XmTextGetString(addr_from));
		fprintf(fp, "\n\n\n");

		string = XmTextGetString(addr_to);
		ptr = strtok(string, "\n");
		while ( ptr != NULL )
		{
			fprintf(fp, "\t\t\t%s\n", ptr);
			ptr = strtok(NULL, "\n");
		}
		fclose(fp);


		/*
		 * Build the command string.
		 */
		sprintf(cmd, 
			"nenscript -fCourier%s%s -B -L21 -P%s %s",
			bold, env_strings[pt_size_index], printer, tmpfile);

	}
	else
	{
		/*
	 	 * Write the envelope headers to the temp file.
		 */
		fprintf(fp, "%s\n", XmTextGetString(addr_from));
		fprintf(fp, "\n");
		fprintf(fp, "\n\n\n\n\n");
		fprintf(fp, "%s\n", XmTextGetString(addr_to));
		fclose(fp);


		/*
		 * Build the command string.
		 */
		sprintf(cmd, 
			"nenscript -fCourier%s%s -2 -B -L21 -P%s %s",
			bold, env_strings[pt_size_index], printer, tmpfile);

	}

	/*
	 * Issue the command.
	 */
#ifdef DEBUG
	DBGPrintf(DBG_INFO,("Command used to print the envelope: %s\n", cmd));
	rc = system ( cmd );
	switch ( rc )
	{
		case 127:
			sprintf(cmd,"%s failed: errno=%d", cmd, errno);
			XPErrPopUp(cmd);
			break;
			
		case -1:
			sprintf(cmd,"%s failed: errno=%d", cmd, rc);
			XPErrPopUp(cmd);
			break;
			
		default:
			sprintf(cmd,"%s succeeded", cmd);
			XPMsgWindow(
				cmd, XP_INFO_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK, NULL, 
				NULL, NULL, NULL, NULL, NULL);
			break;
	}
#else
	system ( cmd );
#endif


	/*
 	 * Clean up the temporary file.
	 */
	if ( stat(tmpfile, &stat_buf) == 0 )
	{
		if ( unlink(tmpfile) != 0 )
		{
			sprintf(cmd,"Problems doing cleanup after printing envelope.\n"
				"%s could not be unlinked.\n", tmpfile);
			XPErrPopUp(cmd);
		}
	}
}


/*========================================================================
 *	Name:			GetAddrList
 *	Prototype:	static void GetAddrList()
 *					
 *	Description:
 * Retrieves the addresses currently available from the address database(s)
 * and presents them to the user to select one.
 *
 *	Notes:
 *========================================================================*/
static void
GetAddrList()
{
	Widget		vbox, close_button; 
	Widget		addr_list;
	PilotAddr_T	*current_link;
	XmString		xm_string;
	char			buf[512];
	Arg			args[10];
	int			nargs;
	Dimension	width, height;
	
	sprintf(buf, "%s/%s", app_res.pilot_dir, DefaultPilotDB);
	if ( XPPilotParseAddr(buf) != XP_E_SUCCESS )
	{
		XPErrPopUp(XP_S_LOCDB_FAIL);
		return;
	}

	/*
	 * Allow only one window at a time.
	 */
	if ( addr_dialog != NULL )
		return;

	/*
	 * The addr dialog parent window.
	 */
	addr_dialog = XPCreateGenericShell(AddrListPrompt, CANCEL_BUTTON );

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	sprintf(buf, "*GenericForm%d", (int)addr_dialog );
	vbox = XtNameToWidget(addr_dialog, buf);
	sprintf(buf, "*GenericCancelB%d", (int)addr_dialog);
	close_button = XtNameToWidget(addr_dialog, buf);

	/*
	 * A scrolled list of address.
	 */
	nargs=0;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNvisibleItemCount, 12); nargs++;
   XtSetArg ( args[nargs], XmNselectionPolicy, XmSINGLE_SELECT); nargs++;
	addr_list = XmCreateScrolledList(
				vbox,
				"find_list",
				args,
				nargs);
	XtManageChild(addr_list);

	/*
	 * Populate the list.
	 */
	current_link = Pilot_Address;
	while (current_link != NULL )
	{
		if ( ( current_link->last != NULL ) && ( current_link->first != NULL) )
		{
			if (  ( strlen(current_link->last) != 0 ) &&
					( strlen(current_link->first) != 0 ) )
				sprintf(buf, "%s %s", current_link->first, current_link->last);
			else
			if (  ( strlen(current_link->first) == 0 ) &&
					( strlen(current_link->last) != 0 ) )
				sprintf(buf, "%s", current_link->last);
			else
				sprintf(buf, "%s", current_link->first);
			}
		else
		if ( current_link->last != NULL )
			sprintf(buf, "%s", current_link->last);
		else
		if ( current_link->first != NULL )
			sprintf(buf, "%s", current_link->last);
		else
		if ( current_link->company != NULL )
			sprintf(buf, "%s", current_link->company);

		xm_string = XmStringCreateSimple(buf);
		XmListAddItem(addr_list, xm_string, 0);
		XmStringFree(xm_string); 
		current_link = current_link->next;
	}

	/*
	 * Callback to use when an address has been selected.
	 */
	XtAddCallback( addr_list, XmNsingleSelectionCallback,
		UpdateToText, (XtPointer)NULL);

	/*
	 * Callback to close the window.
	 */
	XtAddCallback( close_button, XmNactivateCallback,
		CloseList, (XtPointer)NULL);

	/*
	 * Display the dialog.
	 */
	XtPopup(addr_dialog, XtGrabNone);
	XPCenterWindow(addr_dialog, envelope_dialog, addr_dialog);

	/*
	 * This bit of code forces the pop up window to stay its
	 * initial size, despite the users attempts to resize it.
	 * By doing this, I avoid having to muck extensively with
	 * aligning objects after a resize.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, &width); nargs++;
	XtSetArg ( args[nargs], XmNheight, &height); nargs++;
	XtGetValues ( addr_dialog, args, nargs);
	nargs = 0;
	XtSetArg ( args[nargs], XmNmaxWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNmaxHeight, height); nargs++;
	XtSetArg ( args[nargs], XmNminWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNminHeight, height); nargs++;
	XtSetValues ( addr_dialog, args, nargs); 

}


/*========================================================================
 *	Name:			CloseList
 *	Prototype:	static void CloseList()
 *					
 *	Description:
 * Closes the address selection dialog.
 *
 *	Notes:
 *========================================================================*/
static void
CloseList()
{
	if ( addr_dialog != NULL )
	{
		XtDestroyWidget(addr_dialog);
		addr_dialog = NULL;
	}
}


/*========================================================================
 *	Name:			UpdateToText
 *	Prototype:	static void UpdateToText()
 *					
 *	Description:
 * Updates the To: address box with the selected entries data.
 *
 *	Notes:
 *========================================================================*/
static void
UpdateToText(
	Widget		list_widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
#ifdef DEBUG
	char		*fname="UpdateToText()";
#endif

	int			i;
	PilotAddr_T	*current_link;
	char			name[1024];
	char			buf[2048];

	XmListCallbackStruct *cbs = (XmListCallbackStruct *)call_data;

	DBGEnter();
	
	current_link = Pilot_Address;
	for ( i=1; i<cbs->item_position; i++ )
		current_link = current_link->next;

	if ( ( current_link->last != NULL ) && ( current_link->first != NULL ) )
	{
		if (	( strlen(current_link->last) != 0 ) && 
				( strlen(current_link->first) != 0 ) )
			sprintf(name, "%s %s", current_link->first, current_link->last);
		else
		if (	( strlen(current_link->first) == 0 ) && 
				( strlen(current_link->last) != 0 ) )
			sprintf(name, "%s", current_link->last);
		else
			sprintf(name, "%s", current_link->first);
	}
	else
	if ( current_link->last != NULL )
		sprintf(name, "%s", current_link->last);
	else
	if ( current_link->first != NULL )
		sprintf(name, "%s", current_link->last);
	else
	if ( current_link->company != NULL )
		sprintf(name, "%s", current_link->company);

	DBGPrintf(DBG_INFO, ("name: *%s*\n", name));

	sprintf(buf, "%s\n%s\n%s %s\n%s",
		name,
		current_link->address,
		current_link->city,
		current_link->state,
		current_link->zip);

	XmTextSetString(addr_to, buf);
}


/*========================================================================
 *	Name:			SaveFromId
 *	Prototype:	static void SaveFromId()
 *					
 *	Description:
 * Save the From address to the local id file.
 *
 *	Notes:
 *========================================================================*/
static void
SaveFromId()
{
#ifdef DEBUG
	char		*fname="SaveFromId()";
#endif

	FILE		*fd;
	char		*string;
	char		buf[MAXPATHLEN];

	DBGEnter();

	string = XmTextGetString(addr_from);

	DBGPrintf(DBG_INFO,("From address: \n%s\n", string));

	if ( strlen(string) > 0 )
	{
		sprintf(buf,"%s/%s", app_res.note_dir, DefaultIdFile);
		fd = fopen(buf, "w");
		fprintf(fd, "%s", string);
		fclose(fd);
	}

	XtFree(string);
}

