/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: findnote.c
 *            DESCRIPTION: routines for finding a note
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPCreateFindNotePrompt
 *      PRIVATE FUNCTIONS: FindPopDown, FindPopUpChild
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_FINDNOTE_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>

#include "motif.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void FindPopDown();
static void FindPopUpChild();


/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;
extern Widget			Top_Level;
extern Display			*display;


/* === Static Variables === */
static Widget		find_widget;
static PNLinks		*find_list_data=NULL;



/*========================================================================
 *	Name:			XPCreateFindNotePrompt
 *	Prototype:	void XPCreateFindNotePrompt(StickyNote *notes, char hidden)
 *					
 *	Description:
 * Create a scrollable window in which all notes can be listed.
 *
 *	Input Arguments:
 *	StickyNote	*notes		link list of all notes
 *	char			hidden		TRUE:  only find hidden notes
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPCreateFindNotePrompt(
	StickyNote	*notes,
	char			hidden
)
{
#ifdef DEBUG
	char				*fname="XPCreateFindNotePrompt";
#endif

	Arg				args[20];
	int				nargs;
	char				found;
	Widget			cancel;
	Widget			vbox;
	Widget			find_list;
	StickyNote		*pn;
	char				buf[256];
	PNLinks			*newlink, *lastlink=NULL;
	XmString			xm_string;

	DBGEnter();

	/*
	 * Allow only one copy at a time.
	 */
	if ( find_widget != NULL )
		return;

	/*
	 * Create a dialog and set its title.
	 */
	DBGPrintf(DBG_INFO,("Creating find dialog.\n"));
	if ( hidden )
		find_widget = XPCreateGenericShell(StickyNoteList, 
						CANCEL_BUTTON );
	else
		find_widget = XPCreateGenericShell(NotesFindPrompt, 
						CANCEL_BUTTON );

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	sprintf(buf, "*GenericForm%d", (int)find_widget );
	vbox = XtNameToWidget(find_widget, buf);
	sprintf(buf, "*GenericCancelB%d", (int)find_widget);
	cancel = XtNameToWidget(find_widget, buf);
 

	/*
	 * A scrolled list to display the available notes.
	 */
	DBGPrintf(DBG_INFO,("Creating scrolled list window.\n"));
	nargs=0;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNvisibleItemCount, 12); nargs++;
   XtSetArg ( args[nargs], XmNselectionPolicy, XmSINGLE_SELECT); nargs++;
	find_list = XmCreateScrolledList(
				vbox,
				"find_list",
				args,
				nargs);
	XtManageChild(find_list);

	/*
	 * Cancel button callback.
	 */
	XtAddCallback( cancel, XmNactivateCallback, 
		FindPopDown, (XtPointer)find_widget);

	/*
	 * Populate the list.
	 */
	DBGPrintf(DBG_INFO,("Adding list items.\n"));
	found = FALSE;
	for (pn = notes; pn != NULL; pn = pn->pn_next)
	{
		if ( hidden ) 
		{
			DBGPrintf(DBG_INFO,("looking for hidden notes only.\n"));
			if ( pn->pn_hidden )
			{
				DBGPrintf(DBG_INFO,("found hidden note: %s\n", pn->pn_name));
				xm_string = XmStringCreateSimple(pn->pn_name);
				XmListAddItem(find_list, xm_string, 0);
				XmStringFree(xm_string);

				newlink = (PNLinks *)XPSafeAlloc(sizeof(PNLinks));
				newlink->next = NULL;
				newlink->pn = pn;
				if ( find_list_data == NULL )
					find_list_data = newlink;
				else
					lastlink->next = newlink;
				lastlink = newlink;

				found=TRUE;
			}
			else
				DBGPrintf(DBG_INFO,("Note not hidden: %s\n", pn->pn_name));
		}
		else
		{
			xm_string = XmStringCreateSimple(pn->pn_name);
			XmListAddItem(find_list, xm_string, 0);
			XmStringFree(xm_string);

			newlink = (PNLinks *)XPSafeAlloc(sizeof(PNLinks));
			newlink->next = NULL;
			newlink->pn = pn;
			if ( find_list_data == NULL )
				find_list_data = newlink;
			else
				lastlink->next = newlink;
			lastlink = newlink;

			found=TRUE;
		}
	}

	if ( !found )
	{
		DBGPrintf(DBG_INFO,("No notes found - displaying error message.\n"));
		if ( hidden )
		{
			strcpy(buf, XP_S_NO_HIDDEN_NOTES);
			XPErrPopUp(buf);
		}
		else
		{
			strcpy(buf, XP_S_NO_NOTES);
			XPErrPopUp(buf);
		}
		FindPopDown();
		return;
	}
	else
	{
		XtAddCallback( find_list, XmNsingleSelectionCallback, 
			FindPopUpChild, (XtPointer)find_list_data);
	}

	/*
	 * Set the default width for the window.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XtNwidth, XP_FIND_WIDTH); nargs++;
	XtSetArg ( args[nargs], XtNheight, XP_FIND_HEIGHT); nargs++;
	XtSetValues( find_list, args, nargs );

	/*
	 * Stick the dialog in the middle of * the display.
	 */
	XtRealizeWidget(find_widget);
	XPCenterWindow(find_widget, NULL, find_widget);
	XtPopup(find_widget, XtGrabNone);
}


/*========================================================================
 *	Name:			FindPopDown and FindChildPopUp
 *					
 *	Description:
 * FindPopDown - Destroy the Find window and clean up data storage
 *	FindChildPopUp - Bring the requested note to the cursor
 *
 *========================================================================*/
static void
FindPopDown()
{
	PNLinks	*newlink, *nextlink;

	if ( find_widget != NULL )
	{
		XtDestroyWidget(find_widget);
		find_widget = NULL;
	}

	newlink = find_list_data;
	while ( newlink != NULL )
	{
		nextlink=newlink->next;
		free(newlink);
		newlink=nextlink;
	}
	find_list_data = NULL;
}

static void
FindPopUpChild(
	Widget		list_widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
	PNLinks			*newlink = (PNLinks *)client_data;
	StickyNote		*pn;
	Arg				args[5];
	int				nargs;
	int				i;
	char				buf[256];

	Window 			root, child;
	unsigned int 	buttons;
	int 				root_x, root_y, child_x, child_y;

	XmListCallbackStruct	*cbs = (XmListCallbackStruct *)call_data;

	if ( newlink == NULL )
	{
		strcpy(buf, "I can't find that note!");
		XPErrPopUp(buf);
	}

	/*
	 * Grab the user data from the specified widget (passed in from caller).
	 * This is the link list of notes.  Use the item number selected to
	 * count through the link list to find the note we really want.
	 */
	for (i=1; i<cbs->item_position; i++)
	{
		newlink=newlink->next;
		if ( newlink == NULL )
		{
			/*
			 * This should never happen, but you never know...
			 */
			strcpy(buf, "I can't find that note!");
			XPErrPopUp(buf);
		}
	}

	pn = (StickyNote *) newlink->pn;

	/*
	 * Place the note by the mouse pointer.
	 */
	XQueryPointer(display, XtWindow(Top_Level), 
		&root, &child, &root_x, &root_y, &child_x, &child_y, &buttons);

	/*
	 * If its a hidden note, then just unhide it.  Its location will
	 * be relative to the current screen, even on multi-workspace 
	 * window managers.
	 */
	if ( pn->pn_hidden )
		pn->pn_hidden = FALSE;
	else
		XtPopdown( pn->pn_shellwidget );

	pn->pn_shellx = root_x;
	pn->pn_shelly = root_y;
	nargs = 0;
	XtSetArg(args[nargs], XmNx, root_x); nargs++;
	XtSetArg(args[nargs], XmNy, root_y); nargs++;
	XtSetValues(pn->pn_shellwidget, args, nargs);
	XtPopup( pn->pn_shellwidget, XtGrabNone );


	/* 
	 * Close the Find window
	 */
	FindPopDown();
}

