/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: findtext.c
 *            DESCRIPTION: routines for finding text in a note
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPFindTextPrompt
 *      PRIVATE FUNCTIONS: FindPopDown
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_FINDTEXT_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>

#include "motif.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void FindPopDown();
static void FindNext();
static void FindPrev();
static void FindHome();


/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;
extern Widget			Top_Level;
extern Display			*display;


/* === Static Variables === */
static Widget				find_widget;
static Widget				find_text;
static XmTextPosition	home;



/*========================================================================
 *	Name:			XPFindTextPrompt
 *	Prototype:	void XPFindTextPrompt(Widget widget, int index,
 *								XtPointer call_data)
 *					
 *	Description:
 * Create dialog for searching a note for text.
 *
 *	Input Arguments:
 *	int	index		notes index id
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPFindTextPrompt(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
#ifdef DEBUG
	char				*fname="XPFindTextPrompt";
#endif

	Arg				args[20];
	int				nargs;
	Widget			next_button, prev_button, home_button;
	Widget			cancel;
	Widget			vbox;
	StickyNote		*pn;
	char				buf[256];
	XmString			xm_string;
	Dimension		width, height;

	DBGEnter();

	/*
	 * Allow only one copy at a time.
	 */
	if ( find_widget != NULL )
		return;

	/*
	 * Grab the note structure.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Create a dialog and set its title.
	 */
	find_widget = XPCreateGenericShell(FindTextDialog, 
						CANCEL_BUTTON );

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	sprintf(buf, "*GenericForm%d", (int)find_widget );
	vbox = XtNameToWidget(find_widget, buf);
	sprintf(buf, "*GenericCancelB%d", (int)find_widget);
	cancel = XtNameToWidget(find_widget, buf);
 

	/*
	 * A scrolled list to display the available notes.
	 */
	nargs=0;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
	find_text = XmCreateTextField(
				vbox,
				"find_text",
				args, nargs);
	XtManageChild(find_text);
	XmTextFieldSetMaxLength(find_text, 128);

	/*
	 * Push buttons for Next/Previous/Home (to original spot).
	 */
	xm_string = XmStringCreateSimple(XP_S_NEXT);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, find_text); nargs++;
	next_button = XmCreatePushButton(
				vbox,
				"next_button",
				args, nargs);
	XtManageChild(next_button);
	XmStringFree(xm_string);
	xm_string = XmStringCreateSimple(XP_S_PREV);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNleftWidget, next_button); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, find_text); nargs++;
	prev_button = XmCreatePushButton(
				vbox,
				"prev_button",
				args, nargs);
	XtManageChild(prev_button);
	XmStringFree(xm_string);
	xm_string = XmStringCreateSimple(XP_S_HOME);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNleftWidget, prev_button); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, find_text); nargs++;
	home_button = XmCreatePushButton(
				vbox,
				"home_button",
				args, nargs);
	XtManageChild(home_button);
	XmStringFree(xm_string);

	/*
	 * Callbacks for Next/Prev/Home.
	 */
	XtAddCallback( next_button, XmNactivateCallback, FindNext, (XtPointer)pn);
	XtAddCallback( prev_button, XmNactivateCallback, FindPrev, (XtPointer)pn);
	XtAddCallback( home_button, XmNactivateCallback, FindHome, (XtPointer)pn);

	/*
	 * Cancel button callback.
	 */
	XtAddCallback( cancel, XmNactivateCallback, 
		FindPopDown, (XtPointer)find_widget);

	/*
	 * Set the current location as the "Home" location.
	 */
	home = XmTextGetInsertionPosition(pn->pn_textwidget);

	/*
	 * Stick the dialog in the middle of the note.
	 */
	XtRealizeWidget(find_widget);
	XPCenterWindow(find_widget, pn->pn_shellwidget, find_widget);
	XtPopup(find_widget, XtGrabNone);

	/*
	 * This bit of code forces the pop up window to stay its
	 * initial size, despite the users attempts to resize it.
	 * By doing this, I avoid having to muck extensively with
	 * aligning objects after a resize.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, &width); nargs++;
	XtSetArg ( args[nargs], XmNheight, &height); nargs++;
	XtGetValues ( find_widget, args, nargs);
	nargs = 0;
	XtSetArg ( args[nargs], XmNmaxWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNmaxHeight, height); nargs++;
	XtSetArg ( args[nargs], XmNminWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNminHeight, height); nargs++;
	XtSetValues ( find_widget, args, nargs); 
}


/*========================================================================
 *	Name:			FindPopDown 
 *					
 *	Description:
 * FindPopDown - Destroy the Find window 
 *
 *========================================================================*/
static void
FindPopDown()
{
	if ( find_widget != NULL )
	{
		XtDestroyWidget(find_widget);
		find_widget = NULL;
	}

}


/*========================================================================
 *	Name:			FindNext
 *					
 *	Description:
 * Finds the next occurance of the specified string, if any, in the note.
 *
 *========================================================================*/
static void
FindNext(
	Widget		widget,
	StickyNote	*pn,
	XtPointer	call_data
)
{
	char					*string;
	char					buf[256];
	XmTextPosition		position;
	XmTextPosition		curr_pos;

	/*
	 * Get text string to search for.
	 */
	string = XmTextFieldGetString(find_text);

	if ( strlen(string) == 0 )
		return;

	curr_pos = XmTextGetInsertionPosition(pn->pn_textwidget)+1;

	/*
	 * Search for string in note text.
	 */
	if ( XmTextFindString(pn->pn_textwidget, curr_pos, 
			string, XmTEXT_FORWARD, &position) != True )
	{
		sprintf(buf, "%s: %s", XP_S_FIND_FAIL, string);
		XPMsgWindow(buf, XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK,
			NULL, NULL, NULL, NULL, NULL, NULL);
		return;
	}

	/*
	 * Update current position.
	 */
	curr_pos = position;
	XmTextSetSelection(pn->pn_textwidget, curr_pos, 
		curr_pos+strlen(string), XtLastTimestampProcessed(display));
	XmTextShowPosition(pn->pn_textwidget, curr_pos);
	XmTextSetInsertionPosition(pn->pn_textwidget,curr_pos);

}


/*========================================================================
 *	Name:			FindPrev 
 *					
 *	Description:
 * FindPrev - Finde previous occurance, if any, of string in text widget.
 *
 *========================================================================*/
static void
FindPrev(
	Widget		widget,
	StickyNote	*pn,
	XtPointer	call_data
)
{
	char					*string;
	char					buf[256];
	XmTextPosition		position;
	XmTextPosition		curr_pos;

	/*
	 * Get text string to search for.
	 */
	string = XmTextFieldGetString(find_text);

	if ( strlen(string) == 0 )
		return;

	curr_pos = XmTextGetInsertionPosition(pn->pn_textwidget)-1;

	/*
	 * Search for string in note text.
	 */
	if ( XmTextFindString(pn->pn_textwidget, curr_pos, 
			string, XmTEXT_BACKWARD, &position) != True )
	{
		sprintf(buf, "%s: %s", XP_S_FIND_FAIL, string);
		XPMsgWindow(buf, XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK,
			NULL, NULL, NULL, NULL, NULL, NULL);
		return;
	}

	/*
	 * Update current position.
	 */
	curr_pos = position;
	XmTextSetSelection(pn->pn_textwidget, curr_pos, 
		curr_pos+strlen(string), XtLastTimestampProcessed(display));
	XmTextShowPosition(pn->pn_textwidget, curr_pos);
	XmTextSetInsertionPosition(pn->pn_textwidget,curr_pos);
}


/*========================================================================
 *	Name:			FindHome 
 *					
 *	Description:
 * FindHome - Jump back to original locationin the text widget.
 *
 *========================================================================*/
static void
FindHome(
	Widget		widget,
	StickyNote	*pn,
	XtPointer	call_data
)
{

	XmTextSetInsertionPosition(pn->pn_textwidget, home);
}

