/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: generic.c
 *            DESCRIPTION: generic shell wiindwo for XNotesPlus
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPCreateGenericShell
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_GENERIC_C

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>

#include "motif.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */


/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;
extern Widget			Top_Level;


/* === Static Variables === */



/*========================================================================
 *	Name:			XPCreateGenericShell
 *	Prototype:	XPCreateGenericShell( char *, unsigned char )
 *
 *	Description:
 *		This is the basic shell in which most of the other pop up 
 *		windows will be placed.  This gets called each time a new
 *		window needs to be popped up.
 *
 *	Input Arguments:
 *		char 			*textstring			The title to use for the window
 *		u_char 		button_types		Defines which buttons to include
 *												Valid values are:
 *												0x0001		Accept button
 *												0x0002		Cancel button
 *												0x0008		Overwrite button
 *												0x0010		Append button
 *												0x0020		Help button
 *												0x0040		Print button
 *												0x0100		Close button
 *												0x0fff		All buttons (never used)
 *	Output Arguments:
 *		None.
 *	Return Values:
 *		Widget		widget_id			The widget id of the shell which will be the
 *												parent of the rest of the windows widgets.
 *
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *		In order to use this routine, the caller will need to know the names
 *		of the child widgets of the parent widget id which is returned.
 *		These will be used to set callbacks for the buttons and to find the form
 *		widget in which all the other widgets specific to the caller will go.
 *		The caller can use the Xt function XtNameToWidget() to get the widget
 *		id of the widget for which a callback will be set or for the form widget.
 *
 *		All of these names of these widgets are suffixed by the widget id
 *		of the shell widget (ie the id returned by this routine).
 *		These are the names the caller needs to know:
 *
 *		Widget Name				Purpose
 *		-----------				------------------------------------
 *		GenericForm				the form in which the caller will place
 *									the widgets specific to that caller.
 *		GenericHelpB			The Help button for this shell.
 *		GenericAcceptB			The Accept button for this shell.
 *		GenericCancelB			The Cancel button for this shell.
 *		GenericOverwriteB		The Overwrite button for this shell.
 *		GenericAppendB			The Append button for this shell.
 *
 *		Remember, tack on the widget id of the shell!  For example, if the
 *		shell widget id is 138457 (decimal) then the name of the form widget
 *		for this shell is "GenericForm138457".
 *		
 *========================================================================*/

Widget
XPCreateGenericShell(
	char				*textstring,
	unsigned int	button_types
)
{

#ifdef DEBUG
	char 			fname[]="XPCreateGenericShell()";
#endif

	char			buf[256];
	char			shellname[256];
	Arg			args[20];
	int			nargs;
	XmString		xm_string;
	Widget		shell_id, inside_form, callers_form, callers_frame;
	Widget		button_frame, button_formA, button_form, 
					button_accept, button_cancel, button_overwrite,
					button_help, button_append;


	DBGEnter();

	/*
	 * Create the pop up shell
	 */

	/* new version of title bar */
	(void) sprintf ( buf, "%s - %s", XP_S_PROG_NAME, textstring );

	/*
	 * The Help window is a special case - we want to force a real
	 * widget name so that we can specify the Help windows default
	 * height/width values in the application defaults file.
	 */
	if ( strcmp(textstring, NotesHelpDialog) == 0 )
		sprintf(shellname, "Help");
	else
		sprintf(shellname, "GenericShell");

	nargs=0;
	XtSetArg ( args[nargs], XtNtitle, buf ); nargs++;
	XtSetArg ( args[nargs], XtNallowShellResize, False ); nargs++;
	XtSetArg ( args[nargs], XmNkeyboardFocusPolicy, XmPOINTER ); nargs++;
	XtSetArg ( args[nargs], XmNdeleteResponse, XmDO_NOTHING ); nargs++;
	shell_id = XtCreatePopupShell (
		shellname,							/* widget name */
		transientShellWidgetClass,		/* class */
		Top_Level,							/* parent */
		args,	nargs							/* arg list */
		);
	DBGPrintf(DBG_WINDOWS,("generic shell widget: 0x%08x\n", shell_id));

	
	/* create the inside form - this one is hidden from the caller */
	nargs=0;
	XtSetArg ( args[nargs], XmNborderWidth, 0 ); nargs++;
	inside_form = XtCreateManagedWidget (
		"genericInsideForm",			/* widget name */
		xmFormWidgetClass,			/* class */
		shell_id,						/* parent widget */
		args, nargs						/* arg list */
		);

	/*
	 * create the two frames which define the callers area and the buttons
	 * area.
	 */

	/* create the callers frame */
	nargs=0;
	XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNshadowType, XmSHADOW_ETCHED_OUT ); nargs++;
	XtSetArg ( args[nargs], XmNshadowThickness, 4 ); nargs++;
	callers_frame = XtCreateManagedWidget (
		"callersFrame",				/* widget name */
		xmFrameWidgetClass,			/* class */
		inside_form,					/* parent widget */
		args, nargs						/* arg list */
		);

	/* create the buttons frame */
	nargs=0;
	XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNshadowType, XmSHADOW_ETCHED_OUT ); nargs++;
	XtSetArg ( args[nargs], XmNshadowThickness, 4 ); nargs++;
	button_frame = XtCreateManagedWidget (
		"buttonsFrame",				/* widget name */
		xmFrameWidgetClass,			/* class */
		inside_form,					/* parent widget */
		args, nargs						/* arg list */
		);

	nargs=0;
	XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_WIDGET ); nargs++;
	XtSetArg ( args[nargs], XmNbottomWidget, button_frame ); nargs++;
	XtSetValues( callers_frame, args, nargs );

	/*
	 * create the two forms which go inside the frames 
	 * (for widget positioning)
	 */

	sprintf(buf, "GenericForm%d", (int)shell_id);
	nargs=0;
	XtSetArg ( args[nargs], XmNborderWidth, 0 ); nargs++;
	callers_form = XtCreateManagedWidget (
		buf,								/* widget name */
		xmFormWidgetClass,			/* class */
		callers_frame,					/* parent widget */
		args, nargs						/* arg list */
		);

	nargs=0;
	XtSetArg ( args[nargs], XmNborderWidth, 0 ); nargs++;
	XtSetArg ( args[nargs], XmNrubberPositioning, True ); nargs++;
	XtSetArg ( args[nargs], XmNfractionBase, 100 ); nargs++;
	XtSetArg ( args[nargs], XmNhorizontalSpacing, 10 ); nargs++;
	button_formA = XtCreateManagedWidget (
		"genericButtonsForm",		/* widget name */
		xmFormWidgetClass,			/* class */
		button_frame,					/* parent widget */
		args, nargs						/* arg list */
		);

	nargs=0;
	XtSetArg ( args[nargs], XmNborderWidth, 0 ); nargs++;
	XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_POSITION ); nargs++;
	XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_POSITION ); nargs++;
	XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_SELF ); nargs++;
	XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_SELF ); nargs++;
	XtSetArg ( args[nargs], XmNorientation, XmHORIZONTAL ); nargs++;
	XtSetArg ( args[nargs], XmNpacking, XmPACK_COLUMN ); nargs++;
	XtSetArg ( args[nargs], XmNisAligned, True ); nargs++;
	XtSetArg ( args[nargs], XmNentryAlignment, XmALIGNMENT_CENTER ); nargs++;
	button_form = XmCreateRowColumn (
		button_formA,					/* parent widget */
		"genericButtonsRowColumns",/* widget name */
		args, nargs						/* arg list */
		);
	XtManageChild(button_form);


	/* create the buttons in the button area */

	if ( ( button_types & ACCEPT_BUTTON ) || ( button_types & PRINT_BUTTON ) )
	{
		sprintf(buf, "GenericAcceptB%d", (int)shell_id);
		if ( button_types & ACCEPT_BUTTON )
			xm_string = XmStringCreateSimple("Accept");
		else
			xm_string = XmStringCreateSimple("Print");
		nargs=0;
		XtSetArg ( args[nargs], XmNborderWidth, 0 ); nargs++;
		XtSetArg ( args[nargs], XmNlabelString, xm_string ); nargs++;
		button_accept = XmCreatePushButton(
			button_form,					/* parent widget */
			buf,								/* widget name */
			args, nargs						/* arg list */
			);
		XtManageChild(button_accept);
		XmStringFree(xm_string);
	}
	
	if ( ( button_types & CANCEL_BUTTON ) || ( button_types & CLOSE_BUTTON ) )
	{
		sprintf(buf, "GenericCancelB%d", (int)shell_id);
		if ( button_types & CANCEL_BUTTON )
			xm_string = XmStringCreateSimple("Cancel");
		else
			xm_string = XmStringCreateSimple("Close");
		nargs=0;
		XtSetArg ( args[nargs], XmNborderWidth, 0 ); nargs++;
		XtSetArg ( args[nargs], XmNlabelString, xm_string ); nargs++;
		button_cancel = XmCreatePushButton(
			button_form,					/* parent widget */
			buf,								/* widget name */
			args, nargs						/* arg list */
			);
		XtManageChild(button_cancel);
		XmStringFree(xm_string);
	}

	if ( button_types & OVERWRITE_BUTTON )
	{
		sprintf(buf, "GenericOverwriteB%d", (int)shell_id);
		xm_string = XmStringCreateSimple("Overwrite");
		nargs=0;
		XtSetArg ( args[nargs], XmNborderWidth, 0 ); nargs++;
		XtSetArg ( args[nargs], XmNlabelString, xm_string ); nargs++;
		button_overwrite = XmCreatePushButton(
			button_form,					/* parent widget */
			buf,								/* widget name */
			args, nargs						/* arg list */
			);
		XtManageChild(button_overwrite);
		XmStringFree(xm_string);
	}

	if ( button_types & APPEND_BUTTON )
	{
		sprintf(buf, "GenericAppendB%d", (int)shell_id);
		xm_string = XmStringCreateSimple("Append");
		nargs=0;
		XtSetArg ( args[nargs], XmNborderWidth, 0 ); nargs++;
		XtSetArg ( args[nargs], XmNlabelString, xm_string ); nargs++;
		button_append = XmCreatePushButton(
			button_form,					/* parent widget */
			buf,								/* widget name */
			args, nargs						/* arg list */
			);
		XtManageChild(button_append);
		XmStringFree(xm_string);
	}

	if ( button_types & HELP_BUTTON )
	{
		sprintf(buf, "GenericHelpB%d", (int)shell_id);
		xm_string = XmStringCreateSimple("Help");
		nargs=0;
		XtSetArg ( args[nargs], XmNborderWidth, 0 ); nargs++;
		XtSetArg ( args[nargs], XmNlabelString, xm_string ); nargs++;
		button_help = XmCreatePushButton(
			button_form,					/* parent widget */
			buf,								/* widget name */
			args, nargs						/* arg list */
			);
		XtManageChild(button_help);
		XmStringFree(xm_string);
	}


	DBGExit();

	/* return to the caller */
	return (shell_id);

}

