/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: inserts.c
 *            DESCRIPTION: routines that insert simple text data into notes
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPInsertCalendar, XPInsertDates
 *      PRIVATE FUNCTIONS: CreateInsertPrompt, InsertAccept, InsertPopDown,
 *								 : InsertLeftArrowCB, 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_INSERTS_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include <time.h>

#include "motif.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void InsertPopDown();
static void InsertAccept();
static void InsertLeftArrowCB();
static void CreateInsertPrompt();


/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;
extern Widget			Top_Level;


/* === Static Variables === */
static Widget 		insert_widget=NULL;
static Widget		insert_boxes[12];
static Widget		insert_buttons[2];
static Widget		insert_leftarrows[11];
static int			active_item;



/*========================================================================
 *	Name:			XPInsertCalendar
 *	Prototype:	void XPInsertCalendar(Widget widget, int index, 
 *							XtPointer call_data)
 *					
 *	Description:
 * Insert this months calendar into the note
 *
 *	Notes:
 *========================================================================*/
void
XPInsertCalendar(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	StickyNote	*pn;
	char			tmpfile[512];
	char			cmd[1024];
	int			rc;

	/*
	 * Find the note we're adding the calender to.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Run the calendar generating program to get the text.
	 * We use system() instead of popen() because the latter
	 * doesn't reliably tell us if the process actually ran.
	 * system() will provide the return value of the invoked
	 * program.
	 */
	sprintf(tmpfile,"%s/xp%d", app_res.tmp_dir, (int)getpid());
	sprintf (cmd, "%s > %s", app_res.calendar_cmd, tmpfile );
	if ( (rc=system ( cmd )) != 0 ) 
	{

		sprintf(cmd, XP_S_CANT_RUN_CMD, app_res.calendar_cmd, rc);
		XPErrPopUp(cmd);

		/* cleanup, if necessary */
		sprintf (cmd, "rm %s", tmpfile );
		system ( cmd );
		return;
	}

	/* stuff the output into the note */
	XPInsertText(pn, tmpfile, TRUE, FILE_TYPE);

	/*
	 * Changes occured to the note - set its save widget sensitive.
	 */
	XtSetSensitive ( pn->pn_savewidget, True );
}



/*========================================================================
 *	Name:			XPInsertDates
 *	Prototype:	void XPInsertDates(Widget widget, int index, XtPointer call_data)
 *					
 *	Description:
 * Callback for inserting a formatted date into the note.
 *
 *	Notes:
 *========================================================================*/
void
XPInsertDates(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	StickyNote 	*pn;

	/*
	 * Find the note we're hiding.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	CreateInsertPrompt(pn);

}


/*========================================================================
 *	Name:			CreateInsertPrompt
 *	Prototype:	static void CreateInsertPrompt(StickyNote *pn)
 *					
 *	Description:
 * Create a prompt for insert information
 *
 *	Notes:
 *========================================================================*/
static void
CreateInsertPrompt(
	StickyNote	*pn
)
{
	Widget			label[11];
	Widget			vbox;
	int				i;
	char				buf[256];
	XmString			xm_string;
	Arg				args[10];
	int				nargs;
	Dimension		width, height;

	/*
	 * Prevent multiple windows from being opened at once.
	 */
	if ( insert_widget != NULL )
		InsertPopDown();

	/* initialize marker */
	active_item=0;

	/*
	 * Create a popup shell widget
	 */
	insert_widget = XPCreateGenericShell(NotesInsertPrompt, 
					ACCEPT_BUTTON | CANCEL_BUTTON );

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	sprintf(buf, "*GenericForm%d", (int)insert_widget );
	vbox = XtNameToWidget(insert_widget, buf);
	sprintf(buf, "*GenericAcceptB%d", (int)insert_widget);
	insert_buttons[0] = XtNameToWidget(insert_widget, buf);
	sprintf(buf, "*GenericCancelB%d", (int)insert_widget);
	insert_buttons[1] = XtNameToWidget(insert_widget, buf);

	/*
	 * The set of button/labels that provide the various date formats
	 * to use.
	 */
	for ( i=0; i<11; i++)
	{
		switch(i)
		{
			case 0: sprintf(buf, "mm/dd/yy"); break;
			case 1: sprintf(buf, "<mm/dd/yy>"); break;
			case 2: sprintf(buf, "(mm/dd/yy)"); break;
			case 3: sprintf(buf, "mm/dd/yy - hh:mm"); break;
			case 4: sprintf(buf, "mm/dd/yy(hh:mm)"); break;
			case 5: sprintf(buf, "Jan 1, 1996"); break;
			case 6: sprintf(buf, "Jan 1, 1996 - hh:mm"); break;
			case 7: sprintf(buf, "Jan 1, 1996 (hh:mm)"); break;
			case 8: sprintf(buf, "Tuesday Jan 1, 1996"); break;
			case 9: sprintf(buf, "Tuesday Jan 1, 1996 - hh:mm"); break;
			case 10: sprintf(buf, "Tuesday Jan 1, 1996 (hh:mm)"); break;
		}

		/*
		 * Each row starts with a horizontal box.
		 */
		nargs=0;
		XtSetArg(args[nargs], XmNmarginHeight, 5); nargs++;
		XtSetArg(args[nargs], XmNmarginWidth, 5); nargs++;
		XtSetArg(args[nargs], XmNspacing, 5); nargs++;
		XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
		XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
		XtSetArg(args[nargs], XmNorientation, XmHORIZONTAL); nargs++;
		if ( i == 0 )
		{
			XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
		}
		else
		{
			XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
			XtSetArg(args[nargs], XmNtopWidget, insert_boxes[i-1]); nargs++;
		}
		insert_boxes[i] = XmCreateRowColumn(
								vbox,
								"rowColumn",
								args, nargs
								);
		XtManageChild(insert_boxes[i]);

		/*
		 * First comes a button and a label for each button.
		 */
		xm_string = XmStringCreateSimple("  -  ");
		nargs=0;
		XtSetArg ( args[nargs], XmNlabelString, xm_string ); nargs++;
		insert_leftarrows[i] = XmCreatePushButton(
								insert_boxes[i],
								"pushButton",
								args, nargs
								);
		XmStringFree(xm_string);
		XtManageChild(insert_leftarrows[i]);
		XtAddCallback( insert_leftarrows[i], XmNactivateCallback, 
			InsertLeftArrowCB, (XtPointer)i);

		/*
		 * Then a label with the format type.
		 */
		xm_string = XmStringCreateSimple(buf);
		nargs=0;
		XtSetArg ( args[nargs], XmNlabelString, xm_string ); nargs++;
		label[i] = XmCreateLabel(
							insert_boxes[i],
							"label",
							args, nargs
							);
		XmStringFree(xm_string);
		XtManageChild(label[i]);

	}

	/*
	 * create the Cancel/Accept command buttons, plus a horizontal
	 * box for them.
	 */
	XtAddCallback( insert_buttons[0], XmNactivateCallback, InsertAccept, 
			(XtPointer)pn);
	XtAddCallback( insert_buttons[1], XmNactivateCallback, InsertPopDown, 
			(XtPointer)NULL);

	/*
	 * Pop up the window.
	 */
	XtRealizeWidget(insert_widget);
	XPCenterWindow(insert_widget, pn->pn_shellwidget, insert_widget);
	XtPopup( insert_widget, XtGrabNone );

	/*
	 * This bit of code forces the pop up window to stay its
	 * initial size, despite the users attempts to resize it.
	 * By doing this, I avoid having to muck extensively with
	 * aligning objects after a resize.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, &width); nargs++;
	XtSetArg ( args[nargs], XmNheight, &height); nargs++;
	XtGetValues ( insert_widget, args, nargs);
	nargs = 0;
	XtSetArg ( args[nargs], XmNmaxWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNmaxHeight, height); nargs++;
	XtSetArg ( args[nargs], XmNminWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNminHeight, height); nargs++;
	XtSetValues ( insert_widget, args, nargs); 

}


/*========================================================================
 *	Name:			InsertAccept
 *	Prototype:	void InsertAccept(Widget widget, StickyNote *pn, XtPointer call_data)
 *					
 *	Description:
 * Accept selected date value. This routine has the potential to be 
 * very OS specific.
 *
 *	Notes:
 *========================================================================*/
void
InsertAccept(
	Widget		widget,
	StickyNote 	*pn,
	XtPointer	call_data
)
{
	char			cmd[1024];
	time_t		time_tp;
	struct tm	*tm_tp;
	char			dayname[12];
	char			year[5];
	char			month[10];

	/* get todays date and current time */
	time(&time_tp);
	tm_tp = localtime(&time_tp);

	/*
	 * convert numeric values to strings now, even if we don't use them,
	 * since we need these in more than one place.
	 */
	switch ( tm_tp->tm_wday )
	{
		case 0:	sprintf(dayname, "Sunday"); break;
		case 1:	sprintf(dayname, "Monday"); break;
		case 2:	sprintf(dayname, "Tuesday"); break;
		case 3:	sprintf(dayname, "Wednesday"); break;
		case 4:	sprintf(dayname, "Thursday"); break;
		case 5:	sprintf(dayname, "Friday"); break;
		case 6:	sprintf(dayname, "Saturday"); break;
	}
	switch ( tm_tp->tm_mon )
	{
		case 0: sprintf(month, "Jan"); break;
		case 1: sprintf(month, "Feb"); break;
		case 2: sprintf(month, "Mar"); break;
		case 3: sprintf(month, "Apr"); break;
		case 4: sprintf(month, "May"); break;
		case 5: sprintf(month, "Jun"); break;
		case 6: sprintf(month, "Jul"); break;
		case 7: sprintf(month, "Aug"); break;
		case 8: sprintf(month, "Sept"); break;
		case 9: sprintf(month, "Oct"); break;
		case 10: sprintf(month, "Nov"); break;
		case 11: sprintf(month, "Dec"); break;
	}
	if ( tm_tp->tm_year > 100 )
		sprintf(year,"20");
	else
		sprintf(year,"19");

	/*
	 * increment the numeric month, since localtime()
	 * returns the month from 0 to 11.
	 */
	tm_tp->tm_mon++;

	switch (active_item)
	{
		case 0:
			sprintf (cmd, 
				"%02d/%02d/%02d", tm_tp->tm_mon, tm_tp->tm_mday, tm_tp->tm_year );
			break;

		case 1:
			sprintf (cmd, 
				"<%02d/%02d/%02d>", tm_tp->tm_mon, tm_tp->tm_mday, tm_tp->tm_year );
			break;

		case 2:
			sprintf (cmd, 
				"(%02d/%02d/%02d)", tm_tp->tm_mon, tm_tp->tm_mday, tm_tp->tm_year );
			break;

		case 3:
			sprintf (cmd, 
				"%02d/%02d/%02d - %02d:%02d", 
				tm_tp->tm_mon, tm_tp->tm_mday, tm_tp->tm_year, 
				tm_tp->tm_hour, tm_tp->tm_min );
			break;

		case 4:
			sprintf (cmd, 
				"%02d/%02d/%02d(%02d:%02d)", 
				tm_tp->tm_mon, tm_tp->tm_mday, tm_tp->tm_year, 
				tm_tp->tm_hour, tm_tp->tm_min );
			break;

		case 5:
			sprintf (cmd, 
				"%s %d, %s%d", month, tm_tp->tm_mday, year, tm_tp->tm_year);
			break;

		case 6:
			sprintf (cmd, 
				"%s %d, %s%d - %02d:%02d", 
				month, tm_tp->tm_mday, year, tm_tp->tm_year,
				tm_tp->tm_hour, tm_tp->tm_min );
			break;

		case 7:
			sprintf (cmd, 
				"%s %d, %s%d (%02d:%02d)", 
				month, tm_tp->tm_mday, year, tm_tp->tm_year,
				tm_tp->tm_hour, tm_tp->tm_min );
			break;

		case 8:
			sprintf (cmd, 
				"%s %s %d, %s%d", 
				dayname, month, tm_tp->tm_mday, year, tm_tp->tm_year);
			break;

		case 9:
			sprintf (cmd, 
				"%s %s %d, %s%d - %02d:%02d", 
				dayname, month, tm_tp->tm_mday, year, tm_tp->tm_year,
				tm_tp->tm_hour, tm_tp->tm_min );
			break;

		case 10:
			sprintf (cmd, 
				"%s %s %d, %s%d (%02d:%02d)", 
				dayname, month, tm_tp->tm_mday, year, tm_tp->tm_year,
				tm_tp->tm_hour, tm_tp->tm_min );
			break;

	}

	/* stuff the output into the note */
	XPInsertText(pn, cmd, FALSE, TEXT_TYPE);

	XtSetSensitive ( pn->pn_savewidget, True );
	InsertPopDown();
}

/*
 * InsertPopDown - removes the Insert Dialog box
 */
static void
InsertPopDown()
{
	if ( insert_widget != NULL )
	{
		XtDestroyWidget( insert_widget );
		insert_widget = NULL;
	}
}


/*
 * InsertLeftArrowcB - handle a left arrow click
 */
static void
InsertLeftArrowCB(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	int		i;
	Arg		args[1];
	int		nargs;
	XmString	xm_string;

	for (i=0; i<11; i++)
	{
		if ( i == index )
			xm_string = XmStringCreateSimple("  >  ");
		else
			xm_string = XmStringCreateSimple("  -  ");
		nargs=0;
		XtSetArg ( args[nargs], XmNlabelString, xm_string ); nargs++;
		XtSetValues(insert_leftarrows[i], args, nargs);
		XmStringFree(xm_string);
	}

	active_item=index;
}
