/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: menus.c
 *            DESCRIPTION: menus module for XNotesPlus
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPButtonPress, XPCreateMainMenu
 *      PRIVATE FUNCTIONS: HandleMenuSelection
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_MENUS_C

/* === System Headers === */
#include <signal.h>
#include <string.h>
#include <stdlib.h>

#include "motif.h"

/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"

#define MenuCreate_1p5x2	100
#define MenuCreate_2x3		101
#define MenuCreate_3x3		102
#define MenuCreate_3x4		103
#define MenuCreate_3x5		104
#define MenuCreate_4x6		105

#define NoteTypes				0
#define EndNoteTypes			1
#define MenuRaiseAll			2
#define MenuLowerAll			3
#define MenuSaveAll			4
#define MenuShowHidden		5
#define MenuUnHideAll		6
#define MenuHideAll			7
#define MenuCascade			8
#define MenuFindANote		9
#define EndNoteFunctions	10
#define MenuEnvelopes		11
#define MenuProjects			12
#define MenuPilotTool		13
#define MenuToolTips			14
#define EndOtherFunctions	15
#define MenuAbout				16
#define MenuExit				17


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void				HandleMenuSelection();
static void				ToggleTips();


/* === Global Variables === */
extern StickyNote		*notes;
extern Widget			Top_Level;
extern Display			*display;
extern Colormap		XP_Colormap;

int	button1_down;
int	button2_down;
int	button3_down;


/* === Static Variables === */
static char *top_menu_strings[] = {
	"Create Notes",
	"",
	"Raise All Notes",
	"Lower All Notes",
	"Save All Notes",
	"Hidden Notes",
	"Unhide All Notes",
	"Hide All Notes",
	"Cascade Notes",
	"Find A Note",
	"",
	"Envelopes",
	"Projects",
	"Pilot Tool",
	"Disable ToolTips",
	"",
	"About",
	"Exit",
	NULL,
};
static char *note_type_strings[] = {
	"Create 1.5x2 Note",
	"Create 2x3 Note",
	"Create 3x3 Note",
	"Create 3x4 Note",
	"Create 3x5 Note",
	"Create 4x6 Note",
	NULL,
};

static Widget	tool_tips_widget;


/*========================================================================
 *	Name:			XPButtonPress
 *	Prototype:	void XPButtonPress ( Widget widget, XtPointer client_data,
 *							XEvent *event, Boolean do_continue )
 *					
 *	Description:
 *		Handle events that occur within the plaid widget.
 *
 *	Input Arguments:
 *		Standard XtEventHandler arguments.
 *
 *	Output Arguments:
 *	Notes:
 *	We use an event handler instead of ordinary PushButton widget
 *	callbacks because we have lots of special handling we'll want with this
 *	button, beyond simply button clicks.
 *========================================================================*/
void 
XPButtonPress (
	Widget 		widget, 
	XtPointer	client_data,
	XEvent 		*event,
	Boolean		do_continue
)
{
	switch (event->type)
	{
		case ButtonPress:
			switch( (int) (event->xbutton.button) )
			{
				case Button3:
					/*
					 * Post or clear the plaid widgets menu.
					 */
					XmMenuPosition((Widget)client_data, 
						(XButtonPressedEvent *)event);
					XtManageChild((Widget)client_data);
					break;
			}
			break;

		default:
			break;
	}

}


/*========================================================================
 *	Name:			XPCreateMainMenu
 *	Prototype:	XPCreateMainMenu ( Widget button )
 *
 *	Description:
 *	Create the menu in the plaid window.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPCreateMainMenu(
	Widget	button
)
{
	int		i,j;
	Widget	menu, submenu;
	Widget	menu_items, submenu_items;
	Arg		args[5];
	int		nargs;
	XmString	xm_string;
	Pixel		bg, fg;

	/*
	 * Create the pop up menu that goes on the Plaid widget.
	 */
	nargs = 0;
	XtSetArg( args[nargs], XmNbackground, &bg); nargs++;
	XtSetArg( args[nargs], XmNforeground, &fg); nargs++;
	XtGetValues (button, args, nargs);

	nargs = 0;
	XtSetArg( args[nargs], XmNbackground, bg); nargs++;
	XtSetArg( args[nargs], XmNforeground, fg); nargs++;
	XtSetArg( args[nargs], XmNcolormap, XP_Colormap); nargs++;
	menu = XmCreatePopupMenu(
				button,
				"mainmenu",
				args, nargs);

	i = 0;
	while ( top_menu_strings[i] != NULL )
	{
		switch ( i )
		{
			/* 
			 * Create a new menu-item with a label in it and 
			 * add it to the menu
			 */

			case NoteTypes: 
				/*
				 * This item has a submenu.
				 */
				submenu = XmCreatePulldownMenu(
								menu, 
								"mainmenu", 
								NULL, 0);

				xm_string = XmStringCreateSimple(top_menu_strings[i]);
				nargs=0;
				XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
				XtSetArg( args[nargs], XmNsubMenuId, submenu); nargs++;
				XtSetArg( args[nargs], XmNbackground, bg); nargs++;
				XtSetArg( args[nargs], XmNforeground, fg); nargs++;
				XtSetArg( args[nargs], XmNcolormap, XP_Colormap); nargs++;
				menu_items = XtCreateManagedWidget(
								"mainmenu",
								xmCascadeButtonWidgetClass,
								menu,
								args, nargs);
				XmStringFree(xm_string);

				j=0;
				while ( note_type_strings[j] != NULL )
				{
					/* Create a new menu-item with a label in it */
					xm_string = XmStringCreateSimple(note_type_strings[j]);
					nargs=0;
					XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
					XtSetArg( args[nargs], XmNbackground, bg); nargs++;
					XtSetArg( args[nargs], XmNforeground, fg); nargs++;
					XtSetArg( args[nargs], XmNcolormap, XP_Colormap); nargs++;
					submenu_items = XtCreateManagedWidget(
								"mainmenu",
								xmPushButtonWidgetClass,
								submenu,
								args, nargs);
					XmStringFree(xm_string);

					/* Add a callback for this item. */
					XtAddCallback( submenu_items,
						XmNactivateCallback, HandleMenuSelection, 
							(XtPointer)(j+100) );

					j++;
				}
				break;

			case MenuProjects:
				if ( app_res.rgb_file != NULL )
				{
					xm_string = XmStringCreateSimple(top_menu_strings[i]);
					nargs=0;
					XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
					XtSetArg( args[nargs], XmNbackground, bg); nargs++;
					XtSetArg( args[nargs], XmNforeground, fg); nargs++;
					XtSetArg( args[nargs], XmNcolormap, XP_Colormap); nargs++;
					menu_items = XtCreateManagedWidget(
								"mainmenu",
								xmPushButtonWidgetClass,
								menu,
								args, nargs);
					XmStringFree(xm_string);

					/* Add a callback for this item. */
					XtAddCallback( menu_items, XmNactivateCallback, 
						HandleMenuSelection, (XtPointer)i);
				}
				break;

			case MenuPilotTool:
				if (	(	( app_res.pilot_xfer != NULL ) ||
							( app_res.pilot_addr != NULL ) ||
							(	( app_res.pilot_memos != NULL ) &&
								( app_res.pilot_imemos != NULL ) )
						) &&
						( getenv("PILOTPORT") != NULL ) 
					)
				{
					xm_string = XmStringCreateSimple(top_menu_strings[i]);
					nargs=0;
					XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
					XtSetArg( args[nargs], XmNbackground, bg); nargs++;
					XtSetArg( args[nargs], XmNforeground, fg); nargs++;
					XtSetArg( args[nargs], XmNcolormap, XP_Colormap); nargs++;
					menu_items = XtCreateManagedWidget(
								"mainmenu",
								xmPushButtonWidgetClass,
								menu,
								args, nargs);
					XmStringFree(xm_string);

					/* Add a callback for this item. */
					XtAddCallback( menu_items, XmNactivateCallback, 
						HandleMenuSelection, (XtPointer)i);
				}
				break;

			case MenuToolTips:
				nargs=0;

				if ( app_res.no_tips )
				{
					xm_string = XmStringCreateSimple(XP_S_ENABLE_TIPS);
					XtSetArg( args[nargs], XmNuserData, XP_TIPS_OFF); nargs++;
				}
				else
				{
					xm_string = XmStringCreateSimple(XP_S_DISABLE_TIPS);
					XtSetArg( args[nargs], XmNuserData, XP_TIPS_ON); nargs++;
				}

				XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
				XtSetArg( args[nargs], XmNbackground, bg); nargs++;
				XtSetArg( args[nargs], XmNforeground, fg); nargs++;
				XtSetArg( args[nargs], XmNcolormap, XP_Colormap); nargs++;
				tool_tips_widget = menu_items = XtCreateManagedWidget(
							"mainmenu",
							xmPushButtonWidgetClass,
							menu,
							args, nargs);
				XmStringFree(xm_string);

				/* Add a callback for this item. */
				XtAddCallback( menu_items, XmNactivateCallback, 
					HandleMenuSelection, (XtPointer)i);
				break;

			default:
				/*
				 * All other menu items are simple push buttons.
				 */
				xm_string = XmStringCreateSimple(top_menu_strings[i]);
				nargs=0;
				XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
				XtSetArg( args[nargs], XmNbackground, bg); nargs++;
				XtSetArg( args[nargs], XmNforeground, fg); nargs++;
				XtSetArg( args[nargs], XmNcolormap, XP_Colormap); nargs++;
				menu_items = XtCreateManagedWidget(
								"mainmenu",
								xmPushButtonWidgetClass,
								menu,
								args, nargs);
				XmStringFree(xm_string);

				/* Add a callback for this item. */
				XtAddCallback( menu_items, XmNactivateCallback, 
					HandleMenuSelection, (XtPointer)i);

				/*
				 * If the menu item had no text, then this button will not
				 * accept keyboard input - ie its a blank button.  We do
				 * this to keep the look as close to the GTK version as 
				 * possible.
				 */
				if ( strlen(top_menu_strings[i]) == 0 )
					 XtSetSensitive (menu_items, False);

				break;
		}

		i++;
	}

	/*
	 * Tell the plaid button to pop up the menu on a button click.
	 */
	XtAddEventHandler(button, ButtonPressMask,
		False, (XtEventHandler)XPButtonPress, (XtPointer)menu);
	XtAddEventHandler(button, ButtonReleaseMask,
		False, (XtEventHandler)XPButtonPress, (XtPointer)menu);
}



/*========================================================================
 *	Name:			HandleMenuSelection 			
 *	Prototype:	HandleMenuSelection(int)
 *
 *	Description:
 * Callback from menu widget to handle a selection.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
static void
HandleMenuSelection(
	Widget		widget,
	int			menu_item,
	XtPointer	call_data
)
{
#ifdef DEBUG
	char	fname[]="HandleMenuSelection()";
#endif

	DBGEnter();
	DBGPrintf(DBG_INFO,("menu item: %d\n", menu_item));

	/*
	 * Dispatch the command.
	 */
	switch (menu_item) {
	case MenuCreate_1p5x2:
		XPCreateNewNote(StickyNote_1p5x2, NULL);
		break;
	case MenuCreate_2x3:
		XPCreateNewNote(StickyNote_2x3, NULL);
		break;
	case MenuCreate_3x3:
		XPCreateNewNote(StickyNote_3x3, NULL);
		break;
	case MenuCreate_3x4:
		XPCreateNewNote(StickyNote_3x4, NULL);
		break;
	case MenuCreate_3x5:
		XPCreateNewNote(StickyNote_3x5, NULL);
		break;
	case MenuCreate_4x6:
		XPCreateNewNote(StickyNote_4x6, NULL);
		break;
	case MenuRaiseAll:
		XPRaiseAllNotes();
		break;
	case MenuLowerAll:
		XPLowerAllNotes();
		break;
	case MenuSaveAll:
		XPSaveAllNotes(TRUE);
		break;
	case MenuShowHidden:
		XPPopUpList();
		break;
	case MenuUnHideAll:
		XPUnHideAllNotes();
		break;
	case MenuHideAll:
		XPHideAllNotes();
		break;
	case MenuCascade:
		XPCascadeNotes();
		break;
	case MenuFindANote:
		XPCreateFindNotePrompt(notes, FALSE);
		break;
	case MenuEnvelopes:
		XPEnvelope();
		break;
	case MenuProjects:
		XPProject();
		break;
	case MenuPilotTool:
		XPPilot();
		break;
	case MenuToolTips:
		ToggleTips();
		break;
	case MenuAbout:
		XPAbout();
		break;
	case MenuExit:
		XPShutdown();
		break;
	}
}


/*========================================================================
 *	Name:			ToggleTips
 *	Prototype:	ToggleTips()
 *
 *	Description:
 *	Toggle between Tool Tips On and Tool Tips Off.
 *
 *========================================================================*/
static void
ToggleTips()
{
	int		nargs;
	Arg		args[2];
	int		state;
	XmString	xm_string;

	nargs = 0;
	XtSetArg( args[nargs], XmNuserData, &state); nargs++;
	XtGetValues(tool_tips_widget, args, nargs);

	nargs = 0;
	if ( state == XP_TIPS_ON )
	{
		xm_string = XmStringCreateSimple(XP_S_ENABLE_TIPS);
		XtSetArg( args[nargs], XmNuserData, XP_TIPS_OFF); nargs++;
		app_res.no_tips = True;
	}
	else
	{
		xm_string = XmStringCreateSimple(XP_S_DISABLE_TIPS);
		XtSetArg( args[nargs], XmNuserData, XP_TIPS_ON); nargs++;
		app_res.no_tips = False;
	}
	XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetValues(tool_tips_widget, args, nargs);
	XmStringFree(xm_string);
}
