/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995, 1996, 1997, 1998.
 *========================================================================
 *              FILE NAME: name.c 
 *            DESCRIPTION: handle note naming requests made by the user.
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPNameNote
 *      PRIVATE FUNCTIONS: ClearName, UpdateName
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_NAME_C

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include "motif.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void			ClearName();
static void			UpdateName();


/* === Global Variables === */
extern Widget		Top_Level;


/* === Static Variables === */
static Widget		dialogwidget;
static Widget		namewidget;


/*========================================================================
 *	Name:			XPNameNote
 *	Prototype:	XPNameNote(StickyNote *pn)
 *					
 *	Description:
 * Put up a window prompt for the notes name
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPNameNote(
	Widget		widget,
	int			index
)
{
#ifdef DEBUG
	char	fname[]="XPNameNote";
#endif

	Widget			vbox;
	Widget			confirm_button, cancel_button;
	StickyNote		*pn;
	char				buf[256];
	Arg				args[5];
	int				nargs;
	Dimension		width, height;


	/*
	 * Allow only one copy at a time.
	 */
	if ( namewidget != NULL )
		return;

	/*
	 * Grab the note structure.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Create a dialog, set its title, and position it at the cursor.
	 */
	DBGPrintf(DBG_INFO,("Creating find dialog.\n"));
	namewidget = XPCreateGenericShell(StickyNoteDialog, 
					ACCEPT_BUTTON | CANCEL_BUTTON );

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	sprintf(buf, "*GenericForm%d", (int)namewidget );
	vbox = XtNameToWidget(namewidget, buf);
	sprintf(buf, "*GenericCancelB%d", (int)namewidget);
	cancel_button = XtNameToWidget(namewidget, buf);
	sprintf(buf, "*GenericAcceptB%d", (int)namewidget);
	confirm_button = XtNameToWidget(namewidget, buf);
 
	/*
	 * An text entry widget in which the name will go.
	 * If the text already has a name, use it as the default text.
	 */
	nargs=0;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
	dialogwidget = XmCreateTextField(
							vbox,
							"textField",
							args, nargs
							);
	XtManageChild(dialogwidget);

	if ( pn->pn_name != NULL )
		XmTextFieldSetString( dialogwidget, pn->pn_name );

	/*
	 * Confirmation button.
	 */
	XtAddCallback( confirm_button, XmNactivateCallback, 
			UpdateName, (XtPointer)index);

	/*
	 * Cancellation button.
	 */
	XtAddCallback( cancel_button, XmNactivateCallback, 
			ClearName, (XtPointer)NULL);

	/*
	 * Stick the dialog in the middle of the notes shell widget.
	 */
	XtRealizeWidget( namewidget );
	XPCenterWindow(namewidget, pn->pn_shellwidget, namewidget);

	XtPopup(namewidget, XtGrabNone);

	/*
	 * This bit of code forces the pop up window to stay its
	 * initial size, despite the users attempts to resize it.
	 * By doing this, I avoid having to muck extensively with
	 * aligning objects after a resize.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, &width); nargs++;
	XtSetArg ( args[nargs], XmNheight, &height); nargs++;
	XtGetValues ( namewidget, args, nargs);
	nargs = 0;
	XtSetArg ( args[nargs], XmNmaxWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNmaxHeight, height); nargs++;
	XtSetArg ( args[nargs], XmNminWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNminHeight, height); nargs++;
	XtSetValues ( namewidget, args, nargs); 

}

/*
 * ClearName - get rid of the confirmation box.
 */
static void
ClearName()
{
	if ( namewidget != NULL )
	{
		XtDestroyWidget( namewidget );
		namewidget = NULL;
	}
}


static void
UpdateName(
	Widget		widget,
	int			index,
	XtPointer	data
)
{
	char			*text;
	StickyNote	*pn;
	Arg			args[2];
	int			nargs;

	if ((pn = XPFindNote(index)) == NULL)
	{
		XPErrPopUp(XP_S_NOTE_PTR_MISSING);
	}
	else
	{
		text = XmTextFieldGetString( dialogwidget );
		nargs = 0;
		XtSetArg ( args[nargs], XtNtitle, text); nargs++;
		XtSetValues( pn->pn_shellwidget, args, nargs );

		if ( pn->pn_name != NULL )
		{
			free(pn->pn_name);
			pn->pn_name = XPSafeAlloc(strlen(text)+1);
			strcpy(pn->pn_name, text);
		}
	}

	ClearName();
}

