/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: note.c
 *            DESCRIPTION: note handler for XNotesPlus
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPCreateNewNote, XPAllocNote, XPFindNote,
 *								 : XPMakeNoteWidget, XPDestroyNote, XPSaveAllNotes,
 *								 : XPRaiseAllNotes, XPHideAllNotes, XPUnHideAllNotes
 *								 : XPLowerAllNotes, XPToggleRaise, XPToggleShow,
 *								 : XPMakeNewNote, XPSetSaveSensitive, XPCascadeNotes
 *								 : XPSaveNote
 *      PRIVATE FUNCTIONS: NoteIndex, GetNotePosition
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_NOTES_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <string.h>
#include <time.h>
#include <sys/param.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>

#include "motif.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */
void XPCreateNewNote();
void XPRaiseNotes();
void XPRaiseAllNotes();
void XPLowerNotes();
void XPLowerAllNotes();
void XPSaveAllNotes();
void XPPopUpList();
void XPUnHideAllNotes();
void XPHideAllNotes();
void XPCascadeNotes();
void XPMakeNoteListCB();
void XPMakeNoteWidget();
void XPAnchorNote();
void XPUnAnchorNote();
void XPDestroyNote();
void XPSetSaveSensitive();
void XPTearOff();
void XPHideNote();
void XPEraseNote();
void XPSaveNote();
StickyNote	*XPFindNote();
StickyNote	*XPAllocNote();


/* === Private routine prototypes === */
static int 			NoteIndex();
static void 		GetNotePosition();
static void 		DestroyNoteHandler();


/* === Global Variables === */
extern AppRes			app_res;
extern Display			*display;
extern Widget			Top_Level;
extern ProjectType	*projects_list;

StickyNote	*notes = NULL;
Pixmap		alarmon_xbm;
Pixmap		alarmoff_xbm;
Pixmap		blank_xbm;
Pixmap		anchoron_xbm;
Pixmap		anchoroff_xbm;
Pixmap		file_xbm;
Pixmap		notes_xbm;

static int	raisestatus = 1;
static int	showstatus = 1;


/* === Static Variables === */
static char		*File_Menu_Items[] = {
	XP_FILE_SAVE,
	XP_FILE_OPEN,
	XP_FILE_EXPORT,
	XP_FILE_FIND,
	XP_FILE_PRINT,
	XP_FILE_EMAIL,
	NULL
	};
static char		*Options_Menu_Items[] = {
	XP_OPTIONS_HIDE,
	XP_OPTIONS_ERASE,
	XP_OPTIONS_DESTROY,
	XP_OPTIONS_NAME,
	XP_OPTIONS_PROJECT,
	XP_OPTIONS_CALENDAR,
	XP_OPTIONS_DATE,
	NULL
	};
static char		*Alarm_Menu_Items[] = {
	XP_ALARM_ON,
	XP_ALARM_OFF,
	NULL
	};
static char		*Anchor_Menu_Items[] = {
	XP_ANCHOR_ON,
	XP_ANCHOR_OFF,
	NULL
	};



/*========================================================================
 *	Name:			XPCreateNewNote 
 *	Prototype:	XPCreateNewNote(int type, StickyNote **caller_pn)
 *					
 *	Description:
 * Create a new note of the specified type.
 *
 *	Input Arguments:
 *	int	type			used as an index to determine the initial size of 
 *							the note.
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPCreateNewNote(
	int 			type,
	StickyNote	**caller_pn
)
{
#ifdef DEBUG
	char	*fname="XPCreateNewNote()";
#endif

	static 	int hpi = 0;
	static 	int wpi = 0;
	register StickyNote *pn;
	register int hmm, wmm, hpixel, wpixel;

	DBGEnter();

	/*
	 * Find out the number of pixels per inch on the screen.  We
	 * can get the number of pixels, and the size in millimeters.
	 * Then we convert to pixels/inch using the formula
	 *
	 *  2.54 cm     10 mm     pixels     pixels
	 *	--------- x ------- x -------- = --------
	 *	  inch        cm         mm        inch
	 *
	 * The only problem with this is that some servers (like Xsun)
	 * lie about what these numbers really are.
	 */
	if ((hpi == 0) || (wpi == 0)) 
	{
		hpixel = DisplayHeight(display, DefaultScreen(display));
		wpixel = DisplayWidth(display, DefaultScreen(display));

		hmm = DisplayHeightMM (display, DefaultScreen(display));
		wmm = DisplayWidthMM(display, DefaultScreen(display));

		hpi = (int) ((25.4 * hpixel) / (float) hmm + 0.5);
		wpi = (int) ((25.4 * wpixel) / (float) wmm + 0.5);
	}

	/*
	 * Calculate sizes for the note.
	 */
	switch (type) {
	case StickyNote_1p5x2:
		hpixel = 1.5 * hpi;
		wpixel = 2 * wpi;
		break;
	case StickyNote_2x3:
		hpixel = 2 * hpi;
		wpixel = 3 * wpi;
		break;
	case StickyNote_3x3:
		hpixel = 3 * hpi;
		wpixel = 3 * wpi;
		break;
	case StickyNote_3x4:
		hpixel = 3 * hpi;
		wpixel = 4 * wpi;
		break;
	case StickyNote_3x5:
		hpixel = 3 * hpi;
		wpixel = 5 * wpi;
		break;
	case StickyNote_4x6:
		hpixel = 4 * hpi;
		wpixel = 6 * wpi;
		break;
	default:
		hpixel = 1.5 * hpi;
		wpixel = 2 * wpi;
		break;
	}

	/*
	 * Allocate a new note structure.
	 */
	pn = XPAllocNote(NewIndex);

	/*
	 * Set the text window size.
	 */
	pn->pn_textwidth = wpixel;
	pn->pn_textheight = hpixel;

	/*
	 * Make the widget for the note.
	 */
	XPMakeNoteWidget(pn);

	/*
	 * Return the pointer to the new note if the caller requested it.
	 */
	if ( caller_pn != NULL ) 
		*caller_pn = pn;
}


/*========================================================================
 *	Name:			XPAllocNote 
 *	Prototype:	StickyNote *XPAllocNote ( int index )
 *					
 *	Description:
 * Allocate a new note structure and insert in into the list of notes.
 *
 *	Input Arguments:
 *	int	index				
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
StickyNote *
XPAllocNote(
	int index
)
{
#ifdef DEBUG
	char	*fname="XPAllocNote()";
#endif

	register StickyNote	*pn;
	char						label[15];		
	time_t					time_tp;
	struct tm				*tm_tp;

	/*
	 * Allocate a structure.
	 */
	if (notes == NULL) 
	{
		notes = (StickyNote *) XPSafeAlloc(sizeof(StickyNote));
		pn = notes;
	}
	else 
	{
		for (pn = notes; pn->pn_next != NULL; pn = pn->pn_next)
			;

		pn->pn_next = (StickyNote *) XPSafeAlloc(sizeof(StickyNote));
		pn = pn->pn_next;
	}
	DBGPrintf(DBG_INFO,("pn: 0x%08x\n", pn));

	/*
	 * Initialize the note.
	 */
	pn->pn_positionit = FALSE;
	pn->pn_hidden = FALSE;
	pn->pn_anchor = FALSE;
	pn->pn_type = 0;
	pn->pn_alarm = FALSE;
	pn->pn_alarm_audio = NULL;
	pn->pn_alarm_repeat = 0;
	pn->pn_alarm_volume = 50;
	pn->pn_project = NULL;

	/*
	 * Initial position is undefined.
	 */
	pn->pn_shellx = 0xffff;
	pn->pn_shelly = 0xffff;

	/* set todays date in the alarm entries */
	time(&time_tp);
	tm_tp = localtime(&time_tp);
	pn->pn_alarm_mon = tm_tp->tm_mon;
	pn->pn_alarm_day = tm_tp->tm_mday;
	pn->pn_alarm_hour = tm_tp->tm_hour;
	pn->pn_alarm_min = tm_tp->tm_min;

	DBGPrintf(DBG_INFO,("month: %2d\n", pn->pn_alarm_mon));
	DBGPrintf(DBG_INFO,("day  : %2d\n", pn->pn_alarm_day));
	DBGPrintf(DBG_INFO,("hour : %2d\n", pn->pn_alarm_hour));
	DBGPrintf(DBG_INFO,("min  : %2d\n", pn->pn_alarm_min));

	/*
	 * Don't hide by default.
	 * Assign a default text size.
	 * Allocate storage for the text.
	 */
	pn->pn_hide_index = -1;
	pn->pn_textsize = app_res.buf_size;
	pn->pn_text = NULL;

	/*
	 * If the index number was given, use it.  Otherwise,
	 * get a new index number.
	 */
	pn->pn_index = (index == NewIndex ? NoteIndex() : index);

	/* save the name of the note */
	sprintf (label, "Note %d", pn->pn_index);
	pn->pn_name = XPSafeAlloc ( strlen(label) + 1 );
	strcpy(pn->pn_name, label);

	return(pn);
}


/*========================================================================
 *	Name:			NoteIndex 
 *	Prototype:	int NoteIndex ()
 *					
 *	Description:
 * Find the lowest free index number.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
static int
NoteIndex()
{
	register int index;
	register StickyNote *pn;

	/*
	 * This is O(n**2), but the list should be small.
	 */
	for (index = 1; ; index++) 
	{
		if ((pn = XPFindNote(index)) == NULL)
			return(index);
	}
}


/*========================================================================
 *	Name:			XPFindNote
 *	Prototype:	StickyNote *XPFindNote (int index)
 *					
 *	Description:
 * Find the note structure with the given index number.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
StickyNote *
XPFindNote(
	register int index
)
{
	register StickyNote *pn;

	for (pn = notes; pn != NULL; pn = pn->pn_next) 
	{
		if (pn->pn_index == index)
			return(pn);
	}

	return(NULL);
}


/*========================================================================
 *	Name:			XPMakeNoteWidget 
 *	Prototype:	void XPMakeNoteWidget ( StickyNote *pn )
 *					
 *	Description:
 * Create a new note window.
 *
 *	Input Arguments:
 *		StickyNote		*pn		pointer to note's data structure
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPMakeNoteWidget(
	StickyNote *pn
)
{
#ifdef DEBUG
	char				*fname="XPMakeNoteWidget";
#endif

	Arg			args[15];
	int			nargs;
	Widget		menu_items;
	Widget		file_menu, options_menu;
	Widget		vbox, hbox, text;
	Widget		alarm_button, alarm_menu;
	Widget		anchor_button, anchor_menu;
	Widget		file_button;
	Widget		notes_button;
	XmString		xm_string;
	ProjectType	*item;

	XpmAttributes	attributes;
	int				i;
	Position			shellx, shelly;

	static char		firsttime = TRUE;

	DBGEnter();

	/*
	 * Make the shell widget for this note.  We just use
	 * a pop-up shell for this.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XtNx, pn->pn_shellx ); nargs++;
	XtSetArg ( args[nargs], XtNy, pn->pn_shelly ); nargs++;
	XtSetArg ( args[nargs], XmNwidth, pn->pn_textwidth); nargs++;
	XtSetArg ( args[nargs], XmNheight, pn->pn_textheight); nargs++;
	XtSetArg ( args[nargs], XtNtitle, pn->pn_name ); nargs++;
	XtSetArg ( args[nargs], XtNallowShellResize, True ); nargs++;
	XtSetArg ( args[nargs], XmNkeyboardFocusPolicy, XmPOINTER ); nargs++;
	XtSetArg ( args[nargs], XmNdeleteResponse, XmDO_NOTHING ); nargs++;
	pn->pn_shellwidget = XtCreatePopupShell (
					"StickyNote",
					transientShellWidgetClass,
					Top_Level,
					args, nargs
					);

	/* A form inside which the menubar and text area will go */
	nargs=0;
	XtSetArg ( args[nargs], XtNborderWidth, 0); nargs++;
   vbox = XmCreateForm(
         pn->pn_shellwidget,
         "Form",
         args, nargs
         );
   XtManageChild(vbox);

	/* A menubar, and the alarm and anchor icons */
   nargs = 0;
   XtSetArg ( args[nargs], XtNborderWidth, 0 ); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNheight, 21 ); nargs++;
   XtSetArg ( args[nargs], XmNresizeHeight, False ); nargs++;
   XtSetArg ( args[nargs], XmNspacing, 0 ); nargs++;
   XtSetArg ( args[nargs], XmNmarginHeight, 0 ); nargs++;
   hbox = XmCreateMenuBar (
      		vbox,
      		"menuBar",
      		args, nargs
      		);
   XtManageChild(hbox);

	/*
	 * Only create the pixmaps once - we can reuse them.
	 */
	if ( firsttime )
	{
   	XPCreatePixmap(&alarmon_xbm, NULL, XP_ALARM_ON_XPM, 
			&attributes, vbox);
   	XPCreatePixmap(&alarmoff_xbm, NULL, XP_ALARM_OFF_XPM, 
			&attributes, vbox);
   	XPCreatePixmap(&anchoron_xbm, NULL, XP_ANCHOR_XPM, 
			&attributes, vbox);
   	XPCreatePixmap(&anchoroff_xbm, NULL, XP_ANCHOR_OFF_XPM, 
			&attributes, vbox);
   	XPCreatePixmap(&blank_xbm, NULL, XP_ALARM_OFF_XPM, 
			&attributes, vbox);
   	XPCreatePixmap(&file_xbm, NULL, XP_FILE_XPM, 
			&attributes, vbox);
   	XPCreatePixmap(&notes_xbm, NULL, XP_NOTES_XPM, 
			&attributes, vbox);

		firsttime = FALSE;
	}

	if ( pn->pn_alarm )
		pn->pn_alarm_pixmapid = alarmon_xbm;
	else
		pn->pn_alarm_pixmapid = alarmoff_xbm;

	/*
	 * The Alarm button.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelType, XmPIXMAP); nargs++;
	XtSetArg ( args[nargs], XmNlabelPixmap, pn->pn_alarm_pixmapid); nargs++;
   pn->pn_alarmwidget = 
		alarm_button = XtCreateManagedWidget(
				"alarmButton",
				xmCascadeButtonWidgetClass,
				hbox,
				args, nargs
				);


	/*
	 * The anchor button.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelType, XmPIXMAP); nargs++;
	if ( pn->pn_anchor )
	{
		XtSetArg ( args[nargs], XmNlabelPixmap, anchoron_xbm); nargs++;
	}
	else
	{
		XtSetArg ( args[nargs], XmNlabelPixmap, anchoroff_xbm); nargs++;
	}
	pn->pn_anchorwidget = 
	anchor_button = XtCreateManagedWidget(
				"anchorButton",
				xmCascadeButtonWidgetClass,
				hbox,
				args, nargs
				);

	/*
	 * Add the alarm and anchor menus.
	 */
	alarm_menu = XmCreatePulldownMenu(
				hbox,
				"menu",
				NULL, 0);
	anchor_menu = XmCreatePulldownMenu(
				hbox,
				"menu",
				NULL, 0);

	nargs=0;
	XtSetArg ( args[nargs], XmNsubMenuId, alarm_menu); nargs++;
	XtSetValues ( alarm_button, args, nargs );
	nargs=0;
	XtSetArg ( args[nargs], XmNsubMenuId, anchor_menu); nargs++;
	XtSetValues ( anchor_button, args, nargs );

	for(i = 0; Alarm_Menu_Items[i] != NULL; i++)
	{
		xm_string = XmStringCreateSimple(Alarm_Menu_Items[i]);
		switch (i) 
		{
			case XP_ALARM_ON_ID:
				nargs=0;
				XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
				menu_items = XtCreateManagedWidget(
								"alarmon_button",
								xmPushButtonWidgetClass,
								alarm_menu,
								args, nargs);
				XmStringFree(xm_string);
				pn->pn_settimewidget = menu_items;

				/* Add a callback for this item. */
				XtAddCallback( menu_items, XmNactivateCallback, 
					XPCreateTimerPrompt, (XtPointer)pn->pn_index);
				break;

			case XP_ALARM_OFF_ID:
				nargs=0;
				XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
				menu_items = XtCreateManagedWidget(
								"alarmoff_button",
								xmPushButtonWidgetClass,
								alarm_menu,
								args, nargs);
				XmStringFree(xm_string);
				pn->pn_unsettimewidget = menu_items;

				/* Add a callback for this item. */
				XtAddCallback( menu_items, XmNactivateCallback, 
					XPAlarmOff, (XtPointer)pn->pn_index);
				break;

		}
	}

	for(i = 0; Anchor_Menu_Items[i] != NULL; i++)
	{
		xm_string = XmStringCreateSimple(Anchor_Menu_Items[i]);
		switch (i) 
		{
			case XP_ANCHOR_ON_ID:
				nargs=0;
				XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
				menu_items = XtCreateManagedWidget(
								"anchor_button",
								xmPushButtonWidgetClass,
								anchor_menu,
								args, nargs);
				XmStringFree(xm_string);
				pn->pn_anchormenu = menu_items;

				/* Add a callback for this item. */
				XtAddCallback( menu_items, XmNactivateCallback, 
					XPAnchorNote, (XtPointer)pn->pn_index);
				break;

			case XP_ANCHOR_OFF_ID:
				nargs=0;
				XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
				menu_items = XtCreateManagedWidget(
								"unanchor_button",
								xmPushButtonWidgetClass,
								anchor_menu,
								args, nargs);
				XmStringFree(xm_string);
				pn->pn_unanchormenu = menu_items;

				/* Add a callback for this item. */
				XtAddCallback( menu_items, XmNactivateCallback, 
					XPUnAnchorNote, (XtPointer)pn->pn_index);
				break;
		}
	}

	/*
	 * Set anchor sensitivity appropriately.
	 */
	if (pn->pn_anchor)
	{
		XtSetSensitive ( pn->pn_anchormenu, False );
		XtSetSensitive ( pn->pn_unanchormenu, True );
	}
	else
	{
		XtSetSensitive ( pn->pn_anchormenu, True );
		XtSetSensitive ( pn->pn_unanchormenu, False );
	}

	if (pn->pn_alarm)
	{
		XtSetSensitive ( pn->pn_settimewidget, False );
		XtSetSensitive ( pn->pn_unsettimewidget, True );
	}
	else
	{
		XtSetSensitive ( pn->pn_settimewidget, True );
		XtSetSensitive ( pn->pn_unsettimewidget, False );
	}

	/*
	 * Create the menu bar, with all its menus and buttons.  Add
	 * it to the note window.
	 */
	file_menu = XmCreatePulldownMenu(
				hbox,
				"menu",
				NULL, 0);
	options_menu = XmCreatePulldownMenu(
				hbox,
				"menu",
				NULL, 0);

	nargs=0;
	XtSetArg ( args[nargs], XmNlabelType, XmPIXMAP); nargs++;
	XtSetArg ( args[nargs], XmNlabelPixmap, file_xbm); nargs++;
	XtSetArg ( args[nargs], XmNsubMenuId, file_menu); nargs++;
	file_button = XtCreateManagedWidget(
				"fileButton",
				xmCascadeButtonWidgetClass,
				hbox,
				args, nargs
				);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelType, XmPIXMAP); nargs++;
	XtSetArg ( args[nargs], XmNlabelPixmap, notes_xbm); nargs++;
	XtSetArg ( args[nargs], XmNsubMenuId, options_menu); nargs++;
	notes_button = XtCreateManagedWidget(
				"notesButton",
				xmCascadeButtonWidgetClass,
				hbox,
				args, nargs
				);

	/* The actual menus */
	for(i = 0; File_Menu_Items[i] != NULL; i++)
	{
		xm_string = XmStringCreateSimple(File_Menu_Items[i]);
		switch(i)
		{
			case XP_FILE_SAVE_ID:
				nargs=0;
				XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
				menu_items = XtCreateManagedWidget(
								"options_menu",
								xmPushButtonWidgetClass,
								file_menu,
								args, nargs);
				XmStringFree(xm_string);
				pn->pn_savewidget = menu_items;

				/* Add a callback for this item. */
				XtAddCallback( menu_items, XmNactivateCallback, 
					XPSaveNote, (XtPointer)pn->pn_index);

				break;

			case XP_FILE_OPEN_ID:
				nargs=0;
				XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
				menu_items = XtCreateManagedWidget(
								"open_file_button",
								xmPushButtonWidgetClass,
								file_menu,
								args, nargs);
				XmStringFree(xm_string);

				/* Add a callback for this item. */
				XtAddCallback( menu_items, XmNactivateCallback, 
					XPOpenNote, (XtPointer)pn->pn_index);
				break;

			case XP_FILE_EXPORT_ID:	
				nargs=0;
				XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
				menu_items = XtCreateManagedWidget(
								"export_file_button",
								xmPushButtonWidgetClass,
								file_menu,
								args, nargs);
				XmStringFree(xm_string);

				/* Add a callback for this item. */
				XtAddCallback( menu_items, XmNactivateCallback, 
					XPExportNote, (XtPointer)pn->pn_index);
				break;

			case XP_FILE_FIND_ID:	
				nargs=0;
				XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
				menu_items = XtCreateManagedWidget(
								"find_file_button",
								xmPushButtonWidgetClass,
								file_menu,
								args, nargs);
				XmStringFree(xm_string);

				/* Add a callback for this item. */
				XtAddCallback( menu_items, XmNactivateCallback, 
					XPFindTextPrompt, (XtPointer)pn->pn_index);
				break;

			case XP_FILE_PRINT_ID:
				/*
				 * If the print command was not found, we dont' give access
				 * to this feature.  After all, it wouldn't work anyway, right?
				 */
				if ( app_res.print_cmd != NULL )
				{
					nargs=0;
					XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
					menu_items = XtCreateManagedWidget(
								"print_file_button",
								xmPushButtonWidgetClass,
								file_menu,
								args, nargs);
					XmStringFree(xm_string);

					/* Add a callback for this item. */
					XtAddCallback( menu_items, XmNactivateCallback, 
						XPPrintNote, (XtPointer)pn->pn_index);
				}
				break;

			case XP_FILE_EMAIL_ID:
				/*
				 * If the print command was not found, we dont' give access
				 * to this feature.  After all, it wouldn't work anyway, right?
				 */
				if ( app_res.email_cmd != NULL )
				{
					nargs=0;
					XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
					menu_items = XtCreateManagedWidget(
								"email_file_button",
								xmPushButtonWidgetClass,
								file_menu,
								args, nargs);
					XmStringFree(xm_string);

					/* Add a callback for this item. */
					XtAddCallback( menu_items, XmNactivateCallback, 
						XPCreateEmailNotePrompt, (XtPointer)pn->pn_index);
				}
				break;
		}
	}
	for(i = 0; Options_Menu_Items[i] != NULL; i++)
	{

		xm_string = XmStringCreateSimple(Options_Menu_Items[i]);
		switch(i)
		{
			case XP_OPTIONS_HIDE_ID:
				nargs=0;
				XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
				menu_items = XtCreateManagedWidget(
								"hide_button",
								xmPushButtonWidgetClass,
								options_menu,
								args, nargs);
				XmStringFree(xm_string);

				/* Add a callback for this item. */
				XtAddCallback( menu_items, XmNactivateCallback, 
					XPHideNote, (XtPointer)pn->pn_index);
				break;

			case XP_OPTIONS_ERASE_ID:
				nargs=0;
				XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
				menu_items = XtCreateManagedWidget(
								"erase_button",
								xmPushButtonWidgetClass,
								options_menu,
								args, nargs);
				XmStringFree(xm_string);

				/* Add a callback for this item. */
				XtAddCallback( menu_items, XmNactivateCallback, 
					XPEraseNote, (XtPointer)pn->pn_index);
				break;

			case XP_OPTIONS_DESTROY_ID:
				nargs=0;
				XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
				menu_items = XtCreateManagedWidget(
								"destroy_button",
								xmPushButtonWidgetClass,
								options_menu,
								args, nargs);
				XmStringFree(xm_string);

				/* Add a callback for this item. */
				XtAddCallback( menu_items, XmNactivateCallback, 
					XPDestroyNote, (XtPointer)pn);
				break;

			case XP_OPTIONS_NAME_ID:
				nargs=0;
				XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
				menu_items = XtCreateManagedWidget(
								"name_button",
								xmPushButtonWidgetClass,
								options_menu,
								args, nargs);
				XmStringFree(xm_string);

				/* Add a callback for this item. */
				XtAddCallback( menu_items, XmNactivateCallback, 
					XPNameNote, (XtPointer)pn->pn_index);
				break;

			case XP_OPTIONS_PROJ_ID:
				nargs=0;
				XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
				menu_items = XtCreateManagedWidget(
								"project_button",
								xmPushButtonWidgetClass,
								options_menu,
								args, nargs);
				XmStringFree(xm_string);

				/* Add a callback for this item. */
				XtAddCallback( menu_items, XmNactivateCallback, 
					XPNoteProject, (XtPointer)pn->pn_index);
				break;

			case XP_OPTIONS_CALENDAR_ID:
				/*
				 * If the calendar command was not found, we dont' give access
				 * to this feature.  After all, it wouldn't work anyway, right?
				 */
				if ( app_res.calendar_cmd != NULL )
				{
					nargs=0;
					XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
					menu_items = XtCreateManagedWidget(
								"calendar_button",
								xmPushButtonWidgetClass,
								options_menu,
								args, nargs);
					XmStringFree(xm_string);

					/* Add a callback for this item. */
					XtAddCallback( menu_items, XmNactivateCallback, 
						XPInsertCalendar, (XtPointer)pn->pn_index);
				}
				break;

			case XP_OPTIONS_DATE_ID:
				nargs=0;
				XtSetArg( args[nargs], XmNlabelString, xm_string); nargs++;
				menu_items = XtCreateManagedWidget(
								"date_button",
								xmPushButtonWidgetClass,
								options_menu,
								args, nargs);
				XmStringFree(xm_string);

				/* Add a callback for this item. */
				XtAddCallback( menu_items, XmNactivateCallback, 
					XPInsertDates, (XtPointer)pn->pn_index);
				break;
		}
	}

	/*
	 * Add the text field to the box.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNeditMode, XmMULTI_LINE_EDIT); nargs++;
	XtSetArg ( args[nargs], XmNeditable, True); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, hbox); nargs++;
   XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNvisualPolicy, XmCONSTANT); nargs++;
   XtSetArg ( args[nargs], XmNscrollingPolicy, XmAUTOMATIC); nargs++;
   XtSetArg ( args[nargs], XmNscrollBarDisplayPolicy, XmAS_NEEDED); nargs++;
   XtSetArg ( args[nargs], XmNscrollHorizontal, False); nargs++;
   XtSetArg ( args[nargs], XmNspacing, 0); nargs++;
   XtSetArg ( args[nargs], XmNborder, 0); nargs++;
   XtSetArg ( args[nargs], XmNwordWrap, True); nargs++;
	text = XmCreateScrolledText(
				vbox,
				"scrolledText",
				args, nargs);
	XtManageChild(text);
	pn->pn_textwidget = text;

	/*
	 * Set the notes color based on the project, if any.
	 */
	if ( pn->pn_project != NULL )
	{
		if ( strcmp(pn->pn_project, "NONE") != 0 )
		{
			item = projects_list;
			while ( item != NULL )
			{
				if ( strcmp ( item->name, pn->pn_project) == 0 )
				{
					XPUpdateColorBox(pn->pn_shellwidget, 
						pn->pn_textwidget, item->color);
					break;
				}
				item = item->next;
			}
		}
		else
			XPUpdateColorBox(pn->pn_shellwidget, 
					pn->pn_textwidget, "Yellow");
	}
	else
		XPUpdateColorBox(pn->pn_shellwidget, 
				pn->pn_textwidget, "Yellow");

	/*
	 * Add the existing text, if any, to the note.
	 */
	if ( pn->pn_text != NULL )
		XmTextSetString(text, pn->pn_text);

	/*
	 * Event handlers for the text area.
	 */
	XtAddCallback( text, XmNvalueChangedCallback, 
		XPSetSaveSensitive, (XtPointer)pn);

	/*
	 * Realize note.
	 */
	XtRealizeWidget(pn->pn_shellwidget);
	if ( ( pn->pn_shellx == 0xffff ) && ( pn->pn_shellx == 0xffff ) )
		XPCenterWindow(pn->pn_shellwidget, NULL, pn->pn_shellwidget);

	/*
	 * Show the note.
	 */
	if ( pn->pn_hidden == False )
		XtPopup( pn->pn_shellwidget, XtGrabNone );

#ifdef DEBUG
	/*
	 * Save the notes shell coordinates.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XtNx, &shellx); nargs++;
	XtSetArg ( args[nargs], XtNy, &shelly); nargs++;
	XtGetValues ( pn->pn_shellwidget, args, nargs);
	DBGPrintf(DBG_INFO,("Saving shell coords: %d/%d\n", (int)shellx, (int)shelly));
#endif
 
}


/*========================================================================
 *	Name:			XPDestroyNote and DestroyNoteHandler
 *					
 *	Description:
 * Destroy a note window.  The former just opens a confirmation box and
 * the latter is the routine that gets called if the user confirms the
 * destroy request.
 *
 *========================================================================*/
void
XPDestroyNote(
	Widget		widget,
	StickyNote	*pn,
	XtPointer	call_data
)
{

	/*
	 * Create a confirmation box.
	 */
	XPMsgWindow(
		XP_S_DESTROY_CONFIRM, XP_MSG_TYPE | XP_MSG_NOHELP, NULL, 
		pn->pn_shellwidget, DestroyNoteHandler, pn->pn_index, NULL, NULL);
}

void
DestroyNoteHandler(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
#ifdef DEBUG
	char		*fname="DestroyNoteHandler";
#endif

	register StickyNote	*pn, *prev;

	DBGEnter();
	DBGPrintf(DBG_INFO,("index=%d",index));

	/*
	 * Find the note we're destroying.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;
	
	DBGPrintf(DBG_INFO,("Found note."));

	if ( pn != NULL )
	{
		XtDestroyWidget(pn->pn_shellwidget);

		/*
		 * Get rid of the note structure.
		 */
		if (pn != notes) 
		{

			/*
			 * first, free up the name buffer
			 */
			if ( pn->pn_name != NULL )
				free ( pn->pn_name );
	
			for (prev = notes; prev->pn_next; prev = prev->pn_next) 
			{
				if (prev->pn_next == pn)
					break;
			}

			prev->pn_next = pn->pn_next;
		}
		else 
			notes = pn->pn_next;

		/*
		 * Get rid of the file.
		 */
		if (pn->pn_file) 
		{
			unlink(pn->pn_file);
			free(pn->pn_file);
		}

		/*
		 * Free the note text memory that was allocated.
		 */
		if ( pn->pn_text != NULL )
			free(pn->pn_text);
		free(pn);
	}
}


/*========================================================================
 *	Name:			XPSaveAllNotes
 *	Prototype:	void XPSaveAllNotes (int showmsg)
 *					
 *	Description:
 * Save all notes.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPSaveAllNotes(
	int	showmsg
)
{
	register StickyNote *pn;
	char found;

	found = FALSE;
	for (pn = notes; pn != NULL; pn = pn->pn_next)
	{
		XPSaveNote(pn->pn_shellwidget, pn->pn_index, NULL);
		found = TRUE;
	}

	if ( showmsg )
		if ( !found )
			XPErrPopUp("There are no notes to save.");
}


/*========================================================================
 *	Name:			XPSaveNote
 *	Prototype:	void XPSaveNote(Widget window, int index, 
 *							XtPointer call_data) 
 *					
 *	Description:
 * Use temporary files for saves in case the system crashes or xnotes
 * exits unexpectedly during a save.  This will preserve, at a minimum,
 * the last copy of the note.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPSaveNote(
	Widget		window,
	int			index,
	XtPointer	call_data
)
{
#ifdef DEBUG
	char		*fname="XPSaveNote()";
#endif

	register StickyNote	*pn;
	struct stat				stat_buf;

	Arg		args[5];
	int		nargs;
	FILE		*fp;
	int		len, shellx, shelly;
	char		saveit;
	char		tmpfile1[512], tmpfile2[512];
	char		*audio_file, *project;
	char		cmd[512];

	Dimension		texth, textw;
	Position			xpos, ypos;

	static char		*noaudio=XP_S_NO_AUDIO;
	static char		*noproject=XP_S_NO_PROJECT;

	DBGEnter();

	/*
	 * Find the note we're saving.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/* by default, we're not going to save the note */
	saveit = FALSE;

	/*
	 * If the note has changed, the sensitivity of the note
	 * save option will be True.
	 */
	if ( XtIsSensitive(pn->pn_savewidget) == TRUE )
		saveit = TRUE;

	/*
	 * Get the position of the shell window.
	 */
	GetNotePosition(pn->pn_shellwidget, &shellx, &shelly);
	nargs=0;
	XtSetArg( args[nargs], XmNx, &xpos); nargs++;
	XtSetArg( args[nargs], XmNy, &ypos); nargs++;
	XtGetValues( pn->pn_shellwidget, args, nargs );

	DBGPrintf(DBG_INFO,("xy pos:  %d/%d\n", xpos, ypos));
	DBGPrintf(DBG_INFO,("offsets: %d/%d\n", shellx, shelly));

	shellx += xpos;
	shelly += ypos;

	DBGPrintf(DBG_INFO,("absolute: %d/%d\n", shellx, shelly));
	
	if ( (pn->pn_shellx != shellx ) || (pn->pn_shelly != shelly ) )
	{
		pn->pn_shellx = shellx;
		pn->pn_shelly = shelly;
		saveit = TRUE;
	}

	/*
	 * if nothing about the note has changed, don't waste effor saving it
	 */
	if ( !saveit )
		return;

	/*
	 * If it doesn't have a file name, make one.
	 */
	if (pn->pn_file == NULL)
		pn->pn_file = XPMakeFname(pn->pn_index);


	/*
	 * Get the size of the text window.
	 */
	nargs=0;
	XtSetArg( args[nargs], XmNwidth, &textw); nargs++;
	XtSetArg( args[nargs], XmNheight, &texth); nargs++;
	XtGetValues( pn->pn_shellwidget, args, nargs );

	/*
	 * Get the length of the text in the window.
	 */
	if (pn->pn_text != NULL )
		free(pn->pn_text);
	pn->pn_text = XmTextGetString(pn->pn_textwidget);
	len = strlen(pn->pn_text);

	/*
	 * Create the temporary filenames.  These temporary files go in
	 * the same directory as the real notes to avoid problems with
	 * renaming files across file system boundaries, etc.
	 */
	sprintf(tmpfile1,"%s/xp1.%d", app_res.note_dir, (int)getpid());
	sprintf(tmpfile2,"%s/xp2.%d", app_res.note_dir, (int)getpid());

	/*
	 * Open the first temporary file for writing
	 */
	if ((fp = fopen(tmpfile1, "w")) == NULL) 
	{
		fprintf(stderr, "xnotes: ");
		perror(tmpfile1);
		return;
	}

	/*
	 * Determine what to output for the audio file and the project
	 * name.
	 */
	if ( pn->pn_alarm_audio == NULL )
		audio_file = noaudio;
	else
		audio_file = pn->pn_alarm_audio;
	if ( pn->pn_project == NULL )
		project = noproject;
	else
		project = pn->pn_project;

	/*
	 * Print out the information needed to recreate the note.
	 */
	fprintf(fp, 
		"%s %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %s %s\n%s\n", 
		StickyNoteMagic_v3, shellx, shelly, texth, textw, 
		len, pn->pn_hidden, pn->pn_anchor,
		DisplayHeight(display, DefaultScreen(display)),
		DisplayWidth(display, DefaultScreen(display)),
		pn->pn_alarm,
		pn->pn_alarm_mon, pn->pn_alarm_day, 
		pn->pn_alarm_hour, pn->pn_alarm_min, 
		pn->pn_type, pn->pn_alarm_repeat, pn->pn_alarm_volume,
		audio_file, project,
		pn->pn_name);

	/*
	 * Write out the text of the note.
	 */
	if (len)
		fwrite(pn->pn_text, sizeof(char), len, fp);

	fclose(fp);

	/* rename the old note (pn->pn_file, if it exists) to tmp2 */
	if ( stat(pn->pn_file, &stat_buf) == 0 )
	{
		if ( rename(pn->pn_file, tmpfile2) != 0 )
		{
			/* error - couldn't do save */
			sprintf(cmd, "xnotes: %s-%s", pn->pn_file, tmpfile2);
			perror(cmd);
			sprintf(cmd,"Couldn't rename %s\nto %s.\nSave of note failed.\n"
				"%s should contain valid copy of note.", 
				pn->pn_file, tmpfile2, tmpfile2);
			XPErrPopUp(cmd);
			return;
		}

		/* check that rename worked (due to NFS problems) */
		if ( stat(tmpfile2, &stat_buf) != 0 )
		{
			/* rename didn't work! Goddam NFS.... */
			sprintf(cmd,"Rename %s to\n %s failed.\n"
				"Might be NFS problems.\nSave of note failed.", 
				pn->pn_file, tmpfile2);
			XPErrPopUp(cmd);
			return;
		}
	}

	/* rename tmp1 note to old note */
	if ( rename(tmpfile1, pn->pn_file) != 0 )
	{
		/* error - couldn't do save - clean up */
		sprintf(cmd, "xnotes: %s-%s", tmpfile1, pn->pn_file);
		perror(cmd);
		sprintf(cmd,"Couldn't rename %s\nto %s.\n"
			"Save of note failed."
			"%s should contain valid copy of note.", 
			tmpfile1, pn->pn_file, tmpfile1);
		XPErrPopUp(cmd);
		return;
	}

	/* check that rename worked (due to NFS problems) */
	if ( stat(pn->pn_file, &stat_buf) != 0 )
	{
		/* rename didn't work! Goddam NFS.... */
		sprintf(cmd,"Rename %s to\n %s failed.\n"
			"Might be NFS problems.\nSave of note failed.", 
			tmpfile1, pn->pn_file);
		XPErrPopUp(cmd);
		return;
	}

	/* unlink tmp2 */
	if ( stat(tmpfile2, &stat_buf) == 0 )
	{
		if ( unlink(tmpfile2) != 0 )
		{
			sprintf(cmd,"Problems doing cleanup after save.\n"
				"%s could not be unlinked.\n", tmpfile2);
			XPErrPopUp(cmd);
		}
	}

	/* reset save button to be insensitive */
	XtSetSensitive ( pn->pn_savewidget, False );
}


/*========================================================================
 *	Name:			GetNotePosition
 *	Prototype:	GetNotePosition(Widget window, int *x, int *y)
 *					
 *	Description:
 * Find the position of the widget window, taking into
 *	account any borders stuck on by reparenting window
 *	managers.
 *
 *	Input Arguments:
 *		Widget		window		window for which coordinates are needed
 *		int			*x				where to save the x coordinate of window
 *		int			*y				where to save the y coordinate of window
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 * The old version of this used to try to calculate the offset based on
 * any borders added by the window manager.  This version uses
 * XTranslateCoordinates() to translate the absolute position of the 
 * notes shell widget from the shell of the window managers frame.  This
 * appears to do a better job of getting note positions correct.
 *
 * At least I think it does.
 *========================================================================*/
static void
GetNotePosition(
	Widget		widget,
	int			*x, 
	int			*y
)
{
#ifdef DEBUG
	char			*fname="GetNotePosition";
#endif

	Window		*children;
	int			status, nchildren;
	Window		here, parent, root;
	XWindowAttributes win_attributes;
	int			xoffset, yoffset;

	DBGEnter();

	/*
	 * Start with the X Window of the widget passed to us.
	 */
	parent = XtWindow(widget);

	/*
	 * Walk up the tree looking for a parent of this window whose
	 * parent is the root.  That'll either be this window (if there
	 * is no window manager window) or the window added by the
	 * window manager.
	 */
	do {
		here = parent;
		status = XQueryTree(display, here, &root, &parent, &children,
						&nchildren);

		if (!status)
			break;

		if (children)
			XFree(children);
	} while (parent != root);

	/*
	 * Now get the window attributes for this window.
	 */
	XGetWindowAttributes(display, here, &win_attributes);

	/*
	 * Translate the coordinates for the widget passed in relative
	 * to the windows whose attributes we just fetched.  The offset will
	 * be 0,0 if there is no window manager frame, otherwise the offsets
	 * will be the width of the left/right and height of the top/bottom
	 * window frame borders.
	 */
	XTranslateCoordinates(display, 
		here,
		XtWindow(widget),
		0,0, &xoffset, &yoffset, &parent);

	DBGPrintf(DBG_INFO,("xy offsets: %d/%d\n", xoffset, yoffset));

	/*
	 * Pass back the true absolute coordinates for the widget passed in.
	*x = win_attributes.x + xoffset;
	*y = win_attributes.y + yoffset;
	 */
	*x = xoffset;
	*y = yoffset;
}


/*========================================================================
 *	Name:			XPRaiseAllNotes
 *					
 *	Description:
 *	Raise all the notes by raising their shell windows.
 *					
 *	Global Variables:
 *	int	raisestatus		1: notes currently raised (used to toggle status)
 *========================================================================*/
void
XPRaiseAllNotes()
{
	register StickyNote *pn;
	char		found;

	found = False;
	raisestatus = 1;
	for (pn = notes; pn != NULL; pn = pn->pn_next)
	{
		XRaiseWindow(display, XtWindow(pn->pn_shellwidget));
		found = TRUE;
	}

	if ( !found )
		XPErrPopUp(XP_S_NO_NOTES_TO_RAISE);
}


/*========================================================================
 *	Name:			XPHideAllNotes
 *	Prototype:	void XPHideAllNotes()
 *					
 *	Description:
 * Hide all the notes.
 *					
 *	Global Variables:
 *	int	raisestatus		1: notes currently raised (used to toggle status)
 *========================================================================*/
void
XPHideAllNotes()
{
	register StickyNote *pn;
	char		found;

	found = False;
	showstatus = 0;
	for (pn = notes; pn != NULL; pn = pn->pn_next)
	{
		if (pn->pn_hidden == FALSE)
		{
			XtPopdown ( pn->pn_shellwidget );
			pn->pn_hidden = TRUE;
			found = TRUE;
		}
	}

	if ( !found )
		XPErrPopUp(XP_S_NO_NOTES_TO_HIDE);
}


/*========================================================================
 *	Name:			XPUnHideAllNotes
 *	Prototype:	void XPUnHideAllNotes()
 *					
 *	Description:
 * Unhide all hidden notes, if any.
 *					
 *	Global Variables:
 *	int	raisestatus		1: notes currently raised (used to toggle status)
 *========================================================================*/
void
XPUnHideAllNotes()
{
	register StickyNote *pn;
	char		found;

	found = FALSE;
	showstatus = 1;
	for (pn = notes; pn != NULL; pn = pn->pn_next)
		if ( pn->pn_hidden )
		{
			XtPopup(pn->pn_shellwidget, XtGrabNone);
			pn->pn_hidden = FALSE;
			found = TRUE;
		}

	if ( !found )
		XPErrPopUp(XP_S_NO_NOTES_TO_UNHIDE);
}


/*========================================================================
 *	Name:			XPLowerAllNotes
 *	Prototype:	void XPLowerAllNotes()
 *					
 *	Description:
 * Lower all the notes by lowering their shell windows.
 *					
 *	Global Variables:
 *	int	raisestatus		1: notes currently raised (used to toggle status)
 *========================================================================*/
void
XPLowerAllNotes()
{
	register StickyNote *pn;
	char		found;

	found = FALSE;
	raisestatus = 0;
	for (pn = notes; pn != NULL; pn = pn->pn_next)
	{
		XLowerWindow(display, 
			XtWindow(pn->pn_shellwidget));
		found = TRUE;
	}

	if ( !found )
		XPErrPopUp(XP_S_NO_NOTES_TO_LOWER);
}


/*========================================================================
 *	Name:			XPToggleRaise
 *	Prototype:	void XPToggleRaise()
 *					
 *	Description:
 * Toggle raise status
 *					
 *	Global Variables:
 *	int	raisestatus		1: notes currently raised (used to toggle status)
 *========================================================================*/
void
XPToggleRaise() 
{
	if(!showstatus) 
	{
		XPUnHideAllNotes(); 
		XPRaiseAllNotes();
	} 
	else
		raisestatus ? XPLowerAllNotes() : XPRaiseAllNotes();
}


/*========================================================================
 *	Name:			XPToggleShow
 *	Prototype:	void XPToggleShow()
 *					
 *	Description:
 * Toggle show status
 *					
 *	Global Variables:
 *	int	showstatus		1: Hide notes 0: show notes
 *========================================================================*/
void
XPToggleShow() 
{
	showstatus ? XPHideAllNotes() : XPUnHideAllNotes();
}


/*========================================================================
 *	Name:			XPMakeNewNote
 *	Prototype:	void XPMakeNewNote()
 *					
 *	Description:
 * This gets called when the user "tears off" a note from the plaid window.
 *					
 *	Global Variables:
 *========================================================================*/
void
XPMakeNewNote(
)
{
	XPCreateNewNote(StickyNote_1p5x2, NULL);
}


/*========================================================================
 *	Name:			XPSetSaveSensitive
 *	Prototype:	void XPSetSaveSensitive(Widget widget, XtPointer data)
 *					
 *	Description:
 * This gets called when the state of the note changes.  "State" can be
 * changes to text, anchoring, alarms, and so forth.
 *					
 *	Global Variables:
 *========================================================================*/
void
XPSetSaveSensitive(
	Widget		widget,
	XtPointer	data,
	XtPointer	caller_data
)
{
	StickyNote	*pn;

	pn = (StickyNote *)data;
	XtSetSensitive(pn->pn_savewidget, True);
}


/*========================================================================
 *	Name:			XPCascadeNotes
 *	Prototype:	void XPCascadeNotes()
 *					
 *	Description:
 * Cascade the notes on any anchor notes.
 * This isn't the best, but it distributes the notes among the notes
 * marked as anchors fairly evenly.
 *					
 *	Global Variables:
 *========================================================================*/
void
XPCascadeNotes()
{
#ifdef DEBUG
	char *fname="XPCascadeNotes";
#endif

	register StickyNote	*pn;
	PNLinks 					*linklist, *pnlink, *prevlink;
	Arg						args[5];
	int						nargs;
	int						shellx, shelly;
	Position					xpos, ypos;

	DBGEnter();

	/*
	 * get a list of anchor notes
	 */
	linklist = pnlink = prevlink = NULL;
	for (pn = notes; pn != NULL; pn = pn->pn_next)
	{
		if ( pn->pn_anchor )
		{

			pnlink = (PNLinks *)XPSafeAlloc(sizeof (PNLinks));
			pnlink->pn = pn;
			pnlink->next = NULL;

			/*
			 * Get the windows location on the root.
			 */
			GetNotePosition(pn->pn_shellwidget, &shellx, &shelly);
			nargs=0;
			XtSetArg( args[nargs], XmNx, &xpos); nargs++;
			XtSetArg( args[nargs], XmNy, &ypos); nargs++;
			XtGetValues( pn->pn_shellwidget, args, nargs );
			shellx += xpos;
			shelly += ypos;
			DBGPrintf(DBG_INFO,("anchor: %d %d\n", shellx, shelly));

			pnlink->x = (int)shellx;
			pnlink->y = (int)shelly;
			if ( linklist == NULL )
				linklist = pnlink;
			if ( prevlink != NULL )
				prevlink->next = pnlink;
			prevlink = pnlink;
		}
	}

	/*
	 * Now run the list of notes and put any non-anchored notes over the
	 * anchored notes.
	 */
	if ( linklist != NULL )
	{
		pnlink = linklist;
		for (pn = notes; pn != NULL; pn = pn->pn_next)
		{
			if ( ( ! pn->pn_anchor ) && ( ! pn->pn_hidden ) )
			{
				/*
				 * reposition the non-anchor down and to
				 * the right of the last note.
				 */
				DBGPrintf(DBG_INFO,("position: %d %d\n", pnlink->x, pnlink->y));
				nargs = 0;
				XtSetArg ( args[nargs], XtNx, pnlink->x+app_res.anchor_offset);
				nargs++;
				XtSetArg ( args[nargs], XtNy, pnlink->y+app_res.anchor_offset);
				nargs++;
				XtSetValues(pn->pn_shellwidget, args, nargs);
				XRaiseWindow(display, XtWindow(pn->pn_shellwidget));

				/*
				 * reset the anchor position
				 */
				pnlink->x = pnlink->x + app_res.anchor_offset;
				pnlink->y = pnlink->y + app_res.anchor_offset;

				/*
				 * Move to the next anchor.  Rotate through the list.
				 */
				if ( pnlink->next != NULL )
					pnlink = pnlink->next;
				else
					pnlink = linklist;
			}
		}
	}

	/*
	 * Clean up that storage
	 */
	if ( linklist != NULL )
	{
		for ( pnlink = linklist, prevlink = linklist; pnlink != NULL; 
			prevlink = pnlink->next, free(pnlink), pnlink = prevlink );
		linklist = NULL;
	}
	else 
		XPErrPopUp(XP_S_NO_ANCHORED_NOTES);
}



/*========================================================================
 *	Name:			XPAnchorNote and XPUnAnchorNote
 *					
 *	Description:
 * Set/Unset the anchor option for a note.
 *					
 *	Global Variables:
 *========================================================================*/
void
XPAnchorNote(
	Widget		widget,
	int			index
)
{
	StickyNote	*pn;
	int			nargs;
	Arg			args[2];

	/*
	 * Find the note we're naming.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	pn->pn_anchor = TRUE;
	nargs = 0;
	XtSetArg ( args[nargs], XmNlabelPixmap, anchoron_xbm); nargs++;
	XtSetValues(pn->pn_anchorwidget, args, nargs);
	XtSetSensitive ( pn->pn_unanchormenu, True );
	XtSetSensitive ( pn->pn_anchormenu, False );
	XtSetSensitive ( pn->pn_savewidget, True );
}

void
XPUnAnchorNote(
	Widget			widget,
	int				index
)
{
#ifdef DEBUG
	char	*fname="XPUnAnchorNote";
#endif

	StickyNote *pn;
	int			nargs;
	Arg			args[2];

	DBGEnter();

	/*
	 * Find the note we're naming.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	pn->pn_anchor = FALSE;
	nargs = 0;
	XtSetArg ( args[nargs], XmNlabelPixmap, anchoroff_xbm); nargs++;
	XtSetValues(pn->pn_anchorwidget, args, nargs);
	XtSetSensitive ( pn->pn_unanchormenu, False );
	XtSetSensitive ( pn->pn_anchormenu, True );
	XtSetSensitive ( pn->pn_savewidget, True );
}


/*========================================================================
 *	Name:			XPHideNote
 *					
 *	Description:
 * Hide a note
 *					
 *	Global Variables:
 *========================================================================*/
void
XPHideNote(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	StickyNote 	*pn;

	/*
	 * Find the note we're hiding.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/* mark it hidden */
	pn->pn_hidden = True;

	/*
	 * pop the window down
	 */
	XtPopdown( pn->pn_shellwidget);
}


/*========================================================================
 *	Name:			XPEraseNote
 *					
 *	Description:
 * Erase a note
 *					
 *	Global Variables:
 *========================================================================*/
void
XPEraseNote(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	StickyNote 			*pn;
	XmTextPosition		last;

	/*
	 * Find the note we're hiding.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	last = XmTextGetLastPosition(pn->pn_textwidget);
	XmTextReplace(pn->pn_textwidget, 0, last, "");
	if ( pn->pn_text != NULL )
		free(pn->pn_text);
}


/*========================================================================
 *	Name:			TearOff
 *					
 *	Description:
 * Tear off a new note from the plaid window.
 *					
 *	Global Variables:
 *========================================================================*/
void
XPTearOff()
{
	XPCreateNewNote(StickyNote_3x3, NULL);
}
