/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: pilot.c
 *            DESCRIPTION: PalmPilot(r) interface routines
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPPilot
 *      PRIVATE FUNCTIONS: see "Private routine prototypes" below
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_PILOT_C

/* === System Headers === */
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif

#include "motif.h"

/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"

/* === external routines === */

/* === Public routine prototypes === */

/* === Private routine prototypes === */
static void ClosePilot();
static void ModeCB();
static void	FetchPilot();
static void	FetchAddresses();
static void FetchCB();
static void ProcessMemos();
static void ToggleChange();
static void HotSyncPrompt();
static void PilotParseSubjects();
static void ParseMemosIntoNotes();
static void ParseNotesIntoMemos();
static void CreateIOPrompt();
static void IOCancelCB();
static void IOAcceptCB();
static void CheckDirEntry();
static void RemoveDir();
static void RemoveDirCB();
static void RenameDir();
static void UpdateName();
static void CloseName();
static void DoPilotSync();

/* === Global Variables === */
extern Widget			Top_Level;
extern AppRes			app_res;
extern StickyNote		*notes;
extern Display			*display;

Widget	pilot_widget;
Widget	memos_list, memos_fetch, memos_process;
Widget	addr_fetch;
Widget	go_button, remove_button, rename_button, file_button, file_text;
Widget	import_button, export_button;
Widget	name_text;
int		mode_index;

static Widget	io_widget;
static Widget	namewidget=NULL;

static char		*pilot_strings[]={
	XP_S_PILOT_MENU_BU,
	XP_S_PILOT_MENU_RESTORE,
	XP_S_PILOT_MENU_SYNC,
	XP_S_PILOT_MENU_UPDATE,
	NULL
};


/*========================================================================
 *	Name:			XPPilot
 *	Prototype:	Widget XPPilot()
 *					
 *	Description:
 *	Provide a front end to the set of Pilot tools accessible from XNotesPlus.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPPilot()
{
#ifdef DEBUG
	char	fname[]="XPPilot";
#endif

	Widget			vbox;
	Widget			cancel_button;
	Widget			left_form, right_form;
	Widget			radio_box;
	Widget			frame, frame2, label;
	Widget			mode_form, menu;
	char				buf[256];
	Arg				args[15];
	int				nargs; 
	int				index;
	int				len;
	Dimension		width, height;
	XmString			xm_string;
	XmString			pilot_xm_string[4];
	XpmAttributes	attributes;
	struct stat		stat_buf;

	static Pixmap	file_xbm;
	static Pixmap	pilot_xbm;
	static Boolean	firsttime=True;

	DBGEnter();

	/*
	 * Allow only one copy at a time.
	 */
	if ( pilot_widget != NULL )
		return;

	/*
	 * Create a dialog, set its title, and position it at the cursor.
	 */
	pilot_widget = XPCreateGenericShell(PilotDialog, CANCEL_BUTTON );


	/*
	 * Get the widget ids of the various parts of the window.
	 */
	sprintf(buf, "*GenericForm%d", (int)pilot_widget );
	vbox = XtNameToWidget(pilot_widget, buf);
	sprintf(buf, "*GenericCancelB%d", (int)pilot_widget);
	cancel_button = XtNameToWidget(pilot_widget, buf);


	/*
	 * Left and right side forms - the boxes inside the rest of the 
	 * components of the window will be placed.
	 */
	nargs = 0;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM); nargs++;
	right_form = XmCreateForm(
						vbox,
						"form",
						args, nargs);
	XtManageChild(right_form);
	nargs = 0;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNrightWidget, right_form); nargs++;
	left_form = XmCreateForm(
						vbox,
						"form",
						args, nargs);
	XtManageChild(left_form);

	/*
	 * Label above scrolled window.
	 */
	xm_string = XmStringCreateLtoR(XP_S_PILOT_MEMOS, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetArg ( args[nargs], XmNchildType, XmFRAME_TITLE_CHILD); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNalignment, XmALIGNMENT_BEGINNING); nargs++;
	label= XmCreateLabel(
					left_form,
					"label",
					args, nargs);
	XmStringFree(xm_string);
	XtManageChild(label);

	/*
	 * A scrolled window in the left form for the list of xnotes or
	 * the list of Pilot memos available - depending on the situation.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNvisibleItemCount, 8 ); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, label); nargs++;
   XtSetArg ( args[nargs], XmNselectionPolicy, XmMULTIPLE_SELECT); nargs++;
	memos_list = XmCreateScrolledList(
							left_form,
							"memoList",
							args, nargs
							);
	XtManageChild(memos_list);

	/* Add a tool tip */
	XtAddEventHandler(memos_list,
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_PILOT_MEMO);


	/*
	 * Radio buttons to determine if we're selecting xnotes to export
	 * to the Pilot, or importing memos to xnotes.
	 */
	nargs=0;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, memos_list); nargs++;
	XtSetArg ( args[nargs], XmNorientation, XmHORIZONTAL); nargs++;
	radio_box = XmCreateRadioBox(
					left_form,
					"label",
					args, nargs);
	xm_string = XmStringCreateLtoR(XP_S_IMPORT, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	import_button = XmCreateToggleButton(
						radio_box,
						"toggleButton",
						args, nargs);
	XtManageChild(import_button);
	XmStringFree(xm_string);
	XtAddCallback( import_button, XmNarmCallback, 
		ToggleChange, (XtPointer)NULL);

	xm_string = XmStringCreateLtoR(XP_S_EXPORT, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	export_button = XmCreateToggleButton(
						radio_box,
						"toggleButton",
						args, nargs);
	XtManageChild(export_button);
	XmStringFree(xm_string);
	XtAddCallback( export_button, XmNarmCallback, 
		ToggleChange, (XtPointer)NULL);
	XtManageChild(radio_box);

	/*
	 * Make the Import option the default.
	 */
	XmToggleButtonSetState(import_button, True, False);

	/*
	 * A button to fetch the memos or xnotes into the scrolled list.
	 */
	xm_string = XmStringCreateSimple(XP_S_FETCH);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, radio_box); nargs++;
	memos_fetch = XmCreatePushButton(
						left_form,
						"button",
						args, nargs);
	XtManageChild(memos_fetch);
	XmStringFree(xm_string);

	/* Add a tool tip */
	XtAddEventHandler(memos_fetch,
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_PILOT_MFET);

	/*
	 * A button to fetch the memos or xnotes into the scrolled list.
	 */
	xm_string = XmStringCreateSimple(XP_S_PROCESS);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNleftWidget, memos_fetch); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, radio_box); nargs++;
	memos_process = XmCreatePushButton(
						left_form,
						"button",
						args, nargs);
	XtManageChild(memos_process);
	XmStringFree(xm_string);

	/* Add a tool tip */
	XtAddEventHandler(memos_process,
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_PILOT_MPRO);

	/*
	 * A framed button to fetch the Pilot addresses database.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN ); nargs++;
	XtSetArg ( args[nargs], XmNshadowThickness, 4 ); nargs++;
	frame = XmCreateFrame(
						right_form,
						"frame",
						args, nargs);
	XtManageChild(frame);
	xm_string = XmStringCreateLtoR(XP_S_PILOT_ADDR, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetArg ( args[nargs], XmNchildType, XmFRAME_TITLE_CHILD); nargs++;
	label= XmCreateLabel(
					frame,
					"frame",
					args, nargs);
	XmStringFree(xm_string);
	XtManageChild(label);
	xm_string = XmStringCreateSimple(XP_S_FETCH);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	addr_fetch = XmCreatePushButton(
						frame,
						"button",
						args, nargs);
	XtManageChild(addr_fetch);
	XmStringFree(xm_string);
	XtAddCallback( addr_fetch, XmNactivateCallback, 
			FetchAddresses, (XtPointer)NULL);

	/* Add a tool tip */
	XtAddEventHandler(addr_fetch,
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_PILOT_ADDR);

	/*
	 * A frame around the Pilot Sync features.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN ); nargs++;
	XtSetArg ( args[nargs], XmNshadowThickness, 4 ); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, addr_fetch); nargs++;
   XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM); nargs++;
	frame2 = XmCreateFrame(
						right_form,
						"frame",
						args, nargs);
	XtManageChild(frame2);
	xm_string = XmStringCreateLtoR(XP_S_PILOT_SYNC, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetArg ( args[nargs], XmNchildType, XmFRAME_TITLE_CHILD); nargs++;
	label= XmCreateLabel(
					frame2,
					"label",
					args, nargs);
	XmStringFree(xm_string);
	XtManageChild(label);

	/*
	 * A form inside the frame to place the Sync window components.
	 */
	mode_form = XmCreateForm(
						frame2,
						"form",
						NULL, 0);
	XtManageChild(mode_form);

	/*
	 * The options menu (with label and cascade button) is created
	 * here.
	 */
	index=0;
	while ( pilot_strings[index] != NULL )
	{
		pilot_xm_string[index] = XmStringCreateSimple(pilot_strings[index]);
		index++;
	}

	xm_string = XmStringCreateSimple(XP_S_PILOT_MODE);
	nargs = 0;
	XtSetArg ( args[nargs], XmNbuttons, pilot_xm_string); nargs++;
	XtSetArg ( args[nargs], XmNoptionLabel, xm_string); nargs++;
	XtSetArg ( args[nargs], XmNbuttonSet, 0); nargs++;
	XtSetArg ( args[nargs], XmNbuttonCount, index); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNsimpleCallback, ModeCB); nargs++;
	menu = XmCreateSimpleOptionMenu(
				mode_form,
				"option_menus",
				args, nargs);
	XmStringFree(xm_string);
	XtManageChild(menu);
	mode_index=0;

	index=0;
	while ( pilot_strings[index] != NULL )
	{
		XmStringFree(pilot_xm_string[index]);
		index++;
	}

	/*
	 * Pushbutton and text field for setting directory.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelType, XmPIXMAP); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, menu); nargs++;
	file_button = XmCreatePushButton(
						mode_form,
						"button",
						args, nargs);
	XtManageChild(file_button);
	XtAddCallback( file_button, XmNactivateCallback, 
			CreateIOPrompt, (XtPointer)NULL);

	/* Add a tool tip */
	XtAddEventHandler(file_button,
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_DIR);

	/*
	 * First time through, create the file pixmap.
	 */
	if ( firsttime )
	{
   	XPCreatePixmap(&file_xbm, NULL, XP_FILE_XPM, 
			&attributes, file_button);
   	XPCreatePixmap(&pilot_xbm, NULL, XP_PILOT_XPM, 
			&attributes, vbox);
		firsttime = False;
	}
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelPixmap, file_xbm); nargs++;
	XtSetValues(file_button, args, nargs);

	nargs=0;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNleftWidget, file_button); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, menu); nargs++;
   XtSetArg ( args[nargs], XmNcolumns, 32); nargs++;
	file_text = XmCreateTextField(
							mode_form,
							"textField",
							args, nargs
							);
	XtManageChild(file_text);
	XmTextFieldSetString(file_text, app_res.pilot_dir);
	len = strlen(app_res.pilot_dir);
	XmTextFieldSetInsertionPosition(file_text, len);
	XmTextFieldShowPosition(file_text, len);
	XtAddCallback( file_text, XmNvalueChangedCallback, 
			CheckDirEntry, (XtPointer)NULL);

	/*
	 * If the backup directory does not exist, create it.
	 */
	if ( stat(app_res.pilot_dir, &stat_buf) != 0 )
		mkdir(app_res.pilot_dir, 0755);

	/*
	 * Backup management buttons (starts sync process, rename dir, remove dir)
	 */
	xm_string = XmStringCreateSimple(XP_S_PILOT_REMOVE);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, file_text); nargs++;
	remove_button = XmCreatePushButton(
						mode_form,
						"button",
						args, nargs);
	XtManageChild(remove_button);
	XmStringFree(xm_string);
	XtAddCallback( remove_button, XmNactivateCallback, 
			RemoveDir, (XtPointer)NULL);

	xm_string = XmStringCreateSimple(XP_S_PILOT_RENAME);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNleftWidget, remove_button); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, file_text); nargs++;
	rename_button = XmCreatePushButton(
						mode_form,
						"button",
						args, nargs);
	XtManageChild(rename_button);
	XmStringFree(xm_string);
	XtAddCallback( rename_button, XmNactivateCallback, 
			RenameDir, (XtPointer)NULL);

	xm_string = XmStringCreateSimple(XP_S_PILOT_START);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNleftWidget, rename_button); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, file_text); nargs++;
	go_button = XmCreatePushButton(
						mode_form,
						"button",
						args, nargs);
	XtManageChild(go_button);
	XmStringFree(xm_string);
	XtAddCallback( go_button, XmNactivateCallback, 
			DoPilotSync, (XtPointer)NULL);

	/*
	 * The Palm Pilot Label
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelType, XmPIXMAP); nargs++;
	XtSetArg ( args[nargs], XmNlabelPixmap, pilot_xbm); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, go_button); nargs++;
	label= XmCreateLabel(
					mode_form,
					"label",
					args, nargs);
	XtManageChild(label);

	/*
	 * Set the callbacks for the various buttons.
	 */
	XtAddCallback( cancel_button, XmNactivateCallback, 
			ClosePilot, (XtPointer)NULL);

	XtAddCallback( memos_fetch, XmNactivateCallback, 
			FetchCB, (XtPointer)NULL);

	XtAddCallback( memos_process, XmNactivateCallback, 
			ProcessMemos, (XtPointer)NULL);

	/*
	 * Disable items for which the pilot-link tool could not be found.
	 */
	if ( app_res.pilot_xfer == NULL )
		XtSetSensitive( frame2, False);
	if ( app_res.pilot_addr == NULL )
		XtSetSensitive( frame, False);
	if ( ( app_res.pilot_memos == NULL ) || ( app_res.pilot_imemos == NULL ) )
		XtSetSensitive( left_form, False);

	/*
	 * Resize window to make room for the text windows.
	 */
	XtRealizeWidget(pilot_widget);
	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, &width); nargs++;
	XtSetArg ( args[nargs], XmNheight, &height); nargs++;
	XtGetValues ( pilot_widget, args, nargs);

	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, width+80); nargs++;
	XtSetValues ( pilot_widget, args, nargs);


	/*
	 * Display the dialog.
	 */
	XtPopup(pilot_widget, XtGrabNone);
	XPCenterWindow(pilot_widget, NULL, pilot_widget);

	/*
	 * This bit of code forces the pop up window to stay its
	 * initial size, despite the users attempts to resize it.
	 * By doing this, I avoid having to muck extensively with
	 * aligning objects after a resize.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, &width); nargs++;
	XtSetArg ( args[nargs], XmNheight, &height); nargs++;
	XtGetValues ( pilot_widget, args, nargs);

	nargs = 0;
	XtSetArg ( args[nargs], XmNmaxWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNmaxHeight, height); nargs++;
	XtSetArg ( args[nargs], XmNminWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNminHeight, height); nargs++;
	XtSetValues ( pilot_widget, args, nargs); 

}


/*========================================================================
 *	Name:			ClosePilot
 *	Prototype:	static void ClosePilot()
 *					
 *	Description:
 * Close the envelopes dialog.  Clean up any extra storage we might have
 * created here, including temp files.
 *
 *	Notes:
 *========================================================================*/
static void
ClosePilot()
{
	char				buf[256];
	struct stat		stat_buf;

	if ( pilot_widget != NULL )
	{
		XtDestroyWidget(pilot_widget);
		pilot_widget = NULL;
	}

	sprintf(buf, "/tmp/xnmemos.%d", (int)getpid());
	if ( stat (buf, &stat_buf ) == 0 )
		unlink(buf);
}


/*========================================================================
 *	Name:			ModeCB
 *	Prototype:	static void ModeCB()
 *					
 *	Description:
 * Close the envelopes dialog.
 *
 *	Notes:
 *========================================================================*/
static void
ModeCB(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	mode_index = index;
}


/*========================================================================
 *	Name:			ToggleChange
 *	Prototype:	static void ToggleChange()
 *					
 *	Description:
 * The import/export toggles have been changed.  Clear the scrolled list.
 *
 *	Notes:
 *========================================================================*/
static void
ToggleChange(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	XmListDeleteAllItems(memos_list);
}


/*========================================================================
 *	Name:			FetchCB
 *	Prototype:	static void FetchCB()
 *					
 *	Description:
 * Retrieve either the list of available notes or the set of memos from
 *	the Palm Pilot, depending on which toggle (export or immport,
 *	respectively) is set.
 *
 *	Notes:
 *========================================================================*/
static void
FetchCB(
	Widget		widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
	XmString		xm_string;
	StickyNote	*pn;

	/*
	 * Clear the scrolled list.
	 */
	XmListDeleteAllItems(memos_list);


	/*
	 * Determine from where to retrieve list items.
	 */
	if ( XmToggleButtonGetState(import_button) == True )
	{
		HotSyncPrompt(XP_PILOT_MEMO);
	}
	else
	{
		/*
		 * Grab the names of the existing notes and add them to
		 * the scrolled list window.
		 */
		for (pn=notes; pn!=NULL; pn=pn->pn_next)
		{
			/*
			 * All notes should have default names - this better not crash!
			 */
			xm_string = XmStringCreateLtoR(pn->pn_name, XmFONTLIST_DEFAULT_TAG);
			XmListAddItem(memos_list, xm_string, 0);
			XmStringFree(xm_string);
		}
	}
}


/*========================================================================
 *	Name:			FetchPilot
 *	Prototype:	static int FetchPilot(GtkWidget *widget, int type)
 *					
 *	Description:
 * Retrieve data from the Palm Pilot.  This will handle memos, addresses,
 * and backups/restores/syncs/updates.
 *
 * Input Arguments:
 *	int	type			one of XP_PILOT_MEMO, XP_PILOT_INST, XP_PILOT_SYNC,
 *							or XP_PILOT_ADDR
 *
 *	Notes:
 *========================================================================*/
static void
FetchPilot(
	Widget		widget,
	int			type,
	XtPointer	client_data
)
{
	FILE				*fd;
	char				buf[MAXPATHLEN];
	char				cmd[MAXPATHLEN];
	char				pilot_cmd[MAXPATHLEN];
	char				title[MAXPATHLEN];
	char				ok_text[MAXPATHLEN];
	struct stat		stat_buf;
	int				rc;
	Boolean			failed;
	DIR				*dp;
	char				mode[5];
	char				*string=NULL;

#ifdef USE_DIR
	register struct direct *d;
#else
	register struct dirent *d;
#endif


	failed = False;

	/*
	 * Type dependent setup for this routine.
	 */
	switch (type)
	{
		case XP_PILOT_MEMO:
			/*
			 * Clean up any old files first.
			 */
			sprintf(buf, "/tmp/xnmemos.%d", (int)getpid());
			if ( stat (buf, &stat_buf ) == 0 )
				unlink(buf);

			sprintf(title, XP_S_PTITLE_MEMO);
			sprintf(pilot_cmd, app_res.pilot_memos);
			sprintf(ok_text, XP_S_PILOT_MEMO_OK);

			break;

		case XP_PILOT_INST:
			sprintf(title, XP_S_PTITLE_IMEMO);
			sprintf(pilot_cmd, app_res.pilot_imemos);
			sprintf(ok_text, XP_S_PILOT_IMEMO_OK);
			break;

		case XP_PILOT_ADDR:
			sprintf(title, XP_S_PTITLE_ADDR);
			sprintf(pilot_cmd, app_res.pilot_addr);
			sprintf(ok_text, XP_S_PILOT_ADDR_OK);
			break;

		case XP_PILOT_SYNC:
			/*
			 * Check for the directory specified.
			 * Create it if its not there.
			 */
			string = XmTextFieldGetString(file_text);
		
			if ( strncmp(string, "/", 1) != 0 )
			{
				sprintf(buf, XP_S_PATH_INVALID);
				XPErrPopUp(buf);
				XtFree(string);
				return;
			}
		
			if ( stat(string, &stat_buf) != 0 )
				mkdir(string, 0755);
			else
				if ( ! (stat_buf.st_mode & S_IFDIR) )
				{
					sprintf(buf, "%s exists and is not a directory!", string);
					XPErrPopUp(buf);
					XtFree(string);
					return;
				}

			switch (mode_index)
			{
				case 0: sprintf(mode, "-b"); break;
				case 1: sprintf(mode, "-r"); break;
				case 2: sprintf(mode, "-s"); break;
				case 3: sprintf(mode, "-u"); break;
			}
			sprintf(title, XP_S_PTITLE_SYNC);
			sprintf(pilot_cmd, app_res.pilot_xfer);
			sprintf(ok_text, XP_S_PILOT_SYNC_OK);
			break;
	}

	/*
	 * Build the shell script to run.  We have to run a script because
	 * 1. we're trying to avoid incorporating the PilotLink software 
	 *		directly in this code (makes it easier to upgrade)
	 * 2. allow shell expansion of "*" to identify all files instead
	 *		of having to list them all with fully-qualified paths.
	 */
	sprintf(cmd, "%s/xnmemos.sh", app_res.pilot_dir);
	fd = fopen(cmd, "w");

	fprintf(fd, "#!/bin/sh\n");
	fprintf(fd, title);
	fprintf(fd, "# This script is automatically generated by xnotes+.\n");
	fprintf(fd, "# \n\n");

	switch (type)
	{
		case XP_PILOT_MEMO:
			fprintf(fd, "%s > /tmp/xnmemos.%d 2>/dev/null \n", 
				app_res.pilot_memos, (int)getpid());
			break;

		case XP_PILOT_INST:
			fprintf(fd, "cd %s/xnmemos\n", app_res.pilot_dir);
			fprintf(fd, "%s -q -c xnotes %s * 2> /dev/null \n", 
				app_res.pilot_imemos, (char *)getenv("PILOTPORT"));
			break;

		case XP_PILOT_ADDR:
			fprintf(fd, "%s %s -c all -w %s/%s 2>/dev/null \n", 
				app_res.pilot_addr, (char *)getenv("PILOTPORT"),
				app_res.pilot_dir, DefaultPilotDB);
			break;

		case XP_PILOT_SYNC:
			fprintf(fd, "pilot-xfer %s %s 2> /dev/null 1>/dev/null \n", 
				mode, string);
			XtFree(string);
			break;
	}

	fprintf(fd, "echo $? > %s/xnmemos.rc\n", app_res.pilot_dir);

	fclose(fd);
	chmod(cmd, S_IRWXU);

	/*
	 * Start the upload process.  This will block other xnotes updates,
	 * including window refreshes, but for now its simpler than trying
	 * to coordinate an exec'd script to handle messaging (ie I'm too
	 * lazy to add signal handling, etc right now).
	 */
	rc=system(cmd);
	switch ( rc )
	{
		case 127:
			sprintf(buf,"%s failed: errno=%d", cmd, errno);
			XPErrPopUp(buf);
			failed=True;
			break;
		case -1:
			sprintf(buf,"%s failed: errno=%d", cmd, rc);
			XPErrPopUp(buf);
			failed=True;
			break;
		default:
			sprintf(buf, "%s/xnmemos.rc", app_res.pilot_dir);
			if ( stat (buf, &stat_buf) == 0 )
			{
				fd = fopen(buf, "r");
				fscanf(fd, "%d", &rc);
				fclose(fd);
				if ( rc != 0 )
				{
					sprintf(buf,XP_S_CANT_RUN_CMD, pilot_cmd, rc);
					XPErrPopUp(buf);
					failed=True;
				}
				else
				{
					XPMsgWindow(ok_text, 
						XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK, NULL, pilot_widget,
						RemoveDirCB, NULL, NULL, NULL);
				}
			}
			else
			{
				sprintf(buf,XP_S_CANT_RUN_CMD2, pilot_cmd);
				XPErrPopUp(buf);
				failed=True;
			}
			break;
	}

	switch ( type )
	{
		case XP_PILOT_MEMO:
			break;

		case XP_PILOT_INST:
			/*
			 * Clean up the temporary files.
			 */
			sprintf(buf, "%s/xnmemos", app_res.pilot_dir);
			if ((dp = opendir(buf)) == NULL)
			{
				fprintf(stderr, "xnotes: ");
				perror(buf);
			}
			else
			{
				while ((d = readdir(dp)) != NULL) 
				{
					if (strcmp(d->d_name, ".") == 0)
						continue;
					if (strcmp(d->d_name, "..") == 0)
						continue;
		
					sprintf(buf, "%s/xnmemos/%s", app_res.pilot_dir, d->d_name);
					unlink (buf);
				}
				closedir(dp);
			}
			break;
	}

	/*
	 * Generic cleanup.
	 */
	sprintf(buf, "%s/xnmemos.sh", app_res.pilot_dir);
	unlink (buf);
	sprintf(buf, "%s/xnmemos.rc", app_res.pilot_dir);
	unlink (buf);


	if ( type == XP_PILOT_MEMO )
	{
		/*
		 * Parse out the subject headers and
		 * add them to the scrolled list window.
		 */
		if ( !failed )
			PilotParseSubjects();
	}
}



/*========================================================================
 *	Name:			ProcessMemos
 *	Prototype:	static void ProcessMemos()
 *					
 *	Description:
 *	Run through the list of items (either xnotes or pilot memos) in the
 * scrolled window and process them for import/export, depending on the
 * state of the toggle buttons.
 *
 *	Notes:
 *========================================================================*/
static void
ProcessMemos(
	Widget		widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
	int				count;
	XmStringTable	positions;
	int				nargs;
	Arg				args[3];
	struct stat		stat_buf;
	char				buf[256];


	/*
	 * If there are no items to process, simply return.
	 */
	nargs=0;
   XtSetArg ( args[nargs], XmNselectedItemCount, &count); nargs++;
	XtGetValues(memos_list, args, nargs);
	if ( count == 0 )
	{
		XPMsgWindow(
			XP_S_LIST_EMPTY, XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOCANCEL, 
			NULL, NULL, NULL, NULL, NULL, NULL);
		return;
	}
	
	/*
	 * Get the list of items to process.
	 */
	nargs=0;
	XtSetArg( args[nargs], XmNselectedItems, &positions ); nargs++;
	XtGetValues( memos_list, args, nargs );


	/*
	 * Determine from where to retrieve list items.
	 */
	if ( XmToggleButtonGetState(import_button) == True )
	{
		/*
		 * These are the subject headers from the memos downloaded
		 * from the Palm Pilot.  Parse their respective memos from the
		 * temporary file and turn them into new notes.
		 */
		ParseMemosIntoNotes(positions, count);

		/*
		 * Remove temp file.
		 */
		sprintf(buf, "/tmp/xnmemos.%d", (int)getpid());
		if ( stat (buf, &stat_buf ) == 0 )
			unlink(buf);
	}
	else
	{
		/*
		 * These are notes to be exported to the Palm Pilot. Write them
		 * to temp files and upload them to the Palm Pilot using "install-memo".
		 */
		ParseNotesIntoMemos(positions, count);
	}
}



/*========================================================================
 *	Name:			HotSyncPrompt
 *	Prototype:	static void HotSyncPrompt(int type)
 *					
 *	Description:
 * Prompt the user to press the HotSync button on the Pilot's cradle.
 *
 *	Notes:
 *========================================================================*/
static void
HotSyncPrompt(
	int 		type
)
{
	XPMsgWindow(
		XP_S_PILOT_HOTSYNC, XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOCANCEL, 
		NULL, NULL, FetchPilot, type, NULL, NULL);
}


/*========================================================================
 *	Name:			PilotParseSubjects
 *	Prototype:	static void PilotParseSubjects()
 *					
 *	Description:
 * Parse the subject headers from the mail-formatted Pilot memos.
 *
 *	Notes:
 *========================================================================*/
static void
PilotParseSubjects(
)
{
#ifdef DEBUG
	char	*fname="PilotParseSubjects()";
#endif

	FILE		*fd;
	char		buf[256];
	char		*token;
	XmString	xm_string;

	struct stat		stat_buf;

	DBGEnter();

	/*
	 * Make sure the temp file is still there.
	 */
	sprintf(buf, "/tmp/xnmemos.%d", (int)getpid());
	if ( stat(buf, &stat_buf) != 0)
	{
		DBGPrintf(DBG_INFO,("Can't find memos temporary file!"));
		XPErrPopUp("Can't find memos temporary file!");
		return;
	}

	/*
	 * Open the temporary file.
	 */
	fd = fopen(buf, "r");

	/*
	 * Read each line, one by one, and test for the Subject: header tag.
	 */
	while (fgets(buf, 256, fd) != NULL)
	{
		if ( !feof(fd) )
		{
			token=strtok(buf, ":");
			if (strcmp(token, "Subject") == 0 )
			{
				/*
				 * Add subjects to scrolled list window.
				 */
				token=strtok(NULL, "\n");
				xm_string = XmStringCreateLtoR(token, XmFONTLIST_DEFAULT_TAG);
				XmListAddItem(memos_list, xm_string, 0);
				XmStringFree(xm_string);
			}
		}
	}

	/*
	 * Close the temporary file.
	 */
	fclose(fd);
}


/*========================================================================
 *	Name:			ParseMemosIntoNotes
 *	Prototype:	static void ParseMemosIntoNotes()
 *					
 *	Description:
 * Parse the memos from the temp file into new notes.
 *
 *	Notes:
 *========================================================================*/
static void
ParseMemosIntoNotes(
	XmStringTable	positions,
	int				list_count
)
{
#ifdef DEBUG
	char	*fname="ParseMemosIntoNotes()";
#endif

	FILE		*fd;
	char		*ptr;
	char		buf[256];
	char		*token, *bufcpy;
	char		*string;
	char		*textbuf=NULL;
	char		subject[1024];
	char		*newbuf=NULL;
	int		i;
	Boolean	keep;

	Arg				args[3];
	int				nargs;
	StickyNote		*pn;
	struct stat		stat_buf;

	DBGEnter();

	/*
	 * Make sure the temp file is still there.
	 */
	sprintf(buf, "/tmp/xnmemos.%d", (int)getpid());
	if ( stat(buf, &stat_buf) != 0)
	{
		DBGPrintf(DBG_INFO,("Can't find memos temporary file!"));
		XPErrPopUp("Can't find memos temporary file!");
		return;
	}

	/*
	 * Open the temporary file.
	 */
	fd = fopen(buf, "r");

	/*
	 * Read each line, one by one.  Look for the "Received:" tag, which will
	 * start a new mail message (skip the "From" since it has 
	 * no colon delimiter and all other colon-delimited tags).
	 */
	keep = False;
	while (fgets(buf, 256, fd) != NULL)
	{
		if ( !feof(fd) )
		{
			/*
			 * Create a copy of the line.
			 */
			bufcpy=(char *)XPSafeAlloc(strlen(buf)+1);
			strcpy(bufcpy, buf);

			/*
			 * Get the token from the line.
			 */
			token=strtok(bufcpy, ":");

			if (strcmp(token, "Received") == 0 )
			{
				/*
				 * We're staring a new memo - reset state info.
				 */
				keep = False;

				/*
				 * Process previous memo, if any.
				 */
				if ( textbuf != NULL )
				{
					/*
					 * Allocate a new note structure.
					 */
					XPCreateNewNote(StickyNote_3x3, &pn);

					/*
					 * Add the text to the window.
					 */
					if ( pn->pn_text != NULL )
						free(pn->pn_text);
					pn->pn_text = XPSafeAlloc(strlen(textbuf)+1);
					strcpy(pn->pn_text, textbuf);
					XmTextSetString(pn->pn_textwidget, pn->pn_text);

					/*
					 * Set the notes title.
					 */
					if ( pn->pn_name != NULL )
						free(pn->pn_name);
					pn->pn_name = XPSafeAlloc(strlen(subject)+1);
					strcpy(pn->pn_name, subject);
					nargs=0;
					XtSetArg ( args[nargs], XtNtitle, pn->pn_name ); nargs++;
					XtSetValues (pn->pn_shellwidget, args, nargs);

					/*
					 * Clean up.
					 */
					free(textbuf);
					textbuf = NULL;
				}
				continue;
			}

			/*
			 * If its one of the other headers, except "Subject:", just
			 * skip the line.
			 */
			if	(	( strcmp(token, "To") == 0 ) ||
					( strcmp(token, "Date") == 0 ) ||
					( strncmp(buf, "From ", 5) == 0 ) )
					
				continue;

			/*
			 * If its the Subject: tag, test it against the list of
			 * subjects we want to turn into notes.  If it matches any,
			 * turn on the "keep" flag.  These are the memos we're going
			 * to turn into notes.
			 */
			if ( strcmp(token, "Subject") == 0 )
			{
				ptr=strtok(NULL,"\n");
				bzero(subject, 1024);
				strcpy(subject, ptr);
				for (i=0; i<list_count; i++)
				{
					XmStringGetLtoR(positions[i], XmFONTLIST_DEFAULT_TAG, &string);
					if ( strcmp(string, subject) == 0 )
						keep = True;
					XtFree(string);
				}
			}
			else
			{
				/*
				 * This must be a text line.  If "keep" is true, add it to 
				 * the temp buffer.
				 */
				if (keep)
				{
					if ( textbuf == NULL )
					{
						textbuf = XPSafeAlloc(strlen(buf) + 1);
						strcpy ( textbuf, buf );
					}
					else
					{
						newbuf = XPSafeAlloc(strlen(buf) + strlen(textbuf) + 1);
						strcpy(newbuf, textbuf);
						strcat(newbuf, buf);
						free(textbuf);
						textbuf = newbuf;
					}
				}
			}

			/*
			 * Free up the copy of the buffer.
			 */
			free(bufcpy);
		}
		else
		{
			/*
			 * One last check, in case the last item in the list was selected.
			 */
			if ( textbuf != NULL )
			{
				/*
				 * Allocate a new note structure.
				 */
				XPCreateNewNote(StickyNote_3x3, &pn);

				/*
				 * Add the text to the window.
				 */
				if ( pn->pn_text != NULL )
					free(pn->pn_text);
				pn->pn_text = XPSafeAlloc(strlen(textbuf)+1);
				strcpy(pn->pn_text, textbuf);
				XmTextSetString(pn->pn_textwidget, pn->pn_text);

				/*
				 * Set the notes title.
				 */
				if ( pn->pn_name != NULL )
					free(pn->pn_name);
				pn->pn_name = XPSafeAlloc(strlen(subject)+1);
				strcpy(pn->pn_name, subject);
				nargs=0;
				XtSetArg ( args[nargs], XtNtitle, pn->pn_name ); nargs++;
				XtSetValues (pn->pn_shellwidget, args, nargs);

				/*
				 * Clean up.
				 */
				free(textbuf);
				textbuf = NULL;
			}
		}
	}

	/*
	 * One last check, in case the last item in the list was selected.
	 */
	if ( textbuf != NULL )
	{
		/*
		 * Allocate a new note structure.
		 */
		XPCreateNewNote(StickyNote_3x3, &pn);

		/*
		 * Add the text to the window.
		 */
		if ( pn->pn_text != NULL )
			free(pn->pn_text);
		pn->pn_text = XPSafeAlloc(strlen(textbuf)+1);
		strcpy(pn->pn_text, textbuf);
		XmTextSetString(pn->pn_textwidget, pn->pn_text);

		/*
		 * Set the notes title.
		 */
		if ( pn->pn_name != NULL )
			free(pn->pn_name);
		pn->pn_name = XPSafeAlloc(strlen(subject)+1);
		strcpy(pn->pn_name, subject);
		nargs=0;
		XtSetArg ( args[nargs], XtNtitle, pn->pn_name ); nargs++;
		XtSetValues (pn->pn_shellwidget, args, nargs);

		/*
		 * Clean up.
		 */
		free(textbuf);
		textbuf = NULL;
	}

	/*
	 * Close the memos temporary file.
	 */
	fclose(fd);

	/*
	 * Clear out the list.
	 */
	 XmListDeleteAllItems(memos_list);
}


/*========================================================================
 *	Name:			ParseNotesIntoMemos
 *	Prototype:	static void ParseNotesIntoMemos()
 *					
 *	Description:
 * Parse notes into a format for uploading as memos.  Upload those new
 * memos.
 *
 *	Notes:
 * This routine builds a script to run install-memo and runs it with the
 * system command.  Its a round about way of doing things, but allows
 * me to use install-memo and not embed code for doin the same thing here
 * in xnotes.  This way I don't have to add the overhead of the Pilot
 * headers, libs, etc just to get xnotes to work.
 *
 * Also, once started the install-memo script will block all other 
 * xnotes work until it completes.  Its a tradeoff between this and having 
 * to get fancy with exec'd scripts and signalling.
 *
 * Finally, because of the use of a script with system(), it doesn't matter 
 * if the install-memo doesn't work or not.  Thats a bonus.
 *========================================================================*/
static void
ParseNotesIntoMemos(
	XmStringTable	positions,
	int				list_count
)
{
#ifdef DEBUG
	char	*fname="ParseNotesIntoMemos()";
#endif

	FILE				*fd;
	char				*string;
	char				filename[MAXPATHLEN];
	char				buf[MAXPATHLEN];
	char				alarm[10];
	char				hidden[10];
	char				anchor[10];
	char				project[512];
	StickyNote		*pn;
	Boolean			found;
	struct stat		stat_buf;
	int				i;

	DBGEnter();

	/*
	 * Check for the memos directory in the users xnotes directory.
	 * Create it if its not there.
	 */
	sprintf(buf, "%s/xnmemos", app_res.pilot_dir);
	if ( stat(buf, &stat_buf) != 0 )
		mkdir(buf, 0755);
	else
		if ( ! (stat_buf.st_mode & S_IFDIR) )
		{
			sprintf(buf, "%s exists and is not a directory!", buf);
			XPErrPopUp(buf);
			return;
		}

	/*
	 * For each note selected in the list, write the text and related
	 * information to a temp file, formatted like this:
	 * Note Title
	 * XNProject: <setting>
	 * XNAlarm: <setting>
	 * XNAnchor: <setting>
	 * XNHidden: <yes or no>
	 * <note text>
	 */
	for (i=0; i<list_count; i++)
	{
		XmStringGetLtoR(positions[i], XmFONTLIST_DEFAULT_TAG, &string);

		/*
		 * Find the note with this text.
		 */
		found = False;
		for (pn=notes; pn!=NULL; pn=pn->pn_next)
		{
			if (strcmp(pn->pn_name, string) == 0 )
			{
				found = True;
				break;
			}
		}
		XtFree(string);

		if ( found )
		{
			/*
			 * Found a matching note - write it to a temp file.
			 */
			sprintf(filename, 
				"%s/xnmemos/%d.%d", app_res.pilot_dir, pn->pn_index, (int)getpid());

			fd = fopen(filename, "w" );

			if ( pn->pn_alarm )
				sprintf(alarm, "True");
			else
				sprintf(alarm, "False");
			if ( pn->pn_anchor )
				sprintf(anchor, "True");
			else
				sprintf(anchor, "False");
			if ( pn->pn_hidden )
				sprintf(hidden, "True");
			else
				sprintf(hidden, "False");
			if ( pn->pn_project == NULL )
				sprintf(project, "None");
			else
				sprintf(project, "%s", pn->pn_project);

			fprintf(fd, 
				"%s\n"
				"XNProject: %s\n"
				"XNAlarm: %s\n"
				"XNAnchor: %s\n"
				"XNHidden: %s\n"
				"%s",
				pn->pn_name, project, alarm, anchor, hidden, 
				pn->pn_text);

			fclose(fd);
			
		}
	}

	/*
	 * Install the notes onto the Pilot.
	 */
	HotSyncPrompt(XP_PILOT_INST);

	/*
	 * Clear out the list.
	 */
	 XmListDeleteAllItems(memos_list);
}


/*========================================================================
 *	Name:			CreateIOPrompt
 *	Prototype:	CreateIOPrompt()
 *					
 *	Description:
 * Create a file selection widget
 *
 *	Notes:
 *========================================================================*/
static void
CreateIOPrompt(
	Widget 		widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
	Arg			args[2];
	int			nargs;
	Widget		component;
	XmString		xm_string;

	/*
	 * Only one copy at a time allowed.
	 */
	if ( io_widget != NULL )
		return;

	/*
	 * Create File Selection Box.
	 */
	xm_string = XmStringCreateSimple(app_res.pilot_dir);
	nargs = 0;
	XtSetArg ( args[nargs], XmNdirectory, xm_string); nargs++;
	XtSetArg ( args[nargs], XmNtitle, NotesPilotBUPrompt); nargs++;
	io_widget = XmCreateFileSelectionDialog(
						Top_Level,
						"FSWin",
						args, nargs);
	XmStringFree(xm_string);
	
	/*
	 * Get the button widgets from the dialog.
	 * Hide the Help button - we don't need it here.
	 */
	component = XmFileSelectionBoxGetChild(io_widget, XmDIALOG_HELP_BUTTON);
	XtUnmanageChild(component);
	component = XmFileSelectionBoxGetChild(io_widget, XmDIALOG_LIST_LABEL);
	XtUnmanageChild(component);
	component = XtNameToWidget(io_widget, "ItemsListSW");
	XtUnmanageChild(component);

	/* The Cancel/Accept buttons */
	XtAddCallback( io_widget, XmNcancelCallback, IOCancelCB, (XtPointer)NULL);
	XtAddCallback( io_widget, XmNokCallback, IOAcceptCB, (XtPointer)NULL);

	/*
	 * pop up the window.
	 */
	XtManageChild(io_widget);
	XPCenterWindow(io_widget,NULL, XtParent(io_widget));
}


/*========================================================================
 *	Name:			IOCancelCB
 *	Prototype:	IOCancelCB()
 *					
 *	Description:
 * Close the file selection dialog.
 *
 *	Notes:
 *========================================================================*/
static void
IOCancelCB()
{
	if (io_widget != NULL )
	{
		XtDestroyWidget(io_widget);
		io_widget=NULL;
	}
}


/*========================================================================
 *	Name:			IOAcceptCB
 *	Prototype:	IOAcceptCB()
 *					
 *	Description:
 * Handle File Selection Accept case - save directory selected in text
 * field, update buttons sensitivity, close file selection dialog.
 *
 *	Notes:
 *========================================================================*/
static void
IOAcceptCB()
{
	Arg		args[2];
	int		nargs;
	XmString	xm_string;
	char		*string;
	int		len;
	struct stat		stat_buf;

	/*
	 * Retrieve the directory name from the File Selection dialog
	 * and update the directory text field.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNdirectory, &xm_string); nargs++;
	XtGetValues ( io_widget, args, nargs);

	XmStringGetLtoR(xm_string, XmFONTLIST_DEFAULT_TAG, &string);
	XmTextFieldSetString(file_text, string);
	len = strlen(string);
	XmTextFieldSetInsertionPosition(file_text, len);
	XmTextFieldShowPosition(file_text, len);

	/*
	 * If the directory exists, set the remove/rename buttons to 
	 * accept button presses.
	 */
	if ( stat(string, &stat_buf) == 0 )
	{
		XtSetSensitive(remove_button, True);
		XtSetSensitive(rename_button, True);
	}
	else
	{
		XtSetSensitive(remove_button, False);
		XtSetSensitive(rename_button, False);
	}

	XtFree(string);

	/*
	 * Close the File Selection window.
	 */
	IOCancelCB();
}


/*========================================================================
 *	Name:			CheckDirEntry
 *	Prototype:	CheckDirEntry()
 *					
 *	Description:
 * Test to see if the directory specified exists.  Do appropriate things
 * if it does.
 *
 *	Notes:
 *========================================================================*/
static void
CheckDirEntry()
{
	struct stat		stat_buf;
	char				*string;

	string = XmTextFieldGetString(file_text);

	if ( stat(string, &stat_buf) == 0 )
	{
		XtSetSensitive(remove_button, True);
		XtSetSensitive(rename_button, True);
	}
	else
	{
		XtSetSensitive(remove_button, False);
		XtSetSensitive(rename_button, False);
	}

	XtFree(string);
}


/*========================================================================
 *	Name:			RenameDir
 *	Prototype:	RenameDir()
 *					
 *	Description:
 * Prompt user for the new name of the directory.
 *
 *	Notes:
 * We can't get here unless the directory already exists.
 *========================================================================*/
static void
RenameDir()
{
#ifdef DEBUG
	char	fname[]="RenameDir";
#endif

	Widget			vbox;
	Widget			confirm_button, cancel_button;
	char				buf[256];
	Arg				args[5];
	int				nargs;
	Dimension		width, height;
	char				*string;
	int				len;


	DBGEnter();

	/*
	 * Allow only one copy at a time.
	 */
	if ( namewidget != NULL )
		return;

	/*
	 * Create a dialog, set its title, and position it at the cursor.
	 */
	namewidget = XPCreateGenericShell(RenameDirDialog, 
					ACCEPT_BUTTON | CANCEL_BUTTON );

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	sprintf(buf, "*GenericForm%d", (int)namewidget );
	vbox = XtNameToWidget(namewidget, buf);
	sprintf(buf, "*GenericCancelB%d", (int)namewidget);
	cancel_button = XtNameToWidget(namewidget, buf);
	sprintf(buf, "*GenericAcceptB%d", (int)namewidget);
	confirm_button = XtNameToWidget(namewidget, buf);
 
	/*
	 * An text entry widget in which the name will go.
	 */
	nargs=0;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNcolumns, 40); nargs++;
	name_text = XmCreateTextField(
							vbox,
							"textField",
							args, nargs
							);
	XtManageChild(name_text);

	string = XmTextFieldGetString(file_text);
	XmTextFieldSetString(name_text, string);
	len = strlen(string);
	XmTextFieldSetInsertionPosition(file_text, len);
	XmTextFieldShowPosition(file_text, len);
	XtFree(string);

	/*
	 * Confirmation button.
	 */
	XtAddCallback( confirm_button, XmNactivateCallback, 
			UpdateName, (XtPointer)NULL);

	/*
	 * Cancellation button.
	 */
	XtAddCallback( cancel_button, XmNactivateCallback, 
			CloseName, (XtPointer)NULL);

	/*
	 * Stick the dialog in the middle of the notes shell widget.
	 */
	XtRealizeWidget( namewidget );
	XPCenterWindow(namewidget, pilot_widget, namewidget);

	XtPopup(namewidget, XtGrabNone);

	/*
	 * This bit of code forces the pop up window to stay its
	 * initial size, despite the users attempts to resize it.
	 * By doing this, I avoid having to muck extensively with
	 * aligning objects after a resize.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, &width); nargs++;
	XtSetArg ( args[nargs], XmNheight, &height); nargs++;
	XtGetValues ( namewidget, args, nargs);
	nargs = 0;
	XtSetArg ( args[nargs], XmNmaxWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNmaxHeight, height); nargs++;
	XtSetArg ( args[nargs], XmNminWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNminHeight, height); nargs++;
	XtSetValues ( namewidget, args, nargs); 

}


/*========================================================================
 *	Name:			UpdateName
 *	Prototype:	UpdateName()
 *					
 *	Description:
 * Change the name of the existing directory.
 *
 *	Notes:
 *========================================================================*/
static void
UpdateName()
{
	char		*old_dir, *new_dir;
	char		buf[256];
	int		len;

	new_dir = XmTextFieldGetString(name_text);
	old_dir = XmTextFieldGetString(file_text);

	if ( rename(old_dir, new_dir) != 0 )
	{
		sprintf(buf, XP_S_RENAME_FAIL, old_dir, new_dir, errno);
		XPErrPopUp(buf);
	}
	else
	{
		XmTextFieldSetString(file_text, new_dir);
		len = strlen(new_dir);
		XmTextFieldSetInsertionPosition(file_text, len);
		XmTextFieldShowPosition(file_text, len);
	}

	XtFree(old_dir);
	XtFree(new_dir);

	CloseName();
}


/*========================================================================
 *	Name:			CloseName
 *	Prototype:	CloseName()
 *					
 *	Description:
 * Close the directory rename dialog.
 *
 *	Notes:
 *========================================================================*/
static void
CloseName()
{
	if (namewidget != NULL )
	{
		XtDestroyWidget(namewidget);
		namewidget=NULL;
	}
}


/*========================================================================
 *	Name:			RemoveDir
 *	Prototype:	RemoveDir()
 *					
 *	Description:
 * Prompt user to verify the removal of an existing directory and its files.
 *
 *	Notes:
 * We can't get here unless the directory already exists.
 *========================================================================*/
static void
RemoveDir()
{
	char				*string;
	char				buf[512];

	string = XmTextFieldGetString(file_text);

	sprintf(buf, XP_S_DELETE_PROMPT, string);
	XPMsgWindow(buf, XP_MSG_TYPE | XP_MSG_NOHELP, NULL, pilot_widget,
		RemoveDirCB, NULL, NULL, NULL);

	XtFree(string);
}


/*========================================================================
 *	Name:			RemoveDirCB
 *	Prototype:	RemoveDirCB()
 *					
 *	Description:
 * Remove an existing directory and its files.
 *
 *	Notes:
 * We can't get here unless the directory already exists.
 *========================================================================*/
static void
RemoveDirCB()
{
	char				*string;
	char				buf[MAXPATHLEN];
	DIR				*dp;
	int				len;

#ifdef USE_DIR
	register struct direct *d;
#else
	register struct dirent *d;
#endif

	string = XmTextFieldGetString(file_text);

	if ((dp = opendir(string)) == NULL)
	{
		fprintf(stderr, "xnotes: ");
		perror(string);
		return;
	}

	while ((d = readdir(dp)) != NULL) 
	{
		if (strcmp(d->d_name, ".") == 0)
			continue;
		if (strcmp(d->d_name, "..") == 0)
			continue;

		sprintf(buf, "%s/%s", string, d->d_name);
		unlink (buf);
	}
	closedir(dp);
	rmdir(string);

	/*
	 * Update the text field.
	 */
	XmTextFieldSetString(file_text, app_res.pilot_dir);
	len = strlen(app_res.pilot_dir);
	XmTextFieldSetInsertionPosition(file_text, len);
	XmTextFieldShowPosition(file_text, len);

	/*
	 * Clean up.
	 */
	XtFree(string);
}


/*========================================================================
 *	Name:			DoPilotSync
 *	Prototype:	DoPilotSync()
 *					
 *	Description:
 * Start the file sync process with the Pilot.
 *
 *	Notes:
 * We *can* get here with a directory specified that doesn't exist!
 *
 * Directories for backups must be fully qualified.  Makes life easier
 * for me.
 *
 * This is just a front end to FetchPilot() for doing backups, etc.
 *========================================================================*/
static void
DoPilotSync()
{
	HotSyncPrompt(XP_PILOT_SYNC);
}


/*========================================================================
 *	Name:			FetchAddresses
 *	Prototype:	static void FetchAddresses()
 *					
 *	Description:
 *
 *	Notes:
 *	Front end to FetchPilot() for fetching addresses from the Pilot.
 *========================================================================*/
static void
FetchAddresses(
	Widget		widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
	HotSyncPrompt(XP_PILOT_ADDR);
}
