/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: pixmaps.c
 *            DESCRIPTION: functions that create pixmaps
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPCreatePixmap
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: X11R4 or later, Xt, Motif 1.2
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: set tabstops=3 for readibility
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *========================================================================
 *
 * MODIFICATION HISTORY:
 *	$Log$
 *========================================================================*/
#ifndef XPIXMAP_C
#define XPIXMAP_C

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>

#include "motif.h"

/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"

/* === external routines === */

/* === Public routine prototypes === */

/* === Private routine prototypes === */

/* === Global Variables defined externally === */
extern Widget	Top_Level;
extern AppRes	app_res;

/* === Global Variables defined locally === */

/* === Static Variables === */


/*========================================================================
 *	Name:			XPCreatePixmap
 *	Prototype:	XPCreatePixmap(Pixmap *pm, Pixmap *mask, char *filename
 *							XpmAttributes *attributes)
 *					
 *
 *	Description:
 *		Create pixmaps for program.
 *
 *	Input Arguments:
 *		char *filename		name of ASCII pixmap file to read.
 *
 *	Output Arguments:
 *		Pixmap			*pm			Place to return the pointer to the new pixmap.
 *		Pixmap			*mask			Place to return the pointer to the new pixmap 
 *											mask.
 *		XpmAttributes	*attributes	Place to return pixmap's attributes.
 *
 *	Return Values:
 *		XpmAttributes	*ptr			points to the attributes returned for pixmap.
 *
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *		
 *========================================================================*/

void
XPCreatePixmap(
	Pixmap			*pixmap,
	Pixmap			*mask,
	char				*filename,
	XpmAttributes	*attributes,
	Widget			parent
)
{
#ifdef DEBUG
	char	*fname="XPCreatePixmap()";
#endif

	Arg				args[5];
	int				nargs;
	int				rc;
	Display			*display;
	Window			win;
	Pixel				bg;
	XpmColorSymbol	color_symbols[8];
	char				buf[MAXPATHLEN*2];

	DBGEnter();

	/*
	 * Grab the Display and Window of the root window, we'll need them later.
	 */
	display = XtDisplay(Top_Level);
	win = XtWindow(Top_Level);

	/*
	 * Use the default colormap for the icons
	 */
	attributes->colormap = DefaultColormap(display, DefaultScreen(display));

	/*
	 * Force the icon to a set of colors already within the default
	 * colormap - makes life a little easier for me, but might cause
	 * icons to be different colors on different systems.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNbackground, &bg ); nargs++;
	XtGetValues( parent, args, nargs );

	/*
	 * Take the color with the symbollic name "None" to be the same color
	 * as the background color of the parent window.
	 */
	color_symbols[0].pixel = bg;
	color_symbols[0].name = "None";
	color_symbols[0].value = NULL;

	/*
	 * We're only specifying one new color for the Pixmap
	 */
	attributes->colorsymbols = color_symbols;
	attributes->numsymbols = 1;

	/*
	 * And we don't need to match the colors in the Pixmap exactly.  Just
	 * get as close as you can to colors already specified in the default
	 * colormap.
	 */
	attributes->exactColors = 0;
	attributes->closeness = 40000;
	
	/*
	 * Create the pixmap
	 */
	attributes->valuemask = XpmColorSymbols | XpmColormap |
									XpmCloseness | XpmReturnPixels;

	/*
	 * Build the full path name to the XPM files.  Its possible to 
	 * overflow the buffer, but only if the xpmdir path is *really* long.
	 */
	sprintf(buf, "%s/%s", app_res.xpm_dir, filename);

	/*
	 * And finally, read in the pixmap
	 */
	rc = XpmReadPixmapFile(display, DefaultRootWindow(display),
				buf, pixmap, mask, attributes);

	DBGPrintf(DBG_INFO,("XpmReadPixmapFile() returned: %d\n", rc));
	return;
}

#endif /* XPIXMAP_C */
