/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: print.c
 *            DESCRIPTION: print routines for XNotesPlus 
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPPrintNote
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES:
 *	
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_PRINT_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <string.h>
#include <time.h>
#include <sys/param.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include "motif.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */


/* === Global Variables === */
extern AppRes		app_res;
extern StickyNote	*notes;
extern Widget		Top_Level;


/* === Static Variables === */



/*========================================================================
 *	Name:			XPPrintNote
 *	Prototype:	XPPrintNote(Widget widget, int index, XtPointer call_data)
 *					
 *	Description:
 * Prints a note.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPPrintNote(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
#ifdef DEBUG
	char			*fname="XPPrintNote";
	int			rc;
#endif

	StickyNote	*pn;
	FILE			*fp;
	int			len;
	char			tmpfile[512];
	char			cmd[1024];
	char			buf[1024];

	DBGEnter();

	/*
	 * Find the note we're printing.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Retrieve the current text from the note.
	 * How do we do this?
	 */

	/*
	 * Make sure there is something to print.
	 */
	if ( pn->pn_text != NULL )
		free(pn->pn_text);
	pn->pn_text = XmTextGetString(pn->pn_textwidget);
	if ( strlen(pn->pn_text) == 0 )
	{
		XPErrPopUp(XP_S_NO_TEXT_TO_PRINT);
		return;
	}

	/*
	 * Open a temporary file
	 */
	sprintf(tmpfile,"%s/xp%d", app_res.tmp_dir, (int)getpid());
	if ((fp = fopen(tmpfile, "w")) == NULL) 
	{
		fprintf(stderr, "xnotes: ");
		perror(tmpfile);
		sprintf(cmd, XP_S_TEMP_OPEN_FAIL, tmpfile);
		XPErrPopUp(cmd);
		return;
	}

	/*
	 * Write note title to temp file.
	 */
	len = strlen(pn->pn_name);
	if (len)
	{
		sprintf(buf, "Note: ");
		fwrite(buf, sizeof(char), strlen(buf), fp);
		fwrite(pn->pn_name, sizeof(char), len, fp);
		sprintf(buf, "\n");
		fwrite(buf, sizeof(char), strlen(buf), fp);
		sprintf(buf, "---------------------------------------------------\n");
		fwrite(buf, sizeof(char), strlen(buf), fp);
	}
	else
	{
		sprintf(buf, XP_S_DEFAULT_NOTE_TITLE);
		fwrite(buf, sizeof(char), strlen(buf), fp);
	}

	/*
	 * Write note text to temp file.
	 */
	len = strlen(pn->pn_text);
	if (len)
		fwrite(pn->pn_text, sizeof(char), len, fp);
	else
	{
		XPErrPopUp(XP_S_NO_TEXT_TO_PRINT);
		return;
	}
	fclose(fp);

	/*
	 * Use the printcmd resource to determine how to print the note
	 */
	sprintf (cmd, app_res.print_cmd, tmpfile );
#ifdef DEBUG
	DBGPrintf(DBG_INFO,("Command used to print the note: %s\n", cmd));
	rc = system ( cmd );
	switch ( rc )
	{
		case 127:
			sprintf(cmd,"%s failed: errno=%d", cmd, errno);
			XPErrPopUp(cmd);
			break;
			
		case -1:
			sprintf(cmd,"%s failed: errno=%d", cmd, rc);
			XPErrPopUp(cmd);
			break;
			
		default:
			sprintf(cmd,"%s succeeded", cmd);
			XPMsgWindow(
				cmd, XP_INFO_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK, NULL, 
				NULL, NULL, NULL, NULL, NULL);
			break;
	}
#else
	system ( cmd );
#endif

	/*
	 * clean up the temporary file
	 */
	sprintf (cmd, "rm %s", tmpfile );
	system ( cmd );

}

