/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995.
 *========================================================================
 *              FILE NAME: projects.c 
 *            DESCRIPTION: handle note project list.
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPProject
 *      PRIVATE FUNCTIONS: UpdateProject ClearProject UpdateRGB ResetWindow
 *								 : WriteProjects PNoteClear PNoteReset PNoteUpdate
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_PROJECT_C

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "motif.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void		UpdateProject();
static void		ClearProject();
static void		UpdateRGB();
static void		ResetWindow();
static void		WriteProjects();
static void		PNoteClear();
static void		PNoteReset();
static void		PNoteUpdate();

/* === Global Variables === */
extern Widget		Top_Level;
extern Display		*display;
extern Colormap	XP_Colormap;

ProjectType			*projects_list=NULL;


/* === Static Variables === */
static Widget		pnote_widget;
static Widget		pnote_color;
static Widget		pnote_list;

static Widget		project_widget;
static Widget		project_color;
static Widget		rgb_list, project_list;
static Widget		project_text;


/*========================================================================
 *	Name:			XPProject
 *	Prototype:	XPProject()
 *					
 *	Description:
 * Dialog window for managing projects and their colors.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPProject()
{
#ifdef DEBUG
	char	fname[]="XPProject";
#endif

	int				count;
	Widget			vbox;
	Widget			confirm_button, cancel_button;
	Widget			label, frame;
	char				buf[512];
	char				*ptr;
	Arg				args[15];
	int				nargs;
	int				dummy1, dummy2, dummy3;
	char				color_name[1028];
	Dimension		width, height;
	XmString			xm_string;
	FILE				*fp;
	ProjectType		*item;
	int				pos; 
	char				*string;
	XmStringTable	positions;

	DBGEnter();

	/*
	 * Allow only one copy at a time.
	 */
	if ( project_widget != NULL )
		return;

	/*
	 * Create a dialog, set its title, and position it at the cursor.
	 */
	project_widget = XPCreateGenericShell(ProjectDialog, 
					ACCEPT_BUTTON | CLOSE_BUTTON );

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	sprintf(buf, "*GenericForm%d", (int)project_widget );
	vbox = XtNameToWidget(project_widget, buf);
	sprintf(buf, "*GenericCancelB%d", (int)project_widget);
	cancel_button = XtNameToWidget(project_widget, buf);
	sprintf(buf, "*GenericAcceptB%d", (int)project_widget);
	confirm_button = XtNameToWidget(project_widget, buf);

	/*
	 * A titled frame around the scrolled list of color choices.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN ); nargs++;
	XtSetArg ( args[nargs], XmNshadowThickness, 4 ); nargs++;
	frame = XmCreateFrame(
						vbox,
						"frame",
						args, nargs);
	XtManageChild(frame);
	xm_string = XmStringCreateLtoR(XP_S_COLOR_CHOICES, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetArg ( args[nargs], XmNchildType, XmFRAME_TITLE_CHILD); nargs++;
	label= XmCreateLabel(
					frame,
					"frame",
					args, nargs);
	XmStringFree(xm_string);
	XtManageChild(label);
 
	/*
	 * An scrolled list in which the set of RGB names will go.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNvisibleItemCount, 15 ); nargs++;
	XtSetArg ( args[nargs], XmNselectionPolicy, XmSINGLE_SELECT ); nargs++;
	rgb_list = XmCreateScrolledList(
							frame,
							"rgbList",
							args, nargs
							);
	XtManageChild(rgb_list);
	XtAddCallback( rgb_list, XmNsingleSelectionCallback, 
		UpdateRGB, (XtPointer)NULL);

	/*
	 * Parse the rgb.txt file into the list of colors.
	 */
	DBGPrintf(DBG_INFO,("rgbfile: %s\n", app_res.rgb_file));
	fp = fopen(app_res.rgb_file, "r");
	while ( ! feof(fp) )
	{
		if ( fscanf(fp, "%d %d %d", &dummy1, &dummy2, &dummy3) == 3 )
		{
			fgets(color_name, 256, fp);
			ptr = strtok(color_name, "\n");
			while ( (ptr[0] == ' ') || (ptr[0] == '\t') )
				ptr++;

			DBGPrintf(DBG_INFO,
				("rgb: %d %d %d *%s*\n", dummy1, dummy2, dummy3, ptr));

			xm_string = XmStringCreateSimple(ptr);
			XmListAddItem(rgb_list, xm_string, 0);
			XmStringFree(xm_string);
		}
		else
			/* just jump to the end of the line and skip it */
			fgets(color_name, 256, fp);
	}
	fclose(fp);


	/*
	 * A label over the scrolled project list.
	 */
	xm_string = XmStringCreateLtoR(XP_S_PROJ_CHOICES, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNleftWidget, frame); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
	label= XmCreateLabel(
					vbox,
					"frame",
					args, nargs);
	XmStringFree(xm_string);
	XtManageChild(label);

	/*
	 * An scrolled list in which the current set of project names will go.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNvisibleItemCount, 8 ); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNleftWidget, frame); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, label); nargs++;
	XtSetArg ( args[nargs], XmNselectionPolicy, XmSINGLE_SELECT ); nargs++;
	project_list = XmCreateScrolledList(
							vbox,
							"projectList",
							args, nargs
							);
	XtManageChild(project_list);
	XtAddCallback( project_list, XmNsingleSelectionCallback, 
		ResetWindow, (XtPointer)NULL);


	/*
	 * Add the projects to the scrolled window.
	 */
	item = projects_list;
	while ( item != NULL )
	{
		xm_string = XmStringCreateSimple(item->name);
		XmListAddItem(project_list, xm_string, 0);
		XmStringFree(xm_string);
		item = item->next;
	}


	/*
	 * Set the color list selected item to be the color of the first
	 * project, if it exists.  Otherwise, set it to the first color 
	 * in the list.
	 */
	if ( projects_list != NULL )
	{
		xm_string = XmStringCreateSimple(projects_list->color);
		pos = XmListItemPos(rgb_list, xm_string);
		if (pos == 0)
			XmListSelectPos(rgb_list, 1, False);
		else
		{
			XmListSelectPos(rgb_list, pos, False);
			XmListSetPos(rgb_list, pos);
		}
		XmStringFree(xm_string);
	}
	else
		XmListSelectPos(rgb_list, 1, False);

	/*
	 * Set the first entry in the project list to selected.
	 */
	XmListSelectPos(project_list, 1, False);

	/*
	 * A text field for the project name.
	 */
	nargs=0;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNleftWidget, frame); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, project_list); nargs++;
	project_text = XmCreateTextField(
							vbox,
							"projectText",
							args, nargs
							);
	XtManageChild(project_text);
	if ( projects_list != NULL )
		XmTextFieldSetString( project_text, projects_list->name);

	/* Add a tool tip */
	XtAddEventHandler(project_text,
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_PRJ_TEXT);

	/*
	 * A colored bar showing the current selected color.
	 */
	xm_string = XmStringCreateSimple(XP_S_SAMPLE_NOTE);
	nargs=0;
   XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNleftWidget, frame); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, project_text); nargs++;
   XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM); nargs++;
	project_color = XmCreateLabel(
							vbox,
							"projectColor",
							args, nargs
							);
	XtManageChild(project_color);
	XmStringFree(xm_string);


	/*
	 * Get the color string of the selected color list item and update the color
	 * bar.
	 */
	nargs=0;
	XtSetArg( args[nargs], XmNselectedItems, &positions ); nargs++;
	XtSetArg( args[nargs], XmNselectedItemCount, &count ); nargs++;
	XtGetValues( rgb_list, args, nargs );
	if ( count > 0 )
	{
		XmStringGetLtoR(positions[0], XmFONTLIST_DEFAULT_TAG, &string);
		XPUpdateColorBox(project_widget, project_color, string);
		XtFree(string);
	}
	else
		XPMsgWindow(XP_S_NO_RGB, XP_INFO_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK,
			NULL, NULL, NULL, NULL, NULL, NULL);


	/*
	 * Confirmation button.
	 */
	XtAddCallback( confirm_button, XmNactivateCallback, 
			UpdateProject, (XtPointer)NULL);

	/*
	 * Cancellation button.
	 */
	XtAddCallback( cancel_button, XmNactivateCallback, 
			ClearProject, (XtPointer)NULL);

	/* Add a tool tip */
	XtAddEventHandler(confirm_button,
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_PRJ_SAVE);

	/* Add a tool tip */
	XtAddEventHandler(cancel_button,
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_PRJ_CANCEL);

	/*
	 * Stick the dialog in the middle of the display.
	 */
	XtRealizeWidget( project_widget );
	XPCenterWindow(project_widget, NULL, project_widget);

	XtPopup(project_widget, XtGrabNone);

	/*
	 * This bit of code forces the pop up window to stay its
	 * initial size, despite the users attempts to resize it.
	 * By doing this, I avoid having to muck extensively with
	 * aligning objects after a resize.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, &width); nargs++;
	XtSetArg ( args[nargs], XmNheight, &height); nargs++;
	XtGetValues ( project_widget, args, nargs);
	nargs = 0;
	XtSetArg ( args[nargs], XmNmaxWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNmaxHeight, height); nargs++;
	XtSetArg ( args[nargs], XmNminWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNminHeight, height); nargs++;
	XtSetValues ( project_widget, args, nargs); 

}


/*========================================================================
 *	Name:			UpdateProject
 *	Prototype:	UpdateProject(Widget widget, XtPointer client_data, 
 *						XtPointer call_data)
 *					
 *	Description:
 * Updates the display and/or lists when the Accept key is hit.
 *
 *	Notes:
 * Grab the name from the text field.  If its an existing project,
 * then grab the setting in the rgb list.  If this doesn't match
 * the projects current setting, update it.  If its a new project,
 * add it to the link list of projects and the project list.  Don't exit.
 *========================================================================*/
static void
UpdateProject(
	Widget		widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
	ProjectType		*item;
	Boolean			found;
	XmStringTable	positions;
	char				*name, *color;
	XmString			xm_string;
	int				nargs;
	Arg				args[2];

	/*
	 * Grab the name from the text field.
	 */
	name = XmTextFieldGetString(project_text);

	/*
	 * Find this project in the link list.
	 */
	item = projects_list;
	found = False;
	if ( item != NULL )
	{
		while ( (!found) && (item != NULL) )
		{
			if ( strcmp(item->name, name) == 0 )
				found = True;
			else
				item = item->next;
		}
	}

	/*
	 * Get color from color list.
	 */
	nargs=0;
	XtSetArg( args[nargs], XmNselectedItems, &positions ); nargs++;
	XtGetValues( rgb_list, args, nargs );
	XmStringGetLtoR(positions[0], XmFONTLIST_DEFAULT_TAG, &color);


	if ( found )
	{
		/*
		 * Project already exists - update its color.
		 */
		if ( item->color != NULL )
			free (item->color);
		item->color = XPSafeAlloc(strlen(color)+1);
		strcpy(item->color, color);
		XPMsgWindow(XP_S_PROJ_UPD, XP_INFO_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK,
			NULL, NULL, NULL, NULL, NULL, NULL);

		/*
		 * Update projects file.
		 */
		WriteProjects();

	}
	else
	{
		/*
		 * New project - add to end of list and select it.
		 */
		item = projects_list;
		if ( item != NULL )
		{
			while (item->next != NULL)
				item = item->next;
			item->next = (ProjectType *)XPSafeAlloc(sizeof(ProjectType));
			item = item->next;
		}
		else
		{
			item = (ProjectType *)XPSafeAlloc(sizeof(ProjectType));
			projects_list = item;
		}
		item->next = NULL;
		item->color = XPSafeAlloc(strlen(color)+1);
		strcpy(item->color, color);
		item->name = XPSafeAlloc(strlen(name)+1);
		strcpy(item->name, name);
		xm_string = XmStringCreateSimple(name);
		XmListAddItem(project_list, xm_string, 0);
		XmStringFree(xm_string);
		XmListSelectPos(project_list, 0, False);

		/*
		 * Update projects file.
		 */
		WriteProjects();

	}

	XtFree(name);
	XtFree(color);

}


/*========================================================================
 *	Name:			ClearProject
 *	Prototype:	ClearProject(Widget widget, XtPointer client_data, 
 *						XtPointer call_data)
 *					
 *	Description:
 * Closes the Projects window.
 *
 *	Notes:
 *========================================================================*/
static void
ClearProject(
	Widget		widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
	if ( project_widget != NULL )
	{
		XtDestroyWidget(project_widget);
		project_widget = NULL;
	}
}


/*========================================================================
 *	Name:			UpdateRGB
 *	Prototype:	UpdateRGB(Widget widget, XtPointer client_data, 
 *						XtPointer call_data)
 *					
 *	Description:
 * Updates the colored box based on the entry selected from the list 
 * of colors.
 *
 *	Notes:
 *========================================================================*/
static void
UpdateRGB(
	Widget		widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
#ifdef DEBUG
	char		*fname="UpdateRGB()";
#endif

	char		*string;

	XmListCallbackStruct	*cbs = (XmListCallbackStruct *)call_data;

	DBGEnter();

	/*
	 * Get the name of the color from the list.
	 */
	XmStringGetLtoR(cbs->item, XmFONTLIST_DEFAULT_TAG, &string);
	DBGPrintf(DBG_INFO,("color name: %s\n", string));

	XPUpdateColorBox(project_widget, project_color, string);
	XtFree(string);

}



/*========================================================================
 *	Name:			XPUpdateColorBox
 *	Prototype:	void XPUpdateColorBox(Widget parent, Widget child, 
 *										char *string)
 *					
 *	Input Arguments:
 *	Widget	parent		parent of child, just make it the shell widget
 *	Widget	child			widget whose background color is to be updated.
 *	char		*string		color name.
 *
 *	Description:
 * Updates the colored box based on the color text string passed in
 *
 *	Notes:
 *========================================================================*/
void
XPUpdateColorBox(
	Widget	parent,
	Widget	child,
	char		*string
)
{
#ifdef DEBUG
	char	*fname="XPUpdateColorBox()";
#endif

	XrmValue	from_string, to_pixel;
	Pixel		color;
	int		nargs;
	Arg		args[2];
	XColor	col, unused;

	DBGEnter();

	if ( ! XAllocNamedColor(display, XP_Colormap, string, &col, &unused) )
	{
		XPErrPopUp(XP_S_CANT_CONV_COLOR);
		return;
	}

	/*
	 * Convert the color name to its Pixel representation.
	 */
	from_string.addr = string;
	from_string.size = strlen(string);
	to_pixel.addr = (caddr_t)&color;
	to_pixel.size = sizeof(Pixel);
	if ( XtConvertAndStore(parent, 
				XtRString, &from_string,
				XtRPixel, &to_pixel) != True )
	{
		XPErrPopUp(XP_S_CANT_CONV_COLOR);
		return;
	}

	DBGPrintf(DBG_INFO,("color value: 0x%08x\n", (int)color));

	/*
	 * Update the color block in the dialog window.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNbackground, color ); nargs++;
	XtSetArg ( args[nargs], XmNborderColor, color ); nargs++;
	XtSetValues( child, args, nargs );
}


/*========================================================================
 *	Name:			ResetWindow
 *	Prototype:	ResetWindow(Widget widget, XtPointer client_data, 
 *						XtPointer call_data)
 *					
 *	Description:
 * Update the color list, the color box, and the text field when an
 * existing project is selected from the projects scrolled list.
 *
 *	Notes:
 *========================================================================*/
static void
ResetWindow(
	Widget		widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
#ifdef DEBUG
	char		*fname="ResetWindow()";
#endif

	Boolean			found;
	char				*string;
	ProjectType		*item;
	XmString			xm_string;
	XmStringTable	positions;
	int				pos;
	int				nargs;
	Arg				args[2];

	XmListCallbackStruct	*cbs = (XmListCallbackStruct *)call_data;

	DBGEnter();

	/*
	 * Get the name of the project from the list.
	 */
	XmStringGetLtoR(cbs->item, XmFONTLIST_DEFAULT_TAG, &string);
	XmTextFieldSetString( project_text, string);

	/*
	 * Find this project in the link list.
	 */
	item = projects_list;
	found = False;
	if ( item != NULL )
	{
		while ( (!found) && (item != NULL) )
		{
			if ( strcmp(item->name, string) == 0 )
				found = True;
			else
				item = item->next;
		}
	}

	/*
	 * Update the color box accordingly.
	 */
	if ( found == True )
	{
		xm_string = XmStringCreateSimple(item->color);
		pos = XmListItemPos(rgb_list, xm_string);
		if (pos == 0)
		{
			XmListSelectPos(rgb_list, 1, False);
			XmListSetPos(rgb_list, 1);
		}
		else
		{
			XmListSelectPos(rgb_list, pos, False);
			XmListSetPos(rgb_list, pos);
		}
		XmStringFree(xm_string);
	}
	else
	{
		XmListSelectPos(rgb_list, 1, False);
		XmListSetPos(rgb_list, 1);
	}

	nargs=0;
	XtSetArg( args[nargs], XmNselectedItems, &positions ); nargs++;
	XtGetValues( rgb_list, args, nargs );
	XmStringGetLtoR(positions[0], XmFONTLIST_DEFAULT_TAG, &string);
	XPUpdateColorBox(project_widget, project_color, string);

	XtFree(string);
}


/*========================================================================
 *	Name:			XPSetUpProjects
 *	Prototype:	XPSetUpProjects()
 *					
 *	Description:
 * Read in the projects file into a link list of projects, if the file
 * exists.
 *
 *	Global Variables:
 *	ProjectType		*projects_list
 *
 *	Notes:
 * This routine gets called before app_res.note_dir is properly set up.
 *========================================================================*/
void
XPSetUpProjects()
{
#ifdef DEBUG
	char	*fname="XPSetUpProjects()";
#endif

	struct stat			*stat_buf;
	char					filename[MAXPATHLEN*2];
	FILE					*fp;
	char					buf[256], *string;
	ProjectType			*item, *lastitem=NULL;
	char					*name, *color;

	DBGEnter();

	/*
	 * Check to see if the projects file exists.
	 */
	sprintf(filename, "%s/%s", app_res.note_dir, DefaultProjectFile);

	DBGPrintf(DBG_INFO,("projects file: %s\n", filename));

	stat_buf = (struct stat *) malloc( sizeof(struct stat) );
	if ( stat ( filename, stat_buf ) == -1 )
	{
		DBGPrintf(DBG_INFO, ("No projects file found.\n"));
		projects_list = NULL;
		return;
	}
	free(stat_buf);

	/*
	 * Open the file.  Read in each line and parse into its name/value
	 * pairs.  Add it to the link list.
	 */
	fp = fopen(filename, "r");

	/*
	 * Verify the file magic id.
	 */
	fgets(buf, 256, fp);
	string=strtok(buf,"\n");
	if ( strcmp(string, StickyProjMagic_v3) != 0 )
	{
		projects_list = NULL;
		return;
	}
	
	while ( fgets(buf, 256, fp) != NULL )
	{
		if ( ! feof(fp) )
		{
			string = XPSafeAlloc(strlen(buf)+1);
			strcpy(string, buf);

			/*
			 * Strip off the newline character.
			 */
			string=strtok(string,"\n");
			name = strtok(string,":");
			color = strtok(NULL,":");

			item = (ProjectType *)XPSafeAlloc(sizeof(ProjectType));
			item->name = XPSafeAlloc(strlen(name)+1);
			strcpy(item->name, name);
			item->color = XPSafeAlloc(strlen(color)+1);
			strcpy(item->color, color);
			item->next = NULL;

			free(string);

			DBGPrintf(DBG_INFO, 
				("project/color: %s %s\n", item->name, item->color));

			/*
			 * Add it to link list.
			 */
			if ( projects_list == NULL )
				projects_list = item;
			else
				lastitem->next = item;
			lastitem = item;

		}
	}
	fclose(fp);

}


/*========================================================================
 *	Name:			WriteProjects
 *	Prototype:	WriteProjects()
 *					
 *	Description:
 * Write the projects file from the link list of projects.
 *
 *	Global Variables:
 *	ProjectType		*projects_list
 *
 *	Notes:
 * Since the projects list will always be relatively small (who has
 * 10,000 projects for a sticky notes program?) we can do this every
 * time a project entry is updated.  The overhead, from a user perspective,
 * is unnoticable.
 *========================================================================*/
static void
WriteProjects()
{
#ifdef DEBUG
	char	*fname="WriteProjects()";
#endif

	char					filename[MAXPATHLEN*2];
	FILE					*fp;
	ProjectType			*item;

	DBGEnter();

	/*
	 * Check to see if the projects file exists.
	 */
	sprintf(filename, "%s/%s", app_res.note_dir, DefaultProjectFile);

	DBGPrintf(DBG_INFO,("projects file: %s\n", filename));

	/*
	 * Open the file.  
	 */
	fp = fopen(filename, "w");
	if ( fp == NULL )
		perror("fopen failed");

	/*
	 * Write the magic id to the file.
	 */
	fprintf(fp, "%s\n", StickyProjMagic_v3);

	/*
	 * Run the link list and write each link to the file.
	 */
	item = projects_list;
	while (item != NULL )
	{
		fprintf(fp, "%s:%s\n", item->name, item->color);
		item = item->next;
	}

	/*
	 * Close the projects file.
	 */
	fclose(fp);

}


/*========================================================================
 *	Name:			XPNoteProject
 *	Prototype:	XPNoteProject()
 *					
 *	Description:
 * Change a notes project configuration.
 *
 *	Global Variables:
 *	ProjectType		*projects_list
 *
 *	Notes:
 *========================================================================*/
void
XPNoteProject(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
#ifdef DEBUG
	char	*fname="XPNoteProject()";
#endif

	Widget		vbox, cancel_button, confirm_button;
	Widget		frame, label;
	XmString		xm_string;
	ProjectType	*item;
	char			buf[256];
	int			nargs;
	Arg			args[10];
	Dimension	width, height;
	StickyNote	*pn;

	DBGEnter();

	/*
	 * Allow only one copy at a time.
	 */
	if ( pnote_widget != NULL )
		return;

	/*
	 * Grab the note structure.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Create a dialog, set its title, and position it at the cursor.
	 */
	pnote_widget = XPCreateGenericShell(ProjectNoteDialog, 
					ACCEPT_BUTTON | CLOSE_BUTTON );

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	sprintf(buf, "*GenericForm%d", (int)pnote_widget );
	vbox = XtNameToWidget(pnote_widget, buf);
	sprintf(buf, "*GenericCancelB%d", (int)pnote_widget);
	cancel_button = XtNameToWidget(pnote_widget, buf);
	sprintf(buf, "*GenericAcceptB%d", (int)pnote_widget);
	confirm_button = XtNameToWidget(pnote_widget, buf);

	/*
	 * A titled frame around the scrolled list of color choices.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM ); nargs++;
	XtSetArg ( args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN ); nargs++;
	XtSetArg ( args[nargs], XmNshadowThickness, 4 ); nargs++;
	frame = XmCreateFrame(
						vbox,
						"frame",
						args, nargs);
	XtManageChild(frame);
	xm_string = XmStringCreateLtoR(XP_S_PROJ_CHOICES, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetArg ( args[nargs], XmNchildType, XmFRAME_TITLE_CHILD); nargs++;
	label= XmCreateLabel(
					frame,
					"frame",
					args, nargs);
	XmStringFree(xm_string);
	XtManageChild(label);
 
	/*
	 * An scrolled list in which the current set of project names will go.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNvisibleItemCount, 6 ); nargs++;
	XtSetArg ( args[nargs], XmNselectionPolicy, XmSINGLE_SELECT ); nargs++;
	pnote_list = XmCreateScrolledList(
							frame,
							"projectList",
							args, nargs
							);
	XtManageChild(pnote_list);
	XtAddCallback( pnote_list, XmNsingleSelectionCallback, 
		PNoteReset, (XtPointer)pn);


	/*
	 * Add the projects to the scrolled window.
	 */
	item = projects_list;
	while ( item != NULL )
	{
		xm_string = XmStringCreateSimple(item->name);
		XmListAddItem(pnote_list, xm_string, 0);
		XmStringFree(xm_string);
		item = item->next;
	}

	/*
	 * A colored bar showing the current selected color.
	 */
	xm_string = XmStringCreateSimple(XP_S_SAMPLE_NOTE);
	nargs=0;
   XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNleftWidget, frame); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM); nargs++;
	pnote_color = XmCreateLabel(
							vbox,
							"projectColor",
							args, nargs
							);
	XtManageChild(pnote_color);
	XmStringFree(xm_string);

	/*
	 * Set the color bar based on the current project color.
	 * If no project is set, then don't do anything.
	 */
	if ( pn->pn_project != NULL )
	{
		if ( strcmp(pn->pn_project, "NONE") != 0 )
		{
			item = projects_list;
			while ( item != NULL )
			{
				if ( strcmp ( item->name, pn->pn_project ) == 0 )
				{
					xm_string = XmStringCreateSimple(item->name);
					XmListSelectItem(pnote_list, xm_string, False);
					XmListSetItem(pnote_list, xm_string);
					XmStringFree(xm_string);
					XPUpdateColorBox(pnote_widget, pnote_color, item->color);
					break;
				}
				item = item->next;
			}
		}
	}

	/*
	 * Confirmation button.
	 */
	XtAddCallback( confirm_button, XmNactivateCallback, 
			PNoteUpdate, (XtPointer)pn);

	/*
	 * Cancellation button.
	 */
	XtAddCallback( cancel_button, XmNactivateCallback, 
			PNoteClear, (XtPointer)NULL);

	/*
	 * Stick the dialog in the middle of the display.
	 */
	XtRealizeWidget( pnote_widget );
	XPCenterWindow(pnote_widget, pn->pn_shellwidget, pnote_widget);

	XtPopup(pnote_widget, XtGrabNone);

	/*
	 * This bit of code forces the pop up window to stay its
	 * initial size, despite the users attempts to resize it.
	 * By doing this, I avoid having to muck extensively with
	 * aligning objects after a resize.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, &width); nargs++;
	XtSetArg ( args[nargs], XmNheight, &height); nargs++;
	XtGetValues ( pnote_widget, args, nargs);
	nargs = 0;
	XtSetArg ( args[nargs], XmNmaxWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNmaxHeight, height); nargs++;
	XtSetArg ( args[nargs], XmNminWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNminHeight, height); nargs++;
	XtSetValues ( pnote_widget, args, nargs); 
}


/*========================================================================
 *	Name:			PNoteClear
 *	Prototype:	PNoteClear(Widget widget, StickyNote *pn, 
 *						XtPointer call_data)
 *					
 *	Description:
 * Close the Note's project dialog.
 *
 *	Notes:
 *========================================================================*/
static void
PNoteClear()
{
	if ( pnote_widget != NULL )
	{
		XtDestroyWidget(pnote_widget);
		pnote_widget = NULL;
	}
}


/*========================================================================
 *	Name:			PNoteUpdate
 *	Prototype:	PNoteUpdate(Widget widget, StickyNote *pn, 
 *						XtPointer call_data)
 *					
 *	Description:
 * Update the color list, the color box
 *
 *	Notes:
 *========================================================================*/
static void
PNoteUpdate(
	Widget		widget,
	StickyNote	*pn,
	XtPointer	call_data
)
{
	XmStringTable	positions;
	int				nargs;
	Arg				args[3];
	char				*string;
	ProjectType		*item;

	nargs=0;
	XtSetArg( args[nargs], XmNselectedItems, &positions ); nargs++;
	XtGetValues( pnote_list, args, nargs );
	XmStringGetLtoR(positions[0], XmFONTLIST_DEFAULT_TAG, &string);

	/*
	 * Found the project in the link list.  Update the list widget
	 * and the color box.
	 */
	item = projects_list;
	while ( item != NULL )
	{
		if ( strcmp ( item->name, string) == 0 )
		{
			if ( pn->pn_project != NULL )
				free(pn->pn_project);
			pn->pn_project = XPSafeAlloc(strlen(string)+1);
			strcpy(pn->pn_project, string);
			XPUpdateColorBox(pn->pn_shellwidget, pn->pn_textwidget, item->color);
			PNoteClear();
			XtFree(string);
			return;
		}
		item = item->next;
	}

	/*
	 * If we get here then we have a serious problem!
	 */
	XPErrPopUp(XP_S_CANT_FIND_PROJ);
	PNoteClear();
	XtFree(string);
}


/*========================================================================
 *	Name:			PNoteReset
 *	Prototype:	PNoteReset(Widget widget, XtPointer client_data, 
 *						XtPointer call_data)
 *					
 *	Description:
 * Update the color list, the color box
 *
 *	Notes:
 *========================================================================*/
static void
PNoteReset(
	Widget		widget,
	StickyNote	*pn,
	XtPointer	call_data
)
{
	ProjectType		*item;
	char				*string;

	XmListCallbackStruct	*cbs = (XmListCallbackStruct *)call_data;

	/*
	 * Get the name of the project from the list.
	 */
	XmStringGetLtoR(cbs->item, XmFONTLIST_DEFAULT_TAG, &string);

	/*
	 * Found the project in the link list.  Update the list widget
	 * and the color box.
	 */
	item = projects_list;
	while ( item != NULL )
	{
		if ( strcmp ( item->name, string) == 0 )
		{
			XPUpdateColorBox(pnote_widget, pnote_color, item->color);
			XtFree(string);
			return;
		}
		item = item->next;
	}

	XtFree(string);

	XPSetSaveSensitive(NULL, pn, NULL);

}
