/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: timer_cb.c
 *            DESCRIPTION: callbacks for timers
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPAutoSave, XPAlarmCheck, XPMakeAlarmPopup,
 *								 : XPCreateTimerPrompt, XPAlarmOff
 *      PRIVATE FUNCTIONS: TimerLeftArrowCB, TimerCancel,
 *								 : TimerRightArrowCB, TimerAccept, UpdateAudioID,
 *								 : Reset
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_TIMERCB_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <signal.h>

#include "motif.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */
void XPMakeAlarmPopup();


/* === Private routine prototypes === */
static void TimerRightArrowCB();
static void TimerLeftArrowCB();
static void TimerCancel();
static void TimerAccept();
static void UpdateAudioID();
static void Reset();


/* === Global Variables === */
extern Widget			Top_Level;
extern Display			*display;
extern AppRes			app_res;
extern StickyNote		*notes;
extern Pixmap			alarmon_xbm;
extern Pixmap			alarmoff_xbm;
extern AudioFileType	*audio_files;


/* === Static Variables === */
static int				audio_id=-1;
static int				reset_val = 0;
static Widget			timer_text[5];
static Widget			timer_widget=NULL;



/*========================================================================
 *	Name:			XPAutoSave
 *	Prototype:	void *XPAutoSave ( XtPointer *user_data, XtIntervalId *id )
 *					
 *	Description:
 * Save all notes automatically when timer expires
 *
 *	Input Arguments:
 *	
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
int
XPAutoSave(
	XtPointer 		user_data,
	XtIntervalId	*id
)
{
	int				timer_interval;
	XtIntervalId	timer;

	XPSaveAllNotes(FALSE);

	/*
	 * Reestablish the timer.
	 */
	timer_interval = app_res.interval * 1000;
	timer = XtAppAddTimeOut (
			XtWidgetToApplicationContext(Top_Level),
			timer_interval,
			(XtTimerCallbackProc)XPAutoSave,
			NULL );

	return(TRUE);
}


/*========================================================================
 *	Name:			XPAlarmCheck
 *	Prototype:	void *XPAlarmCheck ( XtPointer *user_data, XtIntervalId *id )
 *					
 *	Description:
 * Check the notes for any alarms that need to go off.
 *
 *	Input Arguments:
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
int
XPAlarmCheck(
	XtPointer 		user_data,
	XtIntervalId	*id
)
{
#ifdef DEBUG
	char	*fname="XPAlarmCheck()";
#endif

	char	buf[1024];
	char	buf2[MAXPATHLEN];
	char	*earg[257];
	int	i;

	int				alarm_interval;
	XtIntervalId	alarm_timer;

	register StickyNote	*pn;
	time_t					time_tp;
	struct tm				*tm_tp;

	for (pn = notes; pn != NULL; pn = pn->pn_next)
	{
		if (pn->pn_alarm)
		{
			time(&time_tp);
			tm_tp = localtime(&time_tp);
			if ( (pn->pn_alarm_mon == tm_tp->tm_mon) &&
			     (pn->pn_alarm_day == tm_tp->tm_mday) &&
			     (pn->pn_alarm_hour == tm_tp->tm_hour) &&
			     (pn->pn_alarm_min == tm_tp->tm_min) )
			{

				/*
				 * Display the alarm message.
				 */
				XPMakeAlarmPopup(pn);

				/*
				 * Test to see if we need to try to play the audio alarm.
				 */
				if( app_res.noalarm != FALSE )
					DBGPrintf(DBG_INFO,("no alarm is false.\n"));
				if( app_res.audio_cmd == NULL )
					DBGPrintf(DBG_INFO,("audio cmd is null.\n"));
				if ( app_res.noaudio != FALSE)
					DBGPrintf(DBG_INFO,("no audio is false.\n"));
				if ( pn->pn_alarm_audio == NULL)
					DBGPrintf(DBG_INFO,("alarm audio is null.\n"));

				if(( app_res.noalarm == FALSE ) && 
					( app_res.audio_cmd != NULL) &&
					( app_res.noaudio == FALSE) &&
					( pn->pn_alarm_audio != NULL) )
				{

					DBGPrintf(DBG_INFO,("Playing audio alarm.\n"));

					/*
					 * Fork a second process to play the alarm audio,
					 * if requested.
					 */
					if ( fork() == 0 )
					{
						/*
						 * The child process plays the audio.  We fork() because
						 * playing the audio can take a moment and we don't want
						 * to block the rest of the program.
						 *
						 * Errors get written to stderr only.
						 */
						sprintf(buf2, "%s/%s", app_res.audio_dir, pn->pn_alarm_audio);
						sprintf(buf, app_res.audio_cmd, buf2, 
							(pn->pn_alarm_volume/100.0));
						earg[0] = strtok(buf," ");
						for (i=1; i<256; i++)
						{
							earg[i] = strtok(NULL," ");
							if ( earg[i] == NULL )
								break;
						}

						/*
						 * Make sure we NULL terminate the list of args.
						 */
						if (i==256)
						{
							earg[254] = "\"";
							earg[255] = NULL;
						}

						/*
						 * Now exec() the audio command.  We use exec() because
						 * we don't want the child to actually exit - that can
						 * cause problems with widgets shared with the parent
						 * that would go away.
						 */
						if ( execv(earg[0], &earg[0] ) == -1 )
						{
							fprintf(stderr,"%s: command failed (errno=%d)- %s\n",
								XP_S_PROG_NAME, errno, buf);
						}
						/*
						 * Just so its understood - the child stops here!
						 * (Except it should never actually get here.)
						 */
						exit(0);
					}
				}
			}
		}
	}

	/*
	 * Reestablish the timer.
	 */
	alarm_interval = XP_ALARM_PERIOD * 1000;
	alarm_timer = XtAppAddTimeOut (
			XtWidgetToApplicationContext(Top_Level),
			alarm_interval,
			(XtTimerCallbackProc)XPAlarmCheck,
			NULL );

	return(TRUE);
}


/*========================================================================
 *	Name:			XPMakeAlarmPopup
 *	Prototype:	void *XPMakeAlarmPopup ( StickyNote *pn )
 *					
 *	Description:
 * Create dialog for when alarm expires.
 *
 *	Input Arguments:
 *	StickyNote		*pn		pointer to note for which alarm has expired
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPMakeAlarmPopup(
	StickyNote *pn
)
{
	int				i;
	char				string[512];

	/*
	 * The labels with the text message.
	 */
	sprintf(string, "%s %s %s", 
		XP_S_EXPIRED_1, XP_S_EXPIRED_2, pn->pn_name);

	XPMsgWindow(string, 
		XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOCANCEL, NULL, pn->pn_shellwidget,
		NULL, NULL, NULL, NULL);


	/* double-ring bell 3 times */
	for (i=0; i<3; i++)
		XBell(display, 80);
}


/*========================================================================
 *	Name:			XPCreateTimerPrompt
 *	Prototype:	void XPCreateTimerPrompt(Widget widget, int index)
 *					
 *	Description:
 * Create a prompt for setting an alarm.
 *
 *	Input Arguments:
 *	GtkWidget	*widget		widget for which this callback was set
 * int			index			index of note associated with widget
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPCreateTimerPrompt(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
#ifdef DEBUG
	char			*fname="XPCreateTimerPrompt";
#endif 
	Arg				args[10];
	int				nargs;
	StickyNote		*pn;
	Widget			timer_label[4];
	Widget			timer_leftarrow, timer_rightarrow;
	Widget			timer_buttons[2];
	Widget			button;
	Widget			timer_box;
	Widget			frame1, frame2, frame3, vbox, vbox3, vbox4;
	Widget			label, hbox;
	Widget			audio_list;
	Widget			current_setting;
	char				buf[128];
	int				i;
	AudioFileType	*audio_link;
	time_t			time_tp;
	struct tm		*tm_tp;
	XmString			xm_string;
	int				position_index;

	DBGEnter();

	/*
	 * Don't open more than one alarm window.
	 */
	if ( timer_widget != NULL )
		return;

	/*
	 * Find the note of interest.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Create a popup shell widget
	 */
	timer_widget = XPCreateGenericShell(NotesTimerPrompt, ACCEPT_BUTTON | CANCEL_BUTTON);

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	sprintf(buf, "*GenericForm%d", (int)timer_widget );
	vbox = XtNameToWidget(timer_widget, buf);
	sprintf(buf, "*GenericAcceptB%d", (int)timer_widget);
	timer_buttons[0] = XtNameToWidget(timer_widget, buf);
	sprintf(buf, "*GenericCancelB%d", (int)timer_widget);
	timer_buttons[1] = XtNameToWidget(timer_widget, buf);

	/*
	 * 3 Frames around:
	 * 1. The current alarm setting.
	 * 2. Buttons to set the alarm.
	 * 3. Alarm audio setting.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
	XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
	XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
	XtSetArg ( args[nargs], XmNshadowType, XmSHADOW_ETCHED_OUT); nargs++;
	frame1 = XmCreateFrame(
					vbox,
					"frame",
					args, nargs);
	xm_string = XmStringCreateLtoR(XP_S_CURRENT_TIME, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetArg ( args[nargs], XmNchildType, XmFRAME_TITLE_CHILD); nargs++;
	label= XmCreateLabel(
					frame1,
					"frame",
					args, nargs);
	XmStringFree(xm_string);
	XtManageChild(label);
	XtManageChild(frame1);

	nargs=0;
	XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
	XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
	XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
	XtSetArg ( args[nargs], XmNtopWidget, frame1); nargs++;
	XtSetArg ( args[nargs], XmNshadowType, XmSHADOW_ETCHED_OUT); nargs++;
	frame2 = XmCreateFrame(
					vbox,
					"frame",
					args, nargs);
	xm_string = XmStringCreateLtoR(XP_S_SET_TIME, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetArg ( args[nargs], XmNchildType, XmFRAME_TITLE_CHILD); nargs++;
	label= XmCreateLabel(
					frame2,
					"frame",
					args, nargs);
	XmStringFree(xm_string);
	XtManageChild(label);
	XtManageChild(frame2);

	nargs=0;
	XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
	XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
	XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
	XtSetArg ( args[nargs], XmNtopWidget, frame2); nargs++;
	XtSetArg ( args[nargs], XmNshadowType, XmSHADOW_ETCHED_OUT); nargs++;
	frame3 = XmCreateFrame(
					vbox,
					"frame",
					args, nargs);
	xm_string = XmStringCreateLtoR(XP_S_SET_AUDIO, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetArg ( args[nargs], XmNchildType, XmFRAME_TITLE_CHILD); nargs++;
	label= XmCreateLabel(
					frame3,
					"frame",
					args, nargs);
	XmStringFree(xm_string);
	XtManageChild(label);
	XtManageChild(frame3);


	/*
	 * Verticle boxes to place the components inside the frames.
	 */
	nargs=0;
	vbox3 = XmCreateForm(
					frame2,
					"form",
					args, nargs);
	XtManageChild(vbox3);

	nargs=0;
	vbox4 = XmCreateForm(
					frame3,
					"form",
					args, nargs);
	XtManageChild(vbox4);

	/*
	 * The current alarm setting in a label.
	 */
	sprintf(buf, "%02d/%02d %02d:%02d", 
		pn->pn_alarm_mon+1, pn->pn_alarm_day,
		pn->pn_alarm_hour, pn->pn_alarm_min);
	xm_string = XmStringCreateLtoR(buf, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	current_setting = XmCreateLabel(
								frame1,
								"currentSetting",
								args, nargs);
	XmStringFree(xm_string);
	XtManageChild(current_setting);

	/*
	 * Horizontal boxes, one for each line of that sets part of the alarm
	 * setting, plus one for the Accept/Cancel buttons.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNorientation, XmHORIZONTAL); nargs++;
	XtSetArg ( args[nargs], XmNpacking, XmPACK_COLUMN); nargs++;
	XtSetArg ( args[nargs], XmNnumColumns, 4); nargs++;
	XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
	XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
	XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
	timer_box = XmCreateRowColumn(
						vbox3,
						"rowCol",
						args, nargs
						);
	XtManageChild(timer_box);

	/*
	 * A reset button - to make the spin values match the last setting.
	 */
	reset_val = 0;
	xm_string = XmStringCreateSimple(XP_S_TOGGLE);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string ); nargs++;
	XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
	XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
	XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
	XtSetArg ( args[nargs], XmNtopWidget, timer_box); nargs++;
	button = XmCreatePushButton (
					vbox3,
					"toggleButton",
					args, nargs
					);
	XtManageChild(button);
	XmStringFree(xm_string);
	XtAddCallback( button, XmNactivateCallback, Reset, (XtPointer)pn);

	/* Add a tool tip */
	XtAddEventHandler(button,
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_AUDIO_TOG);


	/* The labels of each box */
	for ( i=0; i<4; i++)
	{
		switch(i)
		{
			case 0: sprintf(buf, XP_S_MONTH ); break;
			case 1: sprintf(buf, XP_S_DAY); break;
			case 2: sprintf(buf, XP_S_HOUR); break;
			case 3: sprintf(buf, XP_S_MIN); break;
		}
		xm_string = XmStringCreateLtoR(buf, XmFONTLIST_DEFAULT_TAG);
		nargs=0;
		XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
		timer_label[i]= XmCreateLabel(
					timer_box,
					"frame",
					args, nargs);
		XmStringFree(xm_string);
		XtManageChild(timer_label[i]);
		nargs=0;
		XtSetArg ( args[nargs], XmNalignment, XmALIGNMENT_CENTER); nargs++;
		XtSetValues ( timer_label[i], args, nargs);
	}

	/*
	 * Get the current time.
	 */
	time(&time_tp);
	tm_tp = localtime(&time_tp);

	/*
	 * The arrow buttons and text fields in each box
	 */
	position_index=1;
	for ( i=0; i<4; i++)
	{
		sprintf( buf, " < ");
		xm_string = XmStringCreateSimple(buf);
		nargs=0;
		XtSetArg ( args[nargs], XmNlabelString, xm_string ); nargs++;
		XtSetArg ( args[nargs], XmNpositionIndex, position_index ); nargs++;
		timer_leftarrow = XmCreatePushButton (
					timer_box,
					"pushButton",
					args, nargs
					);
		XmStringFree(xm_string);
		XtManageChild(timer_leftarrow);
		nargs=0;
		XtSetArg ( args[nargs], XmNalignment, XmALIGNMENT_CENTER); nargs++;
		XtSetValues ( timer_leftarrow, args, nargs);
		XtAddCallback( timer_leftarrow, XmNactivateCallback, TimerLeftArrowCB, (XtPointer)i);

		/*
		 * Set the text fields to the notes stored values
		 */
		switch (i)
		{
			case 0: sprintf(buf, "%02d", tm_tp->tm_mon+1); break;
			case 1: sprintf(buf, "%02d", tm_tp->tm_mday); break;
			case 2: sprintf(buf, "%02d", tm_tp->tm_hour); break;
			case 3: sprintf(buf, "%02d", tm_tp->tm_min); break;
			default : sprintf(buf, "   "); break;
		}
		xm_string = XmStringCreateLtoR(buf, XmFONTLIST_DEFAULT_TAG);
		nargs=0;
		XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
		XtSetArg ( args[nargs], XmNborder, 2); nargs++;
		XtSetArg ( args[nargs], XmNpositionIndex, position_index+1 ); nargs++;
		timer_text[i] = XmCreateLabel(
								timer_box,
								"label",
								args, nargs);
		XmStringFree(xm_string);
		XtManageChild(timer_text[i]);
		nargs=0;
		XtSetArg ( args[nargs], XmNalignment, XmALIGNMENT_CENTER); nargs++;
		XtSetValues ( timer_text[i], args, nargs);

		sprintf( buf, " > ");
		xm_string = XmStringCreateSimple(buf);
		nargs=0;
		XtSetArg ( args[nargs], XmNlabelString, xm_string ); nargs++;
		XtSetArg ( args[nargs], XmNpositionIndex, position_index+2 ); nargs++;
		timer_rightarrow = XmCreatePushButton (
					timer_box,
					"pushButton",
					args, nargs
					);
		XmStringFree(xm_string);
		XtManageChild(timer_rightarrow);
		nargs=0;
		XtSetArg ( args[nargs], XmNalignment, XmALIGNMENT_CENTER); nargs++;
		XtSetValues ( timer_rightarrow, args, nargs);
		XtAddCallback( timer_rightarrow, XmNactivateCallback, TimerRightArrowCB, (XtPointer)i);

		position_index += 4;
	}

	/*
	 * A scrolled window with an audio list.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNvisibleItemCount, 5 ); nargs++;
	XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
	XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
	XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
	XtSetArg ( args[nargs], XmNselectionPolicy, XmSINGLE_SELECT); nargs++;
	audio_list = XmCreateScrolledList(
								vbox4,
								"scrolledList",
								args, nargs
								);
	XtManageChild(audio_list);
	XtAddCallback( audio_list, XmNsingleSelectionCallback, 
		UpdateAudioID, (XtPointer)NULL);

	/*
	 * If the audio command is not available, we don't allow setting this
	 * option.
	 */
	if ( app_res.audio_cmd == NULL )
		XtSetSensitive(audio_list, False);

	/*
	 * Populate the list.
	 */
	audio_id = -1;
	audio_link = audio_files;
	while (audio_link != NULL)
	{
		xm_string = XmStringCreateSimple(audio_link->file);
		XmListAddItem(audio_list, xm_string, 0);
		XmStringFree(xm_string);

		/*
		 * If the alarm already has an audio file selected, preselect it.
	 	 */
		if ( pn->pn_alarm_audio != NULL )
		{
			if ( strcmp(pn->pn_alarm_audio, audio_link->file) == 0 )
			{
				XmListSelectPos(audio_list, 0, FALSE);
				audio_id = audio_link->id;
			}
		}

		audio_link = audio_link->next;
	}

	/*
	 * A spinbox for setting the repeat amount.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNorientation, XmHORIZONTAL); nargs++;
	XtSetArg ( args[nargs], XmNpacking, XmPACK_COLUMN); nargs++;
	XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
	XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
	XtSetArg ( args[nargs], XmNtopWidget, audio_list); nargs++;
	hbox = XmCreateRowColumn(
						vbox4,
						"rowCol",
						args, nargs
						);
	XtManageChild(hbox);

	xm_string = XmStringCreateLtoR(XP_S_REPEAT_TEXT, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	label = XmCreateLabel(
				hbox,
				"label",
				args, nargs);
	XtManageChild(label);
	XmStringFree(xm_string);

	sprintf( buf, " < ");
	xm_string = XmStringCreateSimple(buf);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string ); nargs++;
	timer_leftarrow = XmCreatePushButton (
				hbox,
				"pushButton",
				args, nargs
				);
	XmStringFree(xm_string);
	XtManageChild(timer_leftarrow);
	nargs=0;
	XtSetArg ( args[nargs], XmNalignment, XmALIGNMENT_CENTER); nargs++;
	XtSetValues ( timer_leftarrow, args, nargs);
	XtAddCallback( timer_leftarrow, XmNactivateCallback, TimerLeftArrowCB, (XtPointer)i);

	/*
	 * If the audio command is not available, we don't allow setting this
	 * option.
	 */
	if ( app_res.audio_cmd == NULL )
		XtSetSensitive(timer_leftarrow, False);


	sprintf(buf, "%d", pn->pn_alarm_repeat);
	xm_string = XmStringCreateLtoR(buf, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	timer_text[i] = XmCreateLabel(
				hbox,
				"label",
				args, nargs);
	XtManageChild(timer_text[i]);
	XmStringFree(xm_string);
	nargs=0;
	XtSetArg ( args[nargs], XmNalignment, XmALIGNMENT_CENTER); nargs++;
	XtSetValues ( timer_text[i], args, nargs);

	/* Add a tool tip */
	XtAddEventHandler(timer_text[i],
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_AUDIO_REP);

	sprintf( buf, " > ");
	xm_string = XmStringCreateSimple(buf);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string ); nargs++;
	timer_rightarrow = XmCreatePushButton (
				hbox,
				"pushButton",
				args, nargs
				);
	XmStringFree(xm_string);
	XtManageChild(timer_rightarrow);
	nargs=0;
	XtSetArg ( args[nargs], XmNalignment, XmALIGNMENT_CENTER); nargs++;
	XtSetValues ( timer_rightarrow, args, nargs);
	XtAddCallback( timer_rightarrow, XmNactivateCallback, TimerRightArrowCB, (XtPointer)i);

	/*
	 * If the audio command is not available, we don't allow setting this
	 * option.
	 */
	if ( app_res.audio_cmd == NULL )
		XtSetSensitive(timer_rightarrow, False);

	/*
	 * create the Cancel/Accept command buttons 
	 */
	XtAddCallback( timer_buttons[0], XmNactivateCallback, TimerAccept, (XtPointer)pn);
	XtAddCallback( timer_buttons[1], XmNactivateCallback, TimerCancel, (XtPointer)NULL);
 
	/*
	 * Pop up the window.
	 */
	XtRealizeWidget(timer_widget);
	XPCenterWindow(timer_widget, pn->pn_shellwidget, timer_widget);
	XtPopup( timer_widget, XtGrabNone );
}


/*========================================================================
 *	Name:			TimerLeftArrowcB
 *	Prototype:	void TimerLeftArrowcB(Widget widget, int index, XtPointer call_data)
 *					
 *	Description:
 * Handle a left arrow click
 *========================================================================*/
void
TimerLeftArrowCB(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	int		max=0, min=0;
	int		value;
	char		*string;
	char		buf[10];
	Arg		args[2];
	int		nargs;
	XmString	xm_string;

	/* set limits based on which field is being updated */
	switch ( index )
	{
		case 0: max=12; min=1; break; /* month */
		case 2: max=23; min=0; break; /* hour */
		case 3: max=59; min=0; break; /* minute */
		case 4: max=20; min=0; break; /* repeat */

		/*
		 * setting the day max is dependent on which month
		 * is currently set
		 */
		case 1:
			min=1;  
			nargs = 0;
			XtSetArg ( args[nargs], XmNlabelString, &xm_string); nargs++;
			XtGetValues( timer_text[0], args, nargs);
			XmStringGetLtoR(xm_string, XmFONTLIST_DEFAULT_TAG, &string);
			XmStringFree(xm_string);
			value = (int)atoi(string);
			switch ( value )
			{
				case 1:
				case 3:
				case 5:
				case 7:
				case 8:
				case 10:
				case 12:	max=31; break;
				case 2:	max=29; break;
				default:	max=30; break;
			}
			free(string);
			break;
	}

	/* Decrement text field */
	nargs = 0;
	XtSetArg ( args[nargs], XmNlabelString, &xm_string); nargs++;
	XtGetValues( timer_text[index], args, nargs);
	XmStringGetLtoR(xm_string, XmFONTLIST_DEFAULT_TAG, &string);
	XmStringFree(xm_string);
	value = (int)atoi(string) - 1;
	free(string);

	if ( value < min )
		value = max;
	sprintf ( buf, "%02d", value);

	xm_string=XmStringCreateLtoR(buf, XmFONTLIST_DEFAULT_TAG);
	nargs = 0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetValues( timer_text[index], args, nargs);
	XmStringFree(xm_string);

}


/*========================================================================
 *	Name:			TimerRightArrowcB
 *	Prototype:	void TimerRightArrowcB(Widget widget, int index, XtPointer call_data)
 *					
 *	Description:
 * Handle a right arrow click
 *========================================================================*/
void
TimerRightArrowCB(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	Arg		args[2];
	int		nargs;
	int		max=0, min=0;
	char		*string;
	char		buf[25];
	int		value;
	XmString	xm_string;

	/* set limits based on which field is being updated */
	switch ( index )
	{
		case 0: max=12; min=1; break; /* month */
		case 2: max=23; min=0; break; /* hour */
		case 3: max=59; min=0; break; /* minute */
		case 4: max=20; min=0; break; /* repeat */

		/*
		 * setting the day max is dependent on which month
		 * is currently set
		 */
		case 1: 
			min = 1;
			nargs = 0;
			XtSetArg ( args[nargs], XmNlabelString, &xm_string); nargs++;
			XtGetValues( timer_text[0], args, nargs);
			XmStringGetLtoR(xm_string, XmFONTLIST_DEFAULT_TAG, &string);
			XmStringFree(xm_string);
			value = (int)atoi(string);
			switch ( value )
			{
				case 1:
				case 3:
				case 5:
				case 7:
				case 8:
				case 10:
				case 12:	max=31; break;
				case 2:	max=29; break;
				default:	max=30; break;
			}
			free(string);
			break;
	}

	/* increment text field */
	nargs = 0;
	XtSetArg ( args[nargs], XmNlabelString, &xm_string); nargs++;
	XtGetValues( timer_text[index], args, nargs);
	XmStringGetLtoR(xm_string, XmFONTLIST_DEFAULT_TAG, &string);
	XmStringFree(xm_string);
	value = (int)atoi(string) + 1;
	free(string);
	if ( value > max )
		value = min;
	sprintf ( buf, "%02d", value);

	xm_string=XmStringCreateLtoR(buf, XmFONTLIST_DEFAULT_TAG);
	nargs = 0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetValues( timer_text[index], args, nargs);
	XmStringFree(xm_string);
}


/*========================================================================
 *	Name:			TimerCancel
 *	Prototype:	void TimerCancel()
 *					
 *	Description:
 * Removes the "Set Alarm" Dialog box
 *========================================================================*/
static void
TimerCancel()
{
	if ( timer_widget != NULL )
	{
		XtDestroyWidget( timer_widget );
		timer_widget = NULL;
	}
}


/*========================================================================
 *	Name:			TimerAccept
 *	Prototype:	void TimerAccept()
 *					
 *	Description:
 * Accept alarm timer settings.
 *========================================================================*/
static void
TimerAccept(
	Widget		widget,
	StickyNote	*pn
)
{
#ifdef DEBUG
	char			*fname="TimerAccept";
#endif 

	Arg				args[2];
	int				nargs;
	int				i, value;
	char				*string;
	AudioFileType	*audio_link;
	XmString			xm_string;

	DBGEnter();
	
	/*
	 * Fetch the configured alarm values.
	 */
	for (i=0; i<4; i++)
	{
		nargs = 0;
		XtSetArg ( args[nargs], XmNlabelString, &xm_string); nargs++;
		XtGetValues( timer_text[i], args, nargs);
		XmStringGetLtoR(xm_string, XmFONTLIST_DEFAULT_TAG, &string);
		XmStringFree(xm_string);
		value = (int)atoi(string);
		free(string);
		switch (i)
		{
			/* Month has to be adjusted to what localtime() uses. */
			case 0: pn->pn_alarm_mon = value-1; break;
			case 1: pn->pn_alarm_day = value; break;
			case 2: pn->pn_alarm_hour = value; break;
			case 3: pn->pn_alarm_min = value; break;
		}
	}

	/*
	 * Get the repeat count.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNlabelString, &xm_string); nargs++;
	XtGetValues( timer_text[i], args, nargs);
	XmStringGetLtoR(xm_string, XmFONTLIST_DEFAULT_TAG, &string);
	XmStringFree(xm_string);
	pn->pn_alarm_repeat = atoi(string);
	free(string);

	/*
	 * Get the alarm audio setting, if any.
	 */
	audio_link = audio_files;
	DBGPrintf(DBG_INFO,("audio_link = 0x%08x\n", (int)audio_link));
	DBGPrintf(DBG_INFO,("audio_id = %d\n", (int)audio_id));
	while ( audio_link != NULL )
	{
		if (audio_link->id == audio_id)
		{
			DBGPrintf(DBG_INFO,("Setting alarm audio to %s\n", audio_link->file));
			if ( pn->pn_alarm_audio != NULL )
				free(pn->pn_alarm_audio);
			pn->pn_alarm_audio = (char *)XPSafeAlloc(strlen(audio_link->file)+1);
			strcpy(pn->pn_alarm_audio, audio_link->file);
			break;
		}
		audio_link = audio_link->next;
	}

	/*
	 * Turn on the alarm flag.
	 */
	pn->pn_alarm = TRUE;

	/*
	 * Set the sensitivity of various widgets appropriately.
	 */
	XtSetSensitive ( pn->pn_unsettimewidget, True );
	XtSetSensitive ( pn->pn_settimewidget, False );
	XtSetSensitive ( pn->pn_savewidget, True );

	/* set the alarm icon on for this note */
	nargs = 0;
	XtSetArg ( args[nargs], XmNlabelPixmap, alarmon_xbm); nargs++;
	XtSetValues( pn->pn_alarmwidget, args, nargs);

	/*
	 * Close the alarm timer window.
	 */
	TimerCancel();
}


/*========================================================================
 *	Name:			XPAlarmOff
 *	Prototype:	void XPAlarmOff()
 *					
 *	Description:
 * Turn off alarm for a note.
 *
 * Notes:
 * Just like TimerExcept except there is no dialog to fetch data from or 
 * to close.  Also, this routine is called directly as a callback, whereas
 * TimerExcept is called from the timer dialog.
 *========================================================================*/
void
XPAlarmOff(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	Arg			args[2];
	int			nargs;
	StickyNote 	*pn;

	
	/*
	 * Get the note structure.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Turn off the alarm flag.
	 */
	pn->pn_alarm = FALSE;

	/*
	 * Set the sensitivity of various widgets appropriately.
	 */
	XtSetSensitive ( pn->pn_unsettimewidget, False );
	XtSetSensitive ( pn->pn_settimewidget, True );
	XtSetSensitive ( pn->pn_savewidget, True );

	/* set the alarm icon on for this note */
	nargs = 0;
	XtSetArg ( args[nargs], XmNlabelPixmap, alarmoff_xbm); nargs++;
	XtSetValues( pn->pn_alarmwidget, args, nargs);
}



/*========================================================================
 *	Name:			UpdateAudioId
 *	Prototype:	void UpdateAudioID()
 *					
 *	Description:
 * When an audio file is selected, update the global audio id.
 *========================================================================*/
static void
UpdateAudioID(
	Widget		list_widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
#ifdef DEBUG
	char	*fname="UpdateAudioID()";
#endif

	XmListCallbackStruct *cbs = (XmListCallbackStruct *)call_data;

	/*
	 * Get the selection list.  There should only be one since we only
	 * allow a single selection. 
	 */
	audio_id = cbs->item_position-1;
	DBGPrintf(DBG_INFO,("audio id set to %d\n", audio_id));
}


/*========================================================================
 *	Name:			Reset
 *	Prototype:	void Reset()
 *					
 *	Description:
 * Toggle the alarm spin values between the current time and the last
 * setting.
 *========================================================================*/
static void
Reset(
	Widget		widget,
	StickyNote	*pn
)
{
	Arg			args[2];
	int			nargs;
	int			i;
	char			buf[25];
	time_t		time_tp;
	struct tm	*tm_tp;
	XmString		xm_string;

	/*
	 * The arrow buttons and text fields in each box
	 */
	for ( i=0; i<4; i++)
	{
		if ( reset_val == 0 )
		{
			/*
			 * Set the text fields to the notes stored values
			 */
			switch (i)
			{
				case 0: sprintf(buf, "%02d", pn->pn_alarm_mon+1); break;
				case 1: sprintf(buf, "%02d", pn->pn_alarm_day); break;
				case 2: sprintf(buf, "%02d", pn->pn_alarm_hour); break;
				case 3: sprintf(buf, "%02d", pn->pn_alarm_min); break;
			}
			nargs = 0;
			xm_string=XmStringCreateLtoR(buf, XmFONTLIST_DEFAULT_TAG);
			XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
			XtSetValues( timer_text[i], args, nargs);
			XmStringFree(xm_string);
		}
		else
		{
			/*
			 * Set the text fields to the notes stored values
			 */
			time(&time_tp);
			tm_tp = localtime(&time_tp);
			switch (i)
			{
				case 0: sprintf(buf, "%02d", tm_tp->tm_mon+1); break;
				case 1: sprintf(buf, "%02d", tm_tp->tm_mday); break;
				case 2: sprintf(buf, "%02d", tm_tp->tm_hour); break;
				case 3: sprintf(buf, "%02d", tm_tp->tm_min); break;
			}
			nargs = 0;
			xm_string=XmStringCreateLtoR(buf, XmFONTLIST_DEFAULT_TAG);
			XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
			XtSetValues( timer_text[i], args, nargs);
			XmStringFree(xm_string);
		}
	}

	if ( reset_val == 0 )
		reset_val = 1;
	else
		reset_val = 0;
}
