/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: tooltips.c
 *            DESCRIPTION: pop up windows that give help for certain features
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPToolTips
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_TT_C

/* === System Headers === */
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "motif.h"

/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"

/* === external routines === */

/* === Public routine prototypes === */

/* === Private routine prototypes === */

/* === Global Variables === */
extern Widget			Top_Level;
extern AppRes			app_res;
extern StickyNote		*notes;
extern Display			*display;
extern Colormap		XP_Colormap;

static char	*ToolTipText[] = {
	XP_S_TT_DIR,
	XP_S_TT_PRJ_TEXT,
	XP_S_TT_AUDIO_REP,
	XP_S_TT_AUDIO_TOG,
	XP_S_TT_PILOT_ADDR,
	XP_S_TT_PILOT_MFET,
	XP_S_TT_PILOT_MPRO,
	XP_S_TT_PILOT_MEMO,
	XP_S_TT_ENV_SAVE_FROM,
	XP_S_TT_ENV_LOAD_TO,
	XP_S_TT_PRJ_SAVE,
	XP_S_TT_PRJ_CANCEL,
	NULL
};


/*========================================================================
 *	Name:			XPToolTips
 *	Prototype:	Widget XPToolTips()
 *					
 *	Description:
 *	Pop up windows with tips on using certain features.
 *
 *	Notes:
 *		
 *========================================================================*/
void
XPToolTips(
	Widget		w,
	XtPointer	client_data,
	XEvent		*event,
	Boolean		do_continue
)
{
#ifdef DEBUG
	char			fname[]="XPToolTip()";
#endif

	static int		first_time = True;
	static Widget	detail_popup, detail_frame, detail_label;

	Arg				args[10];
	int				nargs;
	XmFontList		font_list;
	Position			root_x, root_y;
	Dimension		width, height;
	Dimension		string_width, string_height;
	XmString			xm_string;

	int				details = (int)client_data;


	DBGEnter();

	/*
	 * Don't show detail windows if user has turned this feature off.
	 */
	if ( app_res.no_tips )
			return;

	if (first_time)
	{
		/*
		 * Create the popup shell, frame and label for our messages
		 * in the pop down window
		 */
		detail_popup = XtVaAppCreateShell (
					"detail_shell",
					"detailShell",
					applicationShellWidgetClass,
					XtDisplayOfObject(w),
					XmNoverrideRedirect, True, 
					XmNcolormap, XP_Colormap,
					NULL);

		nargs = 0;
		XtSetArg(args[nargs], XmNshadowThickness, 2); nargs++;
		XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); nargs++;
		detail_frame = XmCreateFrame (
					detail_popup,
					"detailForm",
					args, nargs
					);
		XtManageChild(detail_frame);

		nargs = 0;
		XtSetArg(args[nargs], XmNalignment, XmALIGNMENT_BEGINNING); nargs++;
		detail_label = XmCreateLabel (
					detail_frame,
					"detailLabel",
					args, nargs
					);
		XtManageChild(detail_label);

		first_time = False;
	}

	switch (event->type)
	{
		case EnterNotify:

			/*
			 * Its possible to get in here with the popup already displayed.
			 * In that case, don't do anything - just return.
		 	 */
			if ( XtIsRealized(detail_popup) )
				return;

			/*
			 * Get the text help message and convert to an XmString.
			 */
			XPMultiLineText(ToolTipText[details], &xm_string);

			/*
			 * Set the help message in the label of the tool tip.
			 */
			nargs = 0;
			XtSetArg(args[nargs], XmNlabelString, xm_string); nargs++;
			XtSetValues(detail_label, args, nargs);
		
			/*
			 * Determine width of the string to use using the font type of our
			 * label widget.
			 */
			nargs = 0;
			XtSetArg(args[nargs], XmNfontList, &font_list); nargs++;
			XtGetValues(detail_label, args, nargs);
		
			string_width = XmStringWidth(font_list, xm_string);
			string_height = XmStringHeight(font_list, xm_string);
		
			XmStringFree(xm_string);
		
			/*
			 * Determine real coordinates for popup window
			 * We want to put it on the lower left edge of the icon, but 
			 * not the lower left edge of the label (since the icon is 
			 * rectangular, but appears non-rectangular, we need the pop-down
			 * window to be near the visible part of the icon).
			 */
			nargs = 0;
			XtSetArg(args[nargs], XmNwidth, &width); nargs++;
			XtSetArg(args[nargs], XmNheight, &height); nargs++;
			XtGetValues(w, args, nargs);
			XtTranslateCoords (w, (Position)(width/2+3), 
					(Position)(height+1), &root_x, &root_y);
		
			/*
			 * Set the width/height/positon for the popup
			 */
			nargs = 0;
			XtSetArg(args[nargs], XmNwidth, string_width+10); nargs++;
			XtSetArg(args[nargs], XmNheight, string_height+10); nargs++;
			XtSetArg(args[nargs], XmNx, root_x); nargs++;
			XtSetArg(args[nargs], XmNy, root_y); nargs++;
			XtSetValues(detail_popup, args, nargs);
		
			XtRealizeWidget(detail_popup);
			break;

		/* case LeaveNotify:
		 */
		default:
			if ( ! first_time )
				if ( XtIsRealized(detail_popup) )
					XtUnrealizeWidget(detail_popup);
			break;
	}
}


