/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: utils.c
 *            DESCRIPTION: generic routines for XNotesPlus
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPMultiLineText, XPInsertText,
 *								 : XPShutdown, XPPopUpWindow, XPCenterWindow
 *      PRIVATE FUNCTIONS: SOECancel, SOEAccept
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_UTILS_C

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>

#include "motif.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */
void XPPopUpWindow ();
void XPCenterWindow();


/* === Private routine prototypes === */
static void SOEAccept();
static void SOECancel();


/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;
extern Widget			Top_Level;
extern Display			*display;


/* === Static Variables === */



/*========================================================================
 *	Name:			XPMultiLineText
 *	Prototype:	void XPMultiLineText(char *string, XmString *xm_string)
 *					
 *	Description:
 * Convert a text string with newline characters into a multi-line Motif
 * string.
 *
 *	Input Arguments:
 *		char	*string			string to place into the labels
 *
 *	Output Arguments:
 *		XmString *xm_string	multiline Motif string that can be used in labels.
 *
 * Notes:
 *========================================================================*/
void
XPMultiLineText(
	char		*msg,
	XmString	*xm_string
)
{
#ifdef DEBUG
	char			*fname="XPMultiLineText";
#endif

	XmString		xm_string2;
	XmString		line;
	XmString		separator;
	char			*ptr;
	char			*msgcopy;
 
	DBGEnter();

	/*
	 * Turn the message into a multiline string, if necessary.
	 */
	msgcopy = XtNewString(msg);
	ptr = strtok(msgcopy,"\n");
	if ( ptr != NULL )
	{
		separator = XmStringSeparatorCreate();
		*xm_string = XmStringCreateLocalized(ptr);
		while ( (ptr = strtok(NULL,"\n")) != NULL )
		{
			line = XmStringCreateLocalized(ptr);
			xm_string2 = XmStringConcat(*xm_string,separator);
			XmStringFree(*xm_string);
			*xm_string = XmStringConcat(xm_string2,line);
			XmStringFree(xm_string2);
			XmStringFree(line);
		}
		XmStringFree(separator);
	}
	else
	{
		*xm_string = XmStringCreateLocalized(msg);
	}
	XtFree(msgcopy);
}



/*========================================================================
 *	Name:			XPInsertText
 *	Prototype:	void InsertText(StickyNote *pn, char *tmpfile, int remove
 *							int type)
 *					
 *	Description:
 * Generic routine for inserting text into a note from a file
 *
 *	Input Arguments:
 *		StickyNote	*pn			pointer to note to update
 *		char			*tmpfile		name of file to read from.
 *		int			remove		remove tmpfile? 1: yes, 0: no
 *		int			type			FILE_TYPE: read from a real file
 *										otherwise the file was a temp file we
 *										created on our own.
 *
 * Notes:
 *========================================================================*/
void
XPInsertText(
	StickyNote	*pn,
	char			*tmpfile,
	int			remove,
	int			type
)
{
	int				fp;
	char				*buf;
	struct stat		stat_buf;
	char				cmd[1024];
	XmTextPosition	position;

	/*
	 * get the position of the cursor in the note
	 */
	position = XmTextGetInsertionPosition(pn->pn_textwidget);

	if ( type == FILE_TYPE )
	{

		/*
		 * insert the text into the note at the cursor location
		 */
		if ( stat(tmpfile, &stat_buf) != 0 )
		{
			sprintf(cmd,"Can't stat file: %s", tmpfile);
			XPErrPopUp(cmd);
			if ( remove )
			{
				sprintf (cmd, "rm %s", tmpfile );
				system ( cmd );
			}
			return;
		}
		if ((fp = open(tmpfile, O_RDONLY)) == -1)
		{
			fprintf(stderr, "xnotes: ");
			perror(tmpfile);
			sprintf(cmd,"Can't open file for\nreading: %s", tmpfile);
			XPErrPopUp(cmd);
			if ( remove )
			{
				sprintf (cmd, "rm %s", tmpfile );
				system ( cmd );
			}
			return;
		}
		buf = (char *) XPSafeAlloc((int)stat_buf.st_size+2);
		while ( read(fp, buf, (int)stat_buf.st_size ) != 0 )
		{
			/*
			 * Insert the text at the current cursor location
			 */
			XmTextInsert(pn->pn_textwidget, position, buf);

			/* 
			 * Update the position of the cursor in the text buffer.
			 */
			position += strlen(buf);
			XmTextSetInsertionPosition(pn->pn_textwidget, position);
		}
		close (fp);
		free(buf);
	
		/*
		 * cleanup
		 */
		if ( remove )
		{
			sprintf (cmd, "rm %s", tmpfile );
			system ( cmd );
		}
	}
	else
	{
		/*
		 * "tmpfile" is actually a text buffer that we want to insert.
		 * This was added as an afterthought (does it show?)
		 */
		XmTextInsert(pn->pn_textwidget, position, tmpfile);
		position += strlen(tmpfile);
		XmTextSetInsertionPosition(pn->pn_textwidget, position);
	}
}
	

/*========================================================================
 *	Name:			XPShutdown
 *	Prototype:	void XPShutdown(GtkWidget *widget, gpointer *data)
 *					
 *	Description:
 *	Graceful exit of XNotesPlus.  This is a callback routine, but can
 * be called directly by other routines if necessary (like signal handlers).
 *
 *	Input Arguments:
 *		Standard Xt Callback arguments.
 *
 *	Notes:
 *========================================================================*/
void 
XPShutdown(
	Widget 		widget, 
	XtPointer	client_data,
	XtPointer	call_data
)
{
	Arg				args[5];
	int				nargs;
	static			int running = FALSE;
	char				buf[256];

	Widget			shell_id;
	Widget			soe_widget, soe_label, soe_buttons[2];

	XmString			xm_string;


	if (app_res.save_notes)
	{
		XPSaveAllNotes(FALSE);
		exit(0);
	}

	/*
	 * Ask the user if they want to save the notes
	 * This is a compile time option.
	 */
	else 
	{
		/*
		 * If there are no notes, just exit.
		 */
		if ( notes == NULL )
			exit(0);

		/*
		 * Only one copy of this window allowed at a time!
		 */
		if ( running )
			return;

		running = TRUE;

		/*
		 * Create a popup shell widget
		 */
		shell_id = XPCreateGenericShell(NotesSaveOnExitPrompt, 
							ACCEPT_BUTTON | CANCEL_BUTTON );
	
		/*
		 * Get the widget ids of the various parts of the window.
		 */
		sprintf(buf, "*GenericForm%d", (int)shell_id );
		soe_widget = XtNameToWidget(shell_id, buf);
		sprintf(buf, "*GenericAcceptB%d", (int)shell_id );
		soe_buttons[0] = XtNameToWidget(shell_id, buf);
		sprintf(buf, "*GenericCancelB%d", (int)shell_id);
		soe_buttons[1] = XtNameToWidget(shell_id, buf);
 
		/*
		 * The windows message.
		 */
		xm_string = XmStringCreateSimple(XP_S_SAVE_NOTES);
		nargs=0;
		XtSetArg ( args[nargs], XmNborderWidth, 0 ); nargs++;
		XtSetArg ( args[nargs], XmNlabelString, xm_string ); nargs++;
		soe_label = XmCreateLabel(
								soe_widget,
								"saveLabel",
								args, nargs
								);
		XtManageChild(soe_label);
		XmStringFree(xm_string);
 

		/*
		 * Set the accept and cancel button callbacks.
		 */
		XtAddCallback ( soe_buttons[0], XmNactivateCallback,
				(XtCallbackProc) SOEAccept, NULL);
		XtAddCallback ( soe_buttons[1], XmNactivateCallback,
				(XtCallbackProc) SOECancel, NULL);
 
		/*
		 * Display the window.
		 */
		XtRealizeWidget(shell_id);
		XPCenterWindow(shell_id, NULL, shell_id);
		XtPopup( shell_id, XtGrabNone );

	}

}


/*========================================================================
 *	Name:			SOECancel and SOEAccept
 *					
 *	Description:
 *	Handle users response to save notes before exiting.
 * SOECancel - exit without saving notes
 * SOEAccept - saving notes and then exit
 *========================================================================*/
static void
SOECancel(
	Widget		widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
	exit(0);
}

static void
SOEAccept(
	Widget		widget,
	StickyNote	*pn,
	XtPointer	call_data
)
{
	XPSaveAllNotes(FALSE);
	exit(0);
}



/*========================================================================
 *	Name:			XPPopUpWindow
 *	Prototype:	XPPopUpWindow()
 *
 *	Description:
 *		callback to pop up a transient shell
 *
 *	Input Arguments:
 *		XtPointer client_data		widget to pop up
 *
 *	Output Arguments:
 *	Return Values:
 *		none.
 *
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *		
 *========================================================================*/
void
XPPopUpWindow (
	Widget w,
	XtPointer client_data,
	XtPointer call_data
)
{

	Arg				args[3];
	int				nargs;
	Window			root, child;
	unsigned int	mask;
	int				root_x, root_y, child_x, child_y;

	/*
	 * The help callbacks may pass a NULL value here, if they are
	 * using Netscape.
	 */
	if ( client_data == NULL )
		return;

	/*
	 * find out where the mouse is and pop up the window there
	 */
	XQueryPointer ( XtDisplay(Top_Level), XtWindow(Top_Level),
		&root, &child,
		&root_x, &root_y,
		&child_x, &child_y,
		&mask
		);

	nargs=0;
	XtSetArg( args[nargs], XtNx, root_x ); nargs++;
	XtSetArg( args[nargs], XtNy, root_y ); nargs++;
	XtSetValues( (Widget) client_data, args, nargs );

	XtPopup ( (Widget) client_data, XtGrabNone );

}


/*========================================================================
 *	Name:			XPCenterWindow
 *	Prototype:	XPCenterWindow(Widget child, Widget parent, Widget shell)
 *
 *	Description:
 *	Position a dialog in the middle of a parent window or in the middle of the
 * display.
 *
 *	Input Arguments:
 *	Widget	child				window that needs positioning
 *	Widget	parent			window in which child is to be centered
 *	Widget	shell				window shell that receives positioning for child
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *	If parent is NULL, then the child is positioned in the middle of the 
 * display.  The child and shell arguments must be specified, but parent.
 * is optional.
 *		
 *========================================================================*/
void
XPCenterWindow(
	Widget	child,
	Widget	parent,
	Widget	shell
)
{
#ifdef DEBUG
	char	*fname="XPCenterWindow()";
#endif

	Arg				args[5];
	int				nargs;
	int				xpos, ypos;
	Dimension		parent_width;
	Dimension		parent_height;
	Dimension		child_width;
	Dimension		child_height;
	Position			root_x, root_y;

	DBGEnter();

	/*
	 * Position the child within the parent.  Be certain
	 * the box will be visible if centering it will place it off the edge
	 * of the screen.
	 */
	if ( parent != NULL )
	{
		nargs=0;
		XtSetArg ( args[nargs], XtNwidth, &parent_width ); nargs++;
		XtSetArg ( args[nargs], XtNheight, &parent_height ); nargs++;
		XtGetValues( parent, args, nargs );
		XtTranslateCoords (parent, 
				(Position)(parent_width/2), (Position)(parent_height/2),
				&root_x, &root_y);
		DBGPrintf(DBG_INFO,
			("rootx/rooty:  %d / %d\n", root_x, root_y));

		nargs=0;
		XtSetArg ( args[nargs], XtNwidth, &child_width ); nargs++;
		XtSetArg ( args[nargs], XtNheight, &child_height ); nargs++;
		XtGetValues( child, args, nargs );
		DBGPrintf(DBG_INFO,
			("childx/childy:  %d / %d\n", child_width, child_height));

		xpos = ((int)root_x - (int)(child_width/2));
		ypos = ((int)root_y - (int)(child_height/2));
		DBGPrintf(DBG_INFO,
			("pre rubber banding display pos:  %d / %d\n", xpos, ypos));

		/*
		 * Rubber band it - make sure window doesn't go off edge
		 * of screen.
		 */
		if (xpos < 1)
			xpos = 1;
		if ( (xpos+child_width) > DisplayWidth(display, DefaultScreen(display)) )
			xpos =  DisplayWidth(display, DefaultScreen(display)) - 
						child_width - 10;
		if (ypos < 1)
			ypos = 1;
		if ( (ypos+child_height) > 
				DisplayHeight(display, DefaultScreen(display)) )
		{
			ypos =  DisplayHeight(display, DefaultScreen(display)) - 
						child_height - 10;
		}
		DBGPrintf(DBG_INFO,
			("post rubber banding display pos:  %d / %d\n", xpos, ypos));

		nargs=0;
		XtSetArg ( args[nargs], XtNx, xpos ); nargs++;
		XtSetArg ( args[nargs], XtNy, ypos ); nargs++;
	 	XtSetValues( shell, args, nargs );
	}
	else
	{
		/*
		 * No parent window supplied, so just stick it in the middle of 
		 * the display.  No rubber banding applied here - the window should
		 * fit on the display when centered!
		 */
		nargs=0;
		XtSetArg ( args[nargs], XtNwidth, &child_width ); nargs++;
		XtSetArg ( args[nargs], XtNheight, &child_height ); nargs++;
		XtGetValues( child, args, nargs );
		DBGPrintf(DBG_INFO,
			("child width/height:  %d / %d\n", child_width, child_height));

		root_x = DisplayWidth(display, DefaultScreen(display));
		root_y = DisplayHeight(display, DefaultScreen(display));
		DBGPrintf(DBG_INFO,
			("display width/height:  %d / %d\n", root_x, root_y));

		xpos = (Dimension)((int)(root_x/2) - ((int)child_width/2));
		ypos = (Dimension)((int)(root_y/2) - ((int)child_height/2));
		DBGPrintf(DBG_INFO,
			("display pos:  %d / %d\n", xpos, ypos));

		nargs=0;
		XtSetArg ( args[nargs], XtNx, xpos ); nargs++;
		XtSetArg ( args[nargs], XtNy, ypos ); nargs++;
	 	XtSetValues( shell, args, nargs );
	}

}

