/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: xnotes.c
 *            DESCRIPTION: Main module for XNotesPlus
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: main
 *      PRIVATE FUNCTIONS: ParseCmdLineOptions, InitConfigParameters
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_C
#define MAIN_C

/* === System Headers === */
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <pwd.h>

#include "motif.h"

/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"

/* === external routines === */

/* === Public routine prototypes === */

/* === Private routine prototypes === */
void WMProtocols();

/* === Global Variables === */
extern AudioFileType		*audio_files;

AppRes						app_res;		/* configurable parameters */
XtAppContext				appcontext;	/* application context */
Widget						Top_Level;	/* top level widget */
Display						*display;
Screen						*screen;
Colormap						XP_Colormap;


/* === Static Variables === */
static XrmOptionDescRec options[] = {
	{ "--bs",			".bufSize",			XrmoptionSepArg,	NULL },
	{ "--notedir",		".noteDir",			XrmoptionSepArg,	NULL },
	{ "--interval",	".interval",		XrmoptionSepArg,	NULL },
	{ "--sb",			".scrollBar",		XrmoptionNoArg,	"true" },
	{ "--sv",			".saveNotes",		XrmoptionNoArg,	"true" },
	{ "--nw",			".nameWidth",		XrmoptionSepArg,	NULL },
	{ "--ns",			".noSave",			XrmoptionNoArg,	"true" },
	{ "--na",			".noAlarm",			XrmoptionNoArg,	"true" },
	{ "--nn",			".noNotice",		XrmoptionNoArg,	"true" },
	{ "-c",				".compatibility",	XrmoptionNoArg,	"true" },
	{ "--help",			".help",				XrmoptionNoArg,	"true" },
	{ "-?",				".help",				XrmoptionNoArg,	"true" },
	{ "--version",		".version",			XrmoptionNoArg,	"true" },
	{ "--ao",			".anchorOffset",	XrmoptionSepArg,	NULL },
	{ "--tmpdir",		".tmpDir",			XrmoptionSepArg,	NULL },
	{ "--printcmd",	".printCmd",		XrmoptionSepArg,	NULL },
	{ "--calendarcmd",".calendarCmd",	XrmoptionSepArg,	NULL },
	{ "--emailcmd",	".emailCmd",		XrmoptionSepArg,	NULL },
	{ "--emailaddr",	".emailAddr",		XrmoptionSepArg,	NULL },
	{ "--homedir",		".homeDir",			XrmoptionSepArg,	NULL },
	{ "--debugfile",	".debugFile",		XrmoptionSepArg,	NULL },
	{ "--debuglevel",	".debugLevel",		XrmoptionSepArg,	NULL },
	{ "--xpmdir",		".xpmDir",			XrmoptionSepArg,	NULL },
	{ "--defaults",	".defaults",		XrmoptionNoArg,	"true" },
	{ "--audiocmd",	".audioCmd",		XrmoptionSepArg,	NULL },
	{ "--audiodir",	".audioDir",		XrmoptionSepArg,	NULL },
	{ "--noaudio",		".noAudio",			XrmoptionNoArg,	"true" },
	{ "--addrdir",		".addrDir",			XrmoptionSepArg,	NULL },
	{ "--install",		".install",			XrmoptionNoArg,	"true" },
	{ "--rgbfile",		".rgbFile",			XrmoptionSepArg,	NULL },
	{ "--pilotdir",	".pilotDir",		XrmoptionSepArg,	NULL },
	{ "--notips",		".noTips",			XrmoptionNoArg,	"true" },
};

/*
 * Fallback resources.  app_defaults.h is built from the *.ad file
 * by the ad2c script during the build process.
 */
static String fallback_resources[] = {
#include "app_defaults.h"
	NULL
};

/*
 * Application resource defaults.  These should be listed in the same
 * order as the options[] values above, just to make it easier to track them.
 */
#define offset(field)	XtOffset(AppResPtr,field)
static XtResource resources[] = {
	{ "bufSize", "BufSize", XtRInt, sizeof(int),
		offset(buf_size), XtRImmediate, (caddr_t) DefaultBufSize },

	{ "noteDir", "NoteDir", XtRString, sizeof(String),
		offset(note_dir), XtRString, DefaultNoteDir },

	{ "interval", "Interval", XtRInt, sizeof(int),
		offset(interval), XtRImmediate, (caddr_t)DefaultInterval },

	{ "scrollBar", "Scroll", XtRBoolean, sizeof(Boolean),
		offset(scroll_bar), XtRImmediate, (caddr_t) False },

	{ "saveNotes", "SaveNotes", XtRBoolean, sizeof(Boolean),
		offset(save_notes), XtRImmediate, (caddr_t) False },

	{ "nameWidth", "NameWidth", XtRInt, sizeof(int),
		offset(name_width), XtRImmediate, (caddr_t) DefaultNameWidth },

	{ "noSave", "NoSave", XtRBoolean, sizeof(Boolean),
		offset(nosave), XtRImmediate, (caddr_t) False },

	{ "noAlarm", "NoAlarm", XtRBoolean, sizeof(Boolean),
		offset(noalarm), XtRImmediate, (caddr_t) False },

	{ "noNotice", "NoNotice", XtRBoolean, sizeof(Boolean),
		offset(nonotice), XtRImmediate, (caddr_t) False },

	{ "compatibility", "Compatibility", XtRBoolean, sizeof(Boolean),
		offset(compatibility), XtRImmediate, (caddr_t) False },

	{ "help", "Help", XtRBoolean, sizeof(Boolean),
		offset(help), XtRImmediate, (caddr_t) False },

	{ "version", "Version", XtRBoolean, sizeof(Boolean),
		offset(version), XtRImmediate, (caddr_t) False },

	{ "anchorOffset", "AnchorOffset", XtRInt, sizeof(int),
		offset(anchor_offset), XtRImmediate, (caddr_t) DefaultAnchorOffset },

	{ "tmpDir", "TmpDir", XtRString, sizeof(String),
		offset(tmp_dir), XtRString, (caddr_t) DefaultTmpDir },

	{ "printCmd", "PrintCmd", XtRString, sizeof(String),
		offset(print_cmd), XtRString, (caddr_t) DefaultPrintCmd },

	{ "calendarCmd", "CalendarCmd", XtRString, sizeof(String),
		offset(calendar_cmd), XtRString, (caddr_t) DefaultCalendarCmd },

	{ "emailCmd", "EmailCmd", XtRString, sizeof(String),
		offset(email_cmd), XtRString, (caddr_t) DefaultEmailCmd },

	{ "emailAddr", "EmailAddr", XtRString, sizeof(String),
		offset(email_addr), XtRString, (caddr_t) DefaultEmailAddr },

	{ "homeDir", "HomeDir", XtRString, sizeof(String),
		offset(home_dir), XtRString, NULL },

	{ "debugFile", "debugFile", XtRString, sizeof(String),
		offset(debugfile), XtRString, "/dev/null" },

	{ "debugLevel", "DebugLevel", XtRString, sizeof(String),
		offset(debuglevel), XtRString, "0" },

	{ "xpmDir", "XpmDir", XtRString, sizeof(String),
		offset(xpm_dir), XtRString, DefaultXPMDir},

	{ "defaults", "Defaults", XtRBoolean, sizeof(Boolean),
		offset(defaults), XtRImmediate, (caddr_t) False},

	{ "audioCmd", "AudioCmd", XtRString, sizeof(String),
		offset(audio_cmd), XtRString, DefaultAudioCmd},

	{ "audioDir", "AudioDir", XtRString, sizeof(String),
		offset(audio_dir), XtRString, DefaultAudioDir},

	{ "noAudio", "NoAudio", XtRBoolean, sizeof(Boolean),
		offset(noaudio), XtRImmediate, (caddr_t) False},

	{ "addrDir", "AddrDir", XtRString, sizeof(String),
		offset(addr_dir), XtRString, DefaultAddrDir},

	{ "install", "Install", XtRBoolean, sizeof(Boolean),
		offset(install), XtRImmediate, (caddr_t) False },

	{ "rgbFile", "RGBFile", XtRString, sizeof(String),
		offset(rgb_file), XtRString, DefaultRGBFile},

	{ "pilotDir", "PilotDir", XtRString, sizeof(String),
		offset(pilot_dir), XtRString, DefaultPilotDir},

	{ "noTips", "NoTips", XtRBoolean, sizeof(Boolean),
		offset(no_tips), XtRImmediate, (caddr_t) False },

};
#undef offset

static Atom wm_delete_window;
static Atom wm_protocols;
static Atom wm_save_yourself;



/*========================================================================
 *	Name:			WMProtocols
 *	Prototype:	void WMProtocols()
 *					
 *	Description:
 *	Handle a window manager "delete window" request, which closes 
 * the application.
 *
 *	Notes:
 *========================================================================*/
void 
WMProtocols(
	Widget	w,
	XEvent	*ev,
	String	*params,
	Cardinal	*n
)
{
	if (ev->type == ClientMessage &&
		ev->xclient.message_type == wm_protocols &&
		ev->xclient.data.l[0] == wm_delete_window) 
	{
			XPShutdown();
	}
}

static XtActionsRec actions[] = {
   { "WMProtocols", WMProtocols }
};
static XtActionsRec drag_actions[] = {
   { "RaiseNotes",	(XtActionProc)XPRaiseAllNotes },
   { "LowerNotes",	(XtActionProc)XPLowerAllNotes },
   { "TearOff",		(XtActionProc)XPTearOff },
   { "RingBell",		(XtActionProc)XPRingBell },
   { "Quit",			(XtActionProc)XPShutdown },
   { "DoNothing",		(XtActionProc)XPNoop },
   { "Alarm",			(XtActionProc)XPActionAlarm },
   { "Anchor",			(XtActionProc)XPActionAnchor },
   { "Destroy",		(XtActionProc)XPActionDestroy },
   { "Calendar",		(XtActionProc)XPActionCal },
   { "Envelope",		(XtActionProc)XPActionEnvelope },
   { "Project",		(XtActionProc)XPActionProject },
   { "Hide",			(XtActionProc)XPActionHide },
   { "Erase",			(XtActionProc)XPActionErase },
   { "Print",			(XtActionProc)XPActionPrint },
   { "Open",			(XtActionProc)XPActionOpen },
   { "Save",			(XtActionProc)XPActionSave },
};




/*========================================================================
 *	Name:			InitConfigParameters
 *	Prototype:	void InitConfigParameters()
 *					
 *	Description:
 *	Initialize the configurable parameters structure.
 *
 *	Notes:
 * We only set values that need changing after XtGetApplicationResources()
 * has completed.  Some items don't need updating since
 * XtGetApplicationResources() will set them correctly, but others have
 * greater dependencies on each other and must be handled manually here.
 *========================================================================*/
void
InitConfigParameters()
{
	char				*home;
	char				*dir=NULL;
	struct passwd	*pwd;


	/*
	 * Set the default Home directory, if not already set.
	 */
	if ( app_res.home_dir == NULL )
	{
		home = getenv("HOME");
		if ( home != NULL )
		{
			app_res.home_dir = XPSafeAlloc(strlen(home)+1);
			strcpy(app_res.home_dir, (char *)home);
		}
		else
		{
			/*
			 * Find the user's home directory.
			 */
			if ((pwd = getpwuid(getuid())) == NULL) 
				app_res.home_dir = NULL;
			home = pwd->pw_dir;
			app_res.home_dir = XPSafeAlloc(strlen(home)+1);
			strcpy(app_res.home_dir, (char *)home);
		}
	}
	else
	{
		if ( ( app_res.home_dir[0] != '/' ) && ( app_res.home_dir[0] != '~' ) )
		{
			printf("Can't use %s for HOME directory.\n", app_res.home_dir);
			printf("Your HOME directory must be a fully qualified path.\n");
			exit (1);
		}
	}

	/*
	 * Now, for each directory that needs to know where the HOME directory
	 * is, update the application resource value.
	 */

	/*
	 * If the current value matches the default, then skip it.
	 */
	if ( strcmp(app_res.xpm_dir, DefaultXPMDir) != 0 )
	{
		/*
		 * If the current value is an absolute path, skip changing it.
		 */
		if ( app_res.xpm_dir[0] != '/' )
		{
			/*
			 * Relative path given - prepend the HOME and Notedir paths to it.
			 */
			dir = app_res.xpm_dir;
			app_res.xpm_dir = 
				XPSafeAlloc(strlen(app_res.home_dir) + 
					strlen(app_res.note_dir) + strlen(dir) + 2 + 1);
			strcpy(app_res.xpm_dir, (char *)app_res.home_dir);
			strcat(app_res.xpm_dir, (char *)"/");
			strcat(app_res.xpm_dir, (char *)app_res.note_dir);
			strcat(app_res.xpm_dir, (char *)"/");
			strcat(app_res.xpm_dir, dir);
		}
	}
	else
	{
		/*
		 * Default dir needs to be prefixed with HOME and Notedir paths.
		 */
		dir = app_res.xpm_dir;
		app_res.xpm_dir = 
			XPSafeAlloc(strlen(app_res.home_dir) + 
				strlen(app_res.note_dir) + strlen(dir) + 2 + 1);
		strcpy(app_res.xpm_dir, (char *)app_res.home_dir);
		strcat(app_res.xpm_dir, (char *)"/");
		strcat(app_res.xpm_dir, (char *)app_res.note_dir);
		strcat(app_res.xpm_dir, (char *)"/");
		strcat(app_res.xpm_dir, dir);
	}


	dir = NULL;
	/*
	 * If the current value matches the default, then skip it.
	 */
	if ( strcmp(app_res.audio_dir, DefaultAudioDir) != 0 )
	{
		/*
		 * If the current value is an absolute path, skip changing it.
		 */
		if ( app_res.audio_dir[0] != '/' )
		{
			/*
			 * Relative path given - prepend the HOME and Notedir paths to it.
			 */
			dir = app_res.audio_dir;
			app_res.audio_dir = 
				XPSafeAlloc(strlen(app_res.home_dir) + 
					strlen(app_res.note_dir) + strlen(dir) + 2 + 1);
			strcpy(app_res.audio_dir, (char *)app_res.home_dir);
			strcat(app_res.audio_dir, (char *)"/");
			strcat(app_res.audio_dir, (char *)app_res.note_dir);
			strcat(app_res.audio_dir, (char *)"/");
			strcat(app_res.audio_dir, dir);
		}
	}
	else
	{
		/*
		 * Default dir needs to be prefixed with HOME and Notedir paths.
		 */
		dir = app_res.audio_dir;
		app_res.audio_dir = 
			XPSafeAlloc(strlen(app_res.home_dir) + 
				strlen(app_res.note_dir) + strlen(dir) + 2 + 1);
		strcpy(app_res.audio_dir, (char *)app_res.home_dir);
		strcat(app_res.audio_dir, (char *)"/");
		strcat(app_res.audio_dir, (char *)app_res.note_dir);
		strcat(app_res.audio_dir, (char *)"/");
		strcat(app_res.audio_dir, dir);
	}


	dir = NULL;
	/*
	 * If the current value matches the default, then skip it.
	 */
	if ( strcmp(app_res.addr_dir, DefaultXPMDir) != 0 )
	{
		/*
		 * If the current value is an absolute path, skip changing it.
		 */
		if ( app_res.addr_dir[0] != '/' )
		{
			/*
			 * Relative path given - prepend the HOME and Notedir paths to it.
			 */
			dir = app_res.addr_dir;
			app_res.addr_dir = 
				XPSafeAlloc(strlen(app_res.home_dir) + 
					strlen(app_res.note_dir) + strlen(dir) + 2 + 1);
			strcpy(app_res.addr_dir, (char *)app_res.home_dir);
			strcat(app_res.addr_dir, (char *)"/");
			strcat(app_res.addr_dir, (char *)app_res.note_dir);
			strcat(app_res.addr_dir, (char *)"/");
			strcat(app_res.addr_dir, dir);
		}
	}
	else
	{
		/*
		 * Default dir needs to be prefixed with HOME and Notedir paths.
		 */
		dir = app_res.addr_dir;
		app_res.addr_dir = 
			XPSafeAlloc(strlen(app_res.home_dir) + 
				strlen(app_res.note_dir) + strlen(dir) + 2 + 1);
		strcpy(app_res.addr_dir, (char *)app_res.home_dir);
		strcat(app_res.addr_dir, (char *)"/");
		strcat(app_res.addr_dir, (char *)app_res.note_dir);
		strcat(app_res.addr_dir, (char *)"/");
		strcat(app_res.addr_dir, dir);
	}

	/*
	 * If the current value matches the default, then skip it.
	 */
	if ( strcmp(app_res.pilot_dir, DefaultPilotDir) != 0 )
	{
		/*
		 * If the current value is an absolute path, skip changing it.
		 */
		if ( app_res.pilot_dir[0] != '/' )
		{
			/*
			 * Relative path given - prepend the HOME and Notedir paths to it.
			 */
			dir = app_res.pilot_dir;
			app_res.pilot_dir = 
				XPSafeAlloc(strlen(app_res.home_dir) + 
					strlen(app_res.note_dir) + strlen(dir) + 2 + 1);
			strcpy(app_res.pilot_dir, (char *)app_res.home_dir);
			strcat(app_res.pilot_dir, (char *)"/");
			strcat(app_res.pilot_dir, (char *)app_res.note_dir);
			strcat(app_res.pilot_dir, (char *)"/");
			strcat(app_res.pilot_dir, dir);
		}
	}
	else
	{
		/*
		 * Default dir needs to be prefixed with HOME and Notedir paths.
		 */
		dir = app_res.pilot_dir;
		app_res.pilot_dir = 
			XPSafeAlloc(strlen(app_res.home_dir) + 
				strlen(app_res.note_dir) + strlen(dir) + 2 + 1);
		strcpy(app_res.pilot_dir, (char *)app_res.home_dir);
		strcat(app_res.pilot_dir, (char *)"/");
		strcat(app_res.pilot_dir, (char *)app_res.note_dir);
		strcat(app_res.pilot_dir, (char *)"/");
		strcat(app_res.pilot_dir, dir);
	}
}


/*========================================================================
 *	Name:			main()
 *
 *	Description:
 *	Just get things rolling.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/

int 
main(
	int argc, 
	char *argv[]
)
{
#ifdef DEBUG
	char	*fname="main()";
#endif

	Widget			button;
	int				timer_interval, alarm_interval;
	XtIntervalId	timer, alarm_timer, disabled_cmds_timer;
	int				rc;
	Window			windows[2];
	Dimension		width, height;
	Arg				args[3];
	int				nargs;


	/*
	 * Ignore child processes exits.
	 */
	signal(SIGCHLD, SIG_IGN);
	signal(SIGTERM, XPShutdown);
	signal(SIGHUP, XPShutdown);
	signal(SIGQUIT, XPShutdown);
	signal(SIGINT, XPShutdown);


	/*
	 * Initialize the application resources for Motif.
	 */
	Top_Level = XtVaAppInitialize(
			&appcontext, 
			StickyNoteClass, 
			options, XtNumber(options), 
			&argc, argv, 
			fallback_resources,
			XmNdeleteResponse, XmDO_NOTHING,
			NULL);

	/*
	 * Window manager save state handlers.
	 */
	wm_delete_window = 
		XmInternAtom( XtDisplay(Top_Level), "WM_DELETE_WINDOW", False);
	wm_protocols = XmInternAtom( XtDisplay(Top_Level), "WM_PROTOCOLS", False);
   wm_save_yourself = 
		XmInternAtom ( XtDisplay (Top_Level), "WM_SAVE_YOURSELF", False);

   XmAddWMProtocols ( Top_Level, &wm_delete_window, 1);
   XmAddWMProtocols ( Top_Level, &wm_save_yourself, 1);

   XmAddWMProtocolCallback ( Top_Level, wm_save_yourself, XPShutdown, 0);

	/*
	 * Save some commonly used values.
	 */
   display = XtDisplay(Top_Level);
   screen = DefaultScreenOfDisplay(display);

	/*
	 * Grab the application resources that have been set.
	 */
	XtGetApplicationResources( Top_Level, &app_res, resources,
		XtNumber(resources), (ArgList)argv, (Cardinal)argc);

	/*
	 * Add default actions to program.
	 */
	XtAppAddActions (appcontext, actions, XtNumber (actions));
	XtAppAddActions (appcontext, drag_actions, XtNumber (drag_actions));

	/*
	 * Update configuration parameters.  We want to prepend directory paths
	 * onto others and we can't know ahead of time what either will be,
	 * so we have to do so after the application resources and command line
	 * have been parsed.
	 */
	InitConfigParameters();

	/*
	 * Set up the audio files for use with the notes.
	 */
	XPSetUpAudio();

	/*
	 * Display Help or Version info, if requested.
	 */
	if ( app_res.help == True )
	{
		printf(USAGE);
		exit(0);
	}
	if ( app_res.version == True )
	{
		printf( "xnotes+ v%s\n", VERSION );
		printf( "Copyright 1994-1998 Michael J. Hammel\n\n");
		exit(0);
	}

#ifdef DEBUG

	/*
	 * set the debug file, if any
	 */
	if ( app_res.debugfile != NULL )
		DBGFile = app_res.debugfile;

	/*
	 * Set the debug level, if any.  The debug level must be set the
	 * same way it is in debug.h - using bit flags.
	 */
	GMDebugLevel = DBGStringToInt(app_res.debuglevel);

#endif

	/*
	 * Debug output is to /dev/console by default if -DDEBUG was set at compile
	 * time, otherwise the DBGxxx macros have no effect.
	 */
	DBGOpen( DEBUG_FD );
	DBGEnter();

	/*
	 * Set up color handling, but only if an installed colormap was
	 * requested.
	 */
	if ( app_res.install == True )
	{
		rc = XPSetGlobalColors(Top_Level, &XP_Colormap, 40);
		switch (rc)
		{
			/*
			 * Colormap successfully created.  We'll install it later.
			 */
			case XP_E_SUCCESS:
				break;
	
			/*
			 * Can't run on the current display.
			 */
			case XP_VISUAL_NOT_SUPPORTED:
				printf(XP_S_MSG_BAD_VISUAL);
				DBGExit();
				DBGClose(DEBUG_FD);
				exit(1);
				break;
	
			/*
			 * We got 256 colors, but we can't install a colormap.  Better
			 * at least notify the user of this case.
			 */
			case XP_CMAP_SMALL_IMMUTABLE:
				printf(XP_S_MSG_LIMITED_VISUAL);
				XP_Colormap = 
					DefaultColormapOfScreen (XtScreen(Top_Level));
				break;
	
			/*
			 * These two should be ok in most cases, so we just keep going
			 * without posting any notices.
			 */
			case XP_CMAP_BIG_IMMUTABLE:
			case XP_CMAP_OK:
				XP_Colormap = 
					DefaultColormapOfScreen (XtScreen(Top_Level));
				break;
		}
	}
	else
		XP_Colormap = DefaultColormapOfScreen (XtScreen(Top_Level));


	/*
	 * Validate application resources that have been set by user.
	 */
	DBGPrintf(DBG_TRACE3, ("Validating Application Resources.\n"));
	XPValidateAppRes();

	/*
	 * Read in the projects list, if any.  We do this now so we can save
	 * debug info if necessary, plus the app_res.note_dir gets set by
	 * XPValidateApRes();
	 */
	XPSetUpProjects();

	/*
	 * Show the configured defaults, help, or version number, if requested.
	 */
	if ( app_res.defaults == True )
		XPShowDefaults();

	/*
	 * Create the Plaid window.
	 */
	DBGPrintf(DBG_TRACE3, ("Creating plaid widget.\n"));
	button = XPCreatePlaidWidget();

	/*
	 * Add pop up menu to Plaid button.
	 */
	DBGPrintf(DBG_TRACE3, ("Adding the popup menu.\n"));
	XPCreateMainMenu(button);

	/*
	 * Set the autosave timer.
	 */
	DBGPrintf(DBG_TRACE3, ("Starting timers.\n"));
	if ( !app_res.nosave )
	{
		timer_interval = app_res.interval * 1000;
		timer = XtAppAddTimeOut (
				XtWidgetToApplicationContext(Top_Level),
				timer_interval,
				XPAutoSave,
				NULL );
	}
	
	/*
	 * Set the alarm timer (expires once a minute). 
	 */
	if ( !app_res.noalarm )
	{
		alarm_interval = XP_ALARM_PERIOD * 1000;
		alarm_timer = XtAppAddTimeOut (
				XtWidgetToApplicationContext(Top_Level),
				alarm_interval,
				XPAlarmCheck,
				NULL );
	}

	/*
	 * Set timer to notify user of disabled functions.
	 */
	if ( !app_res.nonotice )
	{
		disabled_cmds_timer = XtAppAddTimeOut (
			XtWidgetToApplicationContext(Top_Level),
			1500,
			XPShowDisabledCmds,
			NULL );
	}

	/*
	 * make me real, Gepetto!
	 */
	DBGPrint(DBG_INFO, "Calling XtRealizeWidget\n");
	XtRealizeWidget(Top_Level);

	/*
	 * Set the main window to not allow resizing larger - thats so I don't have
	 * to deal with resizing the logo.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, &width); nargs++;
	XtSetArg ( args[nargs], XmNheight, &height); nargs++;
	XtGetValues ( Top_Level, args, nargs);
	nargs = 0;
	XtSetArg ( args[nargs], XmNmaxWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNmaxHeight, height); nargs++;
	XtSetValues ( Top_Level, args, nargs); 


	/*
	 * Load saved notes.  This has to be done after the Top Level is 
	 * realized or else the positioning information will not be honored
	 * for the notes.
	 */
	DBGPrintf(DBG_TRACE3, ("Loading saved notes.\n"));
	XPLoadSavedNotes();


	/*
	 * Install the colormap, if requested.
	 */
	if ( app_res.install == True )
	{
		windows[0] = XP_Colormap;
		XSetWMColormapWindows(
			XtDisplay(Top_Level), XtWindow(Top_Level), windows, 1);
	}

	/*
	 * Sit and spin.
	 */
	DBGPrintf(DBG_TRACE3, ("sit and spin...\n"));
	XtAppMainLoop(appcontext);
	
	return 0;
}
