#!/bin/sh

cwd=`pwd`

echo "====================================================================="
echo "          Welcome to the XNotesPlus Installation Program!"
echo "====================================================================="
echo ""
echo "You will be asked a few questions about where to install the data"
echo "files (audio files, default address and gtkrc files, and so forth)"
echo "and where to install the XNotesPlus binary.  The default values are"
echo "given in brackets.  Just hit ENTER to accept the defaults."
echo ""
echo ""

if [ "$HOME" = "" ]
then
	echo "You need to set your HOME environment variable for this script to"
	echo "work properly."
	exit 1
fi

#
# Determine where data files should go.
#
loop=true
while [ "$loop" = "true" ]
do
	echo "Where do you want to install the XNotesPlus data files? "
	echo "[$HOME/.xnotesplus]-->  "
	read data_dir

	if [ "$data_dir" != "" ]
	then
		if [ "`echo $data_dir | cut -c1`" != "/" ]
		then
			data_dir=$HOME/$data_dir
		fi
	else
		data_dir=$HOME/.xnotesplus
	fi

	echo "Verify: Do you want to to use $data_dir? [Y] "
	read answer

	if [ "$answer" != "" ]
	then
		answer=`echo $answer | cut -c1 | tr 'y' 'Y'`
		if [ "$answer" = "Y" ]
		then
			loop=false
		fi
	else
		loop=false
	fi
done

#
# Determine where the xnotes+ binary should go.
#
loop=true
while [ "$loop" = "true" ]
do
	echo "Type the name of the directory in which to install the binary:"
	echo "[$HOME/bin]-->  "
	read install_dir

	if [ "$install_dir" != "" ] 
	then
		if [ "`echo $install_dir | cut -c1`" != "/" ]
		then
			install_dir=$HOME/$install_dir
		fi
	else
		install_dir=$HOME/bin
	fi

	echo "Verify: Do you want to install in $install_dir? [Y] "
	read answer

	if [ "$answer" != "" ]
	then
		answer=`echo $answer | cut -c1 | tr 'y' 'Y'`
		if [ "$answer" = "Y" ]
		then
			loop=false
		fi
	else
		loop=false
	fi

done


#
# Install the binary.
#
if [ "$install_dir" != "$cwd/bin/xnotes+" ]
then
	if [ -d $install_dir ]
	then
		cp $cwd/bin/xnotes+ $install_dir
	else
		# Attempt to make the directory.
		mkdir $install_dir 2>&1 > /dev/null
		if [ $? -ne 0 ]
		then
			echo "$install_dir does not exist and I can't create it!"
			echo "XNotesPlus installation aborting."
			exit 1
		fi
		cp $cwd/bin/xnotes+ $install_dir
	fi
fi

#
# Install the data files.
#
if [ "$data_dir" != "$cwd" ]
then
	if [ -d $data_dir ]
	then
		cd $cwd
		cp -r * $data_dir
		rm -rf $data_dir/bin 2>&1 > /dev/null 
		rm -f $data_dir/install 2>&1 > /dev/null 
		rm -f $data_dir/README 2>&1 > /dev/null 
		rm -f $data_dir/INSTALL 2>&1 > /dev/null 
	else
		# Attempt to make the directory.
		mkdir $data_dir 2>&1 > /dev/null
		if [ $? -ne 0 ]
		then
			echo "$data_dir does not exist and I can't create it!"
			echo "XNotesPlus installation aborting."
			exit 1
		fi
		cd $cwd
		cp -r * $data_dir
		rm -rf $data_dir/bin 2>&1 > /dev/null 
		rm -f $data_dir/install 2>&1 > /dev/null 
		rm -f $data_dir/README 2>&1 > /dev/null 
		rm -f $data_dir/INSTALL 2>&1 > /dev/null 
	fi
fi


#
# Query for the current color depth.
#
depth=`xdpyinfo | grep "depth of root" | cut -f2 -d":" | awk ' {print $1}'`
case $depth in
8)	xpm=xnotes-warm-colors.xpm;;
*)	xpm=xnotes-high-res.xpm;;
esac

echo "You appear to have a color depth of $depth planes.  The best image for"
echo "the main window will be $xpm.  I'll make this the"
echo "default, but you can modify it later by changing the symbollic link for"
echo "\"xnotes.xpm\" in the $data_dir/xpm-d directory."
echo ""
echo ""

rm -f $data_dir/xpm-d/xnotes.xpm 2>&1 > /dev/null
ln -s $data_dir/xpm-d/$xpm $data_dir/xpm-d/xnotes.xpm

echo "====================================================================="
echo "                 XNotesPlus installation complete."
echo "====================================================================="
echo ""
echo "If you will be using XNotesPlus with the PalmPilot you should add the"
echo "following two variables to your environment:"
echo "bash or ksh users:"
echo "   % export PILOTPORT=/dev/cua1"
echo "   % export PILOTRATE=57600"
echo "csh users:"
echo "   % setenv PILOTPORT /dev/cua1"
echo "   % setenv PILOTRATE 57600"
echo "PILOTPORT should be set to the name of the device to which your Pilot's"
echo "cradle is connected.  For most people on Linux PCs, this will be"
echo "/dev/cua0 or /dev/cua1."
echo ""
echo "Be sure to read the man page before getting started!"
echo ""
echo "                               Enjoy!"
echo "                        Michael J. Hammel"
echo "                    mjhammel@graphics-muse.org"
echo ""
echo ""
