namespace eval gui {
    variable gui

    # Define the fonts used for the Widget labels.
    switch $tkWorld(ostype) {
	windows {
	    set gui(font.label) {helvetica 10 bold}
	}
	default {
	    set gui(font.label) {helvetica 12 bold}
	}
    }
}

# gui::create --
#
#   Method to build the application's GUI components
#
# Args
#
#   None
#
# Returns
#
#   None

proc gui::create { } {
    global tkWorld
    variable gui

    # Create the main widgets that tkWorld will use.
    set tkWorld(toolbar) [toolbar::create .toolbar]
    set tkWorld(cmd_center) [scroll_text::interior \
	    [scroll_text::create .cmd_center 60 5]]
    set tkWorld(log_window) [scroll_text::interior \
	    [scroll_text::create .log_window 60 25]]
    set tkWorld(reg_listbox) [scroll_listbox::interior \
	    [scroll_listbox::create .reg_listbox 12]]

    # Create the GUI components for the shell and working directory
    # drop down list boxes.
    set top [frame .top]
    set shell_label [label $top.shell_label \
	    -text "Shell" \
	    -width 5 \
	    -anchor n]
    set shell_entry [ddlistbox::create $top tkWorld(working_shell) \
	    "sh bash ksh pdksh csh tcsh ash zsh" 10]
    pack $shell_label $shell_entry \
	    -side left \
	    -padx 2 \
	    -pady 1
    pack [seperator::create $top.sep1 sunken] \
	    -side left \
	    -padx 5 \
	    -pady 0 \
	    -fill y
    set wd_button [button $top.working_dir_button \
	    -text "Dir" \
	    -width 5 \
	    -relief flat \
	    -command wdir::current]
    set wd_entry [ddlistbox::create $top tkWorld(working_dir) "" 45]
    pack $wd_button $wd_entry \
	    -side left \
	    -padx 2 \
	    -pady 0 \
	    -fill x

    # Define entry which contains the result of the working
    # directory selection.
    set tkWorld(entry.working_dir) $wd_entry

    # Define the initial directory listing.
    wdir::get $tkWorld(working_dir)

    # Make the dir button raise and lower on mouse entry/leave.
    bind $wd_button <Enter> "$wd_button configure -relief raised"
    bind $wd_button <Leave> "$wd_button configure -relief flat"

    # Define the balloon help messages for the executing shell and
    # the working directory components.
    balloonhelp::for $shell_label "Execute commands using this shell"
    balloonhelp::for $shell_entry "Change the executing shell"
    balloonhelp::for $wd_button "Get the directory listing for\
	    the current working directory"
    balloonhelp::for $wd_entry "The current working directory"

    # Put all of the widgets in a grid.
    grid .toolbar \
	    -row 0 \
	    -column 0 \
	    -columnspan 3 \
	    -sticky ew
    grid [seperator::create .s1] \
	    -row 1 \
	    -column 0 \
	    -columnspan 3 \
	    -sticky ew
    grid $top \
	    -row 2 \
	    -column 0 \
	    -columnspan 3 \
	    -pady 2 \
	    -sticky ew
    grid [seperator::create .s2] \
	    -row 3 \
	    -column 0 \
	    -columnspan 3 \
	    -sticky ew
    grid [label .reg_label \
	    -text Registry \
	    -font $gui(font.label)] \
	    -row 4 \
	    -column 0 \
	    -sticky ew
    grid .reg_listbox \
	    -row 5 \
	    -column 0 \
	    -rowspan 4 \
	    -sticky news
    grid [seperator::create .s3] \
	    -row 4 \
	    -column 1 \
	    -rowspan 5 \
	    -sticky ns
    grid [label .cc_label \
	    -text "Command Center" \
	    -font $gui(font.label)] \
	    -row 4 \
	    -column 2 \
	    -sticky ew
    grid .cmd_center \
	    -row 5 \
	    -column 2 \
	    -sticky news
    grid [seperator::create .s4] \
	    -row 6 \
	    -column 2 \
	    -pady 2 \
	    -sticky ew
    grid [label .lw_label \
	    -text "Log Window" \
	    -font $gui(font.label)] \
	    -row 7 \
	    -column 2 \
	    -sticky ew
    grid .log_window \
	    -row 8 \
	    -column 2 \
	    -sticky news

    # Configure the rows to expand accordingly.
    grid rowconfigure . 0 \
	    -weight 0
    grid rowconfigure . 2 \
	    -weight 0
    grid rowconfigure . 3 \
	    -weight 0
    grid rowconfigure . 5 \
	    -weight 1
    grid rowconfigure . 6 \
	    -weight 0
    grid rowconfigure . 8 \
	    -weight 1

    # Configure the columns to expand accordingly.
    grid columnconfigure . 0 \
	    -weight 0
    grid columnconfigure . 1 \
	    -weight 0
    grid columnconfigure . 2 \
	    -weight 1

    # Put the user at the CC.
    focus $tkWorld(cmd_center)

    # Put all the available commands in the listbox.
    registry::create

    # If the user clicks into these LW and CC widgets, activate the
    # toolbar button group associated with that widget.
    bind $tkWorld(cmd_center) <FocusIn> \
	    {toolbar::group_state cmd_center active; \
	    toolbar::button_state $toolbar::toolbar(stop) disabled}
    bind $tkWorld(log_window) <FocusIn> \
	    {toolbar::group_state log_window active}

    # Setup keyboard bindings for all of the commands available
    # in the CC.
    bind $tkWorld(cmd_center) <Control-r> cmd_center::reset
    bind $tkWorld(cmd_center) <Control-x> cmd_center::run
    bind $tkWorld(cmd_center) <Control-s> cmd_center::shell
    bind $tkWorld(cmd_center) <Control-c> cmd_center::stop
    bind $tkWorld(cmd_center) <Control-p> cmd_center::pipe
    bind $tkWorld(cmd_center) <Control-m> history::create
    bind $tkWorld(cmd_center) <Control-j> \
	    [list focus $tkWorld(log_window)]

    # Setup keyboard bindings for all of the commands available
    # in the LW.
    bind $tkWorld(log_window) <Control-1> log_window::home
    bind $tkWorld(log_window) <Control-2> log_window::end
    bind $tkWorld(log_window) <Control-f> log_window::up
    bind $tkWorld(log_window) <Control-b> log_window::down
    bind $tkWorld(log_window) <Control-r> log_window::clear
    bind $tkWorld(log_window) <Control-s> log_window::save
    bind $tkWorld(log_window) <Control-j> \
	    [list focus $tkWorld(cmd_center)]

    # Configure the window manager for unix systems.
    switch $tkWorld(ostype) {
	unix {
	    wm title . "tkWorld"
	    wm iconbitmap . \
		    @[file join $tkWorld(image_dir) tkWorld.ic.xbm]
	}
    }
}
