namespace eval wdir {
    variable wdir
}

# wdir::get --
#
#   Method which returns a list of the directories in the supplied
#   parent directory.
#
# Args
#
#   p_dir - The parent directory.
#
# Returns
#
#   c_dir - The children in the parent directory.

proc wdir::get { p_dir } {
    global tkWorld

    # First delete from the working directory drop down listbox
    # if it has already been created.
    if [info exists tkWorld(entry.working_dir)] {
	ddlistbox::delete $tkWorld(entry.working_dir)
    }

    # Build the directory hierarchy off the supplied parent
    set c_dir ""; set prv_dir ""
    foreach d [file split $p_dir] {
	if {$d == "/"} {
	    # This is the tree parent.
	    append c_dir "$d "
	} else {
	    # This is the child.
	    append c_dir "$prv_dir/$d "
	    set prv_dir "$prv_dir/$d"
	}
    }

    # Build the listing of all the children in the parent directory.
    foreach f [lsort [glob -nocomplain [file join $p_dir *]]] {
	if [file isdirectory $f] {
	    append c_dir "$f "
	}
    }

    ddlistbox::add $tkWorld(entry.working_dir) \
	    tkWorld(working_dir) $c_dir

    # List complete with at least a return to the parent via ..
    return $c_dir
}

# wdir::current --
#
#   Get the directory listing for the current working directory.
#
# Args
#
#   None.
#
# Returns
#
#   None.

proc wdir::current { } {
    global tkWorld

    # Change the working directory.
    wdir::get $tkWorld(working_dir)

    # Clear the CC and execute a dir command...
    cmd_center::reset
    $tkWorld(cmd_center) insert end dir
    cmd_center::run
}